#!/bin/ksh -hp
#
# Copyright 2011 Oracle Corp. All rights reserved.
# Use is subject to license terms.
#
# Please see README.GDD-OC.sh for more info
#

OUTPUT_PREFIX="/var/tmp" #default temp directory 
CASENUM="11111111"

# sanitize environment, more in initialize and look_for_prog
PATH=/usr/bin:/bin:/usr/sbin:/sbin
export PATH

appName="gdd-oc"
postfixName="collect_noarch"
PRODUCT="Ops Center"
PRODUCT_AB="OC"
scriptDir=`dirname $0`
if [[ -f $scriptDir/version ]]; then
    fver="$scriptDir/version"
    . $scriptDir/functions
else
    fver="$scriptDir/../version"
    . $scriptDir/../functions
fi

VERSION=`head -n 1 $fver`
cmdline="$0 $@"
LANG=C
ARGS=$@
ARGC=$#
SWLINK="https://java.net/projects/oc-doctor/downloads"
DOCLINK="http://docs.oracle.com/cd/E40871_01/index.htm"
BUGEMAIL="https://java.net/projects/oc-doctor/forums/General"
SUPPORTSITE="https://transport.oracle.com/upload/issue/{SR NUMBER}/"
UNAME=`uname`

#set to empty to skip jmxdump, otherwise
#username for domain model dump
JUSER=""

#should we collect big tables ?
COLLECT_BIG_TABLES=""

#should we collect tables from pg regarding locks ?
COLLECT_PGLOCK_TABLES=""

#by default wait for key press when auto collect
TIME=0

#in kB
REQUESTED_TEMP_DISK=600000

NULL=/dev/null
TMP=$OUTPUT_PREFIX

# return codes
# 0 success
# 1 command line parser error, wrong option
# 2 need more privileges (run this script as root)
# 3 runtime error while interacting

#
# print error/warning/info message $@ prefixed
#
error() { echo "`date +%Y-%m-%d-%H-%M` ERROR: $@" | $LOG ; }
warning() { echo "`date +%Y-%m-%d-%H-%M` WARNING: $@" | $LOG ; }
# parameter to debug should be a string, even meta chars will be kept like
# string
debug() { echo "`date +%Y-%m-%d-%H-%M` DEBUG: '$@'" >> $LOGFILE ; }

# log(APPEND!) given command $2-$n to $LOGFILE and save its output to $1
# be warned, since the command is called through *eval*, so you might need to
# escape special chars !!! (e.g. \" or \' )
debug_cmd() {
    log=$1
    [[ x"$log" = x"" ]] && log=$LOGFILE
    shift
    echo "`date +%Y-%m-%d-%H-%M` DEBUG: executing command below, output going to $log" >> $LOGFILE
    echo $@ >> $LOGFILE
    eval $@ >>$log 2>&1
}

info()  { echo "`date +%Y-%m-%d-%H-%M` INFO: $@" | $LOG ; }

#
# Find external binaries we need to use on various platforms
# $1 - binary which we look for
# OS variable as output of uname !!!
# returns 1 if not found in path + error message
#
function look_for_prog
{
    typeset path
    typeset ppath
    typeset progname=${1}

    # initialize takes care of this call to save calls
    #    OS=`$uname`
    if [[ x"$OS" = x"SunOS" ]]; then
        ADDPATH="/usr/sfw/bin:/opt/sfw/bin:/usr/xpg4/bin/:/usr/ucb/:/usr/platform/$ARCH/sbin/"
    elif [[ x"$OS" = x"Linux" ]]; then
        ADDPATH=""
    fi

    ppath=$PATH
    ppath=$ppath:/usr/bin:/usr/sbin:/sbin
    ppath=$ppath:$ADDPATH

    if [[ x"$KSH_VERSION" = x"" ]]; then
	#### Not in KSH - it's probably sh - on some RHEL machines with no ksh
	PATH=$ppath prog=`which ${progname} 2>/dev/null`
	if [ -x "$prog" ]; then
	    echo ${prog}
	else
	    echo "Your $OS does not have: ${progname} in $ppath"
	    exit 1
	fi
    else
	PATH=$ppath prog=`whence ${progname}`
	if [[ -n ${prog} ]]; then
	    print ${prog}
	else
	    echo "Your $OS does not have: ${progname} in $ppath"
	    exit 1
	fi
    fi
    
}

#
# Print usage instructions to stdout
#
usage()
{
    cat <<END
$appName tool helps you to automatically gather debugging data needed
to troubleshoot a problem that you can reproduce. For a quick start
you typically need to run:

$scriptName auto-collect

and then follow the onscreen instructions.
The full usage instructions are available by running:

$scriptName -h

If you understand what $appName is doing you can as well use only:

$scriptName collect

END
}

#
# Print detailed usage instructions to stdout
#
detailed_usage()
{
    cat <<END
usage is: $scriptName [options] <command> 

where <command> is one of the following:

    debug

        Will enable full debugging of ${PRODUCT} and will make the product
        ready for problem reproduction.

    collect

        Will collect all files needed for troubleshooting after the problem
        has been reproduced.

    restore

        Will disable debugging of ${PRODUCT} and will revert it back to
        production configuration after the debugging data has been collected.

    auto-collect

        Automates debug/collect/restore cycle. Prompts the user to reproduce
        the problem once the debugging is enabled and reverts back to
        production configuration once the debugging data had been collected.

and where [options] are:

    -o <output_directory>
        Place the collected files in <output_directory>. Default is to place
        the collected files in /var/tmp/.

    -c <case_number>
        Use the given <case_number> in the output filename.
        The default filename if no <case_number> is specified will be:
        11111111-$appName-hostname-date.tar.gz.

    -u <user_name>
        User name used for registering of agents/proxies to satellite
        (administrator of ${PRODUCT})
        Necessary for dumping of domain model. If not provided jmxdump will
        be skipped.
        Usually it is  root .

    -p <password>
        Optional, if not provided, script will ask for password if -u was used. 
        Note: in 12c there is no need to provide password - only username.
        Note: if password was provided (11g and earlier), during the collection
        the password is visible in plaintext, so only dump the "jmxdump" when needed!

    -t <time>
        To be used with auto-collect option.
        The script will auto collect and restore after the time has passed.
        If you specify 0, it will wait for input from stdin.

    -r
        Don't restart services. Default is to restart components
        automatically to have debugging enabled or disabled.

    -b
        Collect big tables (only to be used when suggested by Oracle support)

    -l
        Collect postgresql lock tables (when suspecting that you have vacuum lock problems)

    -s
        Transfer the collected file to Oracle with sftransport.
        You must be connected to internet with working hostnames resolution.
        This also assumes a correctly configured sftransport installation or 
        curl installed ( http://docs.sun.com/app/docs/doc/820-1190/ ).

    -z
        don't compress the output folder afer done collecting the logs

    -x
        skip explorer data collection

    -i
        skip job logs collection

    -d
        skip database tables collection

    -h
        Display these usage instructions.

    -v
        Display scripts version and simple usage.

Note: 
    OCCLI - environment variable pointing to cli binary

Alternatively visit:
$SWLINK
$DOCLINK1

OCCLI: Info: http://wikis.sun.com/display/EMOC11g1CLI/Home

Please do send feedback at: $BUGEMAIL

END
}

# '
# Parse command line options and do the appropriate work
# $@ - all params passed on to this script
#
parseOptions()
{
    NOCOMPRESS="0"
    SKIPEXPLORER="0"
    SKIPJOBS="0"
    SKIPTABLES="0"

    # Check if user put the command first, instead of options
    COMMAND_ARG=""
    case "$1" in
        debug)
            COMMAND_ARG="debug"
            shift
            ;;
        collect)
            COMMAND_ARG="collect"
            shift
            ;;
        restore)
            COMMAND_ARG="restore"
            shift
            ;;
        auto-collect)
            COMMAND_ARG="auto-collect"
            shift
            ;;
        *)
            ;;
    esac
    
    
    # Parse the options
    while getopts "adlbt:o:c:u:p:rhsxivz?" opt
    do
        case $opt in
            t)
                TIME="$OPTARG"
                ;;
            o)
                OUTPUT_PREFIX="$OPTARG"
                ;;
            c)
                CASENUM="$OPTARG"
                ;;
            u)
                JUSER="$OPTARG"
                ;;
            p)
                JPASSWORD="$OPTARG"
                ;;
            r)
                RESTART="false"
                ;;
            b)
                COLLECT_BIG_TABLES="1"
                ;;
            l)
                COLLECT_PGLOCK_TABLES="1"
                ;;
            s)
                TRANSFER="1"
                ;;
            h)
                detailed_usage
                exit 1
                ;;
            z)
                NOCOMPRESS="1"
                ;;
            x)
                SKIPEXPLORER="1"
                ;;
            i)  SKIPJOBS="1"
                ;;
            d)
                SKIPTABLES="1"
                ;;
            v)
                echo "$appName Version $VERSION"
                usage
                exit 1
                ;;
            # below "works" only for Solaris
            '?')
                detailed_usage
                exit 1
                ;;
            ?)
                echo ""
                echo "Invalid option specified."
                echo ""
                echo "Please use $scriptName -h to see detailed usage instructions."
                echo ""
                exit 1
                ;;
        esac
    done

    if [[ x$JUSER != x"" ]]; then
	# since version 11g, we can use the flag 'wellknown' for the password and do not need to prompt for it
	if [ "$UNAME" = "Linux" ];then
            OCRELEASE=`rpm -q --qf '%{VERSION}' sun-scn-agentfacade-u | cut -d. -f1`
            RETVAL=$?
            [ "${RETVAL}" -ne 0 ] &&  OCRELEASE=0
	else
            OCRELEASE=`/usr/bin/pkgparam SUNWscn-agentfacade-u VERSION | cut -d. -f1`
            [ "${RETVAL}" -ne 0 ] &&  OCRELEASE=0
	fi
	if [[ "${OCRELEASE}" -ge "11" ]]; then
            # 11.x or later
            JPASSWORD="wellknown"
	else
            if [[ x$JPASSWORD = x"" ]]; then
		echo "User name for domain model dump was set, but no password found."
		echo "! WARNING ! : during jmxdump collection the roots password can be seen in process list as parameter, so please avoid unprivileged users looking at processes during this data collection(this is a bug and will be fixed once underlaying tools are fixed)."
		echo "Enter password for $JUSER: "
		stty -echo
		read JPASSWORD
		stty echo
		echo
            fi
	fi
    fi

    if [[ x$COMMAND_ARG = x"" ]]; then
	#### Command wasn't given first (default)
	shift $(($OPTIND - 1))
	#### Use the last parameter as command
	SWITCHON=$1
    else
    	### User provided command first, instead of options...
    	#### Use the first parameter as command
    	SWITCHON=$COMMAND_ARG
    fi
    
    AUTO_COLLECT="false"
    
    # Do the work
    case "$SWITCHON" in
        debug)
            # stop it, enable logging, start it
            checkUser
            prepareOutputDirectory "limited"
            #verifyPermissions
            enableAll
            #removeWorkingDirectory
            ;;
        collect)
            # capture logs
            checkUser
            prepareOutputDirectory
            #verifyPermissions
            collectAll
            compressOutput
            removeWorkingDirectory
            ;;
        restore)
            # stop it, disable logging, start it
            checkUser
            prepareOutputDirectory "limited"
            #verifyPermissions
            disableAll
            #removeWorkingDirectory
            ;;
        auto-collect)
            AUTO_COLLECT="true"
            warning "auto-collect WILL restart your services, consider whether you wanted just use _ collect _ instead"
            # stop it, enable logging, start it
            # pause, capture logs, stop it, disable logging, start it
            checkUser
            prepareOutputDirectory
            #verifyPermissions
            enableAll
            echo ""
            info "Your ${PRODUCT_AB} is now ready for problem reproduction."
            checkTime
            debug_cmd "" sleep $TIME
            if [[ $TIME -eq "0" ]]; then
		info "Please reproduce the problem and press <Enter> when ready to collect."
		read
            fi
            info "Thank You."
            echo ""
            collectAll
            disableAll
            compressOutput
            removeWorkingDirectory
            ;;
        *)
            usage
            exit 1
            ;;
    esac
}

#check whether time from parameter is OK, if not ask for new one
checkTime()
{
    if [[ "$TIME" = ?(+|-)+([0-9]) ]]
    then
	if [[ $TIME -ge "600" && $TIME -le "99999" ]]
	then
            info "Will wait for $TIME seconds before collecting data"
	elif [[ $TIME -eq "0" ]]
	then
            info "No time provided, wait for user input."
	elif [[ $TIME -lt "600" || $TIME -gt "99999" ]]
	then
            echo "Are you really sure you want to wait $TIME seconds? (Y/N)"
            read -r yesNo
            case $yesNo in
		'y'|'Y'|'yes'|'YES')
                    info "Waiting for $TIME seconds before collecting data"
                    ;;
		'n'|'N'|'no'|'NO')
                    echo "Please enter a new time: "
                    read -r TIME
                    checkTime
                    ;;
		*)
                    echo "Please answer Y or N"
                    checkTime
                    ;;
            esac
	fi
    else
	warning "Invalid time entered"
	echo "Please enter a time in seconds for the delay before collecting data:"
	read -r TIME
	checkTime
    fi
}

# force detection of packages in case customers later remove them
detect_packages_sol()
{
    info "Checking for ${PRODUCT_AB} needed packages (satellite/proxy)"
    #options="-R /opt/sun/n1gc/pkgs"
    #pkginfo -q $options $name  (if 0 then installed)
    #pkg list $name > $NULL # for ips

    # PeterCHs Minimalization:
    FIELD="SUNWctpls SUNWxwfnt SUNWxwplr SUNWj5rt SUNWpoolr SUNWpool SUNWadmfr SUNWadmfw SUNWscpr SUNWscpu SUNWlur SUNWluu SUNWlucfg SUNWluzone SUNWzoner SUNWzoneu SUNWtoo SUNWlxsl SUNWinst SUNWgzip SUNWzip SUNWuiu8"

    # prereq scripts
    STOCK="SUNWPython SUNWPython-share SUNWTcl SUNWTk SUNWant SUNWapch2r SUNWapch2u SUNWarc SUNWbash SUNWbip SUNWbinutils SUNWbzip SUNWftpr SUNWftpu SUNWgcc SUNWgccruntime SUNWgcmn SUNWgss SUNWgssc SUNWgssk SUNWgtar SUNWhea SUNWj3rt SUNWj3dev SUNWlexpt SUNWlibm SUNWlibmr SUNWmfrun SUNWnfsckr SUNWnfscr SUNWnfscu SUNWnfsskr SUNWnfssr SUNWnfssu SUNWperl584core SUNWperl584usr SUNWperl-xml-parser SUNWrsgk SUNWsprot SUNWsshcu SUNWsshdr SUNWsshdu SUNWsshr SUNWsshu SUNWwgetr SUNWwgetu SUNWxcu4 SUNWxwice SUNWxwplt SUNWxwrtl SUNWzlib"

    for p in $FIELD $STOCK
    do
	debug_cmd "" pkginfo -q $p
	RC=$?
	if [[ $RC != 0 ]] ; then
            warning "Package $p is missing, the installation was tainted and is unsupported!"
	fi
    done

}

#
# detect installed components
#
detect()
{

    if [[ x"$OS" = x"SunOS" ]]; then
	INSTALLED_LOC=/opt/SUNWxvmoc/bin
    elif [[ x"$OS" = x"Linux" ]]; then
	INSTALLED_LOC=/opt/sun/xvmoc/bin
    else
	error "This OS: $OS is unsupported"
	exit 3
    fi

    if [[ ! -d ${INSTALLED_LOC} ]]; then
	error "Cannot locate ${PRODUCT_AB} directory: $INSTALLED_LOC"
	error "This machine seems not to have any ${PRODUCT_AB} installed, exiting ..."
	exit 3
    fi

    STCLIENT="echo stclient NA"
    IPMITOOL="echo ipmitool NA"
    IPMIFLASH="echo ipmiflash NA"
    AS_CLIENT="echo as_client.pl NA"
    AS_CLIENT_PROXY="echo as_client_proxy.pl NA"

    #common
    if [[ x"$OS" = x"SunOS" ]]; then
	CACAO_BIN=/usr/lib/cacao/bin
	CACAOADM=${CACAO_BIN}/cacaoadm
	CACAO_VAR=/var/cacao/instances
	CACAO_ETC=/etc/cacao
	CACAO_ETCI=$CACAO_ETC/instances
	XVM_VBASE=/var/opt/sun/xvm
	JET_BASE=/opt/SUNWjet
	JET_VAR=/var/opt/sun/jet
	# TODO /var/opt/sun/xvm/bui/work/Catalina/localhost/xvmoc//WEB-INF/classes/logging.properties  - war is not decompressed anymore
	WEBAPP_DIR=${XVM_VBASE}/bui/webapps/xvmoc/
	#OSP_DIR=/var/opt/SUNWscnosp
	STCLIENT=/usr/bin/stclient
	IPMITOOL=/opt/sun/n1gc/pkgs/usr/sbin/ipmitool
	IPMIFLASH=/opt/ipmitool/bin/ipmiflash
	# legacy: /usr/sbin/ipmitool
	# or /opt/ipmitool/bin/ipmitool (obsolete)
	# or internal: /opt/sun/n1gc/pkgs/usr/sbin/ipmitool
	AS_CLIENT=/opt/SUNWscs/sbin/as_client.pl
	AS_CLIENT_PROXY=/opt/SUNWscnosp/sbin/as_client_proxy.pl
	SFTRANSPORT=/opt/SUNWsftransport/bin/sftransport
	DBTOOL=/opt/SUNWscs/sbin/db_tool.pl
	UCE_BACKUP=/opt/UCE_backup
	#TODO below might not be valid for 2.5 clean install ...
	JAVA_HOME=/usr/jdk/latest
    elif [[ x"$OS" = x"Linux" ]]; then
	XVM_VBASE=/var/opt/sun/xvm
	CACAO_BIN=/opt/sun/cacao2/bin
	CACAOADM=${CACAO_BIN}/cacaoadm
	CACAO_VAR=/var/opt/sun/cacao2/instances
	CACAO_ETC=/etc/opt/sun/cacao2
	CACAO_ETCI=$CACAO_ETC/instances
	JET_BASE=/opt/SUNWjet
	JET_VAR=/var/opt/sun/jet
	#TODO below might not be valid for 2.5 : /var/opt/sun/xvm/bui/work/Catalina/localhost/xvmoc//WEB-INF/classes/log4j.xml is used instead  the war just stays there
	WEBAPP_DIR=${XVM_VBASE}/bui/webapps/xvmoc/
	#OSP_DIR=/var/opt/sun/scnosp
	STCLIENT=/opt/sun/servicetag/bin/stclient
	IPMITOOL=/usr/bin/ipmitool
	IPMIFLASH=/usr/bin/ipmiflash
	AS_CLIENT=/opt/sun/scs/sbin/as_client.pl
	AS_CLIENT_PROXY=/opt/sun/scnosp/sbin/as_client_proxy.pl
	SFTRANSPORT="echo sftransport NA"
	DBTOOL=/opt/sun/scs/sbin/db_tool.pl
	UCE_BACKUP=/usr/local/UCE_backup
	JAVA_HOME=/usr/java/latest
    fi

    # check for as_client.pl (since it changed in 2.1)
    if ! [[ -x $AS_CLIENT ]]; then
        if [[ x"$OS" = x"SunOS" ]]; then
            AS_CLIENT=/opt/SUNWscnosp/sbin/as_client.pl
        elif [[ x"$OS" = x"Linux" ]]; then
            AS_CLIENT=/opt/sun/scnosp/sbin/as_client.pl
        fi
    fi

    #sanity check for stclient binary
    if ! [[ -x ${STCLIENT} ]]; then
        x=`look_for_prog stclient`
        if [[ $? -ne 0 ]]; then
            echo WARNING: $x
        else
            STCLIENT=$x
        fi
    fi

    #because of jmxdump port detection(not really needed,just a fall back)
    export CACAO_BIN

    AGENT=0
    AGENTUP=0
    PROXY=0
    PROXYUP=0
    SATELLITE=0
    SATUP=0

    if [[ -x $INSTALLED_LOC/agentadm ]]; then
	info "Detected agent binaries"
	AGENT=1
	if [[ x"$OS" = x"SunOS" ]]; then
            ABASEBIN=/usr/lib/scn/update-agent
            ABASEVAR=/var/scn/update-agent
	elif [[ x"$OS" = x"Linux" ]]; then
            ABASEBIN=/opt/sun/scn/update-agent
            ABASEVAR=/var/opt/sun/scn/update-agent
	fi

	debug_cmd "" $CACAOADM status -i scn-agent
	STATUS=$?
	if [[ $STATUS -eq 0 ]]; then
	    debug_cmd "" "grep \"not running\" $LOG"
	    STATUS=$?
	    if ! [[ $STATUS -eq 0 ]]; then
		info "Agent running"
		AGENTUP=1
	    fi
	fi
    fi

    if [[ -x $INSTALLED_LOC/proxyadm ]]; then
	info "Detected proxy binaries"
	PROXYADM=$INSTALLED_LOC/proxyadm
	PROXY=1

	debug_cmd "" $PROXYADM status
	STATUS=$?
	if [[ $STATUS -eq 0 ]]; then
	    info "Proxy running"
	    PROXYUP=1
	fi

    fi

    if [[ -x $INSTALLED_LOC/satadm ]]; then
	info "Detected satellite binaries"
	SATADM=$INSTALLED_LOC/satadm
	SATELLITE=1

	debug_cmd "" $SATADM status
	STATUS=$?
	if [[ $STATUS -eq 0 ]]; then
	    info "Satellite running"
	    SATUP=1
	fi

    fi

    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then
	if [[ x"$OS" = x"SunOS" ]]; then
	    INSTALLPATH=/opt/SUNWuce
	    #agent is stored in /usr ... but server is still in /opt
	    INSTALLPATH_ETC=/etc/opt/SUNWuce
	    SERVER_ETC=$INSTALLPATH_ETC/server
	    if [ -d /var/opt/sun/xvm/uce/etc.opt/server ]; then			## 12c
	    	SERVER_ETC=/var/opt/sun/xvm/uce/etc.opt/server
	    fi
	    OCCLIP="${OCCLI:-/opt/SUNWoccli/bin}"

            CONSOLE=$INSTALLPATH/console
	    INSTALL=$INSTALLPATH/install
	    SERVER=$INSTALLPATH/server
	    if [ -d /var/opt/sun/xvm/uce/opt/server/ ]; then		# 12c
	    	SERVER=/var/opt/sun/xvm/uce/opt/server
	    fi
	elif [[ x"$OS" = x"Linux" ]]; then
	    INSTALLPATH=/usr/local/uce
	    SERVER_ETC=/etc/uce_server
	    if [ -d /var/opt/sun/xvm/uce/etc ]; then			## 12c
	    	SERVER_ETC=/var/opt/sun/xvm/uce/etc
	    fi
	    OCCLIP="${OCCLI:-/opt/sun/occli/bin}"
	    
	    CONSOLE=$INSTALLPATH/console
	    INSTALL=$INSTALLPATH/install
	    SERVER=$INSTALLPATH/server
	    if [ -d /var/opt/sun/xvm/uce/usr.local/server/ ]; then		# 12c
	    	SERVER=/var/opt/sun/xvm/uce/usr.local/server
	    fi
	fi


	SC_FILE=$WEBAPP_DIR/WEB-INF/classes/logging.properties
	SCPROP_FILE=$WEBAPP_DIR/WEB-INF/classes/log4j.xml
	PUBLIC_FOLDER="$SERVER/public"
	SERVER_LOGS=$SERVER/logs
	if [ "$UNAME" = "Linux" ];then
	    if [ -f /var/opt/sun/xvm/uce/usr.local/server/cgi-bin/.uce.rc ]; then
	        DOT_UCE_RC="/var/opt/sun/xvm/uce/usr.local/server/cgi-bin/.uce.rc"
	        UCE_RC="/var/opt/sun/xvm/uce/usr.local/server/cgi-bin/uce.rc"
	    else
	        DOT_UCE_RC="/usr/local/uce/server/cgi-bin/.uce.rc"
	        UCE_RC="/usr/local/uce/server/cgi-bin/uce.rc"
	    fi
	    if [ -d /var/opt/sun/xvm/uce/usr.local/server/logs ]; then		# 12c
	        SERVER_LOGS=/var/opt/sun/xvm/uce/usr.local/server/logs
	    fi
	    if [ -d /var/opt/sun/xvm/uce/usr.local/server/public/ ]; then	# 12c with no update library goes here
	        PUBLIC_FOLDER=/var/opt/sun/xvm/uce/usr.local/server/public/
	    fi
	else
	    if [ -f /var/opt/sun/xvm/uce/opt/server/cgi-bin/.uce.rc ]; then
	        DOT_UCE_RC="/var/opt/sun/xvm/uce/opt/server/cgi-bin/.uce.rc"
	        UCE_RC="/var/opt/sun/xvm/uce/opt/server/cgi-bin/uce.rc"
	    else
	        DOT_UCE_RC="/opt/SUNWuce/server/cgi-bin/.uce.rc"
	        UCE_RC="/opt/SUNWuce/server/cgi-bin/uce.rc"
	    fi
	    if [ -d /var/opt/sun/xvm/uce/var.opt/server/logs ]; then		# 12c
	        SERVER_LOGS=/var/opt/sun/xvm/uce/var.opt/server/logs
	    fi
	    if [ -d /var/opt/sun/xvm/uce/opt/server/public ]; then		#12c with no update library goes here
	        PUBLIC_FOLDER=/var/opt/sun/xvm/uce/opt/server/public
	    fi
	fi

	### Determine library path
	LIBRARY_FOLDER=""
	PUBLIC_FOLDER_TMP=`grep "public_dir_name" $DOT_UCE_RC 2>/dev/null | awk -F"\"" '{print $2}' 2>/dev/null`
	if [ ! "$PUBLIC_FOLDER_TMP" = "" ]; then
	    if [ -d $PUBLIC_FOLDER_TMP ]; then			### 12c has libraries
		debug "Update library location: $PUBLIC_FOLDER_TMP"
		PUBLIC_FOLDER=$PUBLIC_FOLDER_TMP
		LIBRARY_FOLDER=`dirname $PUBLIC_FOLDER`
	    fi
	fi
    fi

    N1SH=/opt/sun/n1gc/bin/n1sh

    SHOWJOB=/opt/sun/xvm/bin/show_job.pl

    JMXDUMP=/opt/sun/xvm/bin/jmxdump
    if [ -x /opt/sun/xvm/lib/jmxdump/jmxdump ]; then
	JMXDUMP=/opt/sun/xvm/lib/jmxdump/jmxdump
    fi

    # we will no longer collect the entire content since this will collect all the __local MBeans as well
    #JMXDUMP_ARG="*com.sun.hss.domain:*"
    JMXDUMP_ARG="com.sun.hss.domain:*"

    OCBIN="oc"
    OCCLI="$OCCLIP/$OCBIN"

    #OCCLIV=`OCCLI -V`
    # e.g. #OpsCenter Command Line Interface in 11g

    if ! [[ -x ${OCCLI} ]]; then
	# try to fall back to local cli if global not found
	OCCLI="$scriptDir/../cli/$OCBIN"
    fi

    if ! [[ -x ${OCCLI} ]]; then
	warning "${OCCLI} is not found or not set as executable, trying to search for it ... "
	x=`look_for_prog $OCBIN`
	if [[ $? -ne 0 ]]; then
	    warning $x
	    warning "Be aware, that GDD cannot collect info about jobs, notifications or profiles without $OCBIN, so one might need to add screen shots "
	    warning "Will try to use other means to get job listing (show job perl scripts or n1sh if available)"
	else
	    OCCLI=$x
	fi
    fi

    CACAO_INS=`$CACAOADM list-instances | cut -d' ' -f 1`
    # TODO maybe we should just propagate above list according to what was found in detection of instances
    # TODO below log size increase uses : default , scn-proxy and scn-agent as plain strings, should be used in above and kept in variables (detection?) 
    # or maybe not, since in OC 11.1g we went from default to oem-ec

}


#
# Enable debugging on all components
#
enableAll()
{
    if [[ $AGENT -eq 1 ]]; then
	#    info "Enabling agent debug: "
	enableDebugAgent
    fi

    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then
	#    info "Enabling satellite/proxy debug: "
	enableDebugServer
	#enableDebugConsole
    fi

    #info "Enabling $PRODUCT_AB debug:"
    enableDebugXvm
    info "Done enabling debug."
}

#
# Disable debugging on all components
#
disableAll()
{
    if [[ $AGENT -eq 1 ]]; then
	info "Disabling agent debug: "
	restoreDebugAgent
    fi

    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then
	info "Disabling satellite/proxy debug: "
	restoreDebugServer
	#restoreDebugConsole
    fi

    info "Disabling $PRODUCT_AB debug:"
    restoreDebugXvm
    info "Done disabling debug."
}



checkAvailableSpace()
{

    SPACE_MULTIPLIER=2
    if [[ ! x"$KSH_VERSION" = x"" ]]; then		# run this in ksh only
	set -A fileList
    fi

    #checking space required for cacao stuff
    for i in $CACAO_INS
    do
	fileList[$((${#fileList[*]}+1))]="$CACAO_VAR/$i/logs"
	fileList[$((${#fileList[*]}+1))]="$CACAO_VAR/$i/audits"
    done

    #checking space required for satellite and proxy stuff
    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then
	fileList[$((${#fileList[*]}+1))]="${XVM_VBASE}/logs"
	fileList[$((${#fileList[*]}+1))]="${XVM_VBASE}/bui/logs"
	fileList[$((${#fileList[*]}+1))]="${XVM_VBASE}/ha"
	fileList[$((${#fileList[*]}+1))]="${XVM_VBASE}/osp/logs"
	fileList[$((${#fileList[*]}+1))]="${XVM_VBASE}/persistence"

	#checking space required for uce stuff
	fileList[$((${#fileList[*]}+1))]="$SERVER_LOGS/"
	fileList[$((${#fileList[*]}+1))]="$SERVER_ETC/"
	fileList[$((${#fileList[*]}+1))]="$PUBLIC_FOLDER/public_blobs/*.blob.err"
	fileList[$((${#fileList[*]}+1))]="$CONSOLE/logs"
	fileList[$((${#fileList[*]}+1))]="$CONSOLE/bin/*uce.*"
    fi

    totalSize=0
    fileCount=${#fileList[*]}
    for name in ${fileList[*]}
    do 
	#calculating the required space for each directory
	if [[ -f $name || -d $name ]]; then
    	    if [[ ! x"$KSH_VERSION" = x"" ]]; then		# run this in ksh only
        	integer sizeInKB=0
            else
        	sizeInKB=0
            fi
            if [[ x"$OS" = x"SunOS" ]]; then
		sizeInKB=`du -Lks $name | nawk '{print $1}' | tail -1`
            elif [[ x"$OS" = x"Linux" ]]; then
		#Linux requires -P option explicitly for du in for the output to be POSIX compliant!
		sizeInKB=`du -LksP $name | nawk '{print $1}' | tail -1`
            fi 
            totalSize=`expr $(($totalSize + $sizeInKB))`
	fi
    done

    totalSize=`expr $((($totalSize * $SPACE_MULTIPLIER)/1024))`
    info "Approximate space required for log collection: ${totalSize} MB"

    #calculating the free space available in the output directory
    if [[ x"$OS" = x"SunOS" ]]; then
	freeSpace=`df -k ${OUTPUT_PREFIX} | nawk '(NR>1) {print $4}'`
    elif [[ x"$OS" = x"Linux" ]]; then
	#Linux requires -P option explicitly for df in for the output to be POSIX compliant!
	freeSpace=`df -Pk ${OUTPUT_PREFIX} | nawk '(NR>1) {print $4}'`
    fi

    freeSpace=`expr $freeSpace / 1024`
    info "Space available in the output directory (${OUTPUT_PREFIX}): ${freeSpace} MB"

    #exit with error if enough space not available
    if [[ ${totalSize} -gt ${freeSpace} ]]
    then
	error "The output directory $OUTPUT_PREFIX has only $freeSpace MB of free space."
	error "$totalSize MB of free space is approximately required for log collection."
	error "Please free up some space or use -o switch to redirect output somewhere else."
	#debug_cmd "" rm -rf $outdir
	exit 3 
    else
	info "Enough space available ..."
    fi
}

#
# runs the necessary collector scripts or remote probes		
#
collectAll()
{

    if [[ x$SKIPEXPLORER == x"1" ]] ; then
	info "SKIPPING collection of system data as user requested ..."
    else
	info "Collecting system data ..."

	#explorer like
	debug_cmd "$OUTEXPL/alias" alias
	debug_cmd "$OUTnetinfo/netstat-an.out" netstat -an
	debug_cmd "$OUTnetinfo/netstat-rn.out" netstat -rn
	debug_cmd "$OUTnetinfo/nfsstat.out" nfsstat
	debug_cmd "$OUTnetinfo/nfsstat-m.out" nfsstat -m
	debug_cmd "$OUTsysconfig/ifconfig-a.out" ifconfig -a
	debug_cmd "$OUTsysconfig/eeprom.out" eeprom
	debug_cmd "$OUTsysconfig/dispadmin-l.out" dispadmin -l
	debug_cmd "" mkdir $OUTetc/default
	debug_cmd "" cp -p /etc/default/login $OUTetc/default/
	debug_cmd "" cp -p /etc/host* $OUTetc/
	debug_cmd "" cp -p /etc/nsswitch.conf $OUTetc/
	debug_cmd "" cp -p /etc/sysidcfg $OUTetc/
	debug_cmd "" cp -p /etc/resolv.conf $OUTetc/
	debug_cmd "" cp -p /etc/syslog.conf $OUTetc/
	debug_cmd "" cp -p /etc/netmasks $OUTetc/
	debug_cmd "$OUTEXPL/date.out" date
	debug_cmd "" cp -p /etc/profile $OUTEXPL
	debug_cmd "" cp -p ~/.profile $OUTEXPL
	debug_cmd "$OUTEXPL/env" env
	debug_cmd "$OUTEXPL/locale" locale
	debug_cmd "$OUTEXPL/stclient-x" $STCLIENT -x
	debug_cmd "$OUTEXPL/last-50" last -50
	# Get disk status
	debug_cmd "$OUTdisks/df-kl.out" df -kl
	debug_cmd "$OUTdisks/df-k.out" df -k
	debug_cmd "$OUTdisks/df-h.out" df -h
	debug_cmd "$OUTdisks/swap-l.out" swap -l
	debug_cmd "$OUTdisks/swap-s.out" swap -s
	debug_cmd "$OUTdisks/mount-v.out" mount -v
	debug_cmd "$OUTdisks/mount.out" mount
	#jmxdump password workaround
	debug_cmd "$OUTsysconfig/ps-ef.out" "ps -ef | grep -v jmxdump"
	debug_cmd "$OUTsysconfig/vmstat-1-5.out" vmstat 1 5
	debug_cmd "$OUTsysconfig/vmstat-3-3.out" vmstat 3 3
	if [[ x"$OS" = x"SunOS" ]]; then		### only on Solaris
	    debug_cmd "$OUTsysconfig/prstat-L.out" prstat -L 1 1
	    debug_cmd "$OUTsysconfig/prstat.out" prstat 1 1
	fi
	debug_cmd "$OUTsysconfig/uname-a.out" uname -a
	debug_cmd "$OUTsysconfig/umask.out" echo $UMASK
	debug_cmd "$OUTsysconfig/who-r.out" who -r
	debug_cmd "$OUTsysconfig/crontab-l.out" crontab -l
	debug_cmd "" cp -p /etc/inittab $OUTetc

	debug_cmd "$OUTinit/ls-lLa_init.d.out" ls -lLa /etc/init.d
	debug_cmd "" cp -rp /etc/init.d $OUTinit/
	debug_cmd "" cp -rp /etc/rc* $OUTinit/

	debug_cmd "" cp -p /etc/passwd $OUTetc/
	debug_cmd "" cp -p /etc/group $OUTetc/

	debug_cmd "$OUTmessages/dmesg.out" dmesg 

	if [[ x$OS = x"SunOS" ]]; then

	    # collect network info
	    debug_cmd "$OUTnetinfo/ifconfig-a.out" ifconfig -a
	    if [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
		debug_cmd "$OUTnetinfo/ipadm_show-if.out" ipadm show-if
		debug_cmd "$OUTnetinfo/ipadm_show-addr.out" ipadm show-addr
		debug_cmd "$OUTnetinfo/dladm_show-phys.out" dladm show-phys
		debug_cmd "$OUTnetinfo/dladm_show-phys-m.out" dladm show-phys -m
		debug_cmd "$OUTnetinfo/dladm_show-link.out" dladm show-link
		debug_cmd "$OUTnetinfo/dladm_show-link-Z.out" dladm show-link -Z
		debug_cmd "$OUTnetinfo/dladm_show-vnic.out" dladm show-vnic
		debug_cmd "$OUTnetinfo/dladm_show-aggr.out" dladm show-aggr
		debug_cmd "$OUTnetinfo/dladm_show-aggr-x.out" dladm show-aggr -x
	    else
		debug_cmd "$OUTnetinfo/dladm_show-dev" dladm show-dev
		debug_cmd "$OUTnetinfo/dladm_show-link.out" dladm show-link
		debug_cmd "$OUTnetinfo/dladm_show-aggr.out" dladm show-aggr
		debug_cmd "$OUTnetinfo/dladm_show-aggr-L.out" dladm show-aggr -L
	    fi
	    
	    debug_cmd "$OUTsysconfig/coreadm.out" coreadm
	    debug_cmd "" cp -p /etc/coreadm.conf $OUTetc/

	    # collect detailed disk info for Solaris js
	    debug_cmd "" mkdir -p "$OUTdisks/prtvtoc"
	    debug_cmd "$OUTdisks/format.out" "echo 0 | format 2>&1 | sed -e '/^Specify disk/,\$d'"
	    # gather disk partition information
	    for i in `/bin/ls /dev/rdsk/*s2 2>/dev/null`
	    do
		diskname=`echo $i| sed s/.dev.rdsk.//|sed s/s2//| sed s/\$/\&s/`
		# we need devinfo to find a non zero length partition, since prtvtoc
		# will return "No such device or address" on a zero length partition.
		# prtvtoc will be executed on one partition only as the info is the same
		# for all partitions.
		for k in 0 1 2 3 4 5 6 7
		do
		    if test "`devinfo -p /dev/rdsk/$diskname$k 2>$NULL`" != ""
		    then
			debug_cmd "$OUTdisks/prtvtoc/$diskname$k" "prtvtoc /dev/rdsk/$diskname$k"
			break
		    fi
		done
	    done
	    debug_cmd "$OUTEXPL/crle" crle
	    debug_cmd "" cp -p /etc/release $OUTetc
	    debug_cmd "" cp -p /etc/vfstab $OUTetc
	    debug_cmd "" cp -p /etc/defaultrouter $OUTetc
	    debug_cmd "" cp -p /etc/defaultdomain $OUTetc
	    debug_cmd "" cp -p /etc/path_to_inst $OUTetc
	    debug_cmd "$OUTvar/ls-lL_yp_binding.out" ls -lL /var/yp/binding/
	    debug_cmd "" cp -p /var/sadm/system/admin/CLUSTER $OUTvar
	    debug_cmd "" cp -p /var/sadm/install/contents $OUTvar
	    debug_cmd "" cp -p /var/adm/messages* $OUTmessages/
	    debug_cmd "$OUTsysconfig/iostat-E.out" iostat -E 
	    # jmxdump password workaround
	    debug_cmd "$OUTsysconfig/ptree.out" "ptree | grep -v jmxdump"
	    debug_cmd "$OUTsysconfig/ptree-c.out" "ptree -c | grep -v jmxdump"
	    debug_cmd "$OUTsysconfig/ptree-z_global.out" "ptree -z global | grep -v jmxdump"

	    if [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
		debug_cmd "$OUTpatch/pkg-list.out" pkg list
		debug_cmd "$OUTpatch/pkg-publisher.out" pkg publisher
	    else			### no showrev and patchadd in Solaris 11
    		debug_cmd "$OUTpatch/showrev-p.out" showrev -p
    		debug_cmd "$OUTpatch/patchadd-p.out" patchadd -p
    		debug_cmd "$OUTpatch/patch-list" "egrep -e '^Patch' $OUTpatch/showrev-p.out | nawk '{print \$2}' | sort"
	    fi

	    debug_cmd "$OUTpatch/pkginfo.out" pkginfo 
	    debug_cmd "$OUTpatch/pkginfo-p.out" pkginfo -p
	    debug_cmd "$OUTpatch/pkginfo-l.out" pkginfo -l
	    for p in scn hss cacao j6 dhcp scs
	    do
		for i in `grep $p $OUTpatch/pkginfo.out | nawk '{print $2}'`
		do
		    debug_cmd "$OUTpatch/$i.out" pkginfo -l $i
		done
	    done
	    debug_cmd "$OUTpatch/ipmitool.out" pkginfo -l ipmitool
	    debug_cmd "$OUTpatch/SUNWmc.out" pkginfo -l SUNWmc

	    debug_cmd "$OUTsysconfig/psrinfo.out" psrinfo
	    debug_cmd "$OUTsysconfig/isainfo.out" isainfo
	    debug_cmd "$OUTsysconfig/isainfo-kv.out" isainfo -kv
	    debug_cmd "$OUTsysconfig/prtconf-vp.out" prtconf -vp
	    debug_cmd "$OUTsysconfig/prtconf-v.out" prtconf -v

	    # Solaris 10 specifics + collect smf logs + jstacks
	    if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then

		debug_cmd "$OUTsysconfig/zoneadm-list-cp.out" zoneadm list -cp
		debug_cmd "$OUTsysconfig/zoneadm-list-ivc.out" zoneadm list -ivc
		debug_cmd "$OUTsysconfig/zoneadm-list-cv.out" zoneadm list -cv
		debug_cmd "$OUTsysconfig/zoneadm-list-iv.out" zoneadm list -iv
		debug_cmd "$OUTsysconfig/zoneadm-list-ip.out" zoneadm list -ip
		debug_cmd "$OUTsysconfig/zonename.out" zonename

		#collecting zone information
		ZONELIST=`/usr/sbin/zoneadm list -cp | grep -v "global"`
		for ZONE in $ZONELIST
		do
		    ZONENAME=`echo $ZONE | nawk -F: '{print $2}'`
		    ZONEPATH=`echo $ZONE | nawk -F: '{print $4}'`
		    OPT="zlogin:zone:${ZONENAME}"
		    debug_cmd "" mkdir -p "$OUTzones/$ZONENAME/patch+pkg"
		    debug_cmd "" mkdir -p "$OUTzones/$ZONENAME/netinfo"
		    debug_cmd "$OUTzones/$ZONENAME/zonecfg-info" zonecfg -z "$ZONENAME" info
		    debug_cmd "$OUTzones/$ZONENAME/zonecfg-info-fs" zonecfg -z "$ZONENAME" info fs
		    debug_cmd "$OUTzones/$ZONENAME/coreadm.conf" cat $ZONEPATH/root/etc/coreadm.conf
		    debug_cmd "$OUTzones/$ZONENAME/patch+pkg/pkginfo-p" pkginfo -p "${OPT}"
		    debug_cmd "$OUTzones/$ZONENAME/netinfo/nfsstat-m.out" nfsstat -m "${OPT}"

		    if [ -f $ZONEPATH/root/etc/sysidcfg ]
		    then
			debug_cmd "$OUTzones/$ZONENAME/sysidcfg" cat $ZONEPATH/root/etc/sysidcfg
		    fi
		done

		#collecting zfs info
		debug_cmd "$OUTdisks/zpool_status.out" zpool status
		debug_cmd "$OUTdisks/zfs_list.out" zfs list
		
		#   info "Collecting smf information and java jstacks (this might take a while) ..."
		debug_cmd "$OUTsysconfig/svcs-a.out" svcs -a
		debug_cmd "$OUTsysconfig/svcs-xv.out" svcs -xv
		debug_cmd "$OUTsysconfig/svcs_${PRODUCT_AB}.out" svcs \'*scn*\' \'*common*\' \'*dhcp*\' \'*ldom*\' \'*xvm*\' \'*oczones*\' \'*ldmd*\' \'*zstat*\'
		SCNSVCS=svcs-lp_${PRODUCT_AB}.out
		for e in `cat $OUTsysconfig/svcs_${PRODUCT_AB}.out | grep -v doesn\'t | grep -v FMRI | grep -v lrc:/  | nawk '{print $3}'`
		do
		    echo "***************************************************"  >> \
			$OUTsysconfig/$SCNSVCS
		    echo "" >> $OUTsysconfig/$SCNSVCS
		    echo "Service is " $e >> $OUTsysconfig/$SCNSVCS
		    echo "" >> $OUTsysconfig/$SCNSVCS
		    CMD="svcs -lp $e | tee $OUTsmf/tmp >> $OUTsysconfig/$SCNSVCS"
		    debug $CMD
		    eval $CMD 2>&1
		    # now collect the rest
		    SMFLOG=`cat $OUTsmf/tmp | grep logfile | nawk '{print $2}'`
		    if [[ -f $SMFLOG ]] ; then
			debug_cmd "" cp $SMFLOG $OUTsmf/
		    fi
		    # TODO test if java, if yes, grab jstack
		    # not needed yet, if needed, grep for process , grep java , test $? for 0 
		    # jstack -F (newgrp sys might be needed)
		    debug_cmd "" rm $OUTsmf/tmp
		    echo "" >> $OUTsysconfig/$SCNSVCS
		done

		# full list of required SMF logs for Solaris 10 single user patching issues
		debug_cmd "" cp -p "/var/svc/log/milestone-single-user:default.log" "$OUTsmf"
		debug_cmd "" cp -p "/var/svc/log/milestone-patching:default.log" "$OUTsmf"
		debug_cmd "" cp -p "/var/svc/log/application-scn-singleuser-agent:default.log" "$OUTsmf"
		debug_cmd "" cp -p "/var/svc/log/application-scn-update-agent:default.log" "$OUTsmf"
		debug_cmd "" cp -p "/var/svc/log/system-patchchk:default.log" "$OUTsmf"

		debug_cmd "$OUTsysconfig/prtdiag-v.out" prtdiag -v

		#zstat specific
		debug_cmd "$OUTEXPL/zstatadm_-f_process.out" /opt/sun/xvm/lib/zstat/zstatadm -f process
		debug_cmd "$OUTEXPL/zstatadm_-q_process.out_rc" /opt/sun/xvm/lib/zstat/zstatadm -q process
		echo $? >> "$OUTEXPL/zstatadm_-q_process.out_rc"
		debug_cmd "$OUTEXPL/zstatadm_-q_task.out_rc" /opt/sun/xvm/lib/zstat/zstatadm -q task
		echo $? >> "$OUTEXPL/zstatadm_-q_task.out_rc"
		debug_cmd "$OUTEXPL/zstatadm_-q_flow.out_rc" /opt/sun/xvm/lib/zstat/zstatadm -q flow
		echo $? >> "$OUTEXPL/zstatadm_-q_flow.out_rc"
		debug_cmd "" cp -p "/etc/opt/sun/oc/zstat.conf" "$OUTEXPL"
		debug_cmd "$OUTEXPL/ls-la_exacct.out" ls -la /var/adm/exacct/
		debug_cmd "$OUTEXPL/acctadm.out" acctadm

		LDM=/usr/sbin/ldm #TODO this should be an alias, but we don't want to produce warning on systems without ldoms
		debug_cmd "$OUTldom/ldm_list.out" $LDM list
		debug_cmd "$OUTldom/ldm_list-l.out" $LDM list -l
		debug_cmd "$OUTldom/ldm_list-devices_a.out" $LDM list-devices -a
		debug_cmd "$OUTldom/ldm_list-services.out" $LDM list-services
		debug_cmd "$OUTldom/ldm_list-io.out" $LDM list-io
		debug_cmd "$OUTldom/ldm_list-io-l.out" $LDM list-io -l
		debug_cmd "$OUTldom/ldm_list_o_physio.out" $LDM list -o physio
		debug_cmd "$OUTldom/ldm_list_o_network.out" $LDM list -o network
		debug_cmd "$OUTldom/ldm_list_po_network.out" $LDM list -p -o network
		debug_cmd "$OUTldom/ldm_list_spconfig.out" $LDM list-spconfig 
		debug_cmd "$OUTldom/ldm-V.out" $LDM -V
	    fi

	elif [[ x$OS = x"Linux" ]]; then
	    # This is Linux(?) explorer like section.
	    debug_cmd "" cp -p /etc/grub.conf $OUTetc
	    debug_cmd "" cp -p /etc/kdump.conf $OUTetc
	    debug_cmd "" cp -p /etc/*release $OUTetc
	    debug_cmd "" cp -p /etc/fstab  $OUTetc
	    debug_cmd "" cp -p /var/log/messages* $OUTmessages/
	    debug_cmd "" cp -p /proc/cpuinfo $OUTsysconfig
	    debug_cmd "$OUTsysconfig/chkconfig--list" chkconfig --list
	    debug_cmd "$OUTrpm/rpm-qa.out" rpm -qa
	    for p in scn hss cacao j6 dhcp jdk ipmitool scs
	    do
		for i in `grep $p $OUTrpm/rpm-qa.out`
		do
		    debug_cmd "$OUTrpm/$i.out" rpm -qi $i
		done
	    done
	fi

    fi # SKIPEXPLORER

    #common
    info "Collecting CACAO ..."

    # list all cacao dir for cores !
    debug_cmd "$OUTCACAO/find-ls_CACAO_VAR" find $CACAO_VAR -ls
    debug_cmd "$OUTCACAO/cacaoadm-V.out" $CACAOADM -V
    # below should probably go to explorer
    debug_cmd "" mkdir "$OUTCACAO/etc"
    debug_cmd "" cp -r $CACAO_ETC $OUTCACAO/etc/

    echo $CACAO_INS > $OUTCACAO/cacao_instances

    #get cacao logs from all available instances
    for i in $CACAO_INS
    do
	debug_cmd "" mkdir "$OUTCACAO/$i"
	debug_cmd "" mkdir "$OUTCACAO/$i/logs"
	debug_cmd "" mkdir "$OUTCACAO/$i/audits"
	debug_cmd "" cp -p $CACAO_VAR/$i/logs/* $OUTCACAO/$i/logs/
	debug_cmd "" cp -p $CACAO_VAR/$i/audits/* $OUTCACAO/$i/audits/
	debug_cmd "$OUTCACAO/$i/status" $CACAOADM status -i $i
	debug_cmd "$OUTCACAO/$i/list-keystore" "/bin/echo trustpass | \
                   /usr/bin/keytool -list -keystore \
                   /etc/cacao/instances/$i/security/jsse/truststore"
	LIST_MODULES=$OUTCACAO/$i/list_modules
	debug_cmd "$LIST_MODULES" $CACAOADM list-modules -i $i
	#in case we need module status uncomment this, or turn into option
	#otherwise it takes ~ 2min to gather and can be found in cacao logs
	#after startup anyways
	#        STATUS_MODULES=$OUTCACAO/$i/status_modules
	#        for x in `grep -v "modules registered" $LIST_MODULES | cut -d ' ' -f 1`
	#        do
	#         echo "Module: $x" >>$STATUS_MODULES
	#         CMD="$CACAOADM status -i $i $x >>$STATUS_MODULES"
	#         debug $CMD
	#         eval $CMD 2>&1
	#        done
	debug_cmd "$OUTCACAO/$i/list-filters" $CACAOADM list-filters -i $i
	debug_cmd "$OUTCACAO/$i/list-params" $CACAOADM list-params -i $i
	debug_cmd "$OUTCACAO/$i/verify-configuration" $CACAOADM verify-configuration -i $i
    done

    if [ -d /guests -o -d /var/mnt/virtlibs -o -d /var/mnt/oc-zpools ]; then
        info "Collecting local and NFS Library content ..."
        [ -d /guests ] && ls -lLad /guests/* /guests/*/* /guests/*/*/* > $OUTLIB/ls-lLa_guests.out 2>/dev/null
        [ -d /var/mnt/virtlibs ] && (cd /var/mnt/virtlibs; ls -lLad */* */*/* */*/*/* > $OUTLIB/ls-lLa_virtlibs.out 2>/dev/null)
        [ -d /var/mnt/oc-zpools ] && ls -lLad /var/mnt/oc-zpools/* /var/mnt/oc-zpools/*/* /var/mnt/oc-zpools/*/*/* > $OUTLIB/ls-lLa_oc-zpools.out 2>/dev/null
        # Collecting all metadata from libraries located in /guests and /var/mnt/virtlibs
        # 1st step: generating a list of files to be collected
        # Caveat: Solaris 11 IPS repos can be located in NFS-Libraries, we must skip them !
        MSR_LIBS=$(ls /var/mnt/virtlibs/*/*/data/pkg5.repository 2>/dev/null | cut -d/ -f1-5)
        if [ -z "$MSR_LIBS" ]; then
                find /var/mnt/virtlibs/ /guests -type f 2>/dev/null | egrep  ".txt$|.xml$|.properties$|.mount$|.bmpl$|.lnk$" | cut -c2- > /var/tmp/ocdoctor-include.lst.$$
        else
                find /guests $(ls -d /var/mnt/virtlibs/* 2>/dev/null | egrep -v "$( echo $MSR_LIBS | tr '\n' '|')") -type f 2>/dev/null | \
                egrep  ".txt$|.xml$|.properties$|.mount$|.bmpl$|.lnk$" | cut -c2- > /var/tmp/ocdoctor-include.lst.$$
        fi
	# 2nd step: create a tar-ball out of the files from the above created include list
	(cd /; debug_cmd "" tar cf $OUTLIB/guests+virtlibs-text-files.tar -I /var/tmp/ocdoctor-include.lst.$$ )
	rm -f /var/tmp/ocdoctor-include.lst.$$ 2>/dev/null
    fi

    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then

	if [[ x"$OS" = x"SunOS" ]]; then
	    #TODO is below needed ? oc-doctor does it and has newer db of packages
	    #detect_packages_sol
	    debug_cmd "$OUTSAP/osp/dfshares.out" dfshares
	    debug_cmd "" cp -rp /etc/dfs "$OUTSAP/osp/"
	    debug_cmd "$OUTSAP/osp/share.out" share
	elif [[ x"$OS" = x"Linux" ]]; then
	    debug_cmd "$OUTSAP/osp/exportfs-a.out" exportfs -a
	    #    detect_packages_lin
	    #else
	    #    error "This OS: $OS is unsupported"
	    #    exit 3
	fi

	info "Collecting common satellite & proxy data ..."
	debug_cmd "$OUTSAP/agent_zips.out" ls -la "${XVM_VBASE}/images/agent"
	debug_cmd "$OUTSAP/xvm/ls-laRL_uce.log" ls -laRL "${XVM_VBASE}/uce"
	debug_cmd "" cp -p /var/tmp/installer.log.* /var/tmp/opscenter/installer.log.* "$OUTSAP/install_log"
	debug_cmd "" cp -p /n1gc-setup/.version.properties "$OUTSAP/version.properties"
	debug_cmd "" cp -rp ${XVM_VBASE}/persistence "$OUTSAP/xvm"
	debug_cmd "" cp -p ${XVM_VBASE}/*.properties "$OUTSAP/xvm"
	#debug_cmd "" cp -rp /etc/opt/sun/n1gc "$OUTSAP"
	debug_cmd "" cp -rp ${XVM_VBASE}/osp/logs "$OUTSAP/osp"
	debug_cmd "" cp -p ${XVM_VBASE}/osp/dhcpd* "$OUTSAP/osp"
	debug_cmd "" cp -rp ${XVM_VBASE}/osp/status "$OUTSAP/osp"
	debug_cmd "" cp -rp ${XVM_VBASE}/osp/clients "$OUTSAP/osp"
	debug_cmd "" cp -rp ${XVM_VBASE}/osp/dhcp "$OUTSAP/osp"
	debug_cmd "" cp -p /etc/dhcpd.conf "$OUTSAP/osp"
	debug_cmd "" cp -rp /var/dhcp/SUNWfiles* "$OUTSAP/osp"
	#/etc/dhcp/config/SUNWfiles1_dhcptab - osol
	debug_cmd "" tar cvf $OUTSAP/osp/Templates.tar $JET_BASE/Templates
	debug_cmd "" tar cvf $OUTSAP/osp/Clients.tar $JET_BASE/Clients
	debug_cmd "$OUTSAP/osp/find-ls_tftpboot" find /tftpboot/ -ls
	debug_cmd "" cp -rp ${XVM_VBASE}/logs "$OUTSAP/xvm"
	debug_cmd "" cp -p /opt/sun/n1gc/lib/*.properties "$OUTSAP/lib"
	debug_cmd "" cp -p /opt/sun/n1gc/os/*.properties "$OUTSAP/os"
	debug_cmd "$OUTSAP/ipmitool-V.out" $IPMITOOL -V
	debug_cmd "$OUTSAP/ipmiflash-V.out" $IPMIFLASH -V
	debug_cmd "$OUTSAP/ls-l_UCE_backup.out" ls -l $UCE_BACKUP

	#update bundle info
	# TODO is below done for agents too ? I just saw the agentdb dumped, no other log
	UPDATEDIRS="/var/opt/sun/xvm/update-saved-state /var/scn/update-saved-state"
	for upd in $UPDATEDIRS ; do
	    UPDLOGDIR=`ls $upd 2>/dev/null`
	    if [[ $? -eq 0 ]]; then
		debug_cmd "" mkdir -p $OUTSAP/update
		for logd in $UPDLOGDIR; do
		    debug_cmd "" mkdir -p $OUTSAP/update/$logd
		    debug_cmd "" cp -p $upd/$logd/updatelog.txt "$OUTSAP/update/$logd"
		    debug_cmd "" cp -p $upd/$logd/state-data/Global/*state "$OUTSAP/update/$logd"
		done
	    fi
	done

	debug_cmd "$OUTSAP/sc-console_list-connections.out" sc-console list-connections 
	RC=$?
	if [[ $RC -ne 0 ]]
	then
	    debug_cmd "$OUTSAP/sc-console_return" echo "RETURNED CODE: $RC"
	fi


	#uce server

	debug_cmd "" cp -p $UCE_RC $SERVER_DEST
	debug_cmd "" cp -p $DOT_UCE_RC $SERVER_DEST
	debug_cmd "" cp -p $DOT_UCE_RC $SERVER_DEST/dot.uce.rc
	if [ -f ${SERVER}/.disconnected.properties ]; then
	    debug_cmd "" cp -p ${SERVER}/.disconnected.properties $SERVER_DEST/dot.disconnected.properties
	fi



	debug_cmd "$SERVER_DEST/ls_lL.log" ls -lL $PUBLIC_FOLDER/config_files
	debug_cmd "$SERVER_DEST/ls_laR-server.log" ls -laR $SERVER
	if [ ! "LIBRARY_FOLDER" = "" ]; then
	    debug_cmd "$SERVER_DEST/ls_laR-library-folder.log" ls -laR $LIBRARY_FOLDER
	fi
	if [ -f $SERVER/logs/error.log ]; then
	    debug_cmd "$SERVER_DEST/ls_laR-server-logs.log" ls -laR $SERVER/logs/
	fi
	debug_cmd "" cp -p $PUBLIC_FOLDER/* $SERVER_DEST_PUBLIC
	# below ignore the cyclic symlink ./logs to itself
	debug_cmd "" cp -rpP $SERVER_LOGS/* $SERVER_DEST/logs
	debug_cmd "" cp -rp $SERVER_ETC/* $SERVER_DEST_ETC
	debug_cmd "" mkdir -p $OUTSAP/bui/conf/
	debug_cmd "" cp -p ${XVM_VBASE}/bui/conf/httpd.conf $OUTSAP/bui/conf/
	debug_cmd "" mkdir -p $OUTSAP/osp/web/conf/
	debug_cmd "" cp -rp ${XVM_VBASE}/osp/web/conf/* $OUTSAP/osp/web/conf/
	debug_cmd "" cp -rp $PUBLIC_FOLDER/public_blobs/*.blob.err $SERVER_DEST_PUBLIC
	# get channel info, / at the is important because of dereference
	if [[ -d $PUBLIC_FOLDER/config_files/ ]] ; then
	    for i in `ls $PUBLIC_FOLDER/config_files/`
	    do
		debug_cmd "" mkdir $SERVER_DEST/public/config_files/$i
		debug_cmd "" cp -p $PUBLIC_FOLDER/config_files/${i}/.metainfo.xml \
		    $SERVER_DEST/public/config_files/$i
	    done
	fi

	#debug_cmd "" cp -p $INSTALL/install*.log* $INSTALL_DEST

	#uce engine
	#debug_cmd "" cp -p $ENGINE/bin/*uce.* $ENGINE_DEST
	#debug_cmd "" cp -p $ENGINE/bin/.*uce.* $ENGINE_DEST
	#debug_cmd "" cp -rp $ENGINE/logs/* $ENGINE_DEST/logs
	#debug_cmd "" cp -p $DB/pgdata/postgresql.conf $DB_DEST

	#uce console
	debug_cmd "" cp -p $CONSOLE/bin/*uce.* $CONSOLE_DEST
	#debug_cmd "" cp -p $CONSOLE/bin/.*uce.* $CONSOLE_DEST
	debug_cmd "" cp -rp $CONSOLE/logs $CONSOLE_DEST
	#CONSOLE_USER=$USERHOME/.uce_console
	#CONSOLE_USER_DEST=$TMPDIR/console/user
	#cp -rp $CONSOLE_USER $CONSOLE_USER_DEST

	#uce cli
	#debug_cmd "" cp -p $CLI/bin/*uce.* $CLI_DEST
	#debug_cmd "" cp -p $CLI/bin/.*uce.* $CLI_DEST
	#debug_cmd "" cp -rp $CLI/logs/ $CLI_DEST/logs 
	#CLI_USER=$USERHOME/.uce_cli
	#CLI_USER_DEST=$TMPDIR/cli/user
	#cp -rp $CLI_USER $CLI_USER_DEST

    fi

    #satellite
    if [[ $SATELLITE -eq 1 ]]; then
	info "Collecting satellite specific data ..."
	debug_cmd "" cp -rp ${XVM_VBASE}/bui/logs "$OUTSAT/bui/"

	# collecting Asset Counts 
	ASSET_COUNT=""
	if [ -x $scriptDir/../toolbox/AssetCount.sh ]; then
	    ASSET_COUNT="$scriptDir/../toolbox/AssetCount.sh"
	elif [ -x $scriptDir/toolbox/AssetCount.sh ]; then
	    ASSET_COUNT="$scriptDir/toolbox/AssetCount.sh"
	fi
	if [ -n "$ASSET_COUNT" ]; then
	    info "Collecting Asset Count ..."
	    debug_cmd "$outdir/AssetCount_standard.txt" $ASSET_COUNT -v standard
	    debug_cmd "$outdir/AssetCount_all.txt" $ASSET_COUNT all
	fi


	### HA stuff (requested in Bug 16208228)
	if [ -f ${XVM_VBASE}/ha/EnterpriseController_HA_clusterware.properties ]; then
	    info "Collecting satellite HA specific data ..."
	    ### Copy only specific files to avoid grabbing .tar.gz and .tar the user has created
	    debug_cmd "" cp -rp "${XVM_VBASE}/ha/EnterpriseController*" "$OUTSAT/ha/"
	    debug_cmd "" cp -rp "${XVM_VBASE}/ha/HAECStatus*" "$OUTSAT/ha/"
	    ### Determine CRS folder
	    HOSTNAME=`hostname`
	    LOGFILE_HA=`ls /u01/app/11*/grid/log/${HOSTNAME}/alert${HOSTNAME}.log 2>/dev/null`
	    if [ ! "$LOGFILE_HA" = "" ]; then 
		if [ -f $LOGFILE_HA ]; then
		    debug_cmd "" cp -p $LOGFILE_HA "$OUTSAT/ha/"
		    
		    CRS=`dirname $LOGFILE_HA`
		    APPROOT="${CRS}/agent/crsd/application_root/"
		    if [ -d $APPROOT ]; then
			debug_cmd "" cp -pR "${APPROOT}" "$OUTSAT/ha/application_root/"
		    fi
		    CRSD="${CRS}/crsd/"
		    if [ -d $CRSD ]; then
			debug_cmd "" cp -pR "${CRSD}" "$OUTSAT/ha/crsd/"
		    fi
		fi
	    fi
	fi
	

	if [[ -x $AS_CLIENT ]]; then
	    debug_cmd "$OUTSAT/osp/as_client-l.out" $AS_CLIENT -l
	fi

	#for timeout hacks
	if [ -f ${XVM_VBASE}/bui/webapps/xvmoc/WEB-INF/zk.xml ]; then
	    debug_cmd "" cp -p ${XVM_VBASE}/bui/webapps/xvmoc/WEB-INF/zk.xml "$OUTSAT/bui/"
	fi

	if [ -d ${XVM_VBASE}/db ]; then			### no such folder in 12c
	    for db in `ls -1 ${XVM_VBASE}/db/`
	    do
		debug_cmd "" cp -rp ${XVM_VBASE}/db/$db/logs "$OUTSAT/db/$db"
		debug_cmd "" cp -p ${XVM_VBASE}/db/$db/postgresql.conf "$OUTSAT/db/$db"
		debug_cmd "" cp -p ${XVM_VBASE}/db/$db/postmaster.opts "$OUTSAT/db/$db"
	    done
	fi 

	get_major_version ""	### provides MAJORVERSION
	if [[ -x ${SATADM} ]] && [[ x"${MAJORVERSION}" = x"12" ]]; then			### 12c EC 
	    info "Detected 12c EC, collecting database 'systemproperty' table and configuration file"
	    
	    touch "$OUTSAT/db/systemproperty.table.txt.tmp"
	    chmod 777 "$OUTSAT/db/systemproperty.table.txt.tmp"		### otherwise satadm can't create it in this folder
	    
	    SQL_Template /var/tmp/GDD-DB-config.sql.$$ "$OUTSAT/db/systemproperty.table.txt.tmp" "select id ||'|'||subsystem||'|'||pkey||'|'||pvalue||'|'||pdefault||'|'||pvisibility from systemproperty;"
	    echo "@/var/tmp/GDD-DB-config.sql.$$" |  ${SATADM} sqlplus >/dev/null 2>/dev/null
	    
	    echo "id|subsystem|pkey|pvalue|pdefault|pvisibility" > "$OUTSAT/db/systemproperty.table.txt"		### header
	    cat "$OUTSAT/db/systemproperty.table.txt.tmp" >> "$OUTSAT/db/systemproperty.table.txt"
	    rm -f "$OUTSAT/db/systemproperty.table.txt.tmp" "/var/tmp/GDD-DB-config.sql.$$" >/dev/null 2>/dev/null
	    chmod 700 "$OUTSAT/db/systemproperty.table.txt"
	    
	    ### Take localOracleConfig
	    if [ -f /opt/ORCLsysman-db/etc/localOracleConfig.properties ]; then				# solaris
		cp -p /opt/ORCLsysman-db/etc/localOracleConfig.properties "$OUTSAT/db/localOracleConfig.properties.solaris"
	    fi
	    if [ -f /opt/orcl/orcl-sysman-db/etc/localOracleConfig.properties ]; then			# linux
		cp -p /opt/orcl/orcl-sysman-db/etc/localOracleConfig.properties "$OUTSAT/db/localOracleConfig.properties.linux"
	    fi
	    
	    
	fi

	if [[ x$SKIPJOBS == x"1" ]] ; then
	    info "SKIPPING collection of job data as user requested ... "
	else

	    if ! [[ -x ${OCCLI} ]]; then
		if [[ -x ${N1SH} ]]; then
		    debug_cmd "" $N1SH show job count 1
		    N1SHCHK=$?
		fi
		if [[ -x ${N1SH} && $N1SHCHK -eq 0 ]]; then
		    info "Collecting satellite info using n1sh ..."
		    JOB_LIMIT=20
		    debug_cmd "$OUTSAP/n1sh_job.log" $N1SH show job count $JOB_LIMIT
		    for i in `cat $OUTSAP/n1sh_job.log | nawk '{print $1}' | grep -v Job`
		    do
			echo "***************************************************"  >> $OUTSAP/job_info.log
			echo "" >> $OUTSAP/job_info.log
			echo "Job ID" $i >> $OUTSAP/job_info.log
			echo "" >> $OUTSAP/job_info.log
			CMD="$N1SH show job $i >> $OUTSAP/job_info.log"
			debug $CMD
			eval $CMD 2>&1
			echo "" >> $OUTSAP/job_info.log
		    done
		else
		    if [[ -x ${SHOWJOB} ]]; then
			info "Collecting satellite info using show_job.pl ..."
			JOB_LIMIT=20
			debug_cmd "$OUTSAP/showjob.log" "$SHOWJOB | head -`expr $JOB_LIMIT`"
			for i in `cat $OUTSAP/showjob.log | nawk '{print \$1}' | grep '[^a-zA-Z]*\.[0-9]*$' | sed 's/^.*\.\([0-9]*\)$/\1/g'`
			do
			    echo "***************************************************"  >> $OUTSAP/job_info.log
			    echo "" >> $OUTSAP/job_info.log
			    echo "JOB ID" $i >> $OUTSAP/job_info.log
			    echo "" >> $OUTSAP/job_info.log
			    CMD="$SHOWJOB $i >> $OUTSAP/job_info.log"
			    debug $CMD
			    eval $CMD 2>&1
			    echo "" >> $OUTSAP/job_info.log
			done
		    fi
		fi
	    else
		info "Collecting satellite info using $OCCLI ..."
		if [[ -f ~/.jython ]] ; then #backup if jython setting exists
		    mv ~/.jython ~/.jython.old
		fi
		info "Caching directory for $OCBIN cli: ${OUTPUT_PREFIX}/packages (will not be removed to make next run faster)"
		echo "python.cachedir=${OUTPUT_PREFIX}" > ~/.jython
		export JAVA_HOME
		cd `dirname ${OCCLI}`
		echo "yes" | ./$OCBIN 2>$NULL > $NULL
		info "Running $OCBIN cli job list collection now (option -i skips this)"
		joblimit=52
		# Bug 23336316 - Replace old method of calling occli with the '-e' option
		debug "Running: ./$OCBIN -e 'connect;jobs;list' 2>&1 | head -n $joblimit > $OUTSAP/occli_jobs_list.log"
		./$OCBIN -e 'connect;jobs;list' 2>&1 | head -n $joblimit > $OUTSAP/occli_jobs_list.log

		typeset JOBIDS
		JOBIDS=`awk '{if (PRNT==1){print \$1}}; /^\-\-\-\-/ {PRNT=1}' $OUTSAP/occli_jobs_list.log | cut -d\| -f1`
		#### Note that in 12c we have another header line, so the first line will be "---------..."
		rm -f /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$ >/dev/null 2>/dev/null
		echo "connect" > /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$
		echo "jobs" >> /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$
		info "Collecting detailed job information (option -i skips this)"
		for i in $JOBIDS; do			### Combine all detailed requests into one connect... instead of having connect overhead for each
		    echo "list -x $i -l" >> /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$
		done

		# Bug 23336316 - Replace old method of calling occli with the '-c' option
		debug "Running for all jobs: ./$OCBIN -c /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$ 2>&1 >> $OUTSAP/occli_jobs_list-l.log"
		./$OCBIN -c /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$ 2>&1 >> $OUTSAP/occli_jobs_list-l.log
		rm -f /var/tmp/OCDoctor-GDD-list-all-jobs.cli.$$ >/dev/null 2>/dev/null

#
# Bug 22534619  - OCDoctor reports error in sys.excepthook
#
# I tried to root cause this, but whenever I run the same OCCLI command outside of "collectlogs", the issue was not reproducible
# In agreement with Sustaining/RPE, we can live without collecting all Notificattions, so I will just uncomment it out
# for now
# Update: Fix for bug 23336316 also fixes the problem with this issue..Still can't uncomment this section 
# because 'list -l' collects ALL notifications, and this number can be HUGE!
# 'list' under notifications has an option to specify the number of notifications, but that isn't working!
# This can be enabled once the CLI has been fixed

#		debug "Running: ./oc -e \'connect;notifications;list -l\' | ./$OCBIN 2>&1 > $OUTSAP/occli_notifications_list-l.log"
#		./$OCBIN -e 'connect; notifications; list -l' 2>&1 > $OUTSAP/occli_notifications_list-l.log
#		cd - 2>&1 >$NULL
		rm ~/.jython
		if [[ -f ~/.jython.old ]] ; then #restore their jython prop file if it exists
		    mv ~/.jython.old ~/.jython
		fi
	    fi
	fi #SKIPJOBS

	if [[ x$SKIPTABLES == x"1" ]] ; then
	    info "SKIPPING collection of db tables as user requested ... "
	    info "Note that datacenter summary will not be produced."
	else
	    #now collect some database tables
	    info "Collecting satellite database data ..."

	    if [[ "${MAJORVERSION}" == "12" ]]; then
		# collect DB logs in case we are running a local DB
		if [[ "$(awk -F= ' $1 == "mgmtdb.server.type" {print $2}' /var/opt/sun/xvm/db.properties 2>/dev/null)" == "local" ]]; then
		    # collect alert logs
		    debug_cmd "" cp -rp /var/opt/sun/xvm/oracle/diag/rdbms/ocdb/OCDB/trace/alert* $OUTSAT/db/logs/trace
		    if [[ x"$OS" == x"SunOS" ]]; then
			# get SMF logs from DB service
			debug_cmd "" cp -p /var/svc/log/application-scn-oracle:default.log $OUTSAT/db/logs/smf
			debug_cmd "" cp -p /var/svc/log/application-scn-oralistener:default.log $OUTSAT/db/logs/smf
			debug_cmd "" cp -p /var/svc/log/application-scn-db:local.log $OUTSAT/db/logs/smf
		   fi
		fi
	    fi

	    TABLES="channel_instances hd_generated_notifications hd_monitored_attribute hd_persistent_mbean hd_service_tag_source local_users mgmt_backup_path sc_client_registration scn_job_run scn_job_status scn_target_data scn_task_data scn_task_logs scn_task_run uce_todolist scn_target_lock hd_normalize_mbean users account registration_token agent_table hd_correlation_attribute hd_resource_correlation" 

	    for table in $TABLES; do
		if [[ x"${MAJORVERSION}" != x"12" ]]; then			### If not 12c
	    	    debug "executing command : echo select * from $table; | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1"
	    	    echo "select * from $table;" | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1
		else	### 12c
		    debug "executing command : echo select * from $table; | ${SATADM} sqlplus > $OUTSAT/db/$table.selectall 2>&1"
	    	    SQL_Template_full_db /var/tmp/GDD-DB-config.sql.$$ "/var/tmp/GDD-DB-OUT.$$" "select * from $table;"
		    echo "@/var/tmp/GDD-DB-config.sql.$$" |  ${SATADM} sqlplus >/dev/null 2>/dev/null
		    grep "table or view does not exist" /var/tmp/GDD-DB-OUT.$$ > /dev/null
		    if [ $? -ne 0 ]; then
			mv /var/tmp/GDD-DB-OUT.$$ "$OUTSAT/db/$table.selectall"		#### need to save in /tmp/ since oracle user may not have permissions to the dest folder
		    fi
		    rm -f /var/tmp/GDD-DB-config.sql.$$ >/dev/null 2>/dev/null
		fi
	    done

	    BIG_TABLES="hd_cached_mbean uce_updates scn_job"
	    # uce_updates last SJ
	    if [[ x$COLLECT_BIG_TABLES == x"1" ]] ; then
		info "Collecting satellite database data - BIG tables (this can get long & huge) ..."
		for table in $BIG_TABLES; do
		    if [[ x"${MAJORVERSION}" != x"12" ]]; then			### If not 12c
	    		debug "executing command : echo select * from $table; | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1"
    			echo "select * from $table;" | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1
    		    else		### 12 c
    			debug "executing command : echo select * from $table; | ${SATADM} sqlplus > $OUTSAT/db/$table.selectall 2>&1"
	    		SQL_Template_full_db /var/tmp/GDD-DB-config.sql.$$ "$OUTSAT/db/$table.selectall" "select * from $table;"
			echo "@/var/tmp/GDD-DB-config.sql.$$" |  ${SATADM} sqlplus >/dev/null 2>/dev/null
			rm -f /var/tmp/GDD-DB-config.sql.$$ >/dev/null 2>/dev/null
    		    fi
		done
	    fi
	    if [[ x"${MAJORVERSION}" != x"12" ]]; then			### If not 12c
		PGVACUUMLOCKTABLES="pg_stat_activity pg_locks"
		if [[ x$COLLECT_PGLOCK_TABLES == x"1" ]] ; then
		    info "Collecting satellite database data - PG vacuum lock tables ..."
		    for table in $PGVACUUMLOCKTABLES; do
			debug "executing command : echo select * from $table; | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1"
			echo "select * from $table;" | $DBTOOL psql > $OUTSAT/db/$table.selectall 2>&1
		    done
		fi
		
		info "Collecting satellite database table size data ..."
		if [[ -x "$scriptDir/toolbox/get-largest-tables.sh" ]]; then
		    GLTB="$scriptDir/toolbox/get-largest-tables.sh"
		else
		    GLTB="$scriptDir/../toolbox/get-largest-tables.sh"
		fi
		debug_cmd "$OUTSAT/db/get-largest-tables.sh.out" $GLTB
		
		#create a summary from client registration table
		DSUMMARY=$outdir/datacenter_summary.txt
		DSOURCE=$OUTSAT/db/sc_client_registration.selectall
		echo "APPROXIMATE Summary of datacenter from satellite db (refer to $OUTSAT/db/sc_client_registration.selectall for complete view)" > $DSUMMARY
		echo "" >> $DSUMMARY
		echo "Found : " >> $DSUMMARY
		# be uniq on hostnames, because there can be more records per one agent
		debug "Running: grep scn-proxy $DSOURCE | cut -d\| -f 3 | cut -d: -f 2 | sort | uniq | wc -l >> $DSUMMARY"
		PROXY_COUNT=`grep scn-proxy $DSOURCE | cut -d\| -f 3 | cut -d: -f 2 | sort | uniq | wc -l `
		echo $PROXY_COUNT >> $DSUMMARY
		# warn if auto-collect, restart, running on satellite and more than 1 proxy attached
		if [[ ${RESTART} = "true" && ${AUTO_COLLECT} = "true" ]]; then
		    if [[ $PROXY_COUNT -gt 1 ]]; then
			warning "It seems you have more than one proxy attached to satellite.\n Please READ the note on big deployments and restart of satellite(controller)\n services in README.Sun-GDD-${PRODUCT_AB}"
		    fi
		fi
		echo "proxy servers, namely:" >> $DSUMMARY
		debug "Running: grep scn-proxy $DSOURCE | cut -d '|' -f 3 | cut -d: -f 2 | uniq  >> $DSUMMARY"
		grep scn-proxy $DSOURCE | cut -d '|' -f 3 | cut -d: -f 2 | uniq >> $DSUMMARY
		echo "" >> $DSUMMARY
		echo "Total(approx.) agents connected: " >> $DSUMMARY
		echo "(counting only unique host names, ignoring multiple interfaces or re-registrations) " >> $DSUMMARY
		debug "Running: grep scn-agent $DSOURCE | cut -d\| -f 3 | cut -d: -f 2 | uniq | wc -l >> $DSUMMARY"
		grep scn-agent $DSOURCE | cut -d\| -f 3 | cut -d: -f 2 | uniq | wc -l >> $DSUMMARY
		echo "" >> $DSUMMARY
	    fi 	#12c
	fi #SKIPTABLES

	if [[ x$OS = x"SunOS" ]]; then
	    #debug_cmd "" pkginfo -l SUNWucea > $OUTpatch/SUNWucea.out
	    #debug_cmd "" pkginfo -l SUNWucee > $OUTpatch/SUNWucee.out
	    debug_cmd "$OUTpatch/SUNWuces.out" pkginfo -l SUNWuces
	    debug_cmd "$OUTpatch/SUNWucelarge.out" pkginfo -l SUNWucelarge
	    debug_cmd "$OUTpatch/SUNWuceconsole.out" pkginfo -l SUNWuceconsole
	    #debug_cmd "" pkginfo -l SUNWuce-cli > $OUTpatch/SUNWuce-cli.out
	elif [[ x$OS = x"Linux" ]]; then
	    #debug_cmd "" uce-engine > $OUTrpm/sun-uce-engine.out
	    debug_cmd "$OUTrpm/sun-uce-server.out" rpm -qi sun-uce-server
	    debug_cmd "$OUTrpm/sun-uce-console.out" rpm -qi sun-uce-console
	    #debug_cmd "" rpm -qi sun-uce-cli  > $OUTrpm/sun-uce-cli.out
	fi

	# info "Collecting domain model data: CorrelationMap, UserFriendlyName & UnnormalizedObjectNames"
	# debug_cmd "$OUTSAT/jmxdumps/all_ONs.out" "$JMXDUMP s t root wellknown com.sun.hss.domain:* UnnormalizedObjectNames UserFriendlyName CorrelationMap 2>/dev/null"

	info "Collecting domain model data: StatisticsMBean"
	debug_cmd "$OUTSAT/jmxdumps/StatisticsMBean.out" "$JMXDUMP s t root wellknown com.sun.cacao.instrum:type=com.sun.cacao.common.instrum.StatisticsMBean 2>/dev/null"

	if [[ $JUSER != "" ]]; then
	    info "Collecting full satellite domain data ..."
	    info "(this might take a while, check log for details)"
	    debug "HOST=\"\" $JMXDUMP s t $JUSER ******* ${JMXDUMP_ARG} > $OUTSAT/jmxdumps/jmxdump_full.out 2>&1"
	    HOST="" $JMXDUMP s t "$JUSER" "$JPASSWORD" ${JMXDUMP_ARG} > "$OUTSAT/jmxdumps/jmxdump_full.out" 2>&1
	fi

    fi

    if [[ $PROXY -eq 1 ]]; then
	info "Collecting proxy specific data ..."

	debug_cmd "" cp -p ${XVM_VBASE}/proxydb/service.properties \
	    "$OUTPROXY/proxydb"

	if [[ -x $AS_CLIENT_PROXY ]]; then
	    debug_cmd "$OUTPROXY/osp/as_client_proxy-l.out" $AS_CLIENT_PROXY -l
	fi

	if [[ $JUSER != "" ]]; then
	    info "Collecting proxy domain data ..."
	    info "(this might take a while, check log for details)"
	    debug "HOST=\"\" $JMXDUMP p t $JUSER ******* ${JMXDUMP_ARG} > $OUTPROXY/jmxdumps/jmxdump_full.out 2>&1"
	    HOST="" $JMXDUMP p t "$JUSER" "$JPASSWORD" ${JMXDUMP_ARG} > "$OUTPROXY/jmxdumps/jmxdump_full.out" 2>&1

	fi

	info "Collecting domain model data: StatisticsMBean"
	debug_cmd "$OUTPROXY/jmxdumps/StatisticsMBean.out" "$JMXDUMP p t root wellknown com.sun.cacao.instrum:type=com.sun.cacao.common.instrum.StatisticsMBean 2>/dev/null"

    fi

    #agent
    if [[ $AGENT -eq 1 ]]; then
	info "Collecting agent data ..."
	
	debug_cmd "" cp -rp \
	    ${XVM_VBASE}/persistence "$OUTAGENT"
	debug_cmd "" cp -p /opt/sun/n1gc/lib/*.properties "$OUTAGENT/lib"
	debug_cmd "" cp -p /var/tmp/agentadm.* "$OUTAGENT"
	debug_cmd "" cp -p /var/tmp/opscenter/agentadm.* "$OUTAGENT"
	debug_cmd "" cp -p /var/tmp/agent_upgrade_results.log "$OUTAGENT"
	debug_cmd "" cp -p /var/tmp/opscenter/agent_upgrade_results.log "$OUTAGENT"
	debug_cmd "" cp -p /var/tmp/agent_install.log "$OUTAGENT"
	debug_cmd "" cp -p /var/tmp/agent_install "$OUTAGENT"
	
	debug_cmd "$OUTAGENT/install_log" cat /var/scn/install/log
	debug_cmd "" cp -p /tmp/.single_user_mode "$OUTAGENT"
	
	debug_cmd "$OUTAGENT/ls_lLaR.out" ls -lLaR $ABASEBIN
	debug_cmd "$OUTAGENT/ls_lLaR-var.out" ls -lLaR $ABASEVAR
	debug_cmd "$OUTAGENT/ls_lLaR-etc-scn.out" ls -lLaR /etc/scn
	debug_cmd "$OUTAGENT/file_scn_a.out" file /etc/scn/scn_a*
	debug_cmd "$OUTAGENT/ls_ld-etc-scn.out" ls -ld /etc/scn
	debug_cmd "" tar cvf $OUTAGENT/jet.tar $JET_VAR
	
	debug_cmd "" cp -p $ABASEBIN/$ABIN/*uce.* $OUTAGENT/$ABIN
	debug_cmd "" cp -p $ABASEBIN/$ABIN/.*uce.* $OUTAGENT/$ABIN
	debug_cmd "" cp -rp $ABASEVAR/$ALOGS/* $OUTAGENT/$ALOGS
	debug_cmd "" cp -rp $ABASEVAR/$ACONFIG/vtree.xml $OUTAGENT/$ACONFIG
	debug_cmd "" cp -rp $ABASEVAR/$ACONFIG/pending_jobs.xml $OUTAGENT/$ACONFIG
	debug_cmd "" cp -rp $ABASEVAR/$ANOTIF/* $OUTAGENT/$ANOTIF
	debug_cmd "$OUTAGENT/agentadm--version.out" $INSTALLED_LOC/agentadm --version
	debug_cmd "" cp -p /var/run/cacao/instances/scn-agent/run/runtime.properties $OUTAGENT/
	
	debug_cmd "$OUTAGENT/sc-console_list-connections.out" sc-console list-connections
	RC=$?
	if [[ $RC -ne 0 ]];then
	    debug_cmd "$OUTAGENT/sc-console_return" echo "RETURNED CODE: $RC"
	fi
	
	if [[ $JUSER != "" ]]; then
	    info "Collecting agent domain data ..."
	    info "(this might take a while, check log for details)"
	    debug "HOST=\"\" $JMXDUMP a t $JUSER ******* ${JMXDUMP_ARG} > $OUTAGENT/jmxdumps/jmxdump_full.out 2>&1"
	    HOST="" $JMXDUMP a t "$JUSER" "$JPASSWORD" ${JMXDUMP_ARG} > "$OUTAGENT/jmxdumps/jmxdump_full.out" 2>&1
	fi

	info "Collecting domain model data: CorrelationMap, UserFriendlyName & UnnormalizedObjectNames"
	debug_cmd "$OUTAGENT/jmxdumps/all_ONs.out" "$JMXDUMP a t root wellknown com.sun.hss.domain:* UnnormalizedObjectNames UserFriendlyName CorrelationMap 2>/dev/null"

	info "Collecting domain model data: StatisticsMBean"
	debug_cmd "$OUTAGENT/jmxdumps/StatisticsMBean.out" "$JMXDUMP a t root wellknown com.sun.cacao.instrum:type=com.sun.cacao.common.instrum.StatisticsMBean 2>/dev/null"


    fi

    # ER 22173945: OC Doctor should collect OCAI logs
    if [ -d /var/opt/ocai/logs ]; then
        info "Collecting OCAI logs ..."
        debug_cmd "" mkdir $outdir/OCAI
        debug_cmd "" cp -rp /var/opt/ocai/logs/* $outdir/OCAI
    fi 

    # collecting startup times of EC, PC & Agent using toolbox script report_startup_times.sh
    STARTUP_TIMES=""
    if [ -x $scriptDir/../toolbox/report_startup_times.sh ]; then
	STARTUP_TIMES="$scriptDir/../toolbox/report_startup_times.sh"
    elif [ -x $scriptDir/toolbox/report_startup_times.sh ]; then
	STARTUP_TIMES="$scriptDir/toolbox/report_startup_times.sh"
    fi
    if [ -n "$STARTUP_TIMES" ]; then
	info "Collecting Startup times of cacao instances..."
	debug_cmd "$outdir/report_startup_times.txt" $STARTUP_TIMES
	debug_cmd "$outdir/report_startup_times.csv" $STARTUP_TIMES -c
    fi

    OCDOC_BIN=""
    if [ -x $scriptDir/../OCDoctor.sh ]; then
	OCDOC_BIN="$scriptDir/../OCDoctor.sh"
    elif [ -x /var/opt/sun/xvm/OCDoctor/OCDoctor.sh ]; then
	OCDOC_BIN="/var/opt/sun/xvm/OCDoctor/OCDoctor.sh"
    fi
    if [ ! "$OCDOC_BIN" = "" ]; then
	info "Collecting OCDoctor troubleshoot output..." 
	$OCDOC_BIN --troubleshoot > $outdir/OCDoctor-troubleshoot.txt 2>&1
    fi

    # below is present in all cases (sat, proxy, agent)
    debug_cmd "" cp -p /opt/sun/n1gc/lib/package.cache.xml $outdir/


}

#
# Find external binaries we need to use on various platforms
#
initialize()
{
    uname=/bin/uname
    if ! [[ -x $uname ]] ; then
        uname=/sbin/uname
        if ! [[ -x $uname ]] ; then
            echo ERROR: /bin/uname or /sbin/uname not found or not executable, cannot continue in such environment ...
            exit 1
        fi
    fi
    alias uname=$uname
    OS=`uname`
    OSVER=`uname -r`
    ARCH=`uname -m`
    UMASK=`umask`
    #sanitize umask
    umask 022
    # we don't need expect until we need to interact with something
    #    expect=`look_for_prog expect`
    #    if [[ $? -ne 0 ]]; then echo $expect;
    #    if [[ x"$OS" = x"SunOS" ]]; then
    #        echo "You might try to install SUNWexpect to resolve this problem."
    #    elif [[ x"$OS" = x"Linux" ]]; then
    #        echo "You might try to install expect to resolve this problem."
    #    fi
    #    exit 1; fi

    # env is needed for expect, if you uncomment above, add it below rather than
    # into RECOMMEND_BIN
    DEPEND_BIN="hostname echo basename dirname cat tee df tail awk head \
rm date id tar netstat ifconfig df ps grep mount cut sed sort \
sleep uniq wc egrep" 
    for i in $DEPEND_BIN ; do
	x=`look_for_prog $i`
	if [[ $? -ne 0 ]]; then 
            echo "ERROR: $x , cannot run in such environment ..."
            exit 1
	fi
	alias ${i}=${x}
    done

    if [[ x"$OS" = x"SunOS" ]]; then
    	RECOMMEND_BIN="unzip vmstat crontab dmesg who env find locale gzip compress last sc-console prstat"
    else 	#### Linux doesn't have prstat,  compress is gzip on linux
    	RECOMMEND_BIN="wget unzip vmstat crontab dmesg who env find locale gzip last sc-console"
    fi
    for i in $RECOMMEND_BIN ; do
	x=`look_for_prog ${i}`
	if [[ $? -ne 0 ]]; then 
	    echo WARNING: $x
	    alias ${i}="echo COMMAND NOT FOUND ${i}"
	else
	    alias ${i}=${x}
	fi
    done

    RECOMMEND_BIN_OS=""
    if [[ x"$OS" = x"SunOS" ]]; then
    	if [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then		### no showrev and patchadd in Solaris 11
            RECOMMEND_BIN_OS="iostat ptree zoneadm zonename zonecfg pkginfo psrinfo \
isainfo svcs nawk prtconf prtdiag format devinfo prtvtoc \
share swap dfshares dispadmin coreadm acctadm eeprom nfsstat"
	elif [[ x$OSVER = x"5.10" ]]; then		### 5.10
	    RECOMMEND_BIN_OS="iostat ptree zoneadm zonename zonecfg pkginfo psrinfo \
isainfo svcs nawk showrev patchadd prtconf prtdiag format devinfo prtvtoc \
share swap dfshares dispadmin coreadm acctadm eeprom nfsstat"
	else		### 5.8 & 5.9
	    RECOMMEND_BIN_OS="iostat ptree pkginfo psrinfo \
isainfo nawk showrev patchadd prtconf format devinfo prtvtoc \
share swap dfshares dispadmin coreadm acctadm eeprom nfsstat"
	fi
	
    elif [[ x"$OS" = x"Linux" ]]; then
        RECOMMEND_BIN_OS="rpm nawk exportfs chkconfig"
    fi
    for i in $RECOMMEND_BIN_OS ; do
	x=`look_for_prog $i`
	if [[ $? -ne 0 ]]; then
	    if [[ x"$i" != x"nawk" ]]; then
      		echo WARNING: $x
	    fi
	    alias ${i}="echo COMMAND NOT FOUND ${i}"
	else
	    alias ${i}=${x}
	fi
    done

    test=`nawk 2>$NULL`
    if [[ x$test = x"COMMAND NOT FOUND nawk" ]]; then
	# remove this echo, it looks bad on a normal Linux machine. Also cant use info as the log folder is not there yet # echo "No nawk, will use awk instead."
	alias nawk=awk
    fi

    test=`gzip -V 2>$NULL`
    if [[ x$test = x"COMMAND NOT FOUND gzip -V" ]]; then
	test=`compress -? 2>$NULL`
	if [[ x$test != x"COMMAND NOT FOUND compress -?" ]]; then
	    echo "No gzip, will try to use compress instead."
	    alias gzip=compress
	else
	    echo "WARNING: gzip or compress are not be found on your OS, I will not be \
able to compress output directory !!!"
	fi
    fi

    HOST=`hostname`

    headerdate=`date +%m/%d/%Y`
    datestamp=`date '+%Y-%m-%d-%H-%M'`
    #datestamp=`date "+%y%m%d-%H%M%S"`
    startdate=`date "+%y%m%d-%H%M%S"`

    LOG="tee"

    ID=`id |cut -d"(" -f1|cut -d"=" -f2`

    RESTART="true"

}

checkUser()
{
    if [[ $ID != "0" ]]
    then
        error "Please run this script as root (su -) !"
        exit 2
    fi
}

#
# Prepare output directory and set the follwing variables
# OUTPUT_PREFIX=/var/tmp
# OUTPUT_DIR=<case>-$appName-<hostname>-<datetime>
# LOGFILE=/var/tmp/something...
# LOG=tee -a $LOGFILE
#
prepareOutputDirectory()
{
    LIMITED=$1				#### use this when enabling/disabling debug - so it doesnt do too much stuff just to change that!
    
    OUTPUT_DIR="$CASENUM-$appName-$HOST-$datestamp"
    LOGFILE="$OUTPUT_PREFIX/$scriptName.log"

    outdir="$OUTPUT_PREFIX/$OUTPUT_DIR"

    # verify that $OUTPUT_PREFIX is dir and writable
    if ! [[ -d $OUTPUT_PREFIX ]] || ! [[ -w $OUTPUT_PREFIX ]] ; then
        echo "ERROR: The output directory $OUTPUT_PREFIX is not a writtable directory."
        echo "Please use -o switch to redirect output somewhere else."
        exit 3
    fi
    
    # once we are sure we can write to output, redefine the logging macro to log
    # to a temp log file
    LOG="tee -a $LOGFILE" 
    
    #below not needed, oc-doctor should announce the version
    debug "$appName Version $VERSION"
    info "Logging to $LOGFILE"
    info "User: `id`"
    info "OS: $OS version: $OSVER"
    if [ ! "$LIMITED" = "limited" ]; then
	info "Started as: $cmdline" 
	if [[ x"$KSH_VERSION" = x"" ]]; then
	    info "Shell: not using ksh" 
	else
	    info "Shell: using ksh" 
	fi
    fi
    
    info "Detecting running components ..."
    detect    
    
    if [ ! "$LIMITED" = "limited" ]; then
	info "Checking space requirements(this might take some time) ..."
	checkAvailableSpace   
	
	debug_cmd "" mkdir -p "$outdir"
	LOGFILE="$OUTPUT_PREFIX/$OUTPUT_DIR/$scriptName.log"
	LOG="tee -a $LOGFILE" 
	
	#moving the log file into the output directory
	debug_cmd "" mv "$OUTPUT_PREFIX/$scriptName.log $LOGFILE"
	info "Changing log location to $LOGFILE ..."
	info "Preparing output directories ..."
	#create the output directory structure
	OUTEXPL="$outdir/explorer"
	debug_cmd "" mkdir $OUTEXPL
	OUTpatch="$OUTEXPL/patch+pkg"
	debug_cmd "" mkdir $OUTpatch
	
	# note that variable cannot have - in it's name for eval to work !
	EXPLORER_DIRS="messages cacao disks netinfo init sysconfig var \
	etc j2se rpm smf zones ldom"
	for i in $EXPLORER_DIRS ; do
	    debug_cmd "" mkdir "$OUTEXPL/$i"
	    eval OUT$i="$OUTEXPL/$i"
	done

    	#common
    	OUTCACAO="$outdir/cacao"
    	debug_cmd "" mkdir $OUTCACAO

	OUTLIB="$outdir/library"
	debug_cmd "" mkdir $OUTLIB

    fi
    #component specific

    if [[ $SATELLITE -eq 1 ]] || [[ $PROXY -eq 1 ]] ; then
	OUTSAP="$outdir/satellite_proxy"
	OUTsmf="$OUTSAP/smf"
	SERVER_DEST="$OUTSAP/server"
	SERVER_DEST_PUBLIC=$SERVER_DEST/public
	SERVER_DEST_ETC="$OUTSAP/server_etc"
	CONSOLE_DEST="$OUTSAP/console"
	#    ENGINE_DEST="$OUTSAP/engine"
	#    DB_DEST="$OUTSAP/engine/db"
	#    CLI_DEST="$OUTSAP/cli"
	if [ ! "$LIMITED" = "limited" ]; then
	    debug_cmd "" mkdir "$OUTSAP"
	    debug_cmd "" mkdir $SERVER_DEST $SERVER_DEST_ETC $CONSOLE_DEST
	    #$ENGINE_DEST $DB_DEST
	    debug_cmd "" mkdir -p $SERVER_DEST_PUBLIC
	    debug_cmd "" mkdir $SERVER_DEST_PUBLIC/config_files
	    debug_cmd "" mkdir "$SERVER_DEST/logs"
	    debug_cmd "" mkdir "$OUTSAP/install_log"
	    debug_cmd "" mkdir "$OUTSAP/osp"
	    debug_cmd "" mkdir "$OUTSAP/lib"
	    debug_cmd "" mkdir "$OUTSAP/os"
	    debug_cmd "" mkdir "$OUTSAP/xvm"
	    debug_cmd "" mkdir "$OUTsmf"
	fi
    fi

    if [[ $SATELLITE -eq 1 ]]; then
	OUTSAT="$outdir/satellite"
	if [ ! "$LIMITED" = "limited" ]; then
	    debug_cmd "" mkdir $OUTSAT
	    debug_cmd "" mkdir $OUTSAT/bui
	    debug_cmd "" mkdir $OUTSAT/ha
	    debug_cmd "" mkdir $OUTSAT/osp
	    debug_cmd "" mkdir $OUTSAT/db
	    debug_cmd "" mkdir $OUTSAT/db/logs
	    debug_cmd "" mkdir $OUTSAT/db/logs/trace
	    debug_cmd "" mkdir $OUTSAT/db/logs/smf
	    debug_cmd "" mkdir $OUTSAT/db/mgmt
	    debug_cmd "" mkdir $OUTSAT/db/report
	    debug_cmd "" mkdir $OUTSAT/jmxdumps
	fi
    fi

    if [[ $PROXY -eq 1 ]]; then
	OUTPROXY="$outdir/proxy"
	if [ ! "$LIMITED" = "limited" ]; then
	    debug_cmd "" mkdir $OUTPROXY
	    debug_cmd "" mkdir "$OUTPROXY/proxydb"
	    debug_cmd "" mkdir "$OUTPROXY/osp"
	    debug_cmd "" mkdir "$OUTPROXY/jmxdumps"
	fi
    fi

    if [[ $AGENT -eq 1 ]]; then
	OUTAGENT="$outdir/agent"
	ABIN="bin"
	ALOGS="logs"
	ANOTIF="notifications"
	ACONFIG="config_files"
	if [ ! "$LIMITED" = "limited" ]; then
	    debug_cmd "" mkdir $OUTAGENT
	    debug_cmd "" mkdir "$OUTAGENT/$ABIN"
	    debug_cmd "" mkdir "$OUTAGENT/$ALOGS"
	    debug_cmd "" mkdir "$OUTAGENT/$ANOTIF"
	    debug_cmd "" mkdir "$OUTAGENT/$ACONFIG"
	    debug_cmd "" mkdir "$OUTAGENT/lib"
	    debug_cmd "" mkdir "$OUTAGENT/jmxdumps"
	fi
    fi

}

compressOutput()
{
    debug "--- End of log file ---"
    if [[ x$NOCOMPRESS == x"0" ]] ; then
	EXT_HEAD=""
	if [[ x"$OS" = x"SunOS" ]]; then
            EXT_HEAD="E"
	fi

	testgz=`gzip -V 2>$NULL`
	testc=`compress -? 2>$NULL`
	if [[ x$testgz == x"COMMAND NOT FOUND gzip -V"  ]] && [[ x$testc == x"COMMAND NOT FOUND compress -?" ]]; then
            warning "Tar-ing up the data ..."
            (cd "$OUTPUT_PREFIX" && tar cf$EXT_HEAD "${OUTPUT_DIR}.tar" "${OUTPUT_DIR}")
            error "Will not be able to compress the output directory tar, please do it manually!"
            EXTENSION=""
	else
            EXTENSION=".gz"

	    if [[ x$testgz == x$testc ]]; then
		EXTENSION=".Z"
	    fi
	    info "Compressing the output directory ${OUTPUT_PREFIX}/$OUTPUT_DIR ..."
	    `cd "$OUTPUT_PREFIX" && tar cf$EXT_HEAD - "${OUTPUT_DIR}" | gzip -f > "${OUTPUT_PREFIX}/${OUTPUT_DIR}.tar$EXTENSION"`
	    info "... done"
	fi

	echo ""
	MBSIZE="-h"
	if [[ x$OSVER == x"5.8" ]]; then
            MBSIZE=""
	fi
	fsize=`ls -l $MBSIZE $OUTPUT_PREFIX/${OUTPUT_DIR}.tar$EXTENSION | nawk '{print $5}'`
	info "The following file needs to be sent to Oracle Support"
	info "In case you had problems with the script, "
	info "please report it at $BUGEMAIL ."
	info "case number: ${CASENUM}, file size: $fsize"
	info "Output file: ${OUTPUT_PREFIX}/${OUTPUT_DIR}.tar$EXTENSION"
	sendToSun "${OUTPUT_PREFIX}" "${OUTPUT_DIR}.tar$EXTENSION"
	echo ""
    else
	info "The following directory needs to be sent to Oracle Support"
	info "In case you had problems with the script, "
	info "please report it at $BUGEMAIL ."
	info "Directory with collected logs:"
	info "${OUTPUT_PREFIX}/${OUTPUT_DIR}"
    fi
}

removeWorkingDirectory()
{
    if [[ x$NOCOMPRESS == x"0" ]] ; then
	info "Cleaning up the working directory ..."
	rm -rf "$OUTPUT_PREFIX/$OUTPUT_DIR"
    fi
}

# parameter 1 is the absolute location of file to be sent
# parameter 2 is the file name to be sent to Sun
sendToSun()
{
    # TODO add curl/wget compatibility here:
    if [[ $TRANSFER -eq "1" ]]
    then
	if ! [[ -x $SFTRANSPORT ]]
        then
            x=`look_for_prog sftransport`
            if [[ $? -ne 0 ]]
            then
                error "sftransport not found on this system! " 
                warning "Please upload the file MANUALLY !"
                info "Usually the file is uploaded to $SUPPORTSITE"
            else
                SFTRANSPORT=$x
            fi
	fi
	if [[ -x $SFTRANSPORT ]]
        then
            SFTRANSPORT_DIR=`$SFTRANSPORT -p | grep 'Transfer Directory:' | nawk '{print $3}'`
            SFTRANSPORT_VERSION=`$SFTRANSPORT -v | head -1 | nawk '{print $6}' | cut -d "." -f1`
            if [[ $SFTRANSPORT_VERSION -lt 2 ]]
            then
                error "SFTransport must be at least version 2 !"
                warning "Please upload the file MANUALLY !"
                info "Usually the file is uploaded to $SUPPORTSITE"
            else
		# ONLY NOW we can move the file to get uploaded automagically
		info "Moving output file to SFTransport transfer directory..."
		if [[ -d $SFTRANSPORT_DIR ]]
		then
		    debug_cmd "" mv $1/$2 $SFTRANSPORT_DIR/$2
		    if [[ $? -ne 0 ]]
		    then
			warning "Copy of the file to transport directory($SFTRANSPORT_DIR) failed (disk full?)"
			warning "Please upload the file MANUALLY !"
			info "Usually the file is uploaded to $SUPPORTSITE"
		    else
			info "using SFTransport dir:  $SFTRANSPORT_DIR "
			info "Check SFTransport logs to verify successful upload !!!"
			info "(usually # $SFTRANSPORT -i   will show some details)"
		    fi
		else
                    warning "SFTransport directory not found, please transfer manually"
                    info "Usually the file is uploaded to $SUPPORTSITE"
		fi
            fi
	fi
    else
	info "Usually the file is uploaded to $SUPPORTSITE"
    fi
}

enableDebugAgent()
{
    RC_FILE=${ABASEVAR}/bin/.uce.rc
    RC_FILE_ALL=${ABASEBIN}/bin/uce.rc
    if [[ -a ${RC_FILE} ]]; then
	info "Enabling debug on update agent"
	# Copy the file to a preGDD file, but first check if a pre-file already exists
	if [[ -a ${RC_FILE}.SunGDD ]]
	then
            info "Backup of ${RC_FILE} exists, debug of update agent should be already enabled, only restarting services"
       	    #return 0
	else
            debug_cmd "" cp -rp ${RC_FILE} ${RC_FILE}.SunGDD
            debug_cmd "${RC_FILE}.$$" grep -v default_logger-level ${RC_FILE}
            debug_cmd "" mv ${RC_FILE}.$$ ${RC_FILE}
            debug_cmd "${RC_FILE}.$$" grep -v uce_agent_app-level ${RC_FILE}
            debug_cmd "" mv ${RC_FILE}.$$ ${RC_FILE}
            for i in default_logger-level uce_agent_app-level
            do
		CMD="grep $i ${RC_FILE_ALL} | sed s/\"level, \\\"[^']*{/level, \\\"DETAILED\\\" {/g\" >> ${RC_FILE}"
		debug $CMD
		eval $CMD
            done
	fi
        
	if [[ ${RESTART} = "true" ]]; then
	    if [[ $AGENTUP -eq 1 ]]; then
		info "Restarting update agent"
		info "(if this takes too long, please manually kill uce_agent.bin)"
		if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
                    debug_cmd "" svcadm disable -s scn/update-agent
                    debug_cmd "" svcadm enable -s scn/update-agent
		elif [[ x$OSVER = x"5.9" ]] || [[ x$OSVER = x"5.8" ]];
		then
                    debug_cmd "" /etc/init.d/SUNWscn_update_agent stop
                    debug_cmd "" /etc/init.d/SUNWscn_update_agent start
		else
                    debug_cmd "" /etc/init.d/sun_scn_update_agent stop
                    debug_cmd "" /etc/init.d/sun_scn_update_agent start
		fi
	    else
		warning "Agent wasn't running, hence will not restart it."
	    fi 
	else
            warning "Update agent should be manually restarted to enable debugging"
	fi
    else
	info "No update agent set up, couldn't enable debug"	#'
    fi
}


enableDebugServer()
{
    RC_FILE=${SERVER}/cgi-bin/.uce.rc
    RC_FILE_ALL=${SERVER}/cgi-bin/uce.rc
    if [[ -a $RC_FILE ]]; then
	info "Enabling debug on update server"
	# Copy the file to a preGDD, file but first check if a pre-file already exists
	if [[ -a ${RC_FILE}.SunGDD ]]; then
	    info "Backup of ${RC_FILE} exists, debug of update server should be already enabled, only restarting services"
	    #return 0
	    DO_NOT_RETUN_INSTEAD_RESTART_SERVICES=1
	else
            debug_cmd "" cp -rp ${RC_FILE} ${RC_FILE}.SunGDD
	    for i in log.__file.uce_server_cgi-level log.__file.uce_server_cgi_publisher-level log.__file.uce_server_scheduler-level log.__file.default_logger-level invisible.debug.__group1.debug_mode 
	    do
	        debug_cmd "${RC_FILE}.$$" grep -v ${i} ${RC_FILE}
	        debug_cmd "" mv ${RC_FILE}.$$ ${RC_FILE}
	        debug_cmd "" chown uce-sds ${RC_FILE}
	        debug_cmd "" chgrp uce-sds ${RC_FILE}
	    done

	    for i in log.__file.uce_server_cgi-level log.__file.uce_server_cgi_publisher-level log.__file.uce_server_scheduler-level log.__file.default_logger-level
	    do
		CMD="grep $i ${RC_FILE_ALL} | sed s/\"level, \\\"[^']*{/level, \\\"DETAILED\\\" {/g\" >> ${RC_FILE}"	#'
		debug $CMD
		eval $CMD
	    done
	    CMD="grep invisible.debug.__group1.debug_mode ${RC_FILE_ALL} | sed s/\"false/true/g\" >> ${RC_FILE}"
	    debug $CMD
	    eval $CMD
	fi

	if [[ ${RESTART} = "true" ]]; then
	    if [[ $SATUP -eq 1 ]]; then
		info "Restarting the update server"
		if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
		    debug_cmd "" svcadm disable -s scn/uce-server
		    debug_cmd "" svcadm disable -s scn/uce-scheduler
		    debug_cmd "" svcadm enable -s scn/uce-server
		    debug_cmd "" svcadm enable -s scn/uce-scheduler
   		else
		    debug_cmd "" /etc/init.d/uce_server stop
		    debug_cmd "" /etc/init.d/uce_scheduler stop
		    debug_cmd "" /etc/init.d/uce_server start
		    debug_cmd "" /etc/init.d/uce_scheduler start
		fi
	    else
		warning "Update server wasn't running, hence will not restart it."			#'
	    fi
	else
            warning "Update server should be manually restarted to enable debugging"
	fi
    else
	info "No update server set up, couldn't enable debug"	#'
    fi
}

enableDebugConsole()
{
    RC_FILE=$CONSOLE/bin/.uce.rc
    RC_FILE_ALL=$CONSOLE/bin/uce.rc

    if [[ -a $RC_FILE ]]; then
	info "Enabling debug on update console"
	# Copy the file to a preGDD file, but first check if a pre-file already exists
	if [[ -a ${RC_FILE}.SunGDD ]]; then
	    echo "Backup of ${RC_FILE} exists, debug of update console should be already enabled"
	    return 0
	else
	    debug_cmd "" cp -rp  $RC_FILE ${RC_FILE}.SunGDD
	    debug_cmd "$RC_FILE.$$" grep -v default_logger-level $RC_FILE
	    debug_cmd "" mv $RC_FILE.$$ $RC_FILE
	    debug_cmd "$RC_FILE.$$" grep -v invisible.debug.__group1.debug_mode $RC_FILE
	    debug_cmd "" mv $RC_FILE.$$ $RC_FILE
	    CMD="grep default_logger-level ${RC_FILE_ALL} | sed s/\"level, \\\"[^']*{/level, \\\"DETAILED\\\" {/g\" >> ${RC_FILE}"	#'
	    debug $CMD
	    eval $CMD
	    CMD="grep invisible.debug.__group1.debug_mode $RC_FILE_ALL | sed s/\"false/true/g\">> $RC_FILE"
	    debug $CMD
	    eval $CMD

	    if [[ ${RESTART} = "true" ]]; then
		if [[ $SATUP -eq 1 ]]; then 
		    info "Restarting the update console"
		    if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
			debug_cmd "" /opt/SUNWxvmoc/lib/methods/scn-console restart 
   		    else
			debug_cmd "" /opt/sun/xvmoc/lib/methods/scn-console restart 
		    fi
		else
		    warning "Console wasn't up, hence will not restart it."
		fi
	    else
		warning "Please restart the console to activate debugging"
	    fi
	fi
    else
	info "No update console set up, couldn't enable debug"
    fi
}

#parameter is the cacao instance to set the limits for
increaseCacaoLogs()
{
    CACAOI=$1
    #Increase log file sizes
    LIMIT_PROP_FILE=$CACAO_ETCI/$CACAOI/private/limit
    LIMIT_GDD_PROP_FILE=${LIMIT_PROP_FILE}.SunGDD
    COUNT_PROP_FILE=$CACAO_ETCI/$CACAOI/private/count
    COUNT_GDD_PROP_FILE=${COUNT_PROP_FILE}.SunGDD
    info "Increasing cacao log size for: $CACAOI"
    if [[ -f $LIMIT_GDD_PROP_FILE || -f $COUNT_GDD_PROP_FILE ]]
    then
        info "Already run GDD so $CACAOI log file size should be increased"
    else
        debug_cmd "$LIMIT_GDD_PROP_FILE" $CACAOADM get-param -i $CACAOI log-file-limit
        debug_cmd "$COUNT_GDD_PROP_FILE" $CACAOADM get-param -i $CACAOI log-file-count
        debug_cmd "" $CACAOADM set-param -i $CACAOI log-file-limit=31457280
        debug_cmd "" $CACAOADM set-param -i $CACAOI log-file-count=15
    fi
}

enableDebugXvm()
{
    #info "Enabling Debug on $PRODUCT_AB"
    warning "Performance of affected components WILL degrade due to the debug mode!"

    #Enable debug cacao logs
    # we just need the root loggers, the rest should get its level from parent !
    for i in $CACAO_INS;do
	info "Enabling cacao logs for: $i"
	FILTERS=$CACAO_VAR/$i/logs/$i-filters.SunGDD

	if [ "$i" = "oem-ec" ] && [ -f /opt/sun/xvm/bin/sat_set_cacao_levels.pl ]; then
	    #### Need to stop service, enable debug, start service
	    if [[ ${RESTART} = "true" ]]; then
		info "Stopping EC services, this may take a few minutes..."
		if [[ $SATUP -eq 1 ]]; then
		    debug_cmd "" $SATADM stop -w -v
		fi
	    else
		if [[ $SATUP -eq 1 ]]; then
		    warning "Cacao is running, CANNOT change the EC log file size and number (other debug parameters are still being enabled)"
		fi
	    fi
	    info "Calling 'sat_set_cacao_levels.pl d'"
	    debug_cmd "" /opt/sun/xvm/bin/sat_set_cacao_levels.pl d
	    if [[ ${RESTART} = "true" ]]; then
		info "Starting EC services, this may take a few minutes..."
		if [[ $SATUP -eq 1 ]]; then
		    debug_cmd "" $SATADM start -w -v
		else
		    warning "EC wasn't running, hence will not restart it. Use '$SATADM stop -w -v' to start it manually."		#'
		fi
		
	    fi
	elif [ "$i" = "scn-proxy" ] && [ -f /opt/sun/xvm/bin/proxy_set_cacao_levels.pl ]; then
	    #### Need to stop service, enable debug, start service
	    if [[ ${RESTART} = "true" ]]; then
		info "Stopping Proxy services, this may take a few minutes..."
		if [[ $PROXYUP -eq 1 ]]; then
		    debug_cmd "" $PROXYADM stop -w -v
		fi
	    else
		if [[ $PROXYUP -eq 1 ]]; then
		    warning "Cacao is running, CANNOT change the Proxy log file size and number (other debug parameters are still being enabled)"
		fi
	    fi
	    info "Calling 'proxy_set_cacao_levels.pl d'"
	    debug_cmd "" /opt/sun/xvm/bin/proxy_set_cacao_levels.pl d
	    if [[ ${RESTART} = "true" ]]; then
		info "Starting Proxy services, this may take a few minutes..."
		if [[ $PROXYUP -eq 1 ]]; then
		    debug_cmd "" $PROXYADM start -w -v
		else
		    warning "Proxy wasn't running, hence will not restart it. Use '$PROXYADM start -w -v' to start it manually."		#'
		fi
	    fi
	else
	    #Note original levels
	    if [ -d $CACAO_VAR/$i/logs/ ]; then		## The default instance may be there but not enabled, so no folder is there
		## In other words, don't enable debug if the instance is not enabled or not started because we won't have logs anyways...
		debug_cmd "$FILTERS" $CACAOADM list-filters -i $i
		debug_cmd "" $CACAOADM set-filter -p -i $i net.sf.hibernate=INFO
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.hss=ALL
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.cacao=ALL
		debug_cmd "" $CACAOADM set-filter -p -i $i hal=ALL
		# we're not that interested in atom feeds:
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.cacao.atom.impl.AtomServlet=FINER
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.cacao.webserver.impl.security.CacaoAuthenticator=FINER
		
		# satellite specific
		debug_cmd "" $CACAOADM set-filter -p -i $i net.hibernate.util.JDBCExceptionReporter=ALL
		debug_cmd "" $CACAOADM set-filter -p -i $i org.logicalcobwebs.proxool=OFF
		
		#added by field:
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.scn.assetinv.dao.impl=ALL
		debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.scn.assetinv.domain.logic.impl=ALL
		
		#for y in `cat $FILTERS | cut -d "."  -f 1 | $sort | $uniq | \
		#grep -v "=" | grep -v " "`
		#do
		#   debug_cmd "" $CACAOADM set-filter -p -i $i $y=FINEST
		#done
		# add obligatory global filter
		#debug_cmd "" $CACAOADM set-filter -p -i $i global=FINEST
	    fi	### if folder exists
	fi ### if oem-ec AND sat_set_cacao_levels exists
    done

    # Changing BUI Log Levels
    if [[ -a $SC_FILE ]];then
	if [[ -f ${SC_FILE}.SunGDD ]]; then
	    info "Already run GDD so BUI levels are increased"
	else
	    debug_cmd "" cp -rp $SC_FILE ${SC_FILE}.SunGDD
	    CMD="grep -v org.apache.juli.FileHandler.level ${SC_FILE}.SunGDD | grep -v java.util.logging.ConsoleHandler.level > $SC_FILE"
     	    debug $CMD
     	    eval $CMD
     	    CMD="grep org.apache.juli.FileHandler.level ${SC_FILE}.SunGDD | sed s/\"level =[^']*\"/\"level = FINEST\"/g >> $SC_FILE"
     	    debug $CMD
     	    eval $CMD
     	    CMD="grep java.util.logging.ConsoleHandler.level ${SC_FILE}.SunGDD | sed s/\"level =[^']*\"/\"level = FINEST\"/g >> $SC_FILE"
     	    debug $CMD
     	    eval $CMD
	fi

	if [[ -f $SCPROP_FILE ]]; then
	    if [[ -f ${SCPROP_FILE}.SunGDD ]]; then
        	info "Already run GDD so SCPortal BUI levels are increased"
     	    else
		debug_cmd "" cp -rp $SCPROP_FILE ${SCPROP_FILE}.SunGDD
		CMD="grep -v log4j.rootLogger ${SCPROP_FILE}.SunGDD | grep -v log4j.appender.FILE.MaxFileSize > $SCPROP_FILE"
        	debug $CMD
        	eval $CMD
        	CMD="grep log4j.rootLogger ${SCPROP_FILE}.SunGDD | sed s/\"=[^']*\"/\"=FINEST,FILE\"/g >> $SCPROP_FILE"
        	debug $CMD
        	eval $CMD
        	CMD="grep log4j.appender.FILE.MaxFileSize ${SCPROP_FILE}.SunGDD | sed s/\"=[^']*\"/\"=40000KB\"/g >> $SCPROP_FILE"
        	debug $CMD
        	eval $CMD
     	    fi
	fi
	#else 
	#    info "BUI not installed on this node..."
    fi
    # Restart services to leave in a sane state (only if wasn't done before! this below is the old way. The new way above is using the .pl scripts to set levels. This restart below is for when those scripts dont exist
    if [ ! -f /opt/sun/xvm/bin/sat_set_cacao_levels.pl ] && [ ! -f /opt/sun/xvm/bin/proxy_set_cacao_levels.pl ]; then
	if [[ ${RESTART} = "true" ]]; then
	    info "Restarting services and increasing log size, this may take a few minutes:"
	    if [[ $PROXY -eq 1 ]]; then
		if [[ $PROXYUP -eq 1 ]]; then
	            debug_cmd "" $PROXYADM stop -w -v
	            increaseCacaoLogs scn-proxy
		else
	            warning "Proxy wasn't running, hence will not stop it."
		fi
	    fi
	    if [[ $SATELLITE -eq 1 ]]; then
		if [[ $SATUP -eq 1 ]]; then
	            debug_cmd "" $SATADM stop -w -v
	            increaseCacaoLogs default
	            debug_cmd "" $SATADM start -w -v
		else
	            warning "Satellite wasn't running, hence will not restart it."
		fi
	    fi
	    if [[ $PROXY -eq 1 ]]; then
		if [[ $PROXYUP -eq 1 ]]; then
	            debug_cmd "" $PROXYADM start -w -v
		else
	            warning "Proxy wasn't running, hence will not start it."		#'
		fi
	    fi
	    
	else
	    warning "Satellite/Proxy services need to be manually restarted to enable debugging!"
	fi
    fi
    if [[ ${RESTART} = "true" ]]; then				#### need to restart agent in new and old methods (because there is no .pl script for it)
	if [[ $AGENT -eq 1 ]]; then
            if [[ $AGENTUP -eq 1 ]]; then
        	info "Restarting agent services and increasing log size, this may take a few minutes:"
         	debug_cmd "" $CACAOADM stop -i scn-agent
         	increaseCacaoLogs scn-agent
         	debug_cmd "" $CACAOADM start -i scn-agent
            else
          	warning "Agent wasn't running, hence will not restart it."		#'
            fi
	fi
    else
	if [[ $AGENT -eq 1 ]] && [[ $AGENTUP -eq 1 ]]; then
	    warning "Agent services need to be manually restarted to enable debugging!"
	fi
    fi
}

restoreDebugAgent()
{
    RC_FILE=${ABASEVAR}/bin/.uce.rc
    RC_FILE_ALL=${ABASEBIN}/bin/uce.rc

    if [[ -a ${RC_FILE} ]]; then
	info "Restoring debug on update agent"
	if [[ -a ${RC_FILE}.SunGDD ]]; then
	    debug_cmd "" mv ${RC_FILE}.SunGDD ${RC_FILE}
	    if [[ ${RESTART} = "true" ]]; then
		if [[ $AGENTUP -eq 1 ]]; then 
		    info "Restarting the update agent"
		    info "(if this takes too long, please manually kill uce_agent.bin)"
		    if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
			debug_cmd "" svcadm disable -s scn/update-agent
			debug_cmd "" svcadm enable -s scn/update-agent
		    elif [[ x$OSVER = x"5.9" ]] || [[ x$OSVER = x"5.8" ]];
		    then
			debug_cmd "" /etc/init.d/SUNWscn_update_agent stop
			debug_cmd "" /etc/init.d/SUNWscn_update_agent start
		    else
			debug_cmd "" /etc/init.d/sun_scn_update_agent stop
			debug_cmd "" /etc/init.d/sun_scn_update_agent start
		    fi
		else
		    warning "Agent wasn't running, hence will not restart it"	#'
		fi
	    else 
		warning "Update agent should be manually restarted to disable debugging"
	    fi
	else
	    error "${RC_FILE}.SunGDD file was not found."
	    error "Cannot revert or debug is already off...."
	fi
    else
	info "No update agent installed, cannot restore it."
    fi
}

restoreDebugServer()
{
    RC_FILE=$SERVER/cgi-bin/.uce.rc
    RC_FILE_ALL=$SERVER/cgi-bin/uce.rc

    if [[ -a $RC_FILE ]]; then
	info "Restoring debug on update server"
	if [[ -a ${RC_FILE}.SunGDD ]]
	then
            debug_cmd "" mv ${RC_FILE}.SunGDD $RC_FILE
            if [[ ${RESTART} = "true" ]]; then
		if [[ $SATUP -eq 1 ]]; then
		    info "Restarting the update server"
		    if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
			debug_cmd "" svcadm disable -s scn/uce-server
			debug_cmd "" svcadm disable -s scn/uce-scheduler
			debug_cmd "" svcadm enable -s scn/uce-server
			debug_cmd "" svcadm enable -s scn/uce-scheduler
		    else
			debug_cmd "" /etc/init.d/uce_server stop
			debug_cmd "" /etc/init.d/uce_scheduler stop
			debug_cmd "" /etc/init.d/uce_server start
			debug_cmd "" /etc/init.d/uce_scheduler start
		    fi
		else
		    warning "Update server wasn't running, hence will not restart it."
		fi
            else
		warning "Server should be manually restarted to disable debugging"
            fi
	else
            error "${RC_FILE}.SunGDD was not found."
            error "Cannot revert or debug is already off...."
	fi
    else
	info "No update server installed, cannot restore it."
    fi
}

restoreDebugConsole()
{
    RC_FILE=$CONSOLE/bin/.uce.rc
    RC_FILE_ALL=$CONSOLE/bin/uce.rc

    if [[ -a $RC_FILE ]]; then
	info "Restoring debug on update console"
	if [[ -a ${RC_FILE}.SunGDD ]]
	then
	    debug_cmd "" mv ${RC_FILE}.SunGDD $RC_FILE
	    if [[ ${RESTART} = "true" ]]; then
		if [[ $SATUP -eq 1 ]]; then
		    info "Restarting the update console"
		    if [[ x$OSVER = x"5.10" ]] || [[ x$OSVER = x"5.11" ]] || [[ x$OSVER = x"5.12" ]]; then
			debug_cmd "" /opt/SUNWxvmoc/lib/methods/scn-console restart
		    else
			debug_cmd "" /opt/sun/xvmoc/lib/methods/scn-console restart
		    fi
		else
		    warning "Console wasn't running, hence will not restart it."	#'
		fi 
	    else
		warning "Please restart the console to activate debugging"
	    fi
	else
	    error "${RC_FILE}.SunGDD was not found."
	    error "Cannot revert or debug is already off...."
	fi
    else
	info "No update console installed, cannot restore it."
    fi
}

#parameter is the cacao instance to restore the limits for
restoreCacaoLogs()
{
    CACAOI=$1
    #Set default log file sizes
    LIMIT_PROP_FILE=$CACAO_ETCI/$CACAOI/private/limit
    LIMIT_GDD_PROP_FILE=${LIMIT_PROP_FILE}.SunGDD
    COUNT_PROP_FILE=$CACAO_ETCI/$CACAOI/private/count
    COUNT_GDD_PROP_FILE=${COUNT_PROP_FILE}.SunGDD
    info "Setting default cacao log size for: $CACAOI"
    if [[ -f $LIMIT_GDD_PROP_FILE && -f $COUNT_GDD_PROP_FILE ]]
    then
        LIMIT=`cat $LIMIT_GDD_PROP_FILE`
        COUNT=`cat $COUNT_GDD_PROP_FILE`
        debug_cmd "" $CACAOADM set-param -i $CACAOI $LIMIT
        debug_cmd "" $CACAOADM set-param -i $CACAOI $COUNT
        debug_cmd "" rm $LIMIT_GDD_PROP_FILE $COUNT_GDD_PROP_FILE
    else
        info "No old GDD file was found."
        info "Cannot revert or debug is already off...."
    fi
}

restoreDebugXvm()
{
    #info "Restoring Debug on $PRODUCT_AB"
    info "Performance of affected components should improve back."
    
    #Disable debug cacao logs
    # we just need the root loggers, the rest should get its level from parent !
    for i in $CACAO_INS
    do
	info "Disabling cacao logs for: $i"
	FILTERS=$CACAO_VAR/$i/logs/$i-filters.SunGDD
	
	if [ "$i" = "oem-ec" ] && [ -f /opt/sun/xvm/bin/sat_set_cacao_levels.pl ]; then
	    #### Need to stop service, enable debug, start service
	    if [[ ${RESTART} = "true" ]]; then
		info "Stopping EC services, this may take a few minutes..."
		if [[ $SATUP -eq 1 ]]; then
		    debug_cmd "" $SATADM stop -w -v
		fi
	    else
		if [[ $SATUP -eq 1 ]]; then
		    warning "Cacao is running, CANNOT change the EC log file size and number (other debug parameters are still being disabled)"
		fi
	    fi
	    info "Calling 'sat_set_cacao_levels.pl p'"
	    debug_cmd "" /opt/sun/xvm/bin/sat_set_cacao_levels.pl p
	    if [[ ${RESTART} = "true" ]]; then
		info "Starting EC services, this may take a few minutes..."
		if [[ $SATUP -eq 1 ]]; then
		    debug_cmd "" $SATADM start -w -v
		else
		    warning "EC wasn't running, hence will not restart it. Use '$SATADM stop -w -v' to start it manually."		#'
		fi
		
	    fi
	elif [ "$i" = "scn-proxy" ] && [ -f /opt/sun/xvm/bin/proxy_set_cacao_levels.pl ]; then
	    #### Need to stop service, enable debug, start service
	    if [[ ${RESTART} = "true" ]]; then
		info "Stopping Proxy services, this may take a few minutes..."
		if [[ $PROXYUP -eq 1 ]]; then
		    debug_cmd "" $PROXYADM stop -w -v
		fi
	    else
		if [[ $PROXYUP -eq 1 ]]; then
		    warning "Cacao is running, CANNOT change the Proxy log file size and number (other debug parameters are still being disabled)"
		fi
	    fi
	    info "Calling 'proxy_set_cacao_levels.pl p'"
	    debug_cmd "" /opt/sun/xvm/bin/proxy_set_cacao_levels.pl p
	    if [[ ${RESTART} = "true" ]]; then
		info "Starting Proxy services, this may take a few minutes..."
		if [[ $PROXYUP -eq 1 ]]; then
		    debug_cmd "" $PROXYADM start -w -v
		else
		    warning "Proxy wasn't running, hence will not restart it. Use '$PROXYADM start -w -v' to start it manually."		#'
		fi
	    fi
	else
	    #if [[ -f $FILTERS ]] 		### Allow disabling debug even if file does not exist
	    #then
	    debug_cmd "" $CACAOADM set-filter -p -i $i net.sf.hibernate=OFF
	    debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.hss=WARNING
	    debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.cacao=INFO
	    debug_cmd "" $CACAOADM set-filter -p -i $i hal=WARNING
	    
	    #satellite specific
	    debug_cmd "" $CACAOADM set-filter -p -i $i net.hibernate.util.JDBCExceptionReporter=ALL
	    debug_cmd "" $CACAOADM set-filter -p -i $i org.logicalcobwebs.proxool=OFF
	    debug_cmd "" $CACAOADM set-filter -p -i $i javax.management.remote=WARNING
	    
	    #added by field:
	    debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.scn.assetinv.dao.impl=null
	    debug_cmd "" $CACAOADM set-filter -p -i $i com.sun.scn.assetinv.domain.logic.impl=null
	    
	    #Restore levels to NULL (will use the default logger levels)
	    #for y in `cat $FILTERS | cut -d "."  -f 1 | $sort | $uniq | \
	    #grep -v "=" | grep -v " "`
	    #do
	    #    debug_cmd "" $CACAOADM set-filter -p -i $i $y=NULL
	    #done
	    # add obligatory global filter
	    #debug_cmd "" $CACAOADM set-filter -p -i $i global=NULL
	    
	    debug_cmd "" rm $FILTERS
	    # else
	    #    info "$FILTERS was not found."
	    #    info "Cannot revert or debug is already off...."
	    # fi
	fi
    done

    # Changing BUI Log Levels
    if [[ -f ${SC_FILE}.SunGDD ]]
    then
	debug_cmd "" mv ${SC_FILE}.SunGDD $SC_FILE

	if [[ -f ${SCPROP_FILE}.SunGDD ]]
	then
            debug_cmd "" mv ${SCPROP_FILE}.SunGDD $SCPROP_FILE
	else
            info "SCPortal BUI debug was not enabled on this node..."
	fi

    else
	info "BUI debug was not enabled on this node..."
    fi

    # Restart services to leave in a sane state
    if [ ! -f /opt/sun/xvm/bin/sat_set_cacao_levels.pl ] && [ ! -f /opt/sun/xvm/bin/proxy_set_cacao_levels.pl ]; then
	if [[ ${RESTART} = "true" ]]; then
	    info "Restarting services and restoring log size, this may take a few minutes:"
	    if [[ $PROXY -eq 1 ]]; then
		if [[ $PROXYUP -eq 1 ]]; then
	            debug_cmd "" $PROXYADM stop -w -v
	            restoreCacaoLogs scn-proxy
		else
	            warning "Proxy wasn't running, hence will not stop it."
		fi
	    fi
	    if [[ $SATELLITE -eq 1 ]]; then
		if [[ $SATUP -eq 1 ]]; then
	            debug_cmd "" $SATADM stop -w -v
	            restoreCacaoLogs default
	            debug_cmd "" $SATADM start -w -v
		else
	            warning "Satellite wasn't running, hence will not restart it."
		fi
	    fi
	    if [[ $PROXY -eq 1 ]]; then
		if [[ $PROXYUP -eq 1 ]]; then
	            debug_cmd "" $PROXYADM start -w -v
		else
	            warning "Proxy wasn't running, hence will not start it."		#'
		fi
	    fi
	else
	    warning "Satellite/Proxy services need to be manually restarted to disable debugging"
	fi
    fi
    if [[ ${RESTART} = "true" ]]; then				#### need to restart agent in new and old methods (because there is no .pl script for it)
	if [[ $AGENT -eq 1 ]]; then
            if [[ $AGENTUP -eq 1 ]]; then
        	info "Restarting agent services and increasing log size, this may take a few minutes:"
         	debug_cmd "" $CACAOADM stop -i scn-agent
         	restoreCacaoLogs scn-agent
         	debug_cmd "" $CACAOADM start -i scn-agent
            else
          	warning "Agent wasn't running, hence will not restart it."		#'
            fi
	fi
    else
	if [[ $AGENT -eq 1 ]] && [[ $AGENTUP -eq 1 ]]; then
	    warning "Agent services need to be manually restarted to disable debugging!"
	fi
    fi
}


# Script execution starts here
# main

initialize
scriptName=`basename $0`
scriptDir=`dirname $0`
parseOptions $ARGS

umask $UMASK
exit 0

# tw=76
# vim: set et ts=4 sw=4:
