#!/bin/bash
#
# Script to check for a broken .uce.rc config file of the UCE agent
#
# If you are running this script from an Operational Profile, you have use the variable
# "OCDOCTOR_FIX" to determine if the issue should be fixed immediately:
#
# OCDOCTOR_FIX = 0 : Script will exit with failure exit code 1 in case of broken .uce.rc file
#
# OCDOCTOR_FIX = 1 : Script will fix a broken .uce.rc file and restart UCE-Agent
#

#### General variables ###
UNAME=`uname`
if [ "$UNAME" = "Linux" ]; then
        AGENT_DIR="/var/opt/sun/scn/update-agent"
	AWK="/usr/bin/awk"
else
        AGENT_DIR="/var/scn/update-agent"
	AWK="/usr/bin/nawk"
fi

UCERC="${AGENT_DIR}/bin/.uce.rc"
CONFIG="${AGENT_DIR}/config_files"
CHANNELS="${CONFIG}/channels.xml"
DETECT_SCRIPTS="${CONFIG}/*_detect.sh"

get_channel_id(){
	detected_channel=""
	TEMPFILE=/tmp/channel_ids_$$
	if [ -f $CHANNELS ]; then
		egrep '<EXTENSION>|<CHANNEL_ID>' $CHANNELS |\
			sed 's/^.*<EXTENSION>//' |\
			sed 's@</EXTENSION>.*$@@' |\
			sed 's/^.*<CHANNEL_ID>//' |\
			sed 's@</CHANNEL_ID>.*$@@' |\
		while read OS_NAME
		do
			read CHANNEL_ID
			echo "$OS_NAME $CHANNEL_ID"
		done > $TEMPFILE
		for os_detect_script in $DETECT_SCRIPTS
		do
			if $os_detect_script 2>/dev/null; then
				detected_OS=$(basename $os_detect_script _detect.sh)
				detected_channel=$(grep $detected_OS $TEMPFILE | $AWK '{print $2}')
				break
			fi
		done
	fi
	rm -f $TEMPFILE
	echo $detected_channel
}

trigger_uceagent_restart(){
	UCE_AGENT=uce_agent.bin
	AGENT_PID=$(ps -ef | grep -v grep | grep $UCE_AGENT | $AWK '{print $2}')
	if [ -n "$AGENT_PID" ]; then
		kill -HUP $AGENT_PID
		echo "Sleeping 100 seconds to wait for the Update-Agent to restart..."
		echo
		sleep 100
		AGENT_PID=$(ps -ef | grep -v grep | grep $UCE_AGENT | $AWK '{print $2}')
		if [ -n "$AGENT_PID" ]; then
			echo "Update-Agent has restarted successfully"
		else
			echo "Update-Agent has not restarted after 100 seconds. Check Agent Status manually!"
		fi
	else
		echo "Update-Agent is not running, restart the Agent manually!"
	fi
	echo
}

# check if .uce.rc file exists

if [ -f $UCERC ]; then
	
	# check for server_name entry
	if grep 'invisible.server.__general.server_name' $UCERC >/dev/null; then
		SERVER_NAME_OK=true
	else
		SERVER_NAME_OK=false
	fi
	# check for channel_id entry
	if grep 'invisible.startup.actions.__general.last_detect_script_channel_int' $UCERC >/dev/null; then
		CHANNEL_ID_OK=true
	else
		CHANNEL_ID_OK=false
	fi

	if $SERVER_NAME_OK; then
		echo "OK: Config file .uce.rc contains server_name"
	else
		echo
		echo "ERROR: Config file .uce.rc does not contain server_name entry with Proxy Controller IP/hostname"
		echo
		echo "Due to this issue, OS Update actions for this Asset will not be available in the BUI"
		echo
		SERVER_IP=$(sc-console list-connections | grep scn-agent | cut -d: -f2 | cut -c3-)

		# if the channel_id is missing in .uce.rc as well, we will try to re-add it
		$CHANNEL_ID_OK || CHANNEL_ID=$(get_channel_id)

		# we will skip the channel part if channel_id is undefined
		[ -z "$CHANNEL_ID" ] && CHANNEL_ID_OK=true

		if [ "$OCDOCTOR_FIX" = "1" ]; then
			echo "Trying to fix this issue automatically..."
			echo
			echo "( all ) ( invisible.server.__general.server_name, \"${SERVER_IP}\" );" >> $UCERC
			$CHANNEL_ID_OK || echo "( all ) ( invisible.startup.actions.__general.last_detect_script_channel_int, ${CHANNEL_ID} );" >> $UCERC
			echo "Config file $UCERC has been fixed!"
			echo
			trigger_uceagent_restart
		else
			echo "The following line(s) needs to be added to file ${UCERC}:" 
			echo
			echo "( all ) ( invisible.server.__general.server_name, \"${SERVER_IP}\" );"
			$CHANNEL_ID_OK || echo "( all ) ( invisible.startup.actions.__general.last_detect_script_channel_int, ${CHANNEL_ID} );"
			echo
			echo "followed by a restart of the Agent"
			echo
			echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this"
			echo
			exit 1
		fi
	fi
fi
