#!/bin/bash

ZSTAT_PROCESS=/var/adm/exacct/zstat-process

WARNING_SIZE=102400	# in KB, so let's start with 100 MB which is for sure already unhealthy

CONF=/etc/opt/sun/oc/zstat.conf
CONF_LINE="zstat_exacct_allowed=false"

SMF_SERVICE=svc:/application/xvm/zstat:default

SMF_LOG=/var/svc/log/application-xvm-zstat:default.log

EXPECTED_MESSAGE="xvm_zstat: Host not supported.  Lacks zones physical memory accounting features"

CACAO_CMD1="cacaoadm set-filter -i scn-agent com.sun.hss.domain.virtserver.impl.zone=INFO"
CACAO_CMD2="cacaoadm set-filter -i scn-agent -p com.sun.hss.domain.virtserver.impl.zone=INFO"

# Example extract of SMF logfile:
#
# [ Jun 22 06:20:18 Enabled. ]
# [ Jun 22 06:20:18 Executing start method ("/opt/SUNWxvmoc/lib/methods/xvm-zstat start") ]
# xvm_zstat: Host not supported.  Lacks zones physical memory accounting features
# [ Jun 22 06:20:18 Method "start" exited with status 0 ]
# 

if [ ! -f $ZSTAT_PROCESS ]; then
        # file zstat-process exists only in Solaris 10 Global Zones
	exit 0
fi


CURRENT_SIZE=$(du -k $ZSTAT_PROCESS | nawk '{print $1}')
if [ "${CURRENT_SIZE}" -gt $WARNING_SIZE ]; then
	echo 
	echo "ERROR: File $ZSTAT_PROCESS is of size $(du -h $ZSTAT_PROCESS | nawk '{print $1}') !"
	echo "ERROR: Looks like the file is growing faster than its contents can be processed!"
	echo "ERROR: There is a risk that the file system /var and the rpool gets fully filled up!"
	echo "ERROR:"
	echo "ERROR: The current recommendation is to stop collecting zone statistics for now!"
	echo
	if [ -f $CONF ]; then
		# ok, config file exists already, now we have to figure out why the file is still that large!
		#
		# let's check for the right directive
		if grep "^$CONF_LINE" $CONF >/dev/null 2>&1; then
			# ok, config file looks good
			#
			# did we restart the SMF service ? Let's check if the SMF log is newer than the config file
			#
			FILENAME=$(find $SMF_LOG -prune -newer $CONF)
			if [ -z "$FILENAME" ]; then
				# ok, SMF service needs to get restarted !
				echo "WARNING: The fix for this issue has already been applied, but the SMF service $SMF_SERVICE has not been restarted!"
				if [ "$OCDOCTOR_FIX" = "1" ];then
					echo "Trying to automatically fix this..."
					svcadm disable -s $SMF_SERVICE
					cp /dev/null $ZSTAT_PROCESS
					svcadm enable $SMF_SERVICE
					$CACAO_CMD1
					$CACAO_CMD2
					echo "Done."
				else
					echo
					echo "To fix this, run the following commands:"
					echo
					echo "svcadm disable -s $SMF_SERVICE"
					echo "cp /dev/null $ZSTAT_PROCESS"
					echo "svcadm enable $SMF_SERVICE"
					echo "$CACAO_CMD1"
					echo "$CACAO_CMD2"
					echo
					echo "The cacaoadm commands will avoid that the cacao logs will get filled with warnings,"
					echo "stating that the collection of zone statistics has been disabled."
					echo
					echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
				fi
			else
				# ok, SMF service has been restarted correctly
				#
				# could it be that the filesize has not ben reset to 0 ?
				FILENAME=$(find $SMF_LOG -prune -newer $ZSTAT_PROCESS)
				if [ -n "$FILENAME" ]; then
					# ok, the file itself has not been truncated back to 0 size !
					echo "WARNING: The fix for this issue has already been applied, but the file $ZSTAT_PROCESS has not been truncated!"
					if [ "$OCDOCTOR_FIX" = "1" ]; then
						echo "Trying to automatically fix this..."
						cp /dev/null $ZSTAT_PROCESS
						echo "Done."
					else
						echo
						echo "To fix this, run the following command:"
						echo
						echo "cp /dev/null $ZSTAT_PROCESS"
						echo
						echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
					fi
				else
					echo "WARNING: The correct workaround is already in place, but the issue is still there!"
					echo "WARNING: Please, open a Service Request to get the issue resolved!"
					echo
				fi
			fi
		else
			# config file is there but does not have the right directive
			echo "WARNING: The config file $CONF should have just this single line:"
			echo
			echo "$CONF_LINE"
			echo
			echo "WARNING: but the current content looks like this:"
			echo
			cat $CONF
			echo
			if [ "$OCDOCTOR_FIX" = "1" ]; then
				echo "Trying to automatically fix this..."
				echo "$CONF_LINE" > $CONF
				svcadm disable -s $SMF_SERVICE
				cp /dev/null $ZSTAT_PROCESS
				svcadm enable $SMF_SERVICE
				$CACAO_CMD1
				$CACAO_CMD2
				echo "Done."
			else
				echo
				echo "To fix this, run the following commands:"
				echo
				echo "echo $CONF_LINE > $CONF"
				echo "svcadm disable -s $SMF_SERVICE"
				echo "cp /dev/null $ZSTAT_PROCESS"
				echo "svcadm enable $SMF_SERVICE"
				echo "$CACAO_CMD1"
				echo "$CACAO_CMD2"
				echo
				echo "The cacaoadm commands will avoid that the cacao logs will get filled with warnings,"
				echo "stating that the collection of zone statistics has been disabled."
				echo
				echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
			fi
		fi
	else
		# config file does not yet exist
		if [ "$OCDOCTOR_FIX" = "1" ]; then
			echo "Trying to automatically fix this..."
			echo "$CONF_LINE" > $CONF
			svcadm disable -s $SMF_SERVICE
			cp /dev/null $ZSTAT_PROCESS
			svcadm enable $SMF_SERVICE
			$CACAO_CMD1
			$CACAO_CMD2

			echo "Done."
		else
			echo
			echo "To fix this, run the following commands:"
			echo
			echo "echo $CONF_LINE > $CONF"
			echo "svcadm disable -s $SMF_SERVICE"
			echo "cp /dev/null $ZSTAT_PROCESS"
			echo "svcadm enable $SMF_SERVICE"
			echo "$CACAO_CMD1"
			echo "$CACAO_CMD2"
			echo
			echo "The cacaoadm commands will avoid that the cacao logs will get filled with warnings,"
			echo "stating that the collection of zone statistics has been disabled."
			echo
			echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
		fi
	fi
	echo
else
       echo "OK: File $ZSTAT_PROCESS is of reasonable size"
fi
