#!/bin/bash
#
# this check is covering multiple bugs:
#
# 1) Bug 21464720 - OPScenter 12.3 jdk- 6 update to 101 rev break agent communication
#
#    Affected systems: S10 & S11 GZs running JDK 6_101
#
# 2) Bug 21566146 - AC on CDOM with S10 cannot connect to ldmd and thus to PC also
#    Bug 21620080 - Connection between Agent and Proxy cannot be established: No negotiable cipher
#    Bug 22256330 - LDM to OC communication via XMPP keeps getting SSL handshake failure
#    Bug 22545191 - Upgrading a CDOM Agent is overwriting file /opt/sun/n1gc/etc/ldoms.properties
#
#    Affected systems: S10 CDOMs if Agent uses 1.6.0_101, 1.7.0_85 or 1.8.0_51 (or later)
#                      S11 CDOMs if Agent uses 1.7.0_85 (or later)
#                      S11 CDOMs running older SRUs like S11.1 SRU 18.5 are not able to work with latest JDK8 versions
#
#    Latest Update for S10 CDOMs:
#
#           Patch 151934-02 for LDOM 3.2 fixes the following bug
#
#           15816108 Ops Center Java XMPP client doesn't work on Java 1.7.0_07
#
#           The fix is relevant for all Java versions used by the Ops Center Agent
#           Important here: the fix allows to remove the following directive
#
#	    xmpp.ciphersuite.excluded=.*_CBC_.*
#
#           from file /opt/sun/n1gc/etc/ldoms.properties
#
#	    Excluding the CBC type cipher suites did cause all the issues introduced by the latest Java updates for JDK 6, 7 &8
#
#           On S10 CDOMs we will now check for
#		- LDOM 3.2
#		- installed patch 151934-02
#		- removed or commented out directive xmpp.ciphersuite.excluded=.*_CBC_.*
#
#	    Only then, any Java version or release will work fine 
#
#
#
#           In parallel, Sustaining/Support did recommend to use a private JRE for the CDOM Agent.
#           Therefore, we need to check if a privat JRE is in use, no actions required then.
#                       
#
# 3) Bug 21470341 - SSL handshake error encountered on upgrade
#
#    This is not an Agent issue, but we can see if the PC needs to get fixed and will tell that
#
#
# 2015/08/17 - Update: New Agent bundles are available to address issue #1 (Bug 21464720)
#
#         12.2.1:  Fixed Agent Version is 12.2.1.813 or higher
#         12.2.2:  Fixed Agent Version is 12.2.2.1108 or higher
#         12.3.0:  Fixed Agent Version is 12.3.0.1410 or higher
#
# We need to check if the fixed bundles are in use to not throw a false alarm
#
# MOS Doc 2036973.1 contains the download links, the error/warning messages will point to it
#
# 2015/10/02 - Update: Added check for bug 21620080
#


#################################################################################################################################
#
# definitions and functions
#
#################################################################################################################################

UNAME=$(uname)
if [ "$UNAME" = "SunOS" ]; then
    CACAOADM=/usr/sbin/cacaoadm
    CACAOLOG=/var/cacao/instances/scn-agent/logs/cacao.0
else
    CACAOADM=/opt/sun/cacao2/bin/cacaoadm
    CACAOLOG=/var/opt/sun/cacao2/instances/scn-agent/logs/cacao.0
fi

# exit in case there is no Agent installed
[ -x $CACAOADM ] || exit

AGENT_PARAM=$($CACAOADM get-param -i scn-agent -v java-home 2>/dev/null)
#AGENT_PARAM=/opt/my-java

# no agent configured, if $AGENT_PARAM is null
[ -z "$AGENT_PARAM" ] && exit 0
AGENT_JAVA=$AGENT_PARAM/bin/java

# check for a private version of java used for the Agent
if [ -z "$(echo $AGENT_PARAM | egrep '^/usr/jdk/|^/usr/java/')" ]; then
	# a private version of Java is used, we will skip any further testing
	echo "INFO: Agent is using a private version of java located at $AGENT_JAVA" 
	exit 0
fi

VERSION=$($AGENT_JAVA -fullversion 2>&1 | cut -d\" -f2)
# Example strings:
# VERSION=1.6.0_95-b12
# VERSION=1.6.0_101-b14
# VERSION=1.7.0_80-b35
# VERSION=1.8.0_51-b16

MAJOR_REL=$(echo $VERSION | cut -d. -f2)
#echo "MAJOR_REL=$MAJOR_REL"
UPDATE_REL=$(echo $VERSION | cut -d_ -f2 | cut -d- -f1)
#echo "UPDATE_REL=$UPDATE_REL"
#echo "VERSION=$VERSION MAJOR_REL=$MAJOR_REL UPDATE_REL=$UPDATE_REL"

case "$MAJOR_REL" in
    '6') CRITICAL_VERSION=101;;
    '7') CRITICAL_VERSION=85;;
    '8') CRITICAL_VERSION=51;;
    '*') CRITICAL_VERSION="";;
esac

MY_DIR=`dirname $0`
. $MY_DIR/../../functions
get_agent_version       ### Provide VERSIONNUM
#echo "VERSIONNUM=$VERSIONNUM"
solaris11_stuff		### to define S11 specific parameters

JDK7=/usr/jdk/instances/jdk1.7.0
JDK8=/usr/jdk/instances/jdk1.8.0

if [ -x "$JDK7/bin/java" ]; then
    JDK7_INSTALLED=true
else
    JDK7_INSTALLED=false
fi

if [ -x "$JDK8/bin/java" ]; then
    JDK8_INSTALLED=true
else
    JDK8_INSTALLED=false
fi

show_howto_switch_jdk(){
    if $JDK7_INSTALLED; then
	if [ "$OCDOCTOR_FIX" = "1" -a "$UIDOCTOR" != "1" ]; then
	    echo "Trying to fix this automatically, it will require to shutdown/restart the Agent!"
	    echo "Shutting down Agent"
	    /opt/SUNWxvmoc/bin/agentadm stop
	    echo "Adjusting cacao parameters to use JDK7"
	    cacaoadm set-param -i scn-agent java-home=$JDK7
	    echo "Starting Agent"
	    /opt/SUNWxvmoc/bin/agentadm start
	    echo "done."
	else
	    echo
	    echo "       Here is an example how to change the Agent config to use JDK 7:"
	    echo
	    echo "       1) Stop Agent: /opt/SUNWxvmoc/bin/agentadm stop"
	    echo "       2) Change JDK: cacaoadm set-param -i scn-agent java-home=$JDK7"
	    echo "       3) Start Agent: /opt/SUNWxvmoc/bin/agentadm start"
	    echo
	    echo "       or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
            echo
            echo "       WARNING: A shutdown / restart of the Agent will be part of the fix!"
            echo
	fi
    elif $JDK8_INSTALLED; then
	if [ "$OCDOCTOR_FIX" = "1" -a "$UIDOCTOR" != "1" ]; then
	    echo "Trying to fix this automatically, it will require to shutdown/restart the Agent!"
	    echo "Shutting down Agent"
	    /opt/SUNWxvmoc/bin/agentadm stop
	    echo "Adjusting cacao parameters to use JDK8"
	    cacaoadm set-param -i scn-agent java-home=$JDK8
	    echo "Starting Agent"
	    /opt/SUNWxvmoc/bin/agentadm start
	    echo "done."
	else
            echo
            echo "       Here is an example how to change the Agent config to use JDK 8:"
            echo
            echo "       1) Stop Agent: /opt/SUNWxvmoc/bin/agentadm stop"
            echo "       2) Change JDK: cacaoadm set-param -i scn-agent java-home=$JDK8"
            echo "       3) Start Agent: /opt/SUNWxvmoc/bin/agentadm start"
            echo
	    echo "       or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
            echo
            echo "       WARNING: A shutdown / restart of the Agent will be part of the fix!"
            echo
	fi
    else 
	echo
    fi
}

show_howto_switch_to_jdk8(){
    #
    # let's first check if we have JRE 8 pkgs already installed or if we are able to install them 
    JRE8_PKG="runtime/java/jre-8"
    PKG_INSTALL_CMD_OUTPUT=/tmp/pkg_install_cmd.$$.out
    JRE8_IS_INSTALLABLE=true
    if ! $JDK8_INSTALLED; then
	# we will check if the package runtime/java/jre-8 can be installed
	pkg install -n --no-backup-be $JRE8_PKG >$PKG_INSTALL_CMD_OUTPUT 2>&1
	EXIT_CODE=$?
	if [ $EXIT_CODE -ne 0 ]; then
	    # Houston, we have an issue!
	    # We need JRE 8 for the fix, but it is not installed and we are not able to install it!
	    JRE8_IS_INSTALLABLE=false
	    echo
	    echo "  ERROR: A dry-run to install package $JRE8_PKG failed!"
	    echo "         This package is required to address the issues from Bug 21620080!"
	    echo
	    echo "         Review file $PKG_INSTALL_CMD_OUTPUT to see the reason for the installation failure."
	    echo
	else
	    rm $PKG_INSTALL_CMD_OUTPUT
	fi
    fi

    if $JRE8_IS_INSTALLABLE; then
	# ok, JDK8 is either already installed or we can install it, if required
	if [ "$OCDOCTOR_FIX" = "1" -a "$UIDOCTOR" != "1" ]; then
	    echo
	    echo "         Trying to fix this automatically, it will require to shutdown/restart the Agent!"

	    if ! $JDK8_INSTALLED; then
		# we will now install package jre-8
		pkg install --no-backup-be $JRE8_PKG >$PKG_INSTALL_CMD_OUTPUT 2>&1
		EXIT_CODE=$?
		if [ $EXIT_CODE -ne 0 ]; then
		    echo
		    echo "  ERROR: Unable to install package $JRE8_PKG which is required for the fix!"
		    echo "         This package is required to address the issues from Bug 21620080!"
		    echo
		    echo "         Review file $PKG_INSTALL_CMD_OUTPUT to see the reason for the installation failure."
		    echo
		else
		    # we were able to install JRE 8
		    if [ -x "$JDK8/bin/java" ]; then
			JDK8_INSTALLED=true
			rm $PKG_INSTALL_CMD_OUTPUT
		    else
			echo
			echo "  ERROR: The package $JRE8_PKG got installed successfully, but $JDK8/bin/java does not exist or is not executable!"
			echo
			echo "         Review file $PKG_INSTALL_CMD_OUTPUT to see if it provides a hint why the java binary is not useable."
			echo
			echo "         Without having JRE 8 available, the Agent will not be able to run on Solaris 11.2 SRU 12.6 or higher!"
			echo 
		    fi
		fi
            fi

	    if $JDK8_INSTALLED; then
		echo "Shutting down Agent"
		/opt/SUNWxvmoc/bin/agentadm stop
		echo "Adjusting cacao parameters to use JDK8"
		cacaoadm set-param -i scn-agent java-home=$JDK8
		echo "Starting Agent"
		/opt/SUNWxvmoc/bin/agentadm start
		echo "done."
	    fi
	else
            echo
	    if $JDK8_INSTALLED; then
	        echo "         Here is an example how to change the Agent config to use JRE 8:"
       	        echo
       	        echo "         1) Stop Agent: /opt/SUNWxvmoc/bin/agentadm stop"
               	echo "         2) Change cacao parameter: cacaoadm set-param -i scn-agent java-home=$JDK8"
                echo "         3) Start Agent: /opt/SUNWxvmoc/bin/agentadm start"
                echo
		echo "         or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
                echo
                echo "         WARNING: A shutdown / restart of the Agent will be part of the fix!"
	    elif $JRE8_INSTALLABLE; then
	        echo "         Here is an example how to change the Agent config to use JRE 8:"
       	        echo
		echo "         1) Install JRE 8: pkg install $JRE8_PKG"
       	        echo "         2) Stop Agent: /opt/SUNWxvmoc/bin/agentadm stop"
               	echo "         3) Change cacao parameter: cacaoadm set-param -i scn-agent java-home=$JDK8"
                echo "         4) Start Agent: /opt/SUNWxvmoc/bin/agentadm start"
                echo
		echo "         or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
                echo
                echo "         WARNING: A shutdown / restart of the Agent will be part of the fix!"
	    fi
            echo
	fi
    fi
}

agent_is_broken_already(){
    # ok Agent is broken already
    echo
    echo "ERROR: The installed JRE 6 version cannot be used by the Agent (Bug 21464720)!"
    if $IS_S11; then
	echo "       To fix this, it is recommended to switch to JRE 7/8."
    else
	echo "       MOS Doc 2036973.1 describes the issue in detail and contains download links of fixed Agent Bundles."
	echo "       In parallel, you can fix this by switching to JRE 7/8 or uninstalling JRE 6 patch $LATEST_PATCH."
    fi
    show_howto_switch_jdk
}

#################################################################################################################################
#
# main code with checks
#
#################################################################################################################################

# 
# check for patterns of Bug 21470341 - SSL handshake error encountered on upgrade
#
ERROR_STRING="javax.net.ssl.SSLHandshakeException: Remote host closed connection during handshake"

if tail -500 $CACAOLOG 2>/dev/null | grep "$ERROR_STRING" >/dev/null; then
    # ok, we can see SSL handshake errors in cacao log, we are not able to connect to the EC
    echo
    echo "WARNING: SSL handshake errors found in cacao log (MOS DocId 2036973.1)"
    echo "         A temporary unavailable Proxy could have caused these errors as well."
    echo "         Please run latest OCDoctor on the EC and all remote PCs to rule out any persistent connection issue!"
    echo
fi

VIRTUALIZATION_TYPE=$(grep '^virtualization.type=' /opt/sun/n1gc/lib/XVM_AGENT.properties 2>/dev/null | cut -d= -f2)
LDOMS_PROPERTIES=/opt/sun/n1gc/etc/ldoms.properties

#
# check for Bug 21566146 - AC on CDOM with S10 cannot connect to ldmd and thus to PC also
#
# Furthermore, if JDK 6 is used, we have to ensure that we are using a fixed Agent version or higher:
# 12.2.1.813
# 12.2.2.1108
# 12.3.0.1410
#
OS_RELEASE=$(uname -r)
if [ "$UNAME" = "SunOS" -a "$OS_RELEASE" = "5.10" ]; then
    if [ "$VIRTUALIZATION_TYPE" = "ldom" ]; then
	LDOM_VERSION=$(ldm -V 2>/dev/null| nawk '/Logical Domains Manager/ {print $5}' | cut -d. -f1-2 | tr '.' '0')
	[ -z "$LDOM_VERSION" ] && LDOM_VERSION=0
	#LDOM_VERSION=301
	#echo "LDOM_VERSION=$LDOM_VERSION"
        INSTALLED_PATCHREV="1$(installed_rev_of_patch 151934)"
	#INSTALLED_PATCHREV=1
	#echo "INSTALLED_PATCHREV=$INSTALLED_PATCHREV"
	[ -z "$INSTALLED_PATCHREV" ] && INSTALLED_PATCHREV=0
        if grep '^xmpp.ciphersuite.excluded.*=.*_CBC_.*' $LDOMS_PROPERTIES 1>/dev/null 2>&1; then
		CBC_ALLOWED=false
	else
		CBC_ALLOWED=true
	fi
	AGENT_UPGRADE_REQUIRED=false
	AGENT_REQUIRED_MINIMUM_VERSION=""
	if [ "$MAJOR_REL" -eq 6 ]; then
		# check for fixed Agent versions
		if [ "$VERSIONNUM" -lt "12210813" ]; then
			AGENT_UPGRADE_REQUIRED=true
			AGENT_REQUIRED_MINIMUM_VERSION="12.2.1.813"
		elif [ "$VERSIONNUM" -lt "12221108" ]; then
			AGENT_UPGRADE_REQUIRED=true
			AGENT_REQUIRED_MINIMUM_VERSION="12.2.2.1108"
		elif [ "$VERSIONNUM" -lt "12301410" ]; then
			AGENT_UPGRADE_REQUIRED=true
			AGENT_REQUIRED_MINIMUM_VERSION="12.3.0.1410"
		fi 
	fi

	
	if [ "$LDOM_VERSION" -eq 302 -a "$INSTALLED_PATCHREV" -ge 102 ] && $CBC_ALLOWED && ! $AGENT_UPGRADE_REQUIRED; then 
		echo "OK: 'CBC' ciphersuites can be used for XMPP connections to ldmd"
		if [ "$VERSIONNUM" -lt "12310000" -a "$UPDATE_REL" -ge "$CRITICAL_VERSION" ]; then
			echo "INFO: Upgrading the Agent to a newer version will overwrite changes in ldoms.properties (Bug 22545191)"
			echo "INFO: The Agent will be broken after upgrade until './OCDoctor.sh --troubleshoot --fix' has been run"
		fi
	else
		#UPDATE_REL="101"
		#echo "UPDATE_REL=$UPDATE_REL"
		#echo "CRITICAL_VERSION=$CRITICAL_VERSION"
		if [ "$UPDATE_REL" -lt "$CRITICAL_VERSION" ]; then
	    		# ok, not yet broken
	    		echo
	    		echo "WARNING: Do not upgrade JDK $MAJOR_REL to version 1.${MAJOR_REL}.0_$CRITICAL_VERSION or higher (Bug 21566146)!"
	    		echo "         The OpsCenter Agent will no longer be able to connect to the ldmd daemon and will be broken!"
		else
	    		# ok, we did upgrade java already and got probably hit by bug 21566146 :-(
   	    		echo
	    		echo "ERROR:   The Java version used by the Agent is 1.${MAJOR_REL}.0_${UPDATE_REL} which will cause connection issues (Bug 21566146)!"
		fi
		echo
		echo "         To be able to use Java versions 1.6.0_101, 1.7.0_85, 1.8.0_51 or higher,"
		echo "         the following condition(s) must be met:"
		echo
		
		if $AGENT_UPGRADE_REQUIRED; then
			echo "         - Agent must be upgraded at minimum to version $AGENT_REQUIRED_MINIMUM_VERSION (MOS Doc 2036973.1)"
		fi

		if [ "$LDOM_VERSION" -lt 302 ]; then
			echo "         - LDOM Manager 3.2 must be used"
		fi
		if [ "$INSTALLED_PATCHREV" -lt 102 ]; then
			echo "         - LDOM 3.2 Patch 151934-02 or higher must be installed"
		fi
		if ! $CBC_ALLOWED; then
			if [ "$LDOM_VERSION" -eq 302 -a "$INSTALLED_PATCHREV" -ge 102 ] && ! $AGENT_UPGRADE_REQUIRED; then
				# ok all conditions apart from ldoms.properties file are in place
				# all is good to offer an auto-fix
				if [ "$OCDOCTOR_FIX" = "1" -a "$UIDOCTOR" != "1" ]; then
					echo "Trying to fix this automatically, it will require to shutdown/restart the Agent!"
					echo "Shutting down Agent"
					/opt/SUNWxvmoc/bin/agentadm stop
					echo "Adjusting file $LDOMS_PROPERTIES"
					cp $LDOMS_PROPERTIES ${LDOMS_PROPERTIES}.ocdoctor.backup
					sed 's/^xmpp\.ciphersuite\.excluded=/#xmpp.ciphersuite.excluded=/' ${LDOMS_PROPERTIES}.ocdoctor.backup > $LDOMS_PROPERTIES
					echo "Starting Agent"
					/opt/SUNWxvmoc/bin/agentadm start
					echo "done."
				else
					echo "         - Remove or comment out the line"
					echo
					echo "           xmpp.ciphersuite.excluded=.*_CBC_.*"
					echo 
					echo "           in file '$LDOMS_PROPERTIES' and restart the Agent."
					echo
					echo "         or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this."
					echo
					echo "         WARNING: A shutdown / restart of the Agent will be part of the fix!"
				fi
			else
				echo "         - Remove or comment out the line"
				echo
				echo "           xmpp.ciphersuite.excluded=.*_CBC_.*"
				echo 
				echo "           in file '$LDOMS_PROPERTIES' and restart the Agent."
			fi
		fi
		echo
	fi
	
	exit 0
    fi
fi



#
# check for Bug 21620080 - Connection between Agent and Proxy cannot be established: No negotiable cipher
#
# S11 CDOMs are not able to use Java 1.7.0_85 or higher, they must switch to use JDK 8
#
if [ "$UNAME" = "SunOS" -a "$OS_RELEASE" = "5.11" ]; then
    if [ "$VIRTUALIZATION_TYPE" = "ldom" ]; then
  	# ok, this is a Control Domain Agent running on Solaris 11
	if [ "$MAJOR_REL" -lt 8 ]; then
	    if [ "$UPDATE_REL" -lt "$CRITICAL_VERSION" ]; then
		# ok, not yet broken
		#
		# ok, we learned from bug 22256330 that it is not a good idea to install JDK 8 on releases 
		# older than S11.1.SRU 19.5. Starting with S11.1 SRU 19.5 there is the pkg java-8-incorporation
		# which ensures that we install a version matching the entire version and not the latest one. 
		#
		if [ "$S11_RELEASE" -ge 2 ] || [ "$S11_RELEASE" -eq 1 -a "$S11_SRU" -ge 19 ]; then 
		    echo
		    echo "WARNING: Do not upgrade the OS to version Solaris 11.2 SRU 12.6 or higher (Bug 21620080)!"
		    echo
                    echo "         The current Java version used by the Agent is 1.${MAJOR_REL}.0_${UPDATE_REL}."
                    echo "         Before upgrading the OS, the Agent configuration must be changed to use Java 8!"
		    show_howto_switch_to_jdk8
		else
		    echo
		    echo "INFO: In case of a planned OS upgrade, do not immediately upgrade the OS to version"
		    echo "      Solaris 11.2 SRU 12.6 or higher, this will break the Agent (Bug 21620080)!"
		    echo "      To avoid this, you could upgrade first to Solaris 11.2 SRU 11.5 and re-run"
		    echo "      OCDoctor. It will show you the required steps to use Java 8 for the Agent."
		    echo  
		fi
	    else
		# ok, we did upgrade the OS already and got probably hit by bug 21620080 :-(
		echo
		echo "ERROR: The Agent is using Java 1.${MAJOR_REL}.0_${UPDATE_REL} which will cause ssl handshake issues (Bug 21620080)!"
		echo "       The Agent configuration must be changed to use JRE 8!"
		show_howto_switch_to_jdk8
	    fi
	else
	    echo "OK: Agent is not using JDK 6 or 7"
        fi
	# we will exit now, no further checks required
	exit 0
    fi
fi

#
# check for Bug 21464720 - OPScenter 12.3 jdk- 6 update to 101 rev break agent communication
#
# it is applicable only for S10 & S11 GZs 
#

ZONENAME="/sbin/zonename"

# check not applicable if we do not have zonename
[ -x "$ZONENAME" ] || exit

# check not applicable if this is not a global zone or kernel zone
[ "$($ZONENAME)" = "global" ] || exit

# check for fixed Agent versions
if [ "$VERSIONNUM" -ge "12210813" -a "$VERSIONNUM" -lt "12220000" ] || \
    [ "$VERSIONNUM" -ge "12221108" -a "$VERSIONNUM" -lt "12300000" ] || \
    [ "$VERSIONNUM" -ge "12301410" ]; then
    echo "OK: Good, found an Agent version with fixes for bug 21464720 in place"
    exit
fi

if [ "$(uname -r)" = "5.11" ]; then
    IS_S11=true
else
    IS_S11=false
    LATEST_PATCH=$(patchadd -p |nawk '/SUNWj6cfg/ {print $2}'|sort|tail -1)
fi

if [ "$MAJOR_REL" -eq 6 ]; then
    if [ "$UPDATE_REL" -lt 101 ]; then
	echo
	if $IS_S11; then
	    echo "INFO:  Do not update to latest Solaris 11.2 SRU 12.x or higher, it will update JDK 6"
	    echo "       to version 1.6.0_101 or higher, which in turn will break the Agent (MOS Doc 2036973.1)!"
	    echo "       It is recommended to switch to JDK 7/8."
	else
	    echo "INFO:  Do not update JDK 6 to version 1.6.0_101 or higher, this will break the Agent!"
	    echo "       MOS Doc 2036973.1 describes the issue in detail and contains download links to fixed Agent Bundles."
	    echo "       In parallel, you can fix this by switching to JDK 7/8."
	fi

	show_howto_switch_jdk

    else
	# ok we are already using a higher version
	# let's see if the Agent is running fine
	AGENT_STATE=$(svcs -H -o state scn-agent)
	if [ "$AGENTSTATE" = "online" ]; then
	    # get status of cacao module com.sun.hss.xvmserver.domainmodel
	    if cacaoadm status -i scn-agent com.sun.hss.xvmserver.domainmodel 2>&1 | grep 'not in good health' >/dev/null; then
		agent_is_broken_already
	    else
		echo "OK: Agent is using Java $VERSION and is not affected by bug 21464720"
	    fi
	else
	    # the Agent is not running, therefore we assume that it is broken already
	    #
	    # we tested already for the fixed Agent Bundle further up
	    #
	    agent_is_broken_already
	fi
    fi
else
    echo "OK: Agent is using JDK $MAJOR_REL and is not affected by bug 21464720"
fi
