#! /usr/bin/bash
#
# script to check cacao certs
#
# Concept from Jim Eggers - CMR Jul 6 2021
# Ver 0.1 Created Jul 8 2021 by Geoff Shipman
# Ver 0.2 Edits by Ted Ward - Jul 8 2021 - Cleaned up output
# Ver 0.3 Fixed check on OC_Services to avoid bad input
#
# Version 0.1 GS
# Version 0.2 TW 
# Version 0.3 GS
#
# Actions to check certs
# date
# cd /etc/cacao/instances/oem-ec/security/jsse
# ls -l
# /usr/bin/openssl x509 -enddate -noout -in agent.cert
# cd /etc/cacao/instances/scn-proxy/security/jsse
# ls -l
# /usr/bin/openssl x509 -enddate -noout -in agent.cert
# cd /etc/cacao/instances/scn-agent/security/jsse
# ls -l
# /usr/bin/openssl x509 -enddate -noout -in agent.cert
#
# Variables

# Locations
ETC_CACAO=/etc/cacao/instances
SECJSSE=security/jsse
VARTMP=/var/tmp

# OS commands
AWK=/usr/bin/awk
CAT=/usr/bin/cat
DATE=/usr/bin/date
GREP=/usr/bin/grep
HOST=/usr/bin/hostname
OPENSSL=/usr/bin/openssl
RM=/usr/bin/rm
SVCS=/usr/bin/svcs
TEE=/usr/bin/tee

# Options
AC=scn-agent						# Ops Center Agent Controller cacao instance
EC=oem-ec							# Ops Center Enterprise Controller cacao instance
PC=scn-proxy						# Ops Center Proxy Controller cacao instance
STDCMD=x509							# X.509 Certificate Data Management.

# Logs
OCServices=oc-smf-service-list.txt	# Ops Center cacao instance list from the host
ScriptLog=CacaoCertCheckLog.txt		# Script log file

# Files
CERT=agent.cert						# Certificate file

## Checks

PreChecks() {
#
# Start pre-checks
#
# Clear any prior logs
#
${CAT} /dev/null > ${VARTMP}/${ScriptLog} 2>/dev/null
${CAT} /dev/null > ${VARTMP}/${OCServices} 2>/dev/null
#
echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
echo "=== Start checks on Ops Center cacao instance(s): " | ${TEE} -a ${VARTMP}/${ScriptLog}
#
# Mark start of log file
#
echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
echo "Log run: "`${DATE} +'%a %b %e %T %Z %Y'` 2>&1 | ${TEE} -a ${VARTMP}/${ScriptLog}
echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
#
# SMF checks for cacao instances in a for loop.
#
	echo "=== Verify Ops Center SMF service(s) exist: " | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
	for INSTANCE in `echo "${EC}\n${PC}\n${AC}"`
	do
		${SVCS} ${INSTANCE} > /dev/null 2>&1
		RESULT=$?
		if [[ ${RESULT} != 0 ]]
		then
			echo "The "`${HOST}`" host does not show any "${INSTANCE}" Ops Center SMF cacao service." | ${TEE} -a ${VARTMP}/${ScriptLog}	
		else
			echo "Found "${INSTANCE} | ${TEE} -a ${VARTMP}/${ScriptLog}
		fi
	done
}

OC_Services() {
# 
# Determine Ops Center services on the host
#
	${SVCS} -v ${EC} ${PC} ${AC} 2>/dev/null | ${GREP} -v "^STATE" | ${AWK} '{print $5}' | ${AWK} -F: '{print $3}' > ${VARTMP}/${OCServices}
}

CertCheck() {
#
# Run the openssl check for expiration against each instances agent.cert file
#
	echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "=== Ops Center SMF cacao instance ===" | ${TEE} -a ${VARTMP}/${ScriptLog}
	for  OCcacao in `${CAT} ${VARTMP}/${OCServices}`
	do
		echo "Checking " ${OCcacao} "certs." | ${TEE} -a ${VARTMP}/${ScriptLog}
		${OPENSSL} ${STDCMD} -enddate -noout -in ${ETC_CACAO}/${OCcacao}/${SECJSSE}/${CERT} | ${TEE} -a ${VARTMP}/${ScriptLog}
		echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
	done	
}

CleanUp() {
	echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "If this check was run for an open service request please include the file or the contents of the file" | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "located "${VARTMP}/${ScriptLog} | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "" | ${TEE} -a ${VARTMP}/${ScriptLog}
	echo "Please review instructions for use of the fix script (ocnewcerts171.sh) in MOS Doc ID 2790192.1" | ${TEE} -a ${VARTMP}/${ScriptLog}
	${RM} ${VARTMP}/${OCServices}
}

## Fix - for future use

#
# Main
#
PreChecks
OC_Services
CertCheck
CleanUp

