#!/bin/bash
#
# Ops Center 12c R3 U1 is using Oracle DB version 12.1.0.2
#
# Ops Center 12c R2 U2 and 12c R3 GA are using Oracle DB version 11.2.0.4
#
# Ops Center versions up to 12c R2 U1 are using Oracle DB version 11.2.0.3
#
# We need to check if the two required zip-files with the new version are in place
# and in parallel, we need to have the former two DB zip-files available in case of roll-back.
#
# Summary:
# 		11g U3 - 12c R1 U4    -> check for the two v11.2.0.3 files
#		12c R2 GA & 12c R2 U1 -> check for the two v11.2.0.3 files and two v11.2.0.4 files
#		12c R2 U2             -> check for the two v11.2.0.4 files
#		12c R3 GA             -> check for the two v11.2.0.4 files and v12.1.0.2 files
#		12c R3 U1             -> check for the two v12.1.0.2 files
#	        

MY_DIR=`dirname $0`
if [ -f $MY_DIR/../../functions ]; then
        . $MY_DIR/../../functions
else
		exit
fi

# check if a local DB is used
DB_PROPERTIES=/var/opt/sun/xvm/db.properties
if [ -f $DB_PROPERTIES ]; then
        #### Determine if this local or remote
        DB_TYPE=`grep "^mgmtdb.server.type" $DB_PROPERTIES | awk -F"=" '{print $2}' 2>/dev/null`
else
        echo "ERROR: $0: DB Properties file $DB_PROPERTIES not found!"
        exit 1
fi

if [ "$DB_TYPE" = "remote" ]; then
	echo "OK: Remote DB is used, no need to check for Oracle DB update requirements"
	exit 0
fi

DB_FILES_11_2_0_3_REQUIRED=false
DB_FILES_11_2_0_4_REQUIRED=false
DB_FILES_12_1_0_2_REQUIRED=false

get_version
if [ $VERSIONNUM -gt "11101571" -a $VERSIONNUM -lt "12200000" ]; then
	DB_FILES_11_2_0_3_REQUIRED=true
elif [ $VERSIONNUM -gt "12200000" -a $VERSIONNUM -lt "12220000" ]; then
	DB_FILES_11_2_0_3_REQUIRED=true
	DB_FILES_11_2_0_4_REQUIRED=true
elif [ $VERSIONNUM -gt "12220000" -a $VERSIONNUM -lt "12300000" ]; then
	DB_FILES_11_2_0_4_REQUIRED=true
elif [ $VERSIONNUM -gt "12300000" -a $VERSIONNUM -lt "12310000" ]; then
	DB_FILES_11_2_0_4_REQUIRED=true
	DB_FILES_12_1_0_2_REQUIRED=true
elif [ $VERSIONNUM -gt "12310000" -a $VERSIONNUM -lt "12320000" ]; then
	DB_FILES_12_1_0_2_REQUIRED=true
else
	# Calypso U2 Development Build
	DB_FILES_12_1_0_2_REQUIRED=true
fi

# until release of 12.3.1, we will not request the new DB files
#DB_FILES_12_1_0_2_REQUIRED=false

DOWNLOADS="/var/tmp/downloads"
UNAME=`uname`
ARCH=`uname -p`
if [ "$UNAME" = "Linux" ]; then
	if [ "$ARCH" = "x86_64" ]; then
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_Linux-x86-64_1of7.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_Linux-x86-64_2of7.zip"
		V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_Linux-x86-64_1of7.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_Linux-x86-64_2of7.zip"
                V12_1_0_2_FILE1="$DOWNLOADS/linuxamd64_12102_database_1of2.zip"
                V12_1_0_2_FILE2="$DOWNLOADS/linuxamd64_12102_database_2of2.zip"
	else
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_LINUX_1of7.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_LINUX_2of7.zip"
                V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_LINUX_1of7.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_LINUX_2of7.zip"
		DB_FILES_12_1_0_2_REQUIRED=false
	fi	
else
	if [ "$ARCH" = "i386" ]; then
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_Solaris86-64_1of6.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_Solaris86-64_2of6.zip"
                V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_Solaris86-64_1of6.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_Solaris86-64_2of6.zip"
                V12_1_0_2_FILE1="$DOWNLOADS/solaris.x64_12102_database_1of2.zip"
                V12_1_0_2_FILE2="$DOWNLOADS/solaris.x64_12102_database_2of2.zip"
	else
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_SOLARIS64_1of7.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_SOLARIS64_2of7.zip"
                V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_SOLARIS64_1of7.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_SOLARIS64_2of7.zip"
                V12_1_0_2_FILE1="$DOWNLOADS/solaris.sparc64_12102_database_1of2.zip"
                V12_1_0_2_FILE2="$DOWNLOADS/solaris.sparc64_12102_database_2of2.zip"
	fi	
fi

if $DB_FILES_11_2_0_3_REQUIRED; then
	if [ ! -d $DOWNLOADS ] || [ ! -f ${V11_2_0_3_FILE1} ] || [ ! -f ${V11_2_0_3_FILE2} ]; then
		echo
		echo "WARNING: The following two Oracle DB files used for EC upgrade or EC fresh install are missing:"
		echo
		echo "${V11_2_0_3_FILE1}"
		echo "${V11_2_0_3_FILE2}"
		echo
		if [ $VERSIONNUM -lt "12100000" ]; then 
			echo "The files are required to upgrade the EC to Ops Center 12.1.0!"
		else
			echo "In case of an EC upgrade failure, the files will be required to re-install the Oracle DB during roll-back!"
		fi
		echo
		echo "To download, login to https://updates.oracle.com/Orion/PatchDetails/process_form?patch_num=10404530"
	
		case "$ARCH" in
			'i386')
				echo "and select the 'Oracle Solaris on X86-64 (64-bit)' platform."
				;;
			'sparc')
				echo "and select the 'Oracle Solaris on SPARC (64-bit)' platform."
				;;
			'x86_64')
				echo "and select the 'Linux X86-64' platform."
				;;
			'i686')
				echo "and select the 'Linux X86' platform."
				;;
		esac
	
		echo
		echo "You only need to download the first 2 files (part 1 and 2), not all of them."
		echo "Place those 2 files in $DOWNLOADS on this machine to be prepared in case of EC upgrade failures."
		echo ""
	else
		echo "OK: Found the two DB files required for EC upgrade up to Ops Center 12.2.1 or roll-back to the current state in case of upgrade failures."
	fi
fi

if $DB_FILES_11_2_0_4_REQUIRED; then
    if [ ! -d $DOWNLOADS ] || [ ! -f ${V11_2_0_4_FILE1} ] || [ ! -f ${V11_2_0_4_FILE2} ]; then
	echo ""
	if [ $VERSIONNUM -gt "12220000" -a $VERSIONNUM -lt "12300000" ]; then 
		echo "WARNING: The upgrade to Ops Center 12.3.0 requires the following two Oracle DB files:"
	elif [ $VERSIONNUM -gt "12300000" -a $VERSIONNUM -lt "12310000" ]; then
		echo "WARNING: A future upgrade of this Ops Center EC will require the following two Oracle DB files:"
	else
		echo "WARNING: The upgrade to Ops Center 12.2.2 or 12.3.0 requires the following two Oracle DB files:"
	fi
	echo ""
	echo "${V11_2_0_4_FILE1}"
	echo "${V11_2_0_4_FILE2}"
	echo ""
	echo "The files are required to upgrade the Oracle DB or to re-install the Oracle DB during roll-back in case of upgrade failures!"
	echo ""
	echo "To download, login to https://updates.oracle.com/Orion/PatchDetails/process_form?patch_num=13390677"
                
	case "$ARCH" in
		'i386')
			echo "and select the 'Oracle Solaris on X86-64 (64-bit)' platform."
			;;
		'sparc')
			echo "and select the 'Oracle Solaris on SPARC (64-bit)' platform."
			;;
		'x86_64')
			echo "and select the 'Linux X86-64' platform."
			;;
		'i686')
			echo "and select the 'Linux X86' platform."
			;;
	esac
                
	echo
	echo "You only need to download the first 2 files (part 1 and 2), not all of them."
	echo "Place those 2 files in $DOWNLOADS on this machine before starting the EC upgrade."
	echo ""
    else
	echo "OK: Found the required DB files for upgrading the EC to Ops Center 12.2.2 or 12.3.0"
    fi
fi

if $DB_FILES_12_1_0_2_REQUIRED; then
    if [ ! -d $DOWNLOADS ] || [ ! -f ${V12_1_0_2_FILE1} ] || [ ! -f ${V12_1_0_2_FILE2} ]; then
	echo ""
	if [ $VERSIONNUM -gt "12300000" -a $VERSIONNUM -lt "12310000" ]; then 
		echo "WARNING: The upgrade to Ops Center 12.3.1 will require the following two Oracle DB files:"
		echo ""
		echo "${V12_1_0_2_FILE1}"
		echo "${V12_1_0_2_FILE2}"
		echo ""
		echo "The files are required to upgrade the Oracle DB to version 12.1.0.2!"
	else
		echo "WARNING: A future upgrade of this Ops Center EC will require the following two Oracle DB files:"
		echo ""
		echo "${V12_1_0_2_FILE1}"
		echo "${V12_1_0_2_FILE2}"
		echo ""
		echo "The files are required to re-install the Oracle DB during roll-back in case of upgrade failures!"
	fi
	echo ""
	echo "To download, login to http://www.oracle.com/technetwork/database/enterprise-edition/downloads/index.html"
                
	case "$ARCH" in
		'i386')
			echo "and select the 'Oracle Solaris on X86-64 (64-bit)' platform."
			;;
		'sparc')
			echo "and select the 'Oracle Solaris on SPARC (64-bit)' platform."
			;;
		'x86_64')
			echo "and select the 'Linux X86-64' platform."
			;;
	esac
                
	echo
	echo "Place those 2 files in $DOWNLOADS on this machine before starting the EC upgrade."
	echo ""
    else
	if [ $VERSIONNUM -gt "12300000" -a $VERSIONNUM -lt "12310000" ]; then 
		echo "OK: Found the required DB files for upgrading the EC to Ops Center 12.3.1"
		
	else
		echo "OK: Found the two DB files required to re-install the Oracle DB during roll-back in case of future upgrade failures!"
	fi
    fi
fi

