#!/bin/bash
#
# During 12c R2 U2 Upgrade, we are recompiling all OC schema objects.
# If there are objects in 'invalid' state, even after the recompile, the compile script will exit with an error causing the upgrade to fail
#
# We need to fix this prior to upgrade!
#
# Check got enhanced with an auto-fix option in case all invalid objects names are matching known patterns:
#
# V_VMB_ORA_PRE_ENG_SYS_CON_*
# V_VDO_OPES_CTRL_COMPONENT_INFO
# V_VDO_DATA_LINK_BINDING
#
# Further enhanced the check to run it on all EC version > 12c R2 GA
# 

MY_DIR=`dirname $0`
if [ -f $MY_DIR/../../functions ]; then
        . $MY_DIR/../../functions
	get_version
else
	echo "ERROR: ${0}: functions file $MY_DIR/../../functions not found!"
	exit 1
fi

# check if a local or remote DB is used
DB_PROPERTIES=/var/opt/sun/xvm/db.properties
if [ -f $DB_PROPERTIES ]; then
        #### Determine if this local or remote
        LOCALREMOTE=`grep "^mgmtdb.server.type" $DB_PROPERTIES | awk -F"=" '{print $2}' 2>/dev/null`
else
        echo "ERROR: $0: DB Properties file $DB_PROPERTIES not found!"
        exit 1
fi

if [ "$LOCALREMOTE" = "remote" ]; then
	echo "OK: EC is using a remote DB, no check required for DB objects in invalid state"
	exit 0
fi

if [ $VERSIONNUM -lt "12200000" ]; then
	echo "OK: EC is not running 12c R2 GA or higher, no need to check for DB Schema Objects in INVALID state"
	exit 0
fi
	

UNAME=`uname`
if [ "$UNAME" = "Linux" ]; then
	AWK="/usr/bin/awk"
        SAT_BIN="/opt/sun/xvmoc/bin/ecadm"
else
	AWK="/usr/bin/nawk"
        SAT_BIN="/opt/SUNWxvmoc/bin/ecadm"
fi

RECOMPILE_SQL=/tmp/recompile.$$.sql
RECOMPILE_OUT=/var/tmp/recomp_inv_obj_oc.log

OCDOCTOR_SQL=/tmp/ocdoctor.sql.$$
OCDOCTOR_OUT=/tmp/ocdoctor.out.$$

cat > $OCDOCTOR_SQL <<EOF
set lines 100
set pages 1000
column object_name format a35
column object_type format a20
column status format a10
spool $OCDOCTOR_OUT
select OBJECT_NAME,OBJECT_TYPE,STATUS from user_objects where status != 'VALID';
spool off;
EOF

cat > $RECOMPILE_SQL <<EOF
-- Copyright 2012, 2013 Oracle Corporation
--
-- Module: recomp_inv_obj_oc.sql
--
-- DESCRIPTION
--   This script provides a packaged interface to recompile invalid
--   PL/SQL modules, Java classes, indextypes and operators in a
--   database sequentially or in parallel.
--   The script will check the status of the OC objects after compilation
--   IF EC schema objects are still in invalid state the script will exit with an error code 1 
--   When EC schema objects in valid states the script will exit status 0
--   Any other errors the script will exit with -999 code

-- This sql script is executed After EC installations configured to use a local
-- database.  It is not called until after all database objects have been created/altered so
-- they can be altered in this script.

spool /var/tmp/recomp_inv_obj_oc.log

prompt "SQLTrace: Entering recomp_inv_obj_oc.sql"

-- we do not want to fail an install/upgrade because of this compilation script.
whenever sqlerror exit failure SQL.SQLCODE;
set serveroutput on

var exitCode number



DECLARE
L_object_name	dba_objects.object_name%type;
L_object_type	dba_objects.object_type%type;


CURSOR C_get_inv_obj IS 
                      select object_name,object_type
			      from dba_objects 
			       where owner ='OC' 
			         and status='INVALID' 
			         order by object_type,object_name
       				 ;
       				 

R_get_inv_obj    C_get_inv_obj%ROWTYPE;       				 
Begin
 dbms_output.enable(buffer_size => NULL);
 
 DBMS_OUTPUT.PUT_LINE('Compiling the OC Schema');
 
 UTL_RECOMP.RECOMP_SERIAL('OC');
 
 DBMS_OUTPUT.PUT_LINE('Done Compiling the OC Schema');
 
 OPEN  C_get_inv_obj;
 
 FETCH C_get_inv_obj into R_get_inv_obj;

IF C_get_inv_obj%NOTFOUND THEN
    :exitCode:=0;
    return; 
ELSE
  DBMS_OUTPUT.PUT_LINE(' Found Objects in invalid states');
END IF;
 
 WHILE (C_get_inv_obj%FOUND)
 Loop
   DBMS_OUTPUT.PUT_LINE('****** Object name '||R_get_inv_obj.object_name ||' Object type '||R_get_inv_obj.object_type||' is in invalid state');
   FETCH C_get_inv_obj into R_get_inv_obj;
   
       
 END LOOP;     
 
 CLOSE C_get_inv_obj;
 :exitCode:= 1;
 
EXCEPTION WHEN OTHERS THEN
  :exitCode:=-999;
  dbms_output.put_line('Error was encountered -'||SQLCODE||'-ERROR- '||SQLERRM);
  
End;
/

prompt "SQLTrace: Exiting recomp_inv_obj_oc.sql"

exit :exitCode;

EOF

# ok first run to check for objects in INVALID state
echo "@$OCDOCTOR_SQL" | ${SAT_BIN} sqlplus >/dev/null 2>/dev/null

if [ -f $OCDOCTOR_OUT ]; then

	if grep 'Not connected' $OCDOCTOR_OUT >/dev/null; then
		echo
		echo "WARNING: DB is not available, unable to check for DB Schema Objects in INVALID state"
		echo
	elif grep 'INVALID' $OCDOCTOR_OUT >/dev/null; then
		# OK, we have DB Schema Objects in INVALID state
		# we will recompile now the invalid DB objects
		su - oracleoc -c "echo @$RECOMPILE_SQL | sqlplus / as sysdba" >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			# even after the recompile, we have objects in INVALID state !
			#
			# now, we will check if the DB Schema Objects in INVALID state are only views with the following name patterns
			#
			# V_VMB_ORA_PRE_ENG_SYS_CON_*
			# V_VDO_OPES_CTRL_COMPONENT_INFO
			# V_VDO_DATA_LINK_BINDING
			#
			# These views should have been dropped as part of upgrade to Diamond, but looks like dropping failed sometimes.
			#
			rm -f $OCDOCTOR_OUT
			echo "@$OCDOCTOR_SQL" | ${SAT_BIN} sqlplus >/dev/null 2>/dev/null
			FILTERED_OUTPUT=/tmp/ocdoctor.filtered.$$.out
			egrep -v '^$|^OBJECT_NAME|^----|rows selected|^V_VMB_ORA_PRE_ENG_SYS_CON_[0-9]*.*VIEW.*INVALID|^V_VDO_DATA_LINK_BINDING.*VIEW.*INVALID|^V_VDO_OPES_CTRL_COMPONENT_INFO.*VIEW.*INVALID' $OCDOCTOR_OUT > $FILTERED_OUTPUT
			FIX_IS_POSSIBLE=true
			if [ -s $FILTERED_OUTPUT ]; then
				# OK, we have unexpected DB Schema Objects in INVALID state or other output, Customer need to open SR and get this fixed
				FIX_IS_POSSIBLE=false
			fi
				
			echo
			echo "ERROR: DB contains Schema Objects in INVALID state!"
			echo
			echo "This will cause EC upgrade failures resulting in a rollback!"
			echo
			echo "List with objects in state 'INVALID':"
			cat $OCDOCTOR_OUT | grep -v '^SQL>'
			echo
			if $FIX_IS_POSSIBLE; then
				if [ "$OCDOCTOR_FIX" = "1" ]; then
					echo "Trying to automatically fix this..."
					for i in $($AWK '/^V_VMB_ORA_PRE_ENG_SYS_CON_[0-9]*.*VIEW.*INVALID|^V_VDO_DATA_LINK_BINDING.*VIEW.*INVALID|^V_VDO_OPES_CTRL_COMPONENT_INFO.*VIEW.*INVALID/ {print $1}' $OCDOCTOR_OUT)
					do
						echo "drop view ${i};" | ${SAT_BIN} sqlplus >/dev/null 2>/dev/null
					done
					echo "Done."	
				else
					echo "To fix this, the listed views must be dropped before upgrading the EC!"
					echo
					echo "Use the following command(s) to drop the view(s) from the DB Schema:"
					echo
					for i in $($AWK '/^V_VMB_ORA_PRE_ENG_SYS_CON_[0-9]*.*VIEW.*INVALID|^V_VDO_DATA_LINK_BINDING.*VIEW.*INVALID|^V_VDO_OPES_CTRL_COMPONENT_INFO.*VIEW.*INVALID/ {print $1}' $OCDOCTOR_OUT)
					do
						echo "# echo 'drop view ${i};' | ${SAT_BIN} sqlplus"
					done
					echo
					echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this"
					echo
				fi
			else	
				echo "Please open a Service Request with Oracle Support to get this issue resolved!"
				echo
				echo "Trying to upgrade the EC in this state will result in a failure and rollback to current state!"
				echo
			fi
		else
			echo "OK: No DB Schema Objects in INVALID state found after recompile"
		fi
	else
		echo "OK: No DB Schema Objects in INVALID state found"
	fi
else
	echo "OK: No DB Schema Objects in INVALID state found"
fi

rm -f $RECOMPILE_SQL $RECOMPILE_OUT $OCDOCTOR_SQL $OCDOCTOR_OUT $FILTERED_OUTPUT
