#!/bin/bash

# Script to check if host requires the fix for 26823628
# 1. Warn during troubleshoot
# 2. Fix with workaround with '--fix'

# Only applicable for EC version 12.3.3
# Since support for OVM 3.5 is added only with 12.3.3

TMP_FILE="/tmp/sql_output.$$.txt"

UNAME=`uname`
if [ "$UNAME" = "Linux" ]; then
    ECADM="/opt/sun/xvmoc/bin/ecadm"
else
    ECADM="/opt/SUNWxvmoc/bin/ecadm"
fi

CURR_DIR=`pwd`
MY_DIR=`dirname $0`
SQL_DIR="${MY_DIR}/../../pis/sql/12.3.3/"
SQL="2017-10-27-seed_product_attributes.sql"
if [ -f $MY_DIR/../../functions ]; then
    source $MY_DIR/../../functions
else
    echo "ERROR: file not found: $MY_DIR/../functions. Exiting ..."
    exit 1
fi


# Only applicable for 12.3.3
get_version ""
if [ $VERSIONNUM -lt 12331801 ] || [ $VERSIONNUM -gt 12401200 ]; then
    echo "OK: Not Ops Center 12.3.3 build 1801."
    exit
fi


check_fix() {

    # Check if the fix has already been applied by looking for one of the product IDs
    # being inserted.

    select_check="select product_uuid from product_attributes where key='LDOM_VERSION_35_FW_REC' and product_uuid='Q7944';"
    echo -e ${select_check} | ${ECADM} sqlplus >${TMP_FILE} 2>&1
    grep PRODUCT_UUID ${TMP_FILE} >/dev/null
    return $?

}

check_fix
if [ $? -eq 0 ]; then
    echo "OK: Fix for Bug 26823628 has already been applied on the Enterprise Controller"
    exit 0
else
    echo
    echo "WARNING: The Enterprise Controller is missing some product attributes in the DB, pertaining to"
    echo "         the support for LDom Manager 3.5 (Bug 26823628). "
    echo
    echo "         To fix this automatically, please run '# ./OCDoctor.sh --troubleshoot --fix'"
    echo "                (or)"
    echo "         Please run the sql script manually to insert the missing attributes:"
    echo "         $ ${ECADM} sqlplus -d ${CURR_DIR}/${SQL_DIR} @${SQL}"
    echo
fi

if [ "$OCDOCTOR_FIX" = "1" ]; then

    echo "INFO: Trying to automatically fix this..."

    if [ -f ${SQL_DIR}/${SQL} ]; then

        echo "INFO: Located the SQL script. Running it now..."
        ${ECADM} sqlplus -d ${CURR_DIR}/${SQL_DIR} @${SQL} >/dev/null 2>&1

        # Check if the values were inserted
        check_fix
        if [ $? -eq 0 ]; then
            echo "INFO: Fix was applied successfully"
            echo
        else
            echo
            echo "ERROR: Could not apply fix automatically. Please run the manual steps provided above"
            echo "       (or) contact Oracle support."
            echo
        fi

    else

        echo
        echo "ERROR: Could not locate the SQL script to run the fix automatically. From the current directory, run:"
        echo "       $ find . -name ${SQL}"
        echo "       Please run the script manually in the same format as mentioned in the warning above."
        echo "       (or) contact Oracle support."

    fi

fi

# Cleanup
rm ${TMP_FILE} >/dev/null
