#!/bin/bash
#
# Bug 20223169 - Ops Center upgrade from 12.2.1 to 12.2.2 with CC agent failed with DB corruption
#
# If an EM Agent has been installed on the EC, the Oracle upgrade will break
#
# Since the issue can happen with any other Oracle Software, the check will be more general
#
# We will check for a file "/var/opt/oracle/oraInst.loc" and if it exists, we will temporary rename it
#
# Only after the upgrade to 12c R2 U2, it should be renamed to the original name. 
#

# source functions
MY_DIR=`dirname $0`
if [ -f $MY_DIR/../../functions ]; then
        . $MY_DIR/../../functions
        get_version ""          ### Provide VERSIONNUM
fi


if [ $VERSIONNUM -lt 12200000 ]; then
	# this check is only applicable for Opscenter 12.2.0 and above
        exit 0
fi

# check if a local or remote DB is used
DB_PROPERTIES=/var/opt/sun/xvm/db.properties
if [ -f $DB_PROPERTIES ]; then
	#### Determine if this local or remote
	LOCALREMOTE=`grep "^mgmtdb.server.type" $DB_PROPERTIES | awk -F"=" '{print $2}' 2>/dev/null`
else
	echo "ERROR: $0: DB Properties file $DB_PROPERTIES not found!"
        exit 1
fi

UNAME=`uname`
if [ "$UNAME" = "Linux" ]; then
        ORAINST_LOC=/etc/oraInst.loc
else
        ORAINST_LOC=/var/opt/oracle/oraInst.loc
fi

ORAINST_LOC_RENAMED=${ORAINST_LOC}_RENAMED_BY_OCDOCTOR

if [ "$LOCALREMOTE" = "local" ]; then
	if [ -f "$ORAINST_LOC_RENAMED" ]; then
		if [ $VERSIONNUM -lt 12220000 ]; then
			echo "OK: File $ORAINST_LOC has already be renamed by OCDoctor"
		else
			echo
			echo "WARNING: File $ORAINST_LOC is still renamed to $ORAINST_LOC_RENAMED !"
			echo
			if [ "$OCDOCTOR_FIX" = "1" ]; then
				printf "Trying to automatically fix this..."
				mv $ORAINST_LOC_RENAMED $ORAINST_LOC
				echo "done."
			else
				echo "To fix this, rename $ORAINST_LOC_RENAMED back to $ORAINST_LOC"
				echo
				echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this"
			fi
			echo
		fi
	elif [ -f "$ORAINST_LOC" ]; then
		if [ $VERSIONNUM -lt 12220000 ]; then
			echo
			echo "ERROR: Existing file $ORAINST_LOC will cause the Upgrade to 12c R2 U2 or higher to fail!"
			echo
			if [ "$OCDOCTOR_FIX" = "1" ]; then
				printf "Trying to automatically fix this..."
				mv $ORAINST_LOC $ORAINST_LOC_RENAMED
				echo "done."
			else
				echo "To fix this, rename $ORAINST_LOC to something like $ORAINST_LOC_RENAMED"
				echo
				echo "or use '# ./OCDoctor.sh --troubleshoot --fix' to automatically fix this"
			fi
			echo
		else
			echo "OK: EC is already running 12c R2 U2 or higher, no check required for other installed Oracle Products"
		fi
	else
		echo "OK: No other Oracle Products installed which could break DB Upgrade" 
	fi	
else
	echo "OK: EC is using a remote DB, no check required for other installed Oracle Products"
fi
