#!/bin/ksh -p
#
# Copyright 2010 Oracle and/or its affiliates. All rights reserved.
# Use is subject to license terms.
#

#trap 'exit' EXIT INT TERM QUIT
#set -eu

alias err='trap '"'"'ERR=$?; [[ $ERR -ne 123 ]] && echo "$0: error on line $LINENO (ret=$ERR)"; return $ERR'"'"' ERR'
trap '[[ $? -eq 123 ]] && exit 1' EXIT

scriptName=`basename $0`
scriptDir=`dirname $0`

cd $scriptDir

if [ ! -f ./${scriptName} ];then
        echo "Please run $scriptName from the its folder"
        exit 1
fi

perl=/usr/bin/perl

OUTPUT_PREFIX="/var/tmp" #default temp directory
CASENUM="11111111"

# sanitize environment, more in initialize and look_for_prog
PATH=/usr/bin:/bin:/usr/sbin:/sbin
export PATH

LOGFILE="/dev/null"
LOG="tee"
# prepareOutputDirectory will set above up

appName="update_pis"
postfixName="log_noarch"
PRODUCT="Ops Center"
PRODUCT_AB="OC"
if [[ -f $scriptDir/version ]]; then
  fver="$criptDirversion"
else
  fver="$scriptDir/../version"
fi
VERSION=`head -1 $fver`
cmdline="$0 $@"
LANG=C
ARGS=$@
ARGC=$#
SWLINK="http://wikis.sun.com/display/OC2dot5/OC+Doctor"
DOCLINK="http://wikis.sun.com/display/OC2dot5/Sun+Gathering+Debug+Data+for+Sun+Ops+Center"
BUGEMAIL="ocdoctor@sun.com"
FEEDBACKEMAIL="ocdoctor@sun.com"
SUPPORTSITE="http://supportfiles.sun.com"


#-----
##
## @function fail
##
## @desc Print message to stderr and exit
##
## @param $1 - Optional text
##
#-----
function fail {
        print -u 2 "$@"
        exit 123
}

#-----
##
## @function usage
##
## @desc Displays brief usage help
##
#-----
function usage {
        err

        cat << EOT
Usage:

$scriptName -s
        - Simulation mode - just do a dry run, proper backup of affected tables, but no new updates

$scriptName -d
        - Deployment mode - install PIS updates


EOT

}

#
# print error/warning/info message $@ prefixed
#
error() { echo "`date +%Y-%m-%d-%H-%M` ERROR: $@" | $LOG ; }
warning() { echo "`date +%Y-%m-%d-%H-%M` WARNING: $@" | $LOG ; }
# parameter to debug should be a string, even meta chars will be kept like
# string
debug() { echo "`date +%Y-%m-%d-%H-%M` DEBUG: '$@'" >> $LOGFILE ; }

# log(APPEND!) given command $2-$n to $LOGFILE and save its output to $1
# be warned, since the command is called through *eval*, so you might need to
# escape special chars !!! (e.g. \" or \' )
debug_cmd() {
    log=$1
    [[ x"$log" = x"" ]] && log=$LOGFILE
    shift
    echo "`date +%Y-%m-%d-%H-%M` \
DEBUG: executing command below, output going to $log" >> $LOGFILE
    echo $@ >> $LOGFILE
    eval $@ >>$log 2>&1
}

info()  { echo "`date +%Y-%m-%d-%H-%M` INFO: $@" | $LOG ; }

#
# Find external binaries we need to use on various platforms
#
initialize()
{
    uname=/bin/uname
    if ! [[ -x $uname ]] ; then
        uname=/sbin/uname
        if ! [[ -x $uname ]] ; then
         echo ERROR: /bin/uname or /sbin/uname not found or not executable, cannot continue in such environment ...
         exit 1
        fi
    fi
    alias uname=$uname
    OS=`uname`
    OSVER=`uname -r`
    ARCH=`uname -m`
    UMASK=`umask`
    #sanitize umask
    umask 022
    HOST=`hostname`

    headerdate=`date +%m/%d/%Y`
    datestamp=`date '+%Y-%m-%d-%H-%M'`
    #datestamp=`date "+%y%m%d-%H%M%S"`
    startdate=`date "+%y%m%d-%H%M%S"`

    LOG="tee"

    ID=`id |cut -d"(" -f1|cut -d"=" -f2`
}

# parameter 1 is the directory to check for space
# parameter 2 is the totalSize needed (will be adjusted by space multiplier)
checkAvailableSpace()
{

CHECKDIR=$1

SPACE_MULTIPLIER=2
par2=$2
totalSize="${par2:-30}"

totalSize=`expr $((($totalSize * $SPACE_MULTIPLIER)))`
info "Approximate space required: ${totalSize} MB"

#calculating the free space available in the output directory
if [[ x"$OS" = x"SunOS" ]]; then
    freeSpace=`df -k ${CHECKDIR} | nawk '(NR>1) {print $4}'`
elif [[ x"$OS" = x"Linux" ]]; then
    #Linux requires -P option explicitly for df in for the output to be POSIX compliant!
    freeSpace=`df -Pk ${CHECKDIR} | nawk '(NR>1) {print $4}'`
fi

freeSpace=`expr $freeSpace / 1024`
info "Space available in the output directory (${CHECKDIR}): ${freeSpace} MB"

#exit with error if enough space not available
if [[ ${totalSize} -gt ${freeSpace} ]]
then
    error "The output directory $CHECKDIR has only $freeSpace MB of free space."
    error "$totalSize MB of free space is approximately required."
    error "Please free up some disk space in target directory."
    #debug_cmd "" rm -rf $outdir
    exit 3
else
    info "Enough space available ..."
fi
}


#
# Prepare output directory and set the follwing variables
# OUTPUT_PREFIX=/var/tmp
# OUTPUT_DIR=<case>-$appName-<hostname>-<datetime>
# LOGFILE=/var/tmp/something...
# LOG=tee -a $LOGFILE
#
prepareOutputDirectory()
{
    OUTPUT_DIR="$CASENUM-$appName-$HOST-$datestamp"
    LOGFILE="$OUTPUT_PREFIX/$scriptName.log"

    outdir="$OUTPUT_PREFIX/$OUTPUT_DIR"

    # verify that $OUTPUT_PREFIX is dir and writable
    if ! [[ -d $OUTPUT_PREFIX ]] || ! [[ -w $OUTPUT_PREFIX ]] ; then
        echo "ERROR: The output directory $OUTPUT_PREFIX is not a writtable directory."
        echo "Please use -o switch to redirect output somewhere else."
        exit 3
    fi

    # once we are sure we can write to output, redefine the logging macro to log
    # to a temp log file
    LOG="tee -a $LOGFILE"

    #below not needed, oc-doctor should announce the version
    debug "$appName Version $VERSION"
    info "Logging to $LOGFILE"
    info "User: `id`"
    info "OS: $OS version: $OSVER"
    info "Started as: $cmdline"

    info "Checking space requirements for logs ..."
    checkAvailableSpace $OUTPUT_PREFIX 5
    info "Checking space requirements for backup directory ..."
    checkAvailableSpace ${backupBase} 10

    debug_cmd "" mkdir -p "$outdir"
    LOGFILE="$OUTPUT_PREFIX/$OUTPUT_DIR/$scriptName.log"
    LOG="tee -a $LOGFILE"

    #moving the log file into the output directory
    debug_cmd "" mv "$OUTPUT_PREFIX/$scriptName.log $LOGFILE"
    info "Changing log location to $LOGFILE ..."

}

if [ "$1" = "" ]; then
	usage
	exit
fi

while getopts "hds?\?" opt
do
  case $opt in  
            d)
                ;;
            s)
                DRYRUN="1"
                ;;
            # below "works" only for Solaris
            h)
                usage
                exit 1
                ;;
            '?')
                usage
                exit 1
                ;;
            ?)
                echo ""
                echo "Invalid option specified."
                echo ""
                echo "Please use $PROGNAME -h to see detailed usage instructions."
                echo ""
                exit 1
                ;;
  esac
done

datestamp=`date '+%Y-%m-%d-%H-%M'`

#TODO get mgmt user from property file too ?
pwfile=/var/opt/sun/xvm/xvm-db.properties
passwd=`grep mgmtdb.password $pwfile`
passwd=${passwd#*"="}

#where the sql are located for current version!
versionfile=/n1gc-setup/.version.properties
version=`grep product.version "$versionfile"`
mversion=${version#*"="} #cut away all before =
mversion=${mversion%.*} #cut away last two .
mversion=${mversion%.*}

if [ "`uname`" = "Linux" ]
then
  sbasedir=/opt/sun/scs/sbin
#TODO cmds should be respective to OC version (2.5 2.6 ?)
  baseBin="/opt/sun/xvmoc/bin"
  dumpCommand="/usr/bin/env LD_LIBRARY_PATH= PGUSER=mgmt PGPASSWORD=$passwd /opt/sun/scs/bin/pg_dump -p 5434 mgmt"
else
  sbasedir=/opt/SUNWscs/sbin
  baseBin="/opt/SUNWxvmoc/bin"
  dumpCommand="/usr/bin/env LD_LIBRARY_PATH=/lib:/usr/lib:/usr/sfw/lib:/opt/SUNWscs/lib PGUSER=mgmt PGPASSWORD=$passwd /opt/SUNWscs/bin/pg_dump -p 5434 mgmt"
fi
dbtool=$sbasedir/db_tool.pl
restartCmd="$baseBin/proxyadm stop -w -v; $baseBin/satadm stop -w -v ; $baseBin/satadm start -w -v ; $baseBin/proxyadm start -w -v"

#below has to be absolute path
dropdir=`pwd`/sql/$mversion
#TODO make below configurable
backupBase=/var/opt/sun/xvm/update-saved-state
#backupBase=/var/tmp
backupDir=${backupBase}/PIS_backup-$datestamp

initialize
prepareOutputDirectory

info "Detected version of OC: $mversion"

info "Gathering tables to modify ..."
typeset tables
debug "$perl -ne '
        print $2, \"\n\" if m/^(update|insert\s+into)\s+([a-zA-Z0-9_]+)/;
' ${dropdir}/*.sql | sort -u"
tables=`$perl -ne '
	print $2, "\n" if m/^(update|insert\s+into)\s+([a-zA-Z0-9_]+)/;
' ${dropdir}/*.sql | sort -u`
debug "Need to backup these tables: $tables"

info "Backing up to $backupDir (this can take some time) ..."
mkdir -p $backupDir
for bt in $tables; do 
 info "Backing up table $bt"
# debug "$dumpCommand -t $bt 2>>&$LOGFILE > $backupDir/backup-$bt.sql"
 $dumpCommand -t $bt 2>>$LOGFILE > $backupDir/backup-$bt.sql
done

if [[ -n "$DRYRUN" ]] ; then
 info "DRY RUN: Would run the command:"
 info "$dbtool load ${dropdir}/*.sql"
else
 info "Loading sql updates now (this might take several minutes to hours)..."
 debug_cmd "$LOGFILE" LC_COLLATE=C $perl $dbtool load ${dropdir}/*.sql
 if [ $? -ne 0 ] ; then
      info "$0: There were some problems with loading of the sqls. Check the logs and send them to support. "
      #fail 1
 fi
 info "Also note that this just updates system info, it will not add new system icons or images."
fi

info "Update script done."
if [[ -z "$DRYRUN" ]] ; then
 info "Please DO restart your EC to complete the update."
 info "($restartCmd)"
fi

