#!/usr/bin/ksh
#
# Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
# preinstall_check [-e]
#
# This script is to verify that the system has all the required resources before installing an Ops Center agent
# onto a system to be managed. The script will check if all the required packages are installed
# and report the missing packages, if any.
#
#
# Exit codes:
#   0 - Pre-install check passed.
#   1 - Pre-install check failed.
# 127 - Invocation error.


E_OK=0		# everything ok.
E_FAIL=1	# some checks failed.

VERBOSE=true		# verbose mode.
LOGFILE=/dev/stdout	# log the output of the script, default is the std output.

ROOT=/
ANONYMOUS_USER="nobody"		# anonymous user required by patchadd on Solaris
PKGINFO="/usr/bin/pkginfo"      # Solaris command to check if a package is installed or not.
PKG="/usr/bin/pkg"				# Solaris 11 IPS
RPM="/bin/rpm"                  # Linux command to check if an rpm is installed or not.

OS_TYPE=`uname -s`
OS_REL=`uname -r`

if [[ "$(/usr/bin/id|cut -d"(" -f1|cut -d"=" -f2)" != "0" ]]; then
        echo "Please run OCDoctor.sh as root"
        exit 1
fi

myname=`basename "$0"`
requiredPackages=""	# this will contain list of all required pkg/rpm.
requiredPatches=""	# this will contain list of all required patches.
specialPackages=""	# mostly for checking if 32-bit version of rpm is installed on Linux.
failure=0               # flag indication failure, 0 meaning everything OK.


#
# Wrap gettext so that you can use variable references.
#
echo_gettext() {
    if [ "$OS_TYPE" = "SunOS" ]; then
	if [ "$OS_REL" = "5.8" -o "$OS_REL" = "5.9" ]; then
	    EG=`echo "$@"`
	else
	    EG=`gettext -s -- "$@"`
	fi
    else
	EG=`gettext -s -- "$@"`
    fi
    eval echo \""$EG"\"
}

#
# isThereNobody
# patchadd command in solaris requires anonymous user to be present.
# This function checks whether anonymous user nobody is enabled on the system
# on S10, check for noaccess instead of nobody.
#
isThereNobody() {
    if [ "$OS_REL" = "5.10" ]; then
        ANONYMOUS_USER="noaccess"
    fi
    /usr/bin/getent passwd $ANONYMOUS_USER > /dev/null 2>&1
    if [ $? -ne 0 ];then
        echo 'ERROR: Unable to find user \"$ANONYMOUS_USER\".'
        echo '       Installation requires user \"$ANONYMOUS_USER\" to be enabled for patchadd to work correctly.'
        failure=1
    else
	$VERBOSE && echo_gettext "OK: Anonymous user \"$ANONYMOUS_USER\" is present on the system." >> $LOGFILE
    fi
}

#
# Determine the processor type of the system with its root at $1.
# Note that this may not be the same as the processor type of the
# system that we're running on.
#
GetArch() {
    processor=`pkgparam -R "$1" SUNWcsu ARCH 2>/dev/null`
    if [ -n "$processor" ]; then
        echo "$processor"
        return 0
    fi

    processor=`pkgparam -R "$root_dir" SUNWcsr ARCH 2>/dev/null`
    if [ -n "$processor" ]; then
        echo "$processor"
        return 0
    fi

    return 1
}

#
# Determine the OS release of the system with its root at $1.
# Note that this may not be the same as the OS release of the
# system that we're running on.
#
GetOSRel() {
    if [ "$1" = "/" ]; then
        uname -r
    else
        sed -n 's/VERSION=/5./p' "$1/var/sadm/softinfo/INST_RELEASE"
    fi
    return 0
}


#
# Check the processor type of the system with its root at $1.
#
isThereDevRandom() {
    if [ ! -c /dev/random ]; then
	echo_gettext 'ERROR: /dev/random does not exist!'
	failure=1
    else
        $VERBOSE && echo_gettext "OK: /dev/random exists" >> $LOGFILE
    fi
    if [ ! -c /dev/urandom ]; then
	echo_gettext 'ERROR: /dev/urandom does not exist!'
	failure=1
    else
        $VERBOSE && echo_gettext "OK: /dev/urandom exists" >> $LOGFILE
    fi
}


#
# Check the processor type of the system with its root at $1.
#
CheckProcessorType() {
    arch=`GetArch $1`
    if [ -z "$arch" ]; then
	echo_gettext 'ERROR: Neither SUNWcsu nor SUNWcsr is installed!' 
        echo_gettext 'Cannot determine target processor type.'
        failure=1
    else
        $VERBOSE && echo_gettext "OK: Processor is $arch" >> $LOGFILE 
    fi
}

#
# Check the OS release of the system with its root at $1.
#
CheckOSRelease() {
    osrel=`GetOSRel $1`
    if [ -z "$osrel" ]; then
        echo_gettext 'ERROR: Cannot determine OS release.'
        failure=1
    else
        $VERBOSE && echo_gettext "OK: OS release is $osrel" >> $LOGFILE
    fi
}

#
# Will figure out which OS and will search for proper Patches
#
CheckInstalledPatches(){
	
	if [ "$OS_REL" = "5.10" ]; then
	    procType=`uname -p`;
		if [ "$procType" = "sparc" ]; then
			CheckInstalledPatchesRevision "124630" "03"
			CheckInstalledPatchesRevision "122660" "07"

		else		### X86
			CheckInstalledPatchesRevision "124631" "03"
			CheckInstalledPatchesRevision "122661" "07"
		fi
    fi


}

CheckInstalledPatchesRevision(){
	patchNumber="$1"
	patchRevision="$2"
	chkPatch=`patchadd -p | grep "Patch: $patchNumber"`
	if [ "$chkPatch" = "" ]; then
		chkPatch=`patchadd -p | grep $patchNumber`
		if [ "$chkPatch" = "" ]; then
			echo "ERROR: Patch $patchNumber-$patchRevision is not installed and it's not in the obsolete list" 		#'
			failure=1
		else
			$VERBOSE && echo_gettext "OK: Patch $patchNumber is in the obsolete list (newer one is installed)" >> $LOGFILE
		fi
	else
		installedPtchRevision=`patchadd -p | grep "Patch: $patchNumber" | awk '{print $2}' | sort | tail -1 | awk -F"-" '{print $2}'`
		if [ $installedPtchRevision -lt $patchRevision ]; then
			echo "ERROR: Patch $patchNumber-$installedPtchRevision is installed, needed revision: $patchRevision" 
			failure=1
		else
			$VERBOSE && echo_gettext "OK: Patch $patchNumber-$installedPtchRevision is installed, needed revision: $patchRevision" >> $LOGFILE
		fi
	fi
}

#
# Check for pkg mismatch - for example, not installed on global but installed on zone
#
CheckMismatchedPackages(){
	pkgsToCheck="$1"
	missing_pkg_count=0                 # count of missing packages.
	error_msg=""                        # error message to be displayed.
	if [ ! "$OS_TYPE" = "SunOS" ]; then
		return
	fi
	if [ ! "$OS_REL" = "5.10" ]; then
		return
	fi
	for pkg in $pkgsToCheck; do
		$PKGINFO -q $pkg
		if [ $? -ne 0 ]; then
			PKGINST=0
		else
			PKGINST=1
		fi
		##### Loop through all the zones as well
		counter=0
		zoneadm list -v 2>/dev/null| grep -v "0 global" | grep " running " | grep " native " | awk '{printf("%s %s\n",$2,$4)}' | while read ZNAME ZPATH; do
		  if [ ! "$ZPATH" = "/" ]; then		### if inside a zone, the zone path is /
			counter=`expr $counter + 1`
			$PKGINFO -q -R $ZPATH/root $pkg
			if [ $? -ne 0 ]; then	#### Not installed on zone
				if [ "$PKGINST" = "1" ]; then
					echo_gettext "WARNING: Package $pkg is installed on the global zone but is NOT installed on $ZNAME!"
					missing_pkg_count=`expr $missing_pkg_count + 1`
				fi
			else			#### Installed on zone
				if [ "$PKGINST" = "0" ]; then
					echo_gettext "WARNING: Package $pkg is NOT installed on the global zone but is installed on $ZNAME!"
					missing_pkg_count=`expr $missing_pkg_count + 1`
				fi
			fi
		  fi
		done
	done
	if [ $missing_pkg_count -ne 0 ]; then
            	error_msg="WARNING:  Found some mismatching packages between zones and global zone."
        	echo_gettext "" >> $LOGFILE
	        $VERBOSE && echo_gettext "$error_msg" >> $LOGFILE
	        $VERBOSE && echo_gettext "" >> $LOGFILE
	        failure=1
	else
		if [ ! "$counter" = "0" ]; then		## only print OK if we have zones
	        	$VERBOSE && echo_gettext "OK: No mismatched packages in zones"
		fi
	fi
}

#
# Check if the packages passed as argument ($1) are installed on
# the system or not. Reports the count and the names of the missing
# packages.
#
CheckInstalledPackages(){
    pkgsToCheck="$1"
    missing_pkg_count=0                 # count of missing packages.
    missing_pkgs=""                     # name of the missing packages.
    error_msg=""                        # error message to be displayed.

    for pkg in $pkgsToCheck
    do
        if [ "$OS_TYPE" = "SunOS" ]; then
        	if [ "$OS_REL" = "5.11" ]; then
        		$PKG list $pkg >/dev/null 2>/dev/null
        	else 
            	$PKGINFO -q $pkg
            fi
        elif [ "$OS_TYPE" = "Linux" ]; then
            $RPM -q $pkg > /dev/null 2>&1
        fi
        if [ $? -ne 0 ]; then
            echo "ERROR: $pkg is not installed." >> $LOGFILE
            missing_pkgs="$missing_pkgs $pkg"
            missing_pkg_count=`expr $missing_pkg_count + 1`
	    PKGINST=0
        else
            #$VERBOSE && echo_gettext "$pkg is installed." >> $LOGFILE
	    PKGINST=1
        fi 
	##### Loop through all the zones as well
	if [ "$PKGINST" = "1" ]; then
		if [ "$OS_TYPE" = "SunOS" ]; then
			if [ "$OS_REL" = "5.10" ]; then
				zoneadm list -v 2>/dev/null| grep -v "0 global" | grep " running " | grep " native " | awk '{printf("%s %s\n",$2,$4)}' | while read ZNAME ZPATH; do
				  if [ ! "$ZPATH" = "/" ]; then           ### if inside a zone, the zone path is /
					$PKGINFO -q -R $ZPATH/root $pkg
					if [ $? -ne 0 ]; then
				            echo_gettext "ERROR: $pkg is installed in the global zone, but not in zone $ZNAME" >> $LOGFILE
				            missing_pkgs="$missing_pkgs $pkg"
				            missing_pkg_count=`expr $missing_pkg_count + 1`
				        else
				            #$VERBOSE && echo_gettext "$pkg is installed on $ZNAME." >> $LOGFILE
				            DONOTHING=1
				        fi
				  fi
				done
			fi
		fi
	fi

    done

    # If some packages are missing, report it to the user.
    if [ $missing_pkg_count -ne 0 ]; then
        if [ $missing_pkg_count -eq 1 ]; then
            error_msg="The following required prerequisite package is not installed. Please install it in order to install the agent."
        else
            error_msg="The following $missing_pkg_count required prerequisite packages are not installed. They must be installed before installing the agent."
        fi

        $VERBOSE && echo_gettext "" >> $LOGFILE
        $VERBOSE && echo_gettext "$error_msg" >> $LOGFILE
        $VERBOSE && echo_gettext "$missing_pkgs" | tr ' ' '\012' >> $LOGFILE    # translate spaces into newline characters
        $VERBOSE && echo_gettext "" >> $LOGFILE
        failure=1
    else
	$VERBOSE && echo_gettext "OK: All required packages are installed"
    fi
}


########## Check conflicting packages #############
#
CheckConflictPackages(){
    pkgsToCheck="$1"
    existing_pkg_count=0             # count of existing packages.
    existing_pkgs=""                 # name of the existing packages.
    error_msg=""                    # error message to be displayed.

    for pkg in $pkgsToCheck
    do
        if [ "$OS_TYPE" = "SunOS" ]; then
            $PKGINFO -q $pkg
        elif [ "$OS_TYPE" = "Linux" ]; then
            $RPM -q $pkg > /dev/null 2>&1
        fi
        if [ $? -eq 0 ]; then
	    THISPKGINSTALLED=1
            echo "ERROR: $pkg is installed - need to remove" >> $LOGFILE
	    existing_pkg_count=`expr $existing_pkg_count + 1`
	else
	    THISPKGINSTALLED=0
        fi
	##### CHECK ZONES TOO ######
	if [ "$THISPKGINSTALLED" = "0" ]; then		### Only check in zones if it's not already reported in global, otherwise we'll have many dup messages
		if [ "$OS_TYPE" = "SunOS" ]; then
			if [ "$OS_REL" = "5.10" ]; then
				zoneadm list -v 2>/dev/null| grep -v "0 global" | grep " running " | grep " native " | awk '{printf("%s %s\n",$2,$4)}' | while read ZNAME ZPATH; do
				  if [ ! "$ZPATH" = "/" ]; then           ### if inside a zone, the zone path is /
					$PKGINFO -q -R $ZPATH/root $pkg
					if [ $? -eq 0 ]; then
						echo "ERROR: $pkg is installed in zone $ZNAME -  - need to remove" >> $LOGFILE
						existing_pkg_count=`expr $existing_pkg_count + 1`
					fi
				  fi
				done
			fi
		fi	
	fi
    done
    if [ $existing_pkg_count -eq 0 ]; then
	echo "OK: No old agent / conflicting packages were found"
    else
	echo "ERROR: the above packages need to be removed (Old agent leftovers?). Try running /var/scn/install/uninstall or remove manually"
	failure=1
	echo
    fi
}

#
# Check if 32-bit version of the rpms passed as argument ($1) are installed
# or not. Reports the count and names of the missing rpms
#
Check32bitPackages() {
    pkgsToCheck="$1"

    missing_pkg_count=0             # count of missing packages.
    missing_pkgs=""                 # name of the missing packages.
    error_msg=""                    # error message to be displayed.
    SUSE_32="32bit"                 # string in rpm name to identify SuSE 32bit compatible rpms

    SUSE_DISTRO=false
    if [ -f /etc/SuSE-release ]; then
        SUSE_DISTRO=true
    fi

    for pkg in $pkgsToCheck
    do
        arch=`$RPM  -q --queryformat '%{ARCH}\n' $pkg`
        # for 32-bit version of an rpm, $arch can be i386, i686, i[0-9]86.
        echo $arch | grep "i[0-9]86" > /dev/null 2>&1
        if [ $? -ne 0 ]; then

            if [ "$SUSE_DISTRO" = "true" ]; then
                # SuSE creates 64 bit rpms that are 32bit compatible rather than true 32bit binaries. These
                # rpms contain "32bit" in the rpm name. For example the 32 bit compatible version
                # of the pam rpm is pam-32bit-0.99.6.3-28.8 even though its arch is reported as
                # x86_64. Therefore we need to check the rpm name rather than its arch. We
                # still first check the arch value even on SuSE just in case true 32bit rpms do
                # show up on SuSE.
                #
                $RPM  -q $pkg-$SUSE_32 > /dev/null 2>&1
                if [ $? -ne 0 ]; then
                    echo "ERROR: 32-bit version $pkg-$SUSE_32 is not installed." >> $LOGFILE
                    missing_pkgs="$missing_pkgs $pkg-$SUSE_32"
                    missing_pkg_count=`expr $missing_pkg_count + 1`
                else
                    $VERBOSE && echo_gettext "32-bit version $pkg-$SUSE_32 is installed." >> $LOGFILE
                fi

            else
                echo "ERROR: 32-bit version of $pkg is not installed." >> $LOGFILE
                missing_pkgs="$missing_pkgs $pkg"
                missing_pkg_count=`expr $missing_pkg_count + 1`
            fi

        else
            $VERBOSE && echo_gettext "OK: 32-bit version of $pkg is installed." >> $LOGFILE
        fi
    done

    if [ $missing_pkg_count -ne 0 ]; then
        if [ $missing_pkg_count -eq 1 ]; then
            error_msg="The following 32-bit rpm is not installed. Please install it in order to install the agent."
        else
            error_msg="The following 32-bit rpms are not installed. They must be installed before installing the agent."
        fi

        $VERBOSE && echo_gettext "" >> $LOGFILE
        $VERBOSE && echo_gettext "$error_msg" >> $LOGFILE
        $VERBOSE && echo_gettext "$missing_pkgs" | tr ' ' '\012' >> $LOGFILE    # translate spaces into newline characters
        $VERBOSE && echo_gettext "" >> $LOGFILE
        failure=1
    fi
}

VERBOSE=true
DELTA=false
if [ "$1" = "-e" ];then
	VERBOSE=false
	if [ "$2" = "12" ]; then		#### when running troubleshoot, on delta we will get this paramter
		DELTA=true
	fi
fi

# If the OS is Open Solaris, do not check the pre-requisites
if [ $DELTA = false ]; then 				#### 5.11 is supported with Delta
	if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.11" ]; then
	    echo "WARNING: Found Solaris 11. Please be aware that Ops Center 11g does NOT support Solaris 11."
	    echo "         Solaris 11 is only supported by the future unreleased version of Ops Center - 12c."
	    DELTA=true		#### Notified user, now assume this is OK from now on
	fi
fi


#**********************************************************************************#
# 				Initialization					   #
#**********************************************************************************#

if [ "$OS_TYPE" = "SunOS" ]; then
	if [ "$OS_REL" = "5.10" ]; then
    	requiredPackages="\
        SUNWadmap \
        SUNWadmc \
        SUNWadmfw \
        SUNWbash \
        SUNWctpls \
        SUNWdtcor \
        SUNWesu \
        SUNWgzip \
        SUNWlibC \
        SUNWlibms \
        SUNWloc \
        SUNWmfrun \
        SUNWswmt \
        SUNWtoo \
        SUNWxcu4 \
        SUNWxwdv \
        SUNWxwfnt \
        SUNWxwice \
        SUNWxwplt \
        SUNWxwrtl \
        SUNWzip \
        SUNWzlib \
		"
	fi
	if [ "$OS_REL" = "5.11" ]; then
    	requiredPackages="\
        shell/bash \
        compress/zip \
		"
	fi

    if [ "$OS_REL" = "5.8" ]; then
	requiredPackages="$requiredPackages \
	SUNWlmsx \
	SUNWnisr \
	SUNWnisu \
	SUNWtltk \
	SUNWxildh \
	SUNWxilow \
	SUNWxilrl \
	SUNWzlibx \
	"
    elif [ "$OS_REL" = "5.9" ]; then
	requiredPackages="$requiredPackages \
        SUNWcpp \
        SUNWgcmn \
        SUNWlibpopt \
	SUNWlmsx \
        SUNWlxml \
        SUNWpl5u \
        SUNWpl5v \
        SUNWzlibx \
	"
    elif [ "$OS_REL" = "5.10" ]; then
	requiredPackages="$requiredPackages \
        SUNWbzip \
        SUNWcpp \
        SUNWgcmn \
        SUNWlibmsr \
        SUNWlibpopt \
        SUNWadmfr \
        SUNWlxml \
        SUNWperl584core \
        SUNWperl584usr \
        SUNWxwplr \
        SUNWTk \
        SUNWTcl \
	"
	mismatchPackages="SUNWj6rt \
	SUNWj6dev \
	SUNWj6cfg \
	"
    fi
elif [ "$OS_TYPE" = "Linux" ]; then
    requiredPackages="\
        coreutils \
        file \
        grep \
        tar \
        unzip \
        xinetd \
	gettext \
	"

    specialPackages="\
	libxml2 \
	pam \
	"
# Temporarily removing e2fsprogs from the list of specialPackages since this is
# causing problem on some Linux distributions we need to support
#      e2fsprogs \

fi

#*************************************************************************************#
# 				Pre-req check starts now			      #
#*************************************************************************************#


if [ "$OS_TYPE" = "SunOS" ]; then
    # Check processor type and OS rel.
    CheckProcessorType "$ROOT"
    CheckOSRelease "$ROOT"
    isThereDevRandom
    # Check if anonymous user is present on Solaris system
    isThereNobody

fi
#*************************************************************************************#

# Check if the required packages are installed on the system.
#$VERBOSE && echo_gettext "Checking if all the required prerequisite packages are installed." >> $LOGFILE
CheckInstalledPackages "$requiredPackages"

# Check mismached packages
CheckMismatchedPackages "$mismatchPackages"

# Check if the required patches are installed
CheckInstalledPatches ""

# Additional check on Linux to verify 32-bit version of some packages are installed or not.
if [ "$OS_TYPE" = "Linux" ]; then
    #$VERBOSE && echo_gettext "Checking for required prerequisite 32-bit rpm versions" >> $LOGFILE
    Check32bitPackages "$specialPackages"
fi

#*************************************************************************************#
#### Check to see if we have any old Service Tags installed (1.1.2)
if [ "$OS_TYPE" = "Linux" ]; then
	PKGNAME="sun-servicetag"
	VERSION=`rpm -q $PKGNAME`
        RETVAL=$?
        VERSION=`echo $VERSION | awk -F"-" '{print $3}'`
        if [ $RETVAL -ne 0 ]; then
		VERSION=""		### not installed
        fi
else
	PKGNAME="SUNWservicetagu"
	VERSION=`/usr/bin/pkgparam $PKGNAME VERSION 2>/dev/null`
	RETVAL=$?
        VERSION=`echo $VERSION | awk -F"," '{print $1}'`
        if [ $RETVAL -ne 0 ]; then
		VERSION=""		### not installed
        fi
fi
if [ "$VERSION" = "" ]; then
	if [ $VERBOSE = true ]; then
		echo "OK: Servicetag is not installed ($PKGNAME)"
	else
		echo "ERROR: Could not find ServiceTag package ($PKGNAME)"
	fi
else
	MINVER="1.1.3"
	STVERNUM=`echo $VERSION | sed -e 's/\.//g' | sed -e 's/-//g'`
	MINVERNUM=`echo $MINVER | sed -e 's/\.//g' | sed -e 's/-//g'`
	if [ $STVERNUM -lt $MINVERNUM ]; then
		failure=1
		if [ "$OS_TYPE" = "Linux" ]; then
		        echo "WARNING: ServiceTag RPM version ($VERSION) is too old and it is recommended to remove it prior to installing the agent"
			echo "You can remove it by running:"
			echo "# rpm -e sun-servicetag"
			echo
		else
		        echo "WARNING: ServiceTag PKG version ($VERSION) is too old and it is recommended to remove it prior to installing the agent"
			echo "You can remove it by running:"
			echo "# pkgrm SUNWstosreg"
			echo "# pkgrm SUNWservicetagu"
			echo "# pkgrm SUNWservicetagr"
			echo
                        echo "WARNING: Do NOT remove further ST packages like SUNWsthwreg or SUNWpsn !!!"
		fi
	else
	        $VERBOSE && echo "OK: ServiceTag version is newer than $MINVER - installed version is $VERSION"
	fi
fi
#*************************************************************************************#

#
# Check for partially installed packages
#
if [ $VERBOSE = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
                                               ### in troubleshoot we run individual tests so in colocated it prints only one message
        if [ ! "$OS_TYPE" = "Linux" ]; then
                OUTPUT=`$PKGINFO -p`
                if [ "$OUTPUT" = "" ]; then
                        echo "OK: No partially installed packages"
                else
                        echo
                        echo "WARNING: Found the following partially installed packages (pkginfo -p), please check if that's a problem:"
                        printf "$OUTPUT\n"
                        echo ""
                fi
		if [ "$OS_REL" = "5.10" ]; then
			zoneadm list -v 2>/dev/null| grep -v "0 global" | grep " running " | grep " native " | awk '{printf("%s %s\n",$2,$4)}' | while read ZNAME ZPATH; do
			  if [ ! "$ZPATH" = "/" ]; then           ### if inside a zone, the zone path is /
				OUTPUT=`$PKGINFO -R $ZPATH/root -p`
				if [ "$OUTPUT" = "" ]; then
		                       	echo "OK: No partially installed packages in zone $ZNAME"
		                else
		                        echo
		                        echo "WARNING: Found the following partially installed packages (pkginfo -p) in zone $ZNAME, please check if that's a problem:"
		                        printf "$OUTPUT\n"
		                        echo ""
		                fi
			  fi
			done
		fi
        fi
fi

### Check if there are any old processes running
if [ $VERBOSE = true ]; then		### Only run in standalone prereq - not in troubleshoot

	########### OLD PACKAGES #########
	if [ "$OS_TYPE" = "Linux" ]; then
		conflictPackages="\
	        sun-scn-agentfacade-r \
	        sun-scn-agentfacade-u \
	        sun-scn-update-util \
	        sun-scnjobmgr-agent-config \
	        sun-scnjobmgr-agent \
	        sun-scn-update-agent-r \
	        sun-scn-update-agent \
	        sun-n1hss-agentdm-config \
	        sun-n1hss-agentdm \
	        sun-scnconnmgt \
	        sun-scnconnmgt-r \
		sun-scn-agent-utils \
		sun-scn-agentupgrade-r \
		sun-scn-agentupgrade-u \
		sun-scnconnmgt-ra \
		"
	else
		conflictPackages="\
		SUNWscn-agentfacade-vc-config \
		SUNWxvm-zstat-init \
		SUNWxvm-zstat \
		SUNWscn-agentupgrade-vc-r \
		SUNWscn-agentupgrade-vc-u \
		SUNWxvm-vc-driver-common \
		SUNWxvm-xvmsvrdm-config \
		SUNWn1hss-core \
		SUNWscnjobmgr-vc-config \
		SUNWscnjobmgr-proxy \
		SUNWscnjobmgr-common-impl \
		SUNWscnjobmgr-common-api \
		SUNWscn-agent-utils \
		SUNWscn-agentupgrade-u \
		SUNWscn-agentfacade-r \
		SUNWscn-agentfacade-u \
		SUNWscn-update-util \
		SUNWscnjobmgr-agent \
		SUNWscn-update-agent-r \
		SUNWscn-update-agent \
		SUNWn1hss-agentdm \
		SUNWscnconnmgt \
		SUNWscnconnmgt-ra \
		"
	fi
	CheckConflictPackages "$conflictPackages"


	########### PROC START HERE ##########
	PROCS=`ps -ef | grep 'uce_agent' | grep -v grep | wc -l`
	PROCS=`echo $PROCS`		### remove TAB
	if [ "$PROCS" = "0" ]; then
		PROCSB=`ps -ef | grep 'scn-agent' | grep -v grep | wc -l`
		PROCSB=`echo $PROCSB`		### remove TAB
		if [ "$PROCSB" = "0" ]; then
			echo "OK: No old agent processes are running"
		else
			echo "ERROR: Found old scn-agent procs running"
			echo
			failure=1
		fi
	else
		echo "ERROR: Found old uce_agent procs running"
		echo
		failure=1
	fi
fi

#***************************** Check for leftover services ***************************#
if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
	if $VERBOSE; then
		OUTPUT=`svcs -a | grep "svc:/application/management/common-agent-container-1:scn-agent"`
		if [ ! "$OUTPUT" = "" ]; then
			echo "ERROR: Found service 'common-agent-container-1:scn-agent' - it needs to be removed before installing an agent"
			echo "       If agent was already uninstalled (don't do this if it's still installed), try:"
			echo "       # svccfg delete common-agent-container-1:scn-agent"
			echo ""
			failure=1
		else
			echo "OK: No scn-agent cacao service leftovers"
		fi
	fi
fi

#**************************** Check for pkgadd lock file *****************************#
if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
	SIZE=`du /tmp/.ai.pkg.zone.lock* | head -1 | awk '{print $1}'`
	NAME=`du /tmp/.ai.pkg.zone.lock* | head -1 | awk '{print $2}'`
	LOCKFILES=0
	if [ ! "$SIZE" = "0" -a ! "$SIZE" = "" ]; then
		echo "WARNING: Found pkgadd/pkgrm lock file $NAME - is there any package management going on?"
		echo "         If not, this could be an old leftover and should be removed, otherwise pkgadd/pkgrm wouldn't work properly"
		echo ""
		LOCKFILES=`expr $LOCKFILES + 1`
	fi
	#### Loop through all zones ####
	zoneadm list -v 2>/dev/null| grep -v "0 global" | grep " running " | awk '{printf("%s %s\n",$2,$4)}' | while read ZNAME ZPATH; do
	  if [ ! "$ZPATH" = "/" ]; then           ### if inside a zone, the zone path is /
		SIZE=`du $ZPATH/root/tmp/.ai.pkg.zone.lock* | head -1 | awk '{print $1}'`
	        NAME=`du $ZPATH/root/tmp/.ai.pkg.zone.lock* | head -1 | awk '{print $2}'`
		if [ ! "$SIZE" = "0" -a ! "$SIZE" = "" ]; then
        	        echo "WARNING: Found pkgadd/pkgrm lock file $NAME (in zone $ZNAME) - is there any package management going on?"
               		echo "         If not, this could be an old leftover and should be removed, otherwise pkgadd/pkgrm wouldn't work properly"
	                echo ""
			LOCKFILES=`expr $LOCKFILES + 1`
	        fi
	  fi
	done
	if [ "$LOCKFILES" = "0" ]; then
		$VERBOSE && echo "OK: No pkgadd/pkgrm lock files"
	else
		failure=1
	fi
fi

		
#*************************************************************************************#

if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
    # Check if /opt is a seperate zfs filesystem and not in vfstab (Bug 15623679)
    #$VERBOSE && echo_gettext "Checking for seperate zfs filesystem /opt." >> $LOGFILE
    zfs list /opt 2>/dev/null | grep /opt >/dev/null
    if [ $? -eq 0 ]; then
        # OK, /opt is a seperate zfs file system
        if  zfs list /opt | grep -w legacy >/dev/null; then
            $VERBOSE && echo 'OK: /opt is a seperate zfs filesystem and is marked as legacy file system.'
        else 
            echo 'ERROR: /opt is a separate zfs filesystem and needs to be marked as legacy file system!'
            echo 'ERROR: Use "zfs set mountpoint=legacy ..." to change this and add a mount entry to /etc/vfstab!'
            failure=1
        fi
    else
        $VERBOSE && echo 'OK: /opt is not a seperate zfs filesystem, nothing to check.'
    fi
fi

#*************************************************************************************#
# Check for LD_LIBRARY_PATH
if $VERBOSE; then		### Only in full check req, in troubleshoot we have special tests
	if [ ! "$LOGNAME" = "root" ]; then
		echo "WARNING: please use 'su - root' before running this test, otherwise can't check LD_LIBRARY_PATH variable"		#'
		echo
	else
		if [ ! "$LD_LIBRARY_PATH_DOCTOR_ORIGINAL" = "" ]; then          ### Saved by the Doctor before setting LD_LIBRARY_PATH
			TEST_LIB=$LD_LIBRARY_PATH_DOCTOR_ORIGINAL
		else
			TEST_LIB=$LD_LIBRARY_PATH
		fi
		if [ "$TEST_LIB" = "" ]; then
		        echo "OK: LD_LIBRARY_PATH is not set"
		else
			if [ "$TEST_LIB" = "EMPTY" ]; then
				echo "OK: LD_LIBRARY_PATH is not set"
			else
			        echo "WARNING: Found that LD_LIBRARY_PATH is set to: $TEST_LIB"
			        echo "         This may (or may not) cause issues while installing the agent."
			        echo ""
			fi
		fi
	fi
fi

#*************************************************************************************#

if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
	if $VERBOSE; then		### Don't run if from --troubleshoot
		#### Check for old Ops Center services
		ZSTAT=`svccfg list | grep "application/xvm/zstat"`
                if [ "$ZSTAT" = "" ]; then
                        echo 'OK: no 'zstat' service found.'
                else
                        echo 'ERROR: found an old 'zstat' service using the command:'
                        echo '       # svccfg list | grep "application/xvm/zstat"'
                        echo
                        echo '       To remove, run the following:'
			echo '       # svcadm disable "application/xvm/zstat"'
                        echo '       # svccfg delete "application/xvm/zstat"'
                        echo ''
                        failure=1
                fi
		OCZONES=`svccfg list | grep "application/oczones"`
		if [ "$OCZONES" = "" ]; then
			echo 'OK: no 'oczones' service found.'
		else
			echo 'ERROR: found an old 'oczones' service using the command:'
			echo '       # svccfg list | grep "application/oczones"' 
			echo
			echo '       To remove, run the following:'
			echo '       # svcadm disable "application/oczones"'
			echo '       # svccfg delete "application/oczones"'
			echo ''
			failure=1
		fi
		####### Check for prtdiag freeze #######
		if [ -f ./prereq/timeout.sh ]; then
		        `./prereq/timeout.sh /usr/sbin/prtdiag >/dev/null 2>/dev/null`
		        RETVAL=$?
		elif [ -f ./timeout.sh ]; then
			`./timeout.sh /usr/sbin/prtdiag >/dev/null 2>/dev/null`
			RETVAL=$?
		else
		        `../prereq/timeout.sh /usr/sbin/prtdiag >/dev/null 2>/dev/null`
		        RETVAL=$?
		fi
		if [ $RETVAL -eq 137 ]; then
		        echo "ERROR: prtdiag seems to be freezing. This is a known Solaris issue (Bug 15394234). To fix this, try the following Solaris workaround:"
		        echo ""
		        echo "       # svcadm restart picl"
		        echo
			failure=1
		else
		        echo "OK: prtdiag doesn't hang"		#'
		        ### if prtdiag doesn't hang - check for T3 ####
	        	OUTPUT=`/usr/sbin/prtdiag | grep " T3-"`
	        	if [ ! "$OUTPUT" = "" ]; then                   ### we have a T3.
	        		RELEASE=`grep Solaris /etc/release | head -1 | cut -f2 -d_ | cut -f1 -dw` ## u1, u2 etc
	        		RELEASE=`echo ${RELEASE} | awk -F"u" '{print $2}'`              ## 1, 2 etc
	        		if [[ $RELEASE -lt 10 ]]; then
	        			if [ -f /platform/sun4v/kernel/drv/n2cp.conf ]; then
	        				OUTPUT=`grep "n2cp-use-ulcwq=0" /platform/sun4v/kernel/drv/n2cp.conf`
	        			else
	        				OUTPUT=""
	        			fi
	        			if [ "$OUTPUT" = "" ]; then
	        				echo "WARNING: This is a T3 machine. You may run into Bug 15722725."
	        				echo "         Ops Center agent may hang during installation."
	        				echo ""
	        				echo "         The workaround is to edit '/platform/sun4v/kernel/drv/n2cp.conf'"
	        				echo "         and insert "n2cp-use-ulcwq=0" to the existing line."
	        				echo ""
	        				echo "         For example, if your current line says this:"
	        				echo "            ddi-forceattach=1 ddi-no-autodetach=1;"
	        				echo "         The new line should look like this:"
	        				echo "            ddi-forceattach=1 ddi-no-autodetach=1 n2cp-use-ulcwq=0;"
	        				echo ""
	        				echo "         Note that in some cases you may need to run 'init 6' for this to take effect, but not always."
	        				echo ""
	        			else
	        				echo "OK: This is a T3 machine and the workaround is applied to '/platform/sun4v/kernel/drv/n2cp.conf'"
	        			fi
	        		else
	        			echo "OK: This is a T3 machine but you're using Solaris 10 U${RELEASE}, not affected by Bug 15676804"	#'
	        		fi
	        	else
	        		echo "OK: This is not a T3 machine (if it was then a workaround may be needed)"
	        	fi
		fi
		####### Check for an old zstatadm process #######
		ZSTATADM=`ps -ef | grep zstatadm | grep -v grep`
		if [ "$ZSTATADM" = "" ]; then
			echo "OK: No 'zstatadm' process found"
		else
			echo "ERROR: Found an old 'zstatadm' process running using the command:"
			echo "       # ps -ef | grep zstatadm | grep -v grep"
			echo ""
			echo "       This is probably a leftover of an old agent (Bug 15647813)."
			echo "       Make sure you kill this process before installing a new agent!"
			echo "       Few machines may require reboot if 'kill -9' doesn't work."
			echo ""
			failure=1
		fi
		####### Check for an old logadm proces (related to zstatadm) ######
		LOGADM=`ps -ef | grep logadm | grep -v grep`
		if [ "$LOGADM" = "" ]; then
			echo "OK: No frozen 'logadm' process found"
		else
			sleep 5		### Allow logadm to finish
			LOGADM=`ps -ef | grep logadm | grep -v grep`
			if [ "$LOGADM" = "" ]; then
				echo "OK: No frozen 'logadm' process found"
			else
				echo "ERROR: Found a frozen 'logadm' process using the command:"
				echo "       # ps -ef | grep logadm | grep -v grep"
				echo ""
				echo "       This is probably due to an old agent installation and due to the zstatadm bug (Bug 15647813)."
				echo "       Make sure this process ends or that you kill it before installing a new agent!"
				echo "       Few machines may require reboot if 'kill -9' doesn't work."
				echo ""
				failure=1
			fi
		fi	
 
	fi
fi

##### checking ssh requirements for OC #######

CHECK=0
cat <<!EOF! | while read file; do
/usr/bin/ssh
/usr/bin/ssh-keygen
/usr/bin/ssh-keyscan
!EOF!
  if [[ -f $file || -h $file ]] ; then
    $PRINTALL #&& printf "."
  else
    printf "WARNING: File or link $file not found\n\n"
    failure=1
    CHECK=1
  fi
done
if [ $CHECK -eq 0 ]; then
    $VERBOSE && echo "OK: Checked for ssh requirements"
fi

#*************************************************************************************#

if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
  # Check if an old zonectrlr.id file exists (Bug 15628914)
  # Do not run check in --troubleshoot mode
  if $VERBOSE; then
    if [ -f /etc/opt/sun/oc/zonectrlr.id ]; then
      echo 'WARNING: Found existing file /etc/opt/sun/oc/zonectrlr.id (Bug 15628914).' 
      echo '         Remove this file if this OS has been created using a flash archive.'
    else
      echo 'OK: No zonectrlr.id leftover file found.'
    fi
  fi
fi

#*************************************************************************************#

if [ "$OS_TYPE" = "SunOS" -a "$OS_REL" = "5.10" ]; then
	# Check if the agent is in multi-user mode
	OUTPUT=`svcs -H multi-user-server | awk '{print $1}'`
	if [ "$OUTPUT" = "online" ]; then
		if $VERBOSE; then
			echo "OK: multi-user-server service is online"
		fi
	else
		echo "ERROR: Service 'multi-user-server' is not online but is required for the agent."
		echo "       Try running 'svcs -x multi-user-server' and 'svcs -l multi-user-server' to see why."
		echo ""
		failure=1
	fi
fi

#*************************************************************************************#

if [ "$OS_TYPE" = "SunOS" ]; then
  # Check if we are using GNU tail which does not have the option "-r"
  TAIL=/usr/bin/tail
  TESTFILE="/tmp/_testfile_.$$"
  touch $TESTFILE

  if [ -x $TAIL ]; then
    if $TAIL -r $TESTFILE 2>/dev/null 1>&2; then
      if $VERBOSE; then
        echo 'OK: Native Solaris "tail" command is available.'
      fi
    else
      echo "WARNING: $TAIL does not support the '-r' option which is required by the Agent!"
      echo "WARNING: Looks like 'GNU tail' is in use by default!"
      failure=1
    fi
  else
    echo "WARNING: The command $TAIL is required by the Agent but could not be found!"
    failure=1
  fi
  rm $TESTFILE
fi

#*************************************************************************************#

# Check if there is an old NFS-Lib entry in /etc/vfstab

if $VERBOSE; then
  if [ -f /etc/vfstab ]; then		### no vfstab in OEL
    VIRTLIBS=`awk '$1 !~ /^#/ && $4 == "nfs" && $3 ~ /^\/var\/mnt\/virtlibs\// {print $3}' /etc/vfstab`

    if [ -z "${VIRTLIBS}" ]; then
      echo 'OK: /etc/vfstab does not contain an old NFS-Lib entry.'
    else
      echo "WARNING: Old NFS-Lib entry found in /etc/vfstab!"
      failure=1
    fi
  fi
fi

#*************************************************************************************#

if [ $failure -eq 1 ]; then
    $VERBOSE && echo_gettext "Please make sure the system meets the above prerequisites before proceeding with the agent installation." >> $LOGFILE
    exit $E_FAIL
fi

# If it gets here, everything is OK.
$VERBOSE && echo_gettext "This system meets the required prerequisites in order to support an agent." >> $LOGFILE
$VERBOSE && echo_gettext "" >> $LOGFILE

exit $E_OK
