#!/bin/bash

if [[ "$(/usr/bin/id|cut -d"(" -f1|cut -d"=" -f2)" != "0" ]]; then
  echo "ERROR: $(basename $0) must be run as root, nothing checked!"
  exit 1
fi

PRINTALL=true
if [ "$1" = "-e" ];then
	PRINTALL=false
fi
ANYERRORS=0

$PRINTALL && echo '========== Checking 12c Enterprise Controller Requirements ... ===================='
#$PRINTALL && echo "OK: This prerequisites script is for 12c EC. Run --ec-prereq-11g if you need the old version"

#
# checking available main memory
#

memory=`/usr/bin/free -m| grep "^Mem:" | awk '{print $2}'`
if [ $memory -lt 5850  ]; then
  printf "ERROR: Not enough Main Memory! 6 GB is the minimum supported, but only $memory MB found\n\n"
  ANYERRORS=1
else
  $PRINTALL && printf "OK: $memory MB of memory\n"
fi

#
# checking swap space
#

swapsize=`/usr/bin/free -m| grep "^Swap:" | awk '{print $2}'`
##### There are 2 swap calculations, one for local DB, one for remote DB
## LOCAL DB
## if RAM = 2 GB and 16 GB then match RAM
## RAM > 16 GB then .75 times RAM
## Swap max is 16G - doesn't need to exceed this
MAXSWAP=16384
if [ ${memory} -lt $MAXSWAP ]; then
	#### if memory is less then max, get equal swap as memory 
	NEEDEDSWAP=$memory
else
	#### if memory is bigger, just use max needed
	NEEDEDSWAP=$MAXSWAP
fi
ComparisonSwap=`expr ${swapsize} + 150`		### For comparison purpose, round up 150MB 
SWAPOK="0"
if [[ ${ComparisonSwap} -lt ${NEEDEDSWAP}  ]]; then
	if [[ ${swapsize} -lt 5850  ]]; then		### Do we have enough swap for remote DB?
		printf "ERROR: Not enough swap space configured. 6 GB is the minimum required, but only ${swapsize} MB found.\n\n"
		ANYERRORS=1
	else 
		#### Only display the following messsages during --prereq, not during --troubleshoot (no need to see it if DB is installed remotely already)
		$PRINTALL && printf "WARNING: Not enough swap space is configured for local database installation.\n"
		$PRINTALL && printf "         The minimum required is ${NEEDEDSWAP} MB, but only ${swapsize} MB found.\n"
		$PRINTALL && printf "         There is, however, enough swap if the database will be installed remotely (min 6 GB needed).\n\n"
		$PRINTALL && ANYERRORS=1
	fi
else
	$PRINTALL && echo "OK: Size of swap space is good for local or remote database installation:  ${swapsize} MB"
	SWAPOK="1"
fi


#
# Checking disk space
#

if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
        MOUNT="/var"
        if [ -d "/var/opt" ]; then
                MOUNT="/var/opt"
                if [ -d "/var/opt/sun" ]; then
                        MOUNT="/var/opt/sun"
                        if [ -d "/var/opt/sun/xvm" ]; then
                                MOUNT="/var/opt/sun/xvm"
                        fi
                fi
        fi

        AVAILABLE=`df -Pk $MOUNT | tail -1 | awk '{print \$4}'`
        AVAILABLEH=`df -Ph $MOUNT | tail -1 | awk '{print \$4}'`
        HARDMIN=19000			### 20G rounded down for comparison
        hHARDMIN="20G"
        RECOMMENDED=69000000		### 70G rounded down for comparison
        hRECOMMENDED="70G"
        if [ $AVAILABLE -lt $RECOMMENDED ]; then
        	if [ $AVAILABLE -lt $HARDMIN ]; then
        		echo "ERROR: Not enough disk space on $MOUNT - Need ${hHARDMIN} minimum (${hRECOMMENDED} recommended) but only have $AVAILABLEH"
        		echo "       Installation will not let you continue."
        		echo ""
        		ANYERRORS=1
        	else
        		echo "WARNING: Not enough recommended disk space on $MOUNT. Required is ${hHARDMIN}, but ${hRECOMMENDED} is recommended. You have ${AVAILABLEH}."
        		echo "         If you are planning on using a local database, you will need additional 10G (+15G is recommended)."
        		echo ""
        	fi
        else
                echo "OK: Found enough disk space on $MOUNT ($AVAILABLEH)"
        fi
        ##### Check /opt
        MOUNT="/opt"
        AVAILABLE=`df -Pk $MOUNT | tail -1 | awk '{print \$4}'`
        AVAILABLEH=`df -Ph $MOUNT | tail -1 | awk '{print \$4}'`
        HARDMIN=1850000			### 2G rounded down for comparison
        hHARDMIN="2G"
        if [ $AVAILABLE -lt $HARDMIN ]; then
        	echo "ERROR: Not enough disk space on $MOUNT - Need ${hHARDMIN} but only have $AVAILABLEH"
        	echo ""
        	ANYERRORS=1
        else
			echo "OK: Found enough disk space on $MOUNT ($AVAILABLEH)"
        fi
        ##### Check /tmp
        MOUNT="/tmp"
        AVAILABLE=`df -Pk $MOUNT | tail -1 | awk '{print \$4}'`
        AVAILABLEH=`df -Ph $MOUNT | tail -1 | awk '{print \$4}'`
        HARDMIN=900000			### 1G rounded down for comparison
        hHARDMIN="1G"
        if [ $AVAILABLE -lt $HARDMIN ]; then
        	echo "ERROR: Not enough disk space on $MOUNT - Need ${hHARDMIN} but only have $AVAILABLEH"
        	echo ""
        	ANYERRORS=1
        else
			echo "OK: Found enough disk space on $MOUNT ($AVAILABLEH)"
        fi
fi


# check for supported OEL or RedHat release
if $PRINTALL; then   # we will run this only in prereq
    SUPPORTED_OS_VERSION=true
    RHELVERFILE="/etc/redhat-release"
    OELVERFILE="/etc/oracle-release"
    if [ -f $OELVERFILE ]; then
        OEL_VERSION=$( awk '/^.* Linux Server release / {print $5}' $OELVERFILE)
        MAJOR_REL=$(echo $OEL_VERSION | cut -d. -f1)
        MINOR_REL=$(echo $OEL_VERSION | cut -d. -f2)
        if [ -n "$MAJOR_REL" -a -n "$MINOR_REL" ]; then
            if [ $MAJOR_REL -ge 8 ]; then
                SUPPORTED_OS_VERSION=false
            fi
            if $SUPPORTED_OS_VERSION; then
                echo "OK: OEL $OEL_VERSION is a supported OS for the Ops Center Enterprise Controller"
            else
                echo
                echo "INFO: Ops Center versions up to 12.3.2 do not support OEL $OEL_VERSION !"
                echo
                echo "Please visit http://docs.oracle.com/cd/E59957_01/doc.123/e59961/toc.htm for the latest Certified Systems Matrix."
                echo
            fi
        fi
    elif [ -f $RHELVERFILE ]; then
        RHEL_VERSION=$( awk '/^.* Linux Server release / {print $7}' $RHELVERFILE)
        MAJOR_REL=$(echo $RHEL_VERSION | cut -d. -f1)
        MINOR_REL=$(echo $RHEL_VERSION | cut -d. -f2)
        if [ -n "$MAJOR_REL" -a -n "$MINOR_REL" ]; then
            if [ $MAJOR_REL -ge 7 ]; then
                SUPPORTED_OS_VERSION=false
            elif [ $MAJOR_REL -eq 6 -a $MINOR_REL -gt 4 ]; then
                SUPPORTED_OS_VERSION=false
            fi
            if $SUPPORTED_OS_VERSION; then
                echo "OK: RHEL $RHEL_VERSION is a supported OS for the Ops Center Enterprise Controller"
            else
                echo
                echo "INFO: Ops Center versions up to 12.3.0 do not support RHEL $RHEL_VERSION !"
                echo
                echo "Please visit http://docs.oracle.com/cd/E59957_01/doc.123/e59961/toc.htm for the latest Certified Systems Matrix."
                echo
            fi
        fi
    else
	printf "ERROR: Could not find $RHELVERFILE ! Only RHEL and OEL are supported Linux distributions.\n\n"
    fi

fi


#
# checking XEN kernel (Bug 15643413)
#

XEN=`uname -r | grep xen`
if [ ! "$XEN" = "" ]; then
	printf "WARNING: 'uname -r' shows that you are running the XEN kernel, which may result in slow EC performance.\n"
	echo "         For optimal performance, it is recommended that the system be reconfigured to use the standard (non-XEN) kernel."
	echo "         This can be achieved by modifying the /boot/grub/menu.lst file and set 'default=N' to a number corresponding to"
	echo "         the non-XEN kernel (requires reboot)."
	echo ""
else
	$PRINTALL && printf "OK: XEN kernel is not loaded\n"
fi


# do we have all required RPMs? 
#

	cat <<!EOF! > /tmp/rpms.$$ 
	xinetd
	tftp-server
	dhcp
	gettext
	perl-XML-Parser
	perl-libintl
        libxml2
	gcc
	httpd
	wget
	make
	bc
	policycoreutils
	keyutils
	compat-libstdc++-33
	libgcc
	elfutils-libelf-devel
	gcc-c++
	libstdc++-devel
	ksh
	libaio-devel
	libstdc++-devel
	openssl
	keyutils-libs
	krb5-libs
	sysstat	
	zip
!EOF!
	while read rpm
	do
	  rpm -q $rpm >/dev/null
	  if [ $? -eq 0 ]; then
	    $PRINTALL && echo "OK: $rpm is installed."
	    #printf "."
	  else
	    printf "WARNING: RPM $rpm is MISSING\n\n"
	    ANYERRORS=1
	  fi
	done < /tmp/rpms.$$
	rm -f /tmp/rpms.$$

#### Check for RHEL 5 specific RPMs
MAJOR_VER=`cat /etc/redhat-release | sed s/.*release\ // | awk -F"." '{print $1}' | awk '{print $1}'`		## 5 or 6
if [ "$MAJOR_VER" = "5" ]; then		### Check for additional 32 bit RPMs
cat <<!EOF! > /tmp/rpms.$$
compat-libcom_err
!EOF!
	while read rpm
	do
        rpm -q $rpm >/dev/null
        if [ $? -eq 0 ]; then
                $PRINTALL && echo "OK: $rpm is installed."
        else
                printf "WARNING: RPM $rpm is MISSING\n\n"
                ANYERRORS=1
        fi
	done < /tmp/rpms.$$
	rm -f /tmp/rpms.$$
fi

#### Check for RHEL 6 specific RPMs
MAJOR_VER=`cat /etc/redhat-release | sed s/.*release\ // | awk -F"." '{print $1}' | awk '{print $1}'`		## 5 or 6
if [ "$MAJOR_VER" = "6" ]; then		### Check for additional 32 bit RPMs
cat <<!EOF! > /tmp/32bitrpms.$$
openssl098e
compat-expat1
libuuid
audit-libs
cracklib
db4
pam
compat-libstdc++-33
glibc
zlib
expat
libxml2
libaio
libstdc++
glibc-devel
!EOF!
	while read rpm
	do
	        # Check for i386/i568/i686 version of the rpm
	        rpm -q --qf "%{NAME}-%{VERSION}-%{RELEASE}:%{ARCH}\n" $rpm | grep i[0-9]86 > /dev/null
	        if [ $? -eq 0 ]; then
	                $PRINTALL && echo "OK: $rpm is installed."
	        else
	                printf "ERROR: 32-bit RPM $rpm is MISSING\n\n"
	                ANYERRORS=1
	        fi
	done < /tmp/32bitrpms.$$
	rm -f /tmp/32bitrpms.$$

fi

#### Check for RHEL 7 specific RPMs
MAJOR_VER=`cat /etc/redhat-release | sed s/.*release\ // | awk -F"." '{print $1}' | awk '{print $1}'`		## 5 or 6
if [ "$MAJOR_VER" = "7" ]; then		### Check for additional 32 bit RPMs

### Check for OEL7 required RPMs
        cat <<!EOF! > /tmp/rpms.$$
        perl-ExtUtils-MakeMaker.noarch
        perl-CGI.noarch
	compat-db47
!EOF!
        while read rpm
        do
          rpm -q $rpm >/dev/null
          if [ $? -eq 0 ]; then
            $PRINTALL && echo "OK: $rpm is installed."
            #printf "."
          else
            printf "WARNING: RPM $rpm is MISSING\n\n"
            ANYERRORS=1
          fi
        done < /tmp/rpms.$$
        rm -f /tmp/rpms.$$

### Check for additional 32 bit RPMs
cat <<!EOF! > /tmp/32bitrpms.$$
keyutils-libs
krb5-libs
libuuid
audit-libs
cracklib
pam
compat-libstdc++-33
glibc
zlib
expat
libxml2
libaio
libstdc++
glibc-devel
openssl098e
!EOF!
	while read rpm
	do
	        # Check for i386/i568/i686 version of the rpm
	        rpm -q --qf "%{NAME}-%{VERSION}-%{RELEASE}:%{ARCH}\n" $rpm | grep i[0-9]86 > /dev/null
	        if [ $? -eq 0 ]; then
	                $PRINTALL && echo "OK: $rpm is installed."
	        else
	                printf "ERROR: 32-bit RPM $rpm is MISSING\n\n"
	                ANYERRORS=1
	        fi
	done < /tmp/32bitrpms.$$
	rm -f /tmp/32bitrpms.$$

fi

### Check Python and Expect
rpm -q python >/dev/null 2>/dev/null
if [ $? -ne 0 ]; then
	CURRENT_INST=0
else
	CURRENT_INST=`rpm -q --qf "%{VERSION}\n" python | head -1 | sed -e 's/\.//g'`
fi
if [ $CURRENT_INST -lt 243 ]; then
	printf "WARNING: RPM 'python' version 2.4.3 or greater is MISSING\n\n"
	ANYERRORS=1
else
	INSTVER=`rpm -q python | head -1`
	$PRINTALL && echo "OK: $INSTVER is installed."
fi
rpm -q expect >/dev/null 2>/dev/null
if [ $? -ne 0 ]; then
	CURRENT_INST=0
else
	CURRENT_INST_MAJOR=`rpm -q --qf "%{VERSION}\n" expect | head -1 | cut -d. -f1`
	CURRENT_INST_MINOR=`rpm -q --qf "%{VERSION}\n" expect | head -1 | cut -d. -f2`
	CURRENT_INST=$(( $CURRENT_INST_MAJOR * 1000 + $CURRENT_INST_MINOR ))
fi
if [ $CURRENT_INST -lt 5043 ]; then
	printf "WARNING: RPM 'expect' version 5.43.0 or greater is MISSING\n\n"
	ANYERRORS=1
else
	INSTVER=`rpm -q expect | head -1`
	$PRINTALL && echo "OK: $INSTVER is installed."
fi

#
# do we have correct umask
#

maskok=0
mask=022

if [[ "$(umask)" != "$mask" && "$(umask)" != "0$mask" ]]; then
  printf "WARNING: umask of user root is not $mask, Ops Center will fail\n"
  ANYERRORS=1
  maskok=1
else
  $PRINTALL && printf "OK: root umask is $mask\n"
fi

for file in /etc/profile* /etc/bash*; do
if [ -f $file ]; then
	OUTPUT=`umask 0$mask ; source $file >/dev/null 2>/dev/null ; umask`
	if [ "$OUTPUT" = "$mask" ] || [ "$OUTPUT" = "0$mask" ]; then
		$PRINTALL && printf "OK: $file has the umask of $mask\n"
	else
		printf "WARNING: it seems like $file may set a umask other than $mask. Ops Center may fail after installation.\n\n"
		ANYERRORS=1
		maskok=1
	fi
fi
done

if [[ "$(grep CMASK /etc/default/init 2> /dev/null|cut -d"#" -f1|egrep -v "(0$mask|$mask)")" != "" ]]; then
  printf "WARNING: it seems like /etc/default/init may set a umask other than $mask. Ops Center may fail after installation.\n\n"
  ANYERRORS=1
  maskok=1
else
  $PRINTALL && printf "OK: /etc/default/init has the umask of $mask\n"
fi

#
# is someone using our ports 
#
if [ $PRINTALL = true ];then		### Run only if we check everything and not through troubleshooting (the ports then SHOULD be in use)
	netports=0
	function portused () {
		port=$1
		comment=$2
		if [[ "$(netstat -an |grep LISTEN|fgrep  "${receive}${port} ")" == "" ]]; then
    			# $PRINTALL && printf "OK: Port $port is not in use\n"
			return 0
		else
    			printf "WARNING: someone is listening on port ${port}! $comment\n\n"
			ANYERRORS=1
			netports=1
			return 1
		fi
	}

	if [[ "$(uname -s)" == "Linux" ]]; then
	  receive=":"
	else
	  receive="*."
	fi

	portused 80 "BUI will connect here"
	portused 9443 "BUI will connect here"
	portused 443 "Proxy communication" 
	portused 21165 "Agent pushes Inventory to Proxy - only needed when Management node is Proxy"
	portused 8002 "Agent downloads from Proxy - only needed when Management node is Proxy"
	portused 8004 "OS provisioning completion status only - only needed when Management node is Proxy and OS Provisioning is wanted"
	portused 11172 "Enterprise Controller Cacao instance oem-ec will use this port"
	portused 162 "SNMP port will be used by Proxy - see the doc in the howto folder to adjust this port"

	if [ $netports -eq 0 ]; then
		$PRINTALL && printf "OK: All ports are available (not in use)\n"
	fi

fi

#
# checking ssh requirements for OC
#

CHECK=0
function checkfile () {
        file=$1
	if [[ -f $file || -h $file ]] ; then
		false && $PRINTALL && printf "."
	else
		printf "ERROR: File or link $file not found\n\n"
		ANYERRORS=1
		CHECK=1
	fi
}
checkfile /usr/bin/scp
checkfile /usr/bin/ssh
checkfile /usr/bin/ssh-keygen
checkfile /usr/bin/ssh-keyscan
if [ $CHECK -eq 0 ]; then
	$PRINTALL && printf "OK: All SSH requirements are met\n"
fi

#
# Check SELinux
#
SECONF="/etc/selinux/config"
if [ -f $SECONF ]; then
	SEL=`grep "^SELINUX=" $SECONF | awk -F"=" '{print $2}'`
	if [ "$SEL" = "enforcing" ]; then
		echo "ERROR: SELinux is enabled in $SECONF - must be disabled before installing Ops Center"
		echo ""
		ANYERRORS=1
	else
		$PRINTALL && echo "OK: SELinux doesn't show up in $SECONF"		#'
	fi
else
	$PRINTALL && echo "OK: No SELinux config file found"
fi
SEBIN="/usr/sbin/selinuxenabled"
if [ -f $SEBIN ]; then
	$SEBIN
	if [ $? -ne 0 ]; then
       		$PRINTALL && echo "OK: SELinux is not enable based on $SEBIN"
	else
	        echo "ERROR: $SEBIN is indicating that SELinux is running - SELinux must be disabled before installing Ops Center"
		echo ""
		ANYERRORS=1
	fi
fi

#
# checking for multiple NICs
#

#if [ $PRINTALL = true ]; then
#	$PRINTALL && printf "Checking for multiple configured NICs:"
#	NIC_NO=`ifconfig -a|grep inet | grep -v 'inet6' |grep -v 'inet addr:127.0.0.1' |grep -v 'inet addr:0.0.0.0'|wc -l`
#	if [ $NIC_NO -gt 1 ] 
#	then
#	  printf "\nWARNING: This system has multiple configured IP addresses !"
#	  printf "\nWARNING: Do not select 'Simple Deployment' during the Initial Configuration !"
#	  printf "\nWARNING: If a local Proxy is required use the command"
#	  printf "\nWARNING:    proxyadm configure -s localhost -a <ip-address-to-be-used> ..."
#	  printf "\nWARNING: to configure the proxy later on! \n"
#	  ANYERRORS=1
#	else
#	  $PRINTALL && printf " OK\n"
#	fi
#fi


#
# Check for a leftover Local Library
#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
    if [ -d /guests ]; then
        echo 'WARNING: Found an existing directory /guests with an Ops Center Local Library!'
        echo 'WARNING: Remove this directory to avoid any conflict due to left content in the Library!'
    else
        echo 'OK: Directory /guests does not exist.'
    fi
fi

#
# Check for proper locale settings
#
if [ $PRINTALL = true ]; then                              ### Only run this in prereq - NOT in troubleshoot (have a specific test for it)
        FAILED=0

        LC_CTYPE=`locale | grep LC_CTYPE | awk -F"=" '{print $2}' | sed -e 's/\"//g'`
        LC_MESSAGES=`locale | grep LC_MESSAGES | awk -F"=" '{print $2}' | sed -e 's/\"//g'`

        LC_CTYPE_EN=`echo $LC_CTYPE|grep -i en_US`
        if [ ! "$LC_CTYPE" = "C" ] && [ "$LC_CTYPE_EN" = "" ]; then
                FAILED=1
                FAILED_L=$LC_CTYPE
        fi

        LC_MESSAGES_EN=`echo $LC_MESSAGES|grep -i en_US`
        if [ ! "$LC_MESSAGES" = "C" ] && [ "$LC_MESSAGES_EN" = "" ]; then
                FAILED=1
                FAILED_L=$LC_MESSAGES
        fi

        if [ $FAILED -eq 0 ]; then
                echo "OK: Locales are OK (\"$LC_CTYPE\")"
        else
                echo "WARNING: Locales are set to \"$FAILED_L\". This may cause issues when upgrading. Make sure you run:"
                echo "         # export LC_ALL=C"
                echo "         # export LANG=C"
                echo "         You can verify the settings by running:"
                echo "         # locale"
                echo ""
                ANYERRORS=1
        fi
fi

#*************************************************************************************#
### Check that hostname resolves properly
HOSTNAME=`hostname`
IPfromGetent=`getent hosts \`hostname\` | awk '{print $1}' 2>/dev/null`
if [ "$IPfromGetent" = "127.0.0.1" ]; then
	echo "ERROR: getent hosts \`hostname\` says my IP is 127.0.0.1, please review /etc/hosts and fix."
	echo ""
	ANYERRORS=1
else
	$PRINTALL && echo "OK: Hostname $HOSTNAME doesn't resolve to 127.0.0.1"				#'
	HostnameNoDot=`echo "$HOSTNAME" | awk -F"." '{print $1}' 2>/dev/null`
	HostFromGetEnt=`getent hosts "$IPfromGetent" | awk '{print $2}' 2>/dev/null | awk -F"." '{print $1}' 2>/dev/null`
	if [ ! "$HostnameNoDot" = "$HostFromGetEnt" ]; then
		echo "ERROR: Using getent, my IP does not resolve to my hostname:"
		echo ""
		echo "getent hosts $HostnameNoDot   -> Got IP \"$IPfromGetent\""
		echo "  then"
		echo "getent hosts $IPfromGetent   -> Got hostname \"$HostFromGetEnt\""
		echo ""
		echo "Please review /etc/hosts and /etc/nsswitch.conf to fix."
		echo ""
		ANYERRORS=1
	else
		$PRINTALL && echo "OK: Hostname $HOSTNAME resolves to $IPfromGetent and back properly"
	fi
fi
#*************************************************************************************#
### Check that localhost resolves properly
IPfromGetent=`getent hosts localhost | awk '{print $1}' 2>/dev/null`
if [ "$IPfromGetent" != "127.0.0.1" ] && [ "$IPfromGetent" != "::1" ]; then
	echo "ERROR: 'getent hosts localhost' says localhost IP is NOT 127.0.0.1, please review /etc/hosts and fix."
	echo ""
	ANYERRORS=1
else
	$PRINTALL && echo "OK: localhost resolves to 127.0.0.1"	
fi
IPfromHosts=`grep -v "^#" /etc/hosts | grep localhost | grep -v "^127.0.0.1" | grep -v "^::1"`
if [ "$IPfromHosts" != "" ]; then
	echo "ERROR: it seems like 'localhost' shows up in /etc/hosts under the wrong IP."
	echo "       Make sure it is only used under 127.0.0.1, otherwise you will have many connectivity issues."
	echo ""
	ANYERRORS=1
fi

#*************************************************************************************#
### Check for OpenDNS-like environment
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot as we have a special troubleshoot test for this
	VIRTNAME="_default_"
	OUTPUT=`getent hosts $VIRTNAME 2>/dev/null`
	if [ $? -eq 0 ]; then
		echo "WARNING: Running 'getent hosts $VIRTNAME' should have returned an error, but instead, it resolved to:"
		echo ""
		printf "$OUTPUT\n"
		IPCHECK=`printf "$OUTPUT\n" | awk '{print $1}'`
		echo "   and the IP $IPCHECK resolves to:"
		getent hosts $IPCHECK
		echo ""
		echo "Immediately after installing Ops Center, run 'OCDoctor.sh --troubleshoot' to see the workaround of a known issue with this DNS config."
		echo ""
	else
		echo "OK: DNS does not unexpectedly resolve hostname '$VIRTNAME'"
	fi
fi

#
# Do old UCE rpms exist on the system? 
#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
cat <<!EOF! > /tmp/uce.$$
sun-uce-agent
sun-uce-server
sun-uce-engine
!EOF!

	while read rpm
	do
	  rpm -q $rpm >/dev/null
	  if [ $? -eq 0 ]; then
	    printf "ERROR: $rpm is installed. You must uninstall $rpm before installing Ops Center\n\n"
	    ANYERRORS=1
	  else
	    $PRINTALL && printf "OK: RPM $rpm not installed\n"
	  fi
	done < /tmp/uce.$$
	rm -f /tmp/uce.$$
fi

if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
	if [ -f /n1gc-setup/.version.properties ]; then
		echo "ERROR: Found an existing /n1gc-setup/.version.properties file, did you uninstall the previous version properly?"
		echo "       Removing the file may not be enough, as there may be some other leftovers."
		echo ""
		ANYERRORS=1
	fi
fi

#
# Do we have a development build of cacao installed? (Bug 15780343)
#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
	OUTPUT=`rpm -qa | grep "^sun-cacao" | grep snapshot`
	if [ "$OUTPUT" = "" ]; then
		echo "OK: No development build of cacao is installed"
	else
		echo "ERROR: Found the following development build of cacao (has 'snapshot' in the name)."
		echo "       Those RPMs must be removed prior to the Enterprise Controller installation:"
		printf "$OUTPUT\n\n"
	fi
fi

#
# Do we have /var/tmp/no_os_selfhealing - the workaround for 17211690?
#
#if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
#	if [ ! -f /var/tmp/no_os_selfhealing ]; then
#		echo "WARNING: There is a known issue with Ops Center Update 4 (12.1.4.2319) and older - bug 17211690."
#		echo "         This bug may cause issues communicating to assets."
#		echo "         Before installing, it is strongly recommended that you create the following file:"
#		echo "         # touch /var/tmp/no_os_selfhealing"
#		echo "         This is a workaround that will prevent this issue from happening."
#		echo ""
#	else
#		echo "OK: /var/tmp/no_os_selfhealing (workaround for bug 17211690) already exists"
#	fi
#fi

#**************** Check ulimit **********************
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
        ULIMIT=`ulimit -Sn`
        if [ $ULIMIT -lt 1024 ]; then
                echo "WARNING: ulimit -Sn (Open files) is $ULIMIT, but it needs to be 1024 to avoid EM installation warnings."
                echo "         To fix, run 'ulimit -Sn 1024' just before you start the installation."
                echo
                echo "         Once ulimit has been fixed and the EC installed, as a post installation task"
                echo "         run OCDoctor.sh --troubleshoot to check for possible follow up instructions."
                echo ""
        else
                echo "OK: ulimit -Sn is $ULIMIT"
        fi
        ULIMIT=`ulimit -Hn`
        if [ $ULIMIT -lt 65536 ]; then
                echo "WARNING: ulimit -Hn (Open files hard limit) is $ULIMIT, but it needs to be 65536 to avoid EM installation warnings."
                echo "         To fix, run 'ulimit -Hn 65536' just before you start the installation."
                echo
                echo "         Once ulimit has been fixed and the EC installed, as a post installation task"
                echo "         run OCDoctor.sh --troubleshoot to check for possible follow up instructions."
                echo ""
        else
                echo "OK: ulimit -Hn is $ULIMIT"
        fi
fi

#**************** Check for an existing /var/js/directory ******************************#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
        if [ -d /var/js ]; then
                echo
                echo "ERROR: Found an existing directory /var/js !"
                echo
                echo "This directory should be removed prior to install the EC. Existing old content can cause installation failures!"
                echo
        else
                echo "OK: No leftover /var/js directory from a previous installation found."
        fi
fi


#
# Check for Java 1.6.0_71 - 1.6.0_74 (Bug 18175308)
#
# We have to test if one of the broken versions is installed and warn the user.
# If there is no Java 6 installed and we are running S11 SRU 15 or SRU16 then 
# the installer will fetch a broken version from the IPS repository, so we have
# to warn the user as well
#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
	INSTALLED_JAVA_6="`ls /usr/java/jdk1.6.0*/bin/java 2>/dev/null`"

	if [ -n "$INSTALLED_JAVA_6" ]; then
		JAVA_VERSION=`$INSTALLED_JAVA_6 -version 2>&1 | awk -F\" '/^java version/ {print $2}'`
		JAVA_MINOR=`echo $JAVA_VERSION | cut -d_ -f2`
		if [ "$JAVA_MINOR" -ge 71 -a "$JAVA_MINOR" -le 74 ]; then
			# found a broken version
			echo		
			echo "ERROR: Java $JAVA_VERSION is installed on this system!"
			echo
			echo "For a Proxy Controller the Java 6 version must not be in the range 1.6.0_71 - 1.6.0_74"
			echo
			echo "The current recommended version for a Proxy Controller is 1.6.0_65"
			echo
			echo "See MOS Doc ID 1439822.1 for download links of this Java 6 version for all platforms"
			echo
			ANYERRORS=1
		else
			# the installed version is OK
			echo "OK: Installed Java 6 version $JAVA_VERSION is not in the range 1.6.0_71 - 1.6.0_74"
		fi
	else
		echo "OK: Java 6 is not installed, the Ops Center installer will install a supported version"
		echo
	fi
fi


#**************** Check for local Oracle DB needed files ******************************#
if [ $PRINTALL = true ]; then                   ### Only run this in prereq - NOT in troubleshoot
        DOWNLOADS="/var/tmp/downloads"
        ARCH=`uname -p`
        if [ "$ARCH" = "x86_64" ]; then
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_Linux-x86-64_1of7.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_Linux-x86-64_2of7.zip"
                V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_Linux-x86-64_1of7.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_Linux-x86-64_2of7.zip"
                V12_1_0_2_FILE1="$DOWNLOADS/linuxamd64_12102_database_1of2.zip"
                V12_1_0_2_FILE2="$DOWNLOADS/linuxamd64_12102_database_2of2.zip"
        else
                V11_2_0_3_FILE1="$DOWNLOADS/p10404530_112030_LINUX_1of7.zip"
                V11_2_0_3_FILE2="$DOWNLOADS/p10404530_112030_LINUX_2of7.zip"
                V11_2_0_4_FILE1="$DOWNLOADS/p13390677_112040_LINUX_1of7.zip"
                V11_2_0_4_FILE2="$DOWNLOADS/p13390677_112040_LINUX_2of7.zip"
                V12_1_0_2_FILE1="$V11_2_0_4_FILE1"
                V12_1_0_2_FILE2="$V11_2_0_4_FILE2"
        fi

        if [ ! -d $DOWNLOADS ] || [ ! -f ${V12_1_0_2_FILE1} ] || [ ! -f ${V12_1_0_2_FILE2} ]; then
                echo ""
                echo "WARNING: If you are planning on using a local database, the following two files are missing:"
                echo ""
                echo "${V12_1_0_2_FILE1}"
                echo "${V12_1_0_2_FILE2}"
                echo ""
                echo "When installing an Enterprise Controller with an embedded database, you need to download those first."
                echo ""
                echo "To download, login to http://www.oracle.com/technetwork/database/enterprise-edition/downloads/index.html"

                case "$ARCH" in
                'i386')
                        echo "and select the 'Oracle Solaris on X86-64 (64-bit)' platform."
                                        ;;
                'sparc')
                        echo "and select the 'Oracle Solaris on SPARC (64-bit)' platform."
                        ;;
                esac


                echo ""
                echo "You do not need those 2 files if you plan to install using a remote database."
                echo ""
                echo "NOTE:"
                echo
                if [ -f ${V11_2_0_4_FILE1} -a -f ${V11_2_0_4_FILE2} ]; then
                        echo "There are DB files in $DOWNLOADS which can be used to install OpsCenter 12.2.2 and 12.3.0!"
                        echo "All is good if you are planning to install one of these OpsCenter versions!"
                        echo
                fi
                if [ -f ${V11_2_0_3_FILE1} -a -f ${V11_2_0_3_FILE2} ]; then
                        echo "There are DB files in $DOWNLOADS which can be used to install OpsCenter up to version 12.2.1!"
                        echo "All is good if you are planning to install one of these OpsCenter versions!"
                        echo
                fi
                if [ ! -f ${V11_2_0_4_FILE1} -a ! -f ${V11_2_0_4_FILE2} -a ! -f ${V11_2_0_3_FILE1} -a ! -f ${V11_2_0_3_FILE2} ]; then
                        echo "If you are planning to install OpsCenter version 12.2.1 or lower, you will need"
                        echo "the following two files in ${DOWNLOADS}:"
                        echo
                        echo "${V11_2_0_3_FILE1}"
                        echo "${V11_2_0_3_FILE2}"
                        echo
                        echo "which are available from https://updates.oracle.com/Orion/PatchDetails/process_form?patch_num=10404530"
                        echo
                        echo "If you are planning to install OpsCenter version 12.2.2 or 12.3.0, you will need"
                        echo "the following two files in ${DOWNLOADS}:"
                        echo
                        echo "${V11_2_0_4_FILE1}"
                        echo "${V11_2_0_4_FILE2}"
                        echo
                        echo "which are available from https://updates.oracle.com/Orion/PatchDetails/process_form?patch_num=13390677"
                        echo
                fi
        else
                echo "OK: Found 2 DB installation files for OpsCenter 12.3.1 or higher in $DOWNLOADS"
                COLOCATED_DB_IS_PLANNED=true
        fi
fi


exit $ANYERRORS
