#!/bin/bash

# Check heap memory of Proxy cacao instance
#
# Two known issues:
# 1) Any max heap value beyond 2000m will get set back to 2000m during upgrade (Bug 20103041)
# 2) If the minimum size has been set to a value larger than 2000m, the SMF service
#    will go into maintenance during Proxy Upgrade. Java will not start due to the 
#    minimum size being larger than the maximum size
#
# Update: Bug 20103041 got fixed in 12c R3, so upgrades of PCs running 12c R2 U2 are no longer at risk to fail.
#         Thecheck will be just skipped on 12c R2 U2 or higher

MY_DIR=`dirname $0`
if [ -f $MY_DIR/../../functions ]; then
    . $MY_DIR/../../functions
elif [ -f /var/opt/sun/xvm/OCDoctor/functions ]; then
   . /var/opt/sun/xvm/OCDoctor/functions
else
    exit 1
fi

# skip check if we are already running 12c R2 U2 or higher
get_version
[ $VERSIONNUM -gt "12220000" ] && exit 0


if [ "$(uname)" = "Linux" ]
then
	CACAOADM=/opt/sun/cacao2/bin/cacaoadm
else
	CACAOADM=/usr/sbin/cacaoadm
fi

JAVA_FLAGS="$($CACAOADM get-param -v -i scn-proxy java-flags 2>/dev/null)"
MINIMUM_HEAP=$(echo $JAVA_FLAGS | sed 's/^.*-Xms//'| awk '{print substr($1,1,length($1)-1)}')
MINIMUM_HEAP_UNIT=$(echo $JAVA_FLAGS | sed 's/^.*-Xms//'| awk '{print substr($1,length($1),1)}')
MAXIMUM_HEAP=$(echo $JAVA_FLAGS | sed 's/^.*-Xmx//'| awk '{print substr($1,1,length($1)-1)}')
MAXIMUM_HEAP_UNIT=$(echo $JAVA_FLAGS |sed 's/^.*-Xmx//'| awk '{print substr($1,length($1),1)}')

#MINIMUM_HEAP=4
#MINIMUM_HEAP_UNIT=G
#MAXIMUM_HEAP=4000
#MAXIMUM_HEAP_UNIT=M

[ "$MINIMUM_HEAP_UNIT" = "g" -o "$MINIMUM_HEAP_UNIT" = "G" ] && MINIMUM_HEAP=$(( $MINIMUM_HEAP * 1024 ))
[ "$MAXIMUM_HEAP_UNIT" = "g" -o "$MAXIMUM_HEAP_UNIT" = "G" ] && MAXIMUM_HEAP=$(( $MAXIMUM_HEAP * 1024 ))
#echo MINIMUM_HEAP=$MINIMUM_HEAP
#echo MINIMUM_HEAP_UNIT=$MINIMUM_HEAP_UNIT
#echo MAXIMUM_HEAP=$MAXIMUM_HEAP
#echo MAXIMUM_HEAP_UNIT=$MAXIMUM_HEAP_UNIT

NO_ERROR_FOUND=true
# issue #1 max heap is larger than 2000m
if [ "$MAXIMUM_HEAP" -gt 2000 ]; then
	echo
	echo "WARNING: Maximum heap size of Proxy cacao instance is set to $MAXIMUM_HEAP MB."
	echo "         During a Proxy upgrade the value will be set back to 2000 MB (Bug 20103041)."
	echo
	echo "Ensure to verify and correct the setting after a Proxy upgrade back to '-Xmx=${MAXIMUM_HEAP}m'!"
	echo
	NO_ERROR_FOUND=false
fi

# issue #2 min heap is larger than 2000m
if [ "$MINIMUM_HEAP" -gt 2000 ]; then
	echo
	echo "ERROR: Start/Minimum heap size of Proxy cacao instance is set to $MINIMUM_HEAP MB."
	echo "       This will cause a Proxy Upgrade failure, since the minimum heap size "
	echo "       will be larger than the new set maximum size of 2000m. This in turn will"
	echo "       cause a startup failure of the Proxy cacao instance during upgrade!"
	echo   
	echo "To fix this, set the value back to equal or below 2000m like '-Xms=200m'!"
	echo
	NO_ERROR_FOUND=false
fi

if $NO_ERROR_FOUND; then
	echo "OK: Memory heap settings of Proxy cacao instance will not cause upgrade issues"
fi 
