#!/bin/bash
#
# Check to see which Java major and minor version is used by the Proxy Controller
#
# We have the following issues:
# 
# 1. Java bug JDK-8030956 / Bug 18287436 - JAVA DB CANNOT RUN WITH JAVA 6 U71
#    This bug is causing issues with embedded Derby DB, affected versions 1.6.0_71 - 1.6.0_74
# 
#    OpsCenter Bug is 18175308
#
#    Comment: S11.1 SRU 17.5 comes with Java 1.6.0_71_b32 (1.6.0.71.32) which includes a fix for bug 18287436.
#             So a Delta Proxy can run with this particular build, but not a Diamond one!
#             The Diamond Proxy is verifying the used Java version and will not start when it is in
#             the range 1.6.0_71 - 1.6.0_74. Therefore we do not recommend using 1.6.0_71_b32 since
#             this Delta Proxy would not be ready for update to Diamond.
#
#	      S11.1 SRU 18.5 comes with Java 1.6.0_75 which is again a supported version
# 
# 2. Java 1.7.0 is not an option since OpsCenter's jsch libraries are having issues with it (bug 15775710)
#
#    Bug 15775710 has been fixed in 12c R1 GA
# 
#
# 3. OVMM x86 code will not work if version of the JDK on the Proxy is not 1.6.0
#    With release of 12c R2 U2 we removed this limitation and are supporting any JDL version
#

# source various common functions
. $(dirname $0)/../../functions

solaris11_stuff

get_version
if [ $VERSIONNUM -lt "12200000" ]; then
	IS_DIAMOND=false
else
	IS_DIAMOND=true
fi

if [ $VERSIONNUM -gt "12220000" ]; then
	echo "OK: Proxy version is newer than 12c R2 U1, no need to check for a supported Java 1.6 version"
	exit 0
fi
 
OS=$(uname)

if [ "${OS}" == "SunOS" ]; then
	CACAOADM=/usr/sbin/cacaoadm
	AWK=/usr/bin/nawk
else
	CACAOADM=/opt/sun/cacao2/bin/cacaoadm
	AWK=/usr/bin/awk
fi

if [ -x $CACAOADM ]; then
	JDK_HOME="$($CACAOADM get-param -i scn-proxy java-home 2>/dev/null | cut -d= -f2)"

	if [ -n "$JDK_HOME" ]; then
		
		JDK_VERSION=$(${JDK_HOME}/bin/java -version 2>&1 | $AWK -F \" ' /^java version/ {print $2}')
		JDK_MAJOR=$( echo $JDK_VERSION | cut -d_ -f1)
		JDK_MINOR=$( echo $JDK_VERSION | cut -d_ -f2)

		if [ "$JDK_MAJOR" = "1.6.0" ]; then
			if [ "$JDK_MINOR" -lt 71 -o "$JDK_MINOR" -gt 74 ]; then		
				if $IS_S11  && [ $S11_RELEASE -eq 1 -a $S11_SRU -lt 15 ]; then
					echo "OK: Proxy is using a supported Java version, do *NOT* upgrade the OS to Solaris 11.1 SRU 15/16/17"
				else
					echo "OK: Proxy is using a supported Java version"
				fi
			else
				echo
				echo "ERROR: Proxy is using Java version $JDK_VERSION which is not supported (Bug 18175308)"
				echo "       Java versions 1.6.0_71 - 1.6.0_74 are causing issues with the derby DB used by the Proxy!"
				echo
				echo "The current recommended versions for a Proxy Controller are 1.6.0_65 and 1.6.0_75 "
				echo
				if $IS_S11; then
					echo "The following document has instructions how to uninstall the current Java 6 version and install 1.6.0_65:"
					echo
					echo "$OCDOCTOR_ROOT/howto/howto_install_java_1.6.0_65.txt"
					echo
				else
					echo "See MOS Doc ID 1439822.1 for download links of this Java 6 version for all platforms"
					echo
				fi
				$IS_DIAMOND || echo "IMPORTANT: Do *NOT* try to upgrade to 12cR2 with this Java version in use !!!" 
			fi
		else
			echo
			echo "ERROR: Proxy is using Java version $JDK_MAJOR but only Java 1.6.0 is supported up to OpsCenter 12c R2 U1!"
			echo
		fi
	else
		# scn-proxy instance does not exist
		echo
		echo "ERROR: Unable to find cacao instance scn-proxy, looks like Proxy Controller has not yet been configured"
		echo
	fi
else
	echo
	echo "ERROR: Ops Center Proxy is not installed properly, unable to find cacaoadm binary." 1>&2
	exit 1
fi   

