#!/bin/bash
#
# Script to enable/disable the Domain Model Navigator on an OpsCenter Agent Controller
#

Usage(){
cat << EOF

        $PROG { enable | disable | status }

	enable - start webserver to access the Agent Domain Model
	disable - stop webserver and perform cleanup 
	status - display current status e.g. URL to access Agent DM if webserver is enabled

	(Version 1.1 2015/02/27)

EOF
exit 1
}

PROG=$(basename $0)

UNAME="$(uname)"

DOMAIN_NAVIGATOR_OUTPUT=/tmp/domain_navigator_output.txt
DM_NAVI_PID=""

dm_navi_is_running(){

	if [ "$UNAME" = "Linux" ]; then
		PS_OUTPUT=/tmp/ps.$$.out
		ps -ef > $PS_OUTPUT
		if grep com.sun.hss.restfuladaptor.xvm.Main $PS_OUTPUT >/dev/null; then
			DM_NAVI_PID=$(awk '/com.sun.hss.restfuladaptor.xvm.Main/ {print $2}' $PS_OUTPUT)
		fi
		rm $PS_OUTPUT
	else
		for i in $(pgrep java)
		do
			if pargs $i | grep '^argv\[4\]: com.sun.hss.restfuladaptor.xvm.Main$' >/dev/null; then
				DM_NAVI_PID=$i
			fi
		done
	fi

	if [ -n "$DM_NAVI_PID" ]; then
		# if the domain-navigator has been started manually, we create a dummy output file with a best guess URL
		[ -f $DOMAIN_NAVIGATOR_OUTPUT ] || echo "http://$(hostname):7000" > $DOMAIN_NAVIGATOR_OUTPUT
		return 0
	else
		return 1
	fi	
}

start_domain_navigator(){

	if dm_navi_is_running; then
		echo
		echo "Webserver to access Domain Model is already running with PID $DM_NAVI_PID!"
		echo
		echo "URL: $(grep '^http://' $DOMAIN_NAVIGATOR_OUTPUT)"
		echo
		exit 0
	fi

	if [ "$UNAME" = "Linux" ]; then
		# workaround for bug 20229123
		[ -h /etc/opt/cacao2 ] || ln -s /etc/opt/sun/cacao2 /etc/opt/cacao2
	fi 
	$BASEDIR/domain-navigator a > $DOMAIN_NAVIGATOR_OUTPUT 2>&1 &

	# we will wait up to 1 minute for the java process to get started and settled down
	for i in {1..60}
	do
		grep '^http://' $DOMAIN_NAVIGATOR_OUTPUT >/dev/null && break
		sleep 1
	done

	if dm_navi_is_running; then
		echo
		echo "Webserver to access Domain Model has been started."
		echo
		echo "URL: $(grep '^http://' $DOMAIN_NAVIGATOR_OUTPUT)"
		echo
	else
		echo
		echo "Unable to start webserver to access Domain Model, please review file"
		echo
		echo $DOMAIN_NAVIGATOR_OUTPUT
		echo
		echo "to determine root cause!"
		echo 
		exit 1
	fi
}

stop_domain_navigator(){
	if dm_navi_is_running; then
		#Webserver to access Domain Model is running with PID $DM_NAVI_PID
		kill $DM_NAVI_PID 2>/dev/null
		echo
		echo "Webserver to access Domain Model is no longer running"
		echo
	else
		echo
		echo "Webserver to access Domain Model has already been stopped"
		echo
	fi
	if [ "$UNAME" = "Linux" ]; then
		# remove workaround for bug 20229123
		[ -h /etc/opt/cacao2 ] && rm /etc/opt/cacao2
	fi
}

domain_navigator_status(){
	if dm_navi_is_running; then 
		echo
		echo "Webserver to access Domain Model is running with PID $DM_NAVI_PID!"
		echo
		echo "URL: $(grep '^http://' $DOMAIN_NAVIGATOR_OUTPUT)"
		echo
		exit 0
	else
		echo
		echo "Webserver to access Domain Model is not running!"
		echo
		exit 1
	fi
}

MY_DIR=$(dirname $0)
BASEDIR=$MY_DIR/lib/domain-navigator

[ $# -ne 1 ] && Usage

# if not running as root, exit
if ! /usr/bin/id | grep "uid=0(root)" >/dev/null 2>&1; then
    echo "$PROG must be run by the root user"
    exit 1
fi

if [ "$UNAME" = "Linux" ]; then
	CACAOADM=/opt/sun/cacao2/bin/cacaoadm
        AGENTADM=/opt/sun/xvmoc/bin/agentadm
else
	CACAOADM=/usr/sbin/cacaoadm
        AGENTADM=/opt/SUNWxvmoc/bin/agentadm
fi

if $CACAOADM status -i scn-agent 2>&1 | grep "scn-agent instance is not running" >/dev/null; then
	echo "OpsCenter Agent Controller is not running, unable to launch Domain Model Navigator!"
	exit 1
fi

case "$1" in
'enable')
	start_domain_navigator
	;;
'disable')
	stop_domain_navigator
	;;
'status')
	domain_navigator_status
	;;
*)	
	Usage
	;;
esac

