#!/bin/bash
#
# clear_old_notifications.sh - script to purge notifications
#
VERSION="1.3 (2016/01/26)"

usage(){
        PROG=$(basename $0)
	cat <<EOF
	
	$PROG [Version: $VERSION]

        The script $PROG is used to manage the notifications stored in the DB.

        Usage:  $PROG -l 
                $PROG -d [-y] [-n count ] [-a] [-q] 

        Available options:
         -l       : List a summary of the notification counts in the DB
         -d       : delete notifications from the DB

        More config options to be used together with '-d' :
         -y       : do not prompt for confirmation
         -n count : number of most recent notifications which should remain in the DB, default is 100
         -a       : delete all notifications without requesting any confirmation, equiv to '-y -n 0'
         -q       : quiet mode, no output apart from error messages

        Note: If the number of generated notifications is huge (e.g. beyond ${HUGE_COUNT}), it is 
              way faster to delete all of them instead of keeping the most recent ones!

        Examples:

	# ./$PROG -d
        Before delete:
        Users: [5]
        User Notifications: [8495]
        Generated Notifications: [1699]

        The script will now purge all but the most recent 100 notifications.
        Press <RETURN> to continue or ^C to exit ...


        Deleting user and generated notifications (this can take a while depending on the amount of notifications)
        After delete:
        Users: [5]
        User Notifications: [500]
        Generated Notifications: [100]


	Example for a crontab entry to delete all notifications every Sunday at midnight:

	0 0 * * 0  /var/opt/sun/xvm/OCDoctor/toolbox/$PROG -d -a -q

EOF
	exit
}

if [ "$(uname)" = "Linux" ]; then
        ECADM="/opt/sun/xvmoc/bin/ecadm"
	AWK="/usr/bin/awk"
else
        ECADM="/opt/SUNWxvmoc/bin/ecadm"
	AWK="/usr/bin/nawk"
fi

sf=/tmp/soa_id.$$
cf=/tmp/cmd.$$

show_un(){
  un=$(echo 'select count(*) FROM HD_USER_NOTIFICATIONS;' | $ECADM sqlplus 2>/dev/null | $AWK '/COUNT/, /^SQL/' | grep '^[ |	]' |grep -v 'COUNT' | $AWK '{print $1}')
  $VERBOSE && echo "User Notifications: [${un}]"
}

show_gn(){
  gn=$(echo 'select count(*) FROM HD_GENERATED_NOTIFICATIONS;' | $ECADM sqlplus 2>/dev/null | $AWK '/COUNT/, /^SQL/' | grep '^[ |	]' | grep -v 'COUNT' | $AWK '{print $1}')
  if [ -z "$1" ]; then
  	$VERBOSE && echo "Generated Notifications: [${gn}]"
  else
	echo ${gn}
  fi

  if [ "${gn}" -ge $HUGE_COUNT -a "${COUNT}" -ne 0 ] && $VERBOSE; then
	echo
	echo "Number of notifications is beyond $HUGE_COUNT ! It is recommended to use option '-d -a' to"
	echo "delete all notifications in a much more efficient way than keeping a certain number of them."
	echo
  fi
}

show_uc(){
  echo 'select count(*), soa_id from hd_user_notifications group by soa_id;' | $ECADM sqlplus 2>/dev/null | $AWK '/COUNT/, /^SQL/' | grep '^[ |	]' | grep -v 'SOA_ID' | $AWK '{$1=""; print}' | cut -c2- > ${sf}
  uc=$(wc -l ${sf} | $AWK '{print $1}')
  $VERBOSE && echo "Users: [${uc}]"
}

delete_all(){
  
  cat > ${cf} <<EOF
  truncate table HD_USER_NOTIFICATIONS;
  alter table HD_USER_NOTIFICATIONS disable constraints HD_USER_NOTIFICATIONS_FK2;
  truncate table HD_GENERATED_NOTIFICATIONS;
  alter table HD_USER_NOTIFICATIONS enable constraints HD_USER_NOTIFICATIONS_FK2;
EOF

  $VERBOSE && echo 'Deleting all user and generated notifications'
  #cp ${cf} /tmp/remove_rows.sql
  #cat ${cf} | $ECADM sqlplus
  cat ${cf} | $ECADM sqlplus >/dev/null 2>&1
 
}

delete_partially(){

  cat > ${cf} <<EOF
DECLARE
CURSOR cdr_cur IS
  SELECT notification_id
      FROM HD_GENERATED_NOTIFICATIONS
  MINUS 
  SELECT *
   FROM (SELECT notification_id
                 FROM HD_GENERATED_NOTIFICATIONS
                       ORDER BY create_dtt DESC)
      WHERE ROWNUM <= ${COUNT};

  TYPE cdr_data IS TABLE OF HD_USER_NOTIFICATIONS.notification_id%TYPE INDEX BY  PLS_INTEGER;
  cdr cdr_data;

BEGIN
 OPEN cdr_cur;
 LOOP
     cdr.DELETE;

     FETCH cdr_cur BULK COLLECT INTO cdr LIMIT 10000;
     DBMS_OUTPUT.PUT_LINE('Totally'||to_char(cdr.COUNT));
     IF cdr.COUNT>0 THEN
        FOR  indx  IN 1 .. cdr.COUNT
        LOOP
        DELETE FROM HD_USER_NOTIFICATIONS WHERE NOTIFICATION_ID=cdr (indx);
        DELETE FROM HD_GENERATED_NOTIFICATIONS WHERE NOTIFICATION_ID=cdr (indx);
        COMMIT;
        END LOOP;
     ELSE
        EXIT;
     END IF;
 END LOOP;

 CLOSE cdr_cur;
 COMMIT;

EXCEPTION
   WHEN OTHERS THEN
      raise_application_error(-20001,'An error was encountered - '||SQLCODE||' -ERROR- '||SQLERRM);
      IF cdr_cur%ISOPEN THEN
         CLOSE cdr_cur;
      END IF;
    
END;
/
EOF

  $VERBOSE && echo 'Deleting user and generated notifications (this can take a while depending on the amount of notifications)'
  #cp ${cf} /tmp/remove_rows.sql
  #cat ${cf} | $ECADM sqlplus
  cat ${cf} | $ECADM sqlplus >/dev/null 2>&1

}

purge_notifications(){

	$VERBOSE && echo "Before delete:"
        $VERBOSE && print_summary

	if $ASK; then
		echo
		echo "The script will now purge all but the most recent $COUNT notifications."
		echo "Press <RETURN> to continue or ^C to exit ..."
		read junk
		echo
	fi	

	if [ "$COUNT" -eq 0 ]; then
		# we will use a more efficient method to delete all notifications
		delete_all
	else
		delete_partially
	fi

	$VERBOSE && echo "After delete:"
        $VERBOSE && print_summary
}

print_summary(){
	show_uc	
	show_un
	show_gn
}

print_gen_notif_count(){		# mainly for OCDoctor check :-)
	COUNT=$(show_gn plain_count 2>/dev/null)
        # check if COUNT is an integer
        if [[ "$COUNT" =~ ^[-0-9]+$ ]]; then
		echo "$COUNT"
	else
		echo "-1"
	fi
}

DELETE_MODE=false
ASK=true
VERBOSE=true
NO_ARGS=true
COUNT=100           ### default number of most recent notifactions to keep
HUGE_COUNT=100000   ### warning limit when to recommend deleting "all" notifications in one run

while getopts lgdyaqhn: option 2>/dev/null
do
	case $option in
                l)      NO_ARGS=false; print_summary;;
                g)      NO_ARGS=false; print_gen_notif_count;;
                d)      NO_ARGS=false; DELETE_MODE=true;;
                y)      NO_ARGS=false; ASK=false;;
                a)      NO_ARGS=false; ASK=false; COUNT=0;;
                q)      NO_ARGS=false; VERBOSE=false;;
                n)      NO_ARGS=false; COUNT=$OPTARG;;
                ?)      usage;;
        esac
done
$NO_ARGS && usage

$DELETE_MODE && purge_notifications

rm -f ${cf} ${sf}

