#!/bin/ksh
#################################################################################
## Desc: This script creates config backups of zones and LDOMs and builds a    ##
##       recovery script that can be used to recreate thes configurations.     ##
##       The recovery files are stored in ${LOGDIR} and is copied to any       ##
##       attached OC METADATA libraries in a directory ${OUTDIR}/`uname -n`.   ##
#################################################################################
#User configuriable variables
MAX_VERSIONS=31 	#Determines the number of copies we keep of the recovery script. Set to 0 to keep all versions.

#Standard program variables
VERSION="0.2"
LOGBASE="/var/log"
OUTDIR="Zone_Ldom_Config_Exports"
LOGDIR="${LOGBASE}/${OUTDIR}"
DATE="`date +%Y%m%d%H%M%S`"
PROG="`basename $0`"
LOCK="$LOGDIR/$PROG.lock"
LOCK_WAIT=10


Lock ()
{
LOOP_MAX=12
LOOP=1

#Create the log directory if it does not exist
if [ ! -d $LOGDIR ]
	then
	echo "INFO: Making the log directory [$LOGDIR]"
	mkdir -p $LOGDIR
fi

#Check if a lock exists and runaround a loop a few times to see if it clears. else create a lock with our PID
while [ $LOOP -le $LOOP_MAX ]
do
	if [ -f $LOCK ]
		then
		echo "INFO: Found existing lock file [$LOCK] for PID `cat $LOCK` [Pass: $LOOP/$LOOP_MAX] .... retrying"
	if [ $LOOP -eq $LOOP_MAX ]
		then
			echo "ERROR:  Exhausted retries waiting for lock to be released [$LOCK]."
			echo "DESC:   Another copy of this program is running [PID:`cat $LOCK`] or has terminated and left a lock file around."
			echo "ACTION: Confirm that another copy of this script is not running and if not remove lock file [$LOCK] manually."
		exit 4
	fi
		sleep $LOCK_WAIT
		LOOP=$((LOOP + 1 ))
		else
		echo   "$$" > $LOCK
		LOOP=$((LOOP_MAX + 1 ))
	fi
done
}

Unlock ()
{
if [ "$$" == "`cat $LOCK`" ]
	then
	rm -f $LOCK
	else
	echo "ERROR: Tried to remove lock file [$LOCK] that is not owned by this process."
	echo "DESC: Lock file [$LOCK] found had PID `cat $LOCK` but my PID was $$."
	echo "ACTION: Confirm that another copy of this script is not running and if not remove lock file [$LOCK] manually."
	exit 3
fi
}

Check_Global_Root ()

{
# check if we are running on a global zone
if [ `zonename` != "global" ]
	then
	echo "WARN: Exiting as this program only runs on a global zone [`zonename`]"
	Unlock
	exit 1
fi
# check if we are running root
if [ `/usr/bin/id|cut -d"(" -f1|cut -d"=" -f2` -ne 0 ]
	then
	echo "WARN: Exiting as this program only runs as a root user."
	Unlock
	exit 2
fi
}

Build_Zone_Recovery_Scripts ()
{
for ZONE in `zoneadm list -ic|grep -vw global`
do
	echo "INFO: ************ Starting processing zone ${ZONE} ************"
	# Set the output file for this zone
	OFILE="${LOGDIR}/zone_recreate_${ZONE}_$DATE.ksh"
	#Lets Build up some variables that will be used in creating the recovery script
	#Determine the zpool for the zone
	ZONE_ZPOOL="`Zone_Zpool`"
	#Select the correct zone attach command
	#ZONE_ATT_CMD="`Zone_Attach_Cmd`" 		# I have disabled this as I don't beileve we need to re-run the P2V scripts again on the attach of a existing branded zone
	ZONE_ATT_CMD="zoneadm -z $ZONE attach -U"

###########################################################################
##                                 HEADER                                ##
## Everything between this header and the subsequent footer form part of ##
## the generated recovery script.                                        ##
###########################################################################
echo "#!/bin/ksh
# First check if the zone is running, if so we do not need to do anything
if [ \`zoneadm list|grep -w ${ZONE}|wc -l\` -ne 0 ]
	then
	echo \"ERROR:  The zone [$ZONE] is already running. Unable to restore the configuration for this zone.\"
	echo \"DESC:   This script is only for restoring the configuration of a zone that has lost its configuration information.\"
	echo \"ACTION: Please check that you have the correct zonename for the config you are trying to restore.\"
	exit 21
fi

# Give a warning and a chance to terminate the script
echo \" ****** WARNING ****** 
* This script was created by $PROG on `date` for zone $ZONE on host `uname -n`.
* This script will recreate the zone config for a host if the existing config has been deleted or lost.
* It does not restore the actual data of the zone filesystems and they should be manually imported and mounted.
* This script does NOT provide any check to confirm that the zone resources (filesystems/IP etc) are available
* and not in use on another GZ. You MUST check this manually.
* This script should be run as root.

Type YES to continue:\"
read CONTINUE
if [ \"\${CONTINUE}\" != \"YES\" ]
	then
	echo \"INFO: Terminating recovery as you did not answer  YES ...[Your answer: \${CONTINUE}]\"
	exit 22
fi

DATE=\"\`date +%Y%m%d%H%M%S\`\"
PROG=\"\`basename \$0 \`\"

# Only recreate the zone if the zone does not exist
if [ \`grep ^${ZONE}: /etc/zones/index|wc -l\` -ne 0 ] || [ -f /etc/zones/${ZONE}.xml ]
	then
	echo \"WARN: Zone $ZONE still exists in the index file or its xml config file still exists. \"
	echo \"INFO: Answer YES to forcibly delete[remove] the zone ${ZONE}.\"
	read DELETE
	if [ \"\${DELETE}\" ==  \"YES\" ]
		then
		cp /etc/zones/index /etc/zones/index.\${PROG}.\${DATE}
		grep -v ^${ZONE}: /etc/zones/index.\${PROG}.\${DATE} > /etc/zones/index
		if [ -f /etc/zones/${ZONE}.xml ]
			then
			mv /etc/zones/${ZONE}.xml /etc/zones/${ZONE}.xml.found.\${DATE}
		fi 
		else
		echo \"ERROR:  Zone $ZONE partially exits and you chose not to let the script delete it [Your Answer: \${DELETE} ].\"
		echo \"DESC:   The exsisting zone configuration must be removed before we can import the backup of the configuration.\"
		echo \"ACTION: Please manually delete the existing zone before running this script, by either using zonecfg\"
		echo \"        commands or deleting it from /etc/zones/index and removing /etc/zones/${ZONE}.xml.\"
	   	exit 23 
	fi
fi

#Import the zone zpool if it is not already inported
if [ \`zpool list|grep -w ${ZONE_ZPOOL}|wc -l\` -eq 0 ]
	then	
	echo \"INFO: Importing zpool ${ZONE_ZPOOL} for zone ${ZONE}.\"
	zpool import ${ZONE_ZPOOL}
	RC=\$?
	if [ \${RC} -ne 0 ]
		then
		echo \"ERROR:  Zpool ${ZONE_ZPOOL} import returned a non zero return code [\${RC}].\"
		echo \"DESC:   The zpool ${ZONE_ZPOOL} failed to import cleanly.\"
		echo \"ACTION: Manually determine the cause of the error importing the zpool and then re-run this script.\"
	   	exit 24 
	fi
	else
	echo \"INFO: Zpool ${ZONE_ZPOOL} for zone ${ZONE} is already imported.\"
fi

# now import the exported config from the seperate file
echo \"INFO: Importing zone configuration.\"
cat ${OFILE:%ksh}zonecfg |zonecfg -z $ZONE 
echo \"INFO: attaching the zone [$ZONE].\" 
${ZONE_ATT_CMD}
echo \"INFO: Zone ${ZONE} has been recreated. Please now boot the zone and answer its sysidcfg questions if required.\"
echo \"INFO: [ # zoneadm -z ${ZONE} boot;zlogin -C ${ZONE} ]\" " >> ${OFILE}

###########################################################################
##                              FOOTER                                   ##
## End of generated recovery script.                                     ##
###########################################################################
	#Set execute permisions
	chmod 755 ${OFILE}

	# Now build a seperate zoneconfig script... We build a seperate zone recovery script and a seperate zone config script just to make it more human readable.
	Zone_Config 
	echo "INFO: Recovery script and config script for $ZONE [${OFILE}, ${OFILE:%ksh}zonecfg] has been created."
	# delete old copies of OFILE
	Del_Old_Files ${OFILE}
	Backup_To_Metadata_Dir ${OFILE} 
	echo "INFO: ************ Finished processing zone ${ZONE} ************"
done
}

Zone_Config ()
{
if [ "${METADATA}" = "YES" ]
	then
	Export_Config_Zone_Metadata
	else
	Export_Config_Zonecfg
fi
}

Export_Config_Zonecfg ()
{
zonecfg -z ${ZONE} export >${OFILE:%ksh}zonecfg
chmod 644 ${OFILE:%ksh}zonecfg
}

Export_Config_Zone_Metadata ()
{
echo "ERR: Not implemented"
}

Zone_Attach_Cmd ()
{
TYPE="`zonecfg -z testzone export|grep brand=|cut -f2 -d=`"
# Now workout which attach command is the correct command
case TYPE in
	solaris) 	# Native zone
		echo "zoneadm -z $ZONE attach -U"
		;;
	solaris8) #Solaris 8 branded
		echo "zoneadm -z $ZONE attach -U;/usr/lib/brand/${TYPE}/s${TYPE:#solaris}_p2v ${ZONE}"
		;;
	solaris9) #Solaris 8 branded
		echo "zoneadm -z $ZONE attach -U;/usr/lib/brand/${TYPE}/s${TYPE:#solaris}_p2v ${ZONE}"
		;;
	solaris10) #Solaris 8 branded
		echo "zoneadm -z $ZONE attach -U;/usr/lib/brand/${TYPE}/s${TYPE:#solaris}_p2v ${ZONE}"
		;;
	*)	#Catchall.. we should never get here
		echo "ERROR: Unkown zone type [${TYPE}]" 1>&2
		echo "DESC:  We did not extract a valid zone type from the zone xml file " 1>&2
		echo "ERROR: Manually check the source recovery file [${PROG:%ksh}zonecfg] and the live zone configuration" 1>&2
		Unlock
		exit 24
		;;
esac
}

Zone_Zpool ()
{
# Detmine zonepath
ZONEPATH="`zonecfg -z ${ZONE} export|grep zonepath=`"
FILESYSTEM="`df -k ${ZONEPATH:#*=}|grep -v Filesystem|awk '{print $1}'`"
ZPOOL="${FILESYSTEM:%%/*}"
echo "${ZPOOL}"
}
		
Build_LDOM_Recovery_Scripts ()
{
for LDOM in `ldm list|grep -vw NAME|grep -vw primary|awk '{print $1}'`
do
	echo "INFO: ************ Starting processing LDOM ${LDOM} ************"
	# Set the output file for this LDOM
	OFILE="${LOGDIR}/LDOM_recreate_${LDOM}_$DATE.ksh"

###########################################################################
##                                 HEADER                                ##
## Everything between this header and the subsequent footer form part of ##
## the generated recovery script.                                        ##
###########################################################################
	echo "#!/bin/ksh
# First check if the LDOM is running, if so we do not need to do anything
if [ \`ldm list|grep -w ${LDOM}|wc -l\` -ne 0 ]
	then
	echo \"ERROR:  The LDOM [$LDOM] is already running/configured. Unable to restore the configuration for this LDOM.\"
	echo \"DESC:   This script is only for restoring the configuration of an LDOM that has lost its configuration information.\"
	echo \"ACTION: Please check that you have the correct LDOM name for the config you are trying to restore\"
	echo \"        or manually delete the existing LDOM before running this script.\"
	exit 31
fi

# Give a warning and a chance to terminate the script
echo \" ****** WARNING ****** 
# This script was created by $PROG on `date` for LDOM $LDOM on host `uname -n`.
# This script will recreate the LDOM config for a host if the existing config has been deleted or lost.
# It does not restore the actual data of the LDOM filesystems and they should be imported and mounted.
# This script does NOT provide any check to confirm that the LDOM resources (filesystems/IP etc) are not
# in use on another CDOM. You MUST check this manually.
# This script should be run as root.

Type YES to continue?:\"
read CONTINUE
if [ \"\${CONTINUE}\" != \"YES\" ]
	then
	echo \"INFO: Terminating recovery as you did not answer YES ...[Your answer: \${CONTINUE}]\"
	exit 32
fi

DATE=\"\`date +%Y%m%d%H%M%S\`\"
PROG=\"\`basename \$0 \`\"

# now import the exported config from the seperate file
echo \"INFO: configuring the LDOM [$LDOM].\"
ldm add-domain -i ${OFILE:%ksh}list-constraints
echo \"INFO: binding the LDOM [$LDOM].\"
ldm bind-domain ${LDOM}
echo \"INFO: LDOM ${LDOM} has been recreated. Please now start the LDOM [ # ldm start ${LDOM} ]\" " >> ${OFILE}
###########################################################################
##                              FOOTER                                   ##
## End of generated recovery script.                                     ##
###########################################################################
	#Set execute permissions
	chmod 755 ${OFILE}

	# Now build a seperate LDOM config file... We build a seperate LDOM recovery script and a seperate LDOM config script just to make it more human readable.
	LDOM_Config 
	echo "INFO: Recovery script and config script for $LDOM [${OFILE}, ${OFILE:%ksh}list-constraints] has been created."
	# delete old copies of OFILE
	Del_Old_Files ${OFILE}
	Backup_To_Metadata_Dir ${OFILE} 
	echo "INFO: ************ Finished processing LDOM ${LDOM} ************"
done
}

LDOM_Config ()
{
if [ "${METADATA}" = "YES" ]
	then
	Export_Config_LDOM_Metadata
	else
	Export_Config_LDOM
fi
}

Export_Config_LDOM ()
{
ldm list-constraints -x ${LDOM}  > ${OFILE:%ksh}list-constraints
chmod 644 ${OFILE:%ksh}list-constraints
}

Export_Config_LDOM_Metadata ()
{
echo "ERR: Not implemented"
}

Backup_To_Metadata_Dir ()
{
SRC_FILE=$1
THIS_HOST="`uname -n`"
for META_DIR in `ls -1d /var/mnt/virtlibs/* 2>/dev/null`
do
	#Make sure a directory for this hostname exists in the metadata share
	if [ ! -d ${META_DIR}/${OUTDIR}/${THIS_HOST} ]
		then
		mkdir -p ${META_DIR}/${OUTDIR}/${THIS_HOST}
	fi
	#Copy a backup of the file to the Metdata share
	SHORT_SRC_FILE=${SRC_FILE:##*/}
	echo "INFO: Creating backup copy of scripts ${SRC_FILE:##*/} in ${META_DIR}/${OUTDIR}/${THIS_HOST}"
	cp ${SRC_FILE:%ksh}*  ${META_DIR}/${OUTDIR}/${THIS_HOST}/
	#set the permisions, first 6440 for all files we copy, then 755 for the recovery script
	chmod 644 ${META_DIR}/${OUTDIR}/${THIS_HOST}/${SHORT_SRC_FILE:%ksh}*
	chmod 755 ${META_DIR}/${OUTDIR}/${THIS_HOST}/${SHORT_SRC_FILE}
	#Check for old copies of the recovery script so we only keep $MAX_VERSIONS
	Del_Old_Files ${META_DIR}/${OUTDIR}/${THIS_HOST}/${SHORT_SRC_FILE}
done
}

Del_Old_Files ()
{
FILE=$1
#if script is called with MAX_VERSIONS variable set to 0 we keep all versions of the script.
if [ ${MAX_VERSIONS} -ne 0 ]
	then
	# Look in the provided directory for other versions of OFILE ... we only keep MAX_VERSIONS
	FILE_VERSION=1
	ls -td1 ${FILE:%_*}*ksh 2>/dev/null |
	while
		read EVAL_FILE
	do
		if [ ${FILE_VERSION} -gt ${MAX_VERSIONS} ]
			then
			echo "INFO: !!! Deleting old copy of the recovery script files .... [COPY:${FILE_VERSION}]"
			#echo "`ls -1 ${EVAL_FILE} ${EVAL_FILE:%ksh}.zonecfg ${EVAL_FILE:%ksh}list-constraints 2>/dev/null`"
			#Delete the script file and the zoneconfig abd list-constraints file if they exist
			rm -f ${EVAL_FILE} ${EVAL_FILE:%ksh}.zonecfg ${EVAL_FILE:%ksh}list-constraints
		fi
		FILE_VERSION=$((FILE_VERSION + 1 ))
	done
fi
}

Usage ()
{
echo "
USAGE: $PROG [ -l | -v ]
	-l	Creates a rebuild script from local zonecfg and ldm list-constraints command output
		for each zone or LDOM currently configured on this host.
	-v	Prints the version of the program
"
}

#
# main
#

# handle command line options
case $1 in
	-m) # Use metadata canfiguration as source
		METADATA="YES"
		;;
	-l) # Use local zondecfg/ldm configuration as source
		METADATA="NO"
		;;
	-v) # Use local zondecfg/ldm configuration as source
		echo "INFO: ${PROG} is version ${VERSION}"
		exit 0
		;;
	*) # catch everything else and display the usage message
		Usage
		exit 10
		;;
esac

#Check that we are root and a GZ/CDOM
Check_Global_Root
#Grab a lock (stops 2 versions of this program running at one time)
Lock
#Collect zone data and build recovery scripts
Build_Zone_Recovery_Scripts
#Collect LDOM data and build recovery scripts - check if ldmd is running
if [ "`svcs svc:/ldoms/ldmd:default 2>/dev/null |grep -v STATE|awk '{print $1}'`" == "online" ]
	then
	Build_LDOM_Recovery_Scripts 
fi
#Release the lock (stops 2 versions of this program running at one time)
Unlock
