#!/usr/bin/ksh
#
# Create guest-domain.properties and guest-domain-FO.properties and best implementer for the guest
#
# Geoff Shipman,
# Created Aug  1 2014
# Updated Dec 31 2014 - Added more checks if 0 length, if non-existent, if owner.vsc.id incorrect
#
# Updated Jan  6 2015 - Added check guest-domain-FO check and replacement if main guest-domain.properties is OK.
#                       Added logging and check for sol 10/11 for ggrep command
#                       Added best implementer check
#
# Updated Jan 14 2015 - Added check for scn-agent online, check library.xml if missing guests run library-check.sh -f <guest UUiD>
#
# Updated Feb 13 2015 - Added additional log information.  Comparing the ldm guest list against the domain guest list and print both
#						if there is a mis-match.   We also check the updates have been written out to file system if changes are made.  
#
# Updated Mar 24 2015 - Adding output from "bad" meta data directories to help determine when the files were updated.  Added jmxdump checks 
#                       to verify the command ran without error and re-run if if has a problem up to 5 retries before exit from script.    
#                       
# Updated Mar 30 2015 - If difference between the ldm and dm data ensuring that all guests are checked regardless of location.
#						Cleaned up presentation when run as operational plan.
#                       
# Juergen Fleischer,
# Updated Oct 20 2015 - Replaced attribute UserFriendlyName with DomainName to get the names of LDom Guests from Domain Model.
#                       Modified grep pattern when grepping for guest names to ensure unique matches
#
# Geoff Shipman,
# Updated Dec 04 2015 - Increased log readability and fixed bug with log file.   Added check for ImageDir existence
#
# Updated Dec 07 2015 - Added check for agent version to skip checks on guest-domain-FO.properties that has been deprecated on and after 12.3 
#
# Updated Dec 10 2015 - Added 12.3 heartbeat adjustments for guest-domain.properties.  This is needed as there is a possibility the file is found at 0 size
#						while the heartbeat does its update.    The new test checks up to 3 times with a 15 second sleep between to verify at least one 
#						20 second heartbeat boundary.
#                       
#                        
#   
# Exit codes:
#	10 =  No scn-agent SMF service found
#	20 =  scn-agent SMF service is in an offline or disabled state
#	30 =  scn-agent SMF service is in a maintenance state  
#	42 =  jmxdump max tries reached for UUID meta data 
#	44 =  jmxdump max tries reached for CDOM host name
#	46 =  jmxdump max tries reached for UniqueIdentifier data 
#	48 =  jmxdump max tries reached for Host IP meta data 
#	50 =  jmxdump max tries reached for VSI entries
#   60 =  guest meta data dir not able to be recreated
#                
#
# Example guest-domain.properties:
# cat /var/mnt/virtlibs/1381127883049/0e3b03cc-9150-4115-903e-251c884c757e/guest-domain.properties
# #Store the owner during create itself. DO NOT HAND EDIT.
# #Wed May 14 19:26:23 GMT 2014
# owner.hostname=burns
# owner.vsc.id=dd7afdb6-db07-4eb7-8fcb-088487bd1191
#
# Example guest-domain-FO.properties:
# #The future owner file. DO NOT HAND EDIT.
# #Mon Dec 22 09:31:39 GMT 2014
# previous.owner.processing=false
# owner.vsc.id=5d3f0465-5a84-46c6-b10b-0c108c904a8a
# 
# Example best implementer file: b123b411-7a25-4545-a314-945bfb23cb7e.bmpl
# #created/updated by xVM Server
# #Mon Dec 22 09:31:28 GMT 2014
# zone_ctrl_id=b123b411-7a25-4545-a314-945bfb23cb7e
# hostname=smithers
# timestamp=1419240688802
# creation_date=22 Dec 2014, 09\:31\:28
# IP=10.165.108.47
#
# Variables
#
AGENTADM=/opt/SUNWxvmoc/bin/agentadm
CP=/usr/bin/cp
CAT=/usr/bin/cat
CUT=/usr/bin/cut
GREP=/usr/bin/grep
EGREP=/usr/bin/egrep
FIND=/usr/bin/find
# for Sol 10 ggrep found in /usr/sfw/bin/
RELEASE=`uname -r`
if [[ $RELEASE = "5.10" ]]
then
	GGREP=/usr/sfw/bin/ggrep
else
	GGREP=/usr/bin/ggrep
fi
AWK=/usr/bin/awk
TEE=/usr/bin/tee
LDM=/usr/sbin/ldm
LS=/usr/bin/ls
JMXDUMP=/opt/sun/xvm/bin/jmxdump
LIBCHK=/var/opt/sun/xvm/OCDoctor/toolbox/library-check.sh
TAIL=/usr/bin/tail
SVCS=/usr/bin/svcs
WC=/usr/bin/wc
LOG=/var/tmp/`hostname`-guest-domain-check-`date '+%yy-%mm-%dd-%HH-%MM-%SS'`.txt
#
# Start pre-checks
#
echo "=== Start pre-checks: " | $TEE -a $LOG
echo "Log run: "`date +'%a %b %e %T %Z %Y'`  2>&1 | $TEE -a $LOG
#
# SMF checks for scn-agent service
#
echo "=== Verify scn-agent service exist: " | $TEE -a $LOG
$SVCS -p scn-agent > /dev/null 2>&1
RESULT="$?"
if [[ $RESULT != 0 ]]
then
	echo "CDOM Host "`hostname`" does not have a scn-agent SMF service."  2>&1 | $TEE -a $LOG
	echo "please verify the scn-agent service and retry."  2>&1 | $TEE -a $LOG
	echo ""  2>&1 | $TEE -a $LOG	
	exit 10
else
	echo "=== Service: scn-agent found " | $TEE -a $LOG
	echo "" | $TEE -a $LOG
fi
echo "=== Verify scn-agent status: " | $TEE -a $LOG
status=`$SVCS -p scn-agent | $EGREP "online|offline|disabled|maintenance" | $AWK '{print $1}'`
if [[ $status == offline || $status == disabled ]]
then
	echo "CDOM host SMF service scn-agent is in "$status" state."  2>&1 | $TEE -a $LOG
	echo ""  2>&1 | $TEE -a $LOG
	exit 20
else
	echo "=== Service: scn-agent is not disabled or offline " | $TEE -a $LOG	
fi
if [[ $status == maintenance ]]
then
	echo "CDOM host SMF service scn-agent is in "$status" state."  2>&1 | $TEE -a $LOG
	echo ""  2>&1 | $TEE -a $LOG
	exit 30
else
	echo "=== Service: scn-agent is online " | $TEE -a $LOG	
	echo "" | $TEE -a $LOG
fi	
#
# Mark start of log file
#
echo "~~~~~~~~~~~~~~~"  | $TEE -a $LOG 
echo "Log run: "`date +'%a %b %e %T %Z %Y'`  2>&1 | $TEE -a $LOG
echo "CDOM host "`hostname`" Ops Center agent is "$status  2>&1 | $TEE -a $LOG
agentVer=`$AGENTADM  -V | $AWK '{print $2}' | $TAIL -1 | cut -c1-4`
echo "The scn-agent version is: "$agentVer | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~"  | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~"  | $TEE -a $LOG
#
# Start collecting meta data information.   
# We need the LdomServerContainer (CDOM Host) UUID (owner.vsc.id) for guest-domain files.
# We use systems hostname output for owner.hostname value.
#
# We need the XVMServer UniqueIdentifier and ApplianceIPAddress for best implementer file.
#
# guest-domain owner.vsc.id value.
# /opt/sun/xvm/bin/jmxdump a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* Uuid
#
echo "===============================================" | $TEE -a $LOG
echo "Collecting the output to create meta data files" | $TEE -a $LOG
echo "===============================================" | $TEE -a $LOG
echo "" | $TEE -a $LOG
#
echo "JMXDump command to generate domain model guest domain Uuid" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$JMXDUMP a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* Uuid > /tmp/uuid.tmp
RESULT="$?"
TRIES=1
MAXTRIES=5
echo " --- Attempt #" $TRIES | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
while [ "$RESULT" != 0 ]
do
	sleep 1
	$JMXDUMP a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* Uuid > /tmp/uuid.tmp
	RESULT="$?"
	let TRIES="$TRIES + 1"
	echo " --- Attempt #" $TRIES | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	if [ $TRIES -gt $MAXTRIES ]
	then
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Max tries reached, jmxdump erred cannot populate meta data files from domain model." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		exit 42
	fi
done
#
# guest-domain owner.hostname value.
# /opt/sun/xvm/bin/jmxdump a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* UserFriendlyName
#
echo "JMXDump command to generate domain model guest domain UserFriendlyName" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$JMXDUMP a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* UserFriendlyName > /tmp/cdomhost.tmp
RESULT="$?"
TRIES=1
MAXTRIES=5
echo " --- Attempt #" $TRIES | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
while [ "$RESULT" != 0 ]
do
	sleep 1
	$JMXDUMP a t root wellknown com.sun.hss.domain:type=LDomServerContainer,name=* UserFriendlyName > /tmp/cdomhost.tmp
	RESULT="$?"
	let TRIES="$TRIES + 1"
	echo " --- Attempt #" $TRIES | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	if [ $TRIES -gt $MAXTRIES ]
	then
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Max tries reached, jmxdump erred cannot populate meta data files from domain model." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		exit 44
	fi
done	
#
# Assign variables for file creation.
#
LSCUuid=`$CAT /tmp/uuid.tmp | $GREP Uuid | $AWK '{print $3}' | $TAIL -1`
LSCHostname=`$CAT /tmp/cdomhost.tmp | $GREP UserFriendlyName | $AWK '{print $3}' | $TAIL -1`
LSCDate=`date +'%a %b %e %T %Z %Y'`
# To be used to set the best implementer time stamp.
echo $LSCDate > /var/tmp/time
LSCGuestDomainProperties=/var/tmp/$LSCHostname.guest-domain.properties
if [ $agentVer -ge "12.3" ]
then
	echo "The scn-agent version is equal to or greater than the 12.3  OC release." | $TEE -a $LOG
	echo "Skipping guest-domain-FO.properties checks as this file has been deprecated." | $TEE -a $LOG
	echo "" | $TEE -a $LOG
else
	LSCGuestDomainFOProperties=/var/tmp/$LSCHostname.guest-domain-FO.properties
fi
#
# Best implementer UniqueIdentifier ApplianceName ApplianceNameIPAddress values.
# /opt/sun/xvm/bin/jmxdump a t root wellknown com.sun.hss.domain:type=xVMServer,name=* UniqueIdentifier ApplianceName ApplianceNameIPAddress
# Where <UUID> is the content of /etc/opt/sun/oc/zonectrlr.id on the owner host.
#
echo "JMXDump command to generate domain model best implementer Unique Identifier" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$JMXDUMP a t root wellknown com.sun.hss.domain:type=xVMServer,name=* UniqueIdentifier > /tmp/unique.tmp
RESULT="$?"
TRIES=1
MAXTRIES=5
echo " --- Attempt #" $TRIES | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
while [ "$RESULT" != 0 ]
do
	sleep 1
	$JMXDUMP a t root wellknown com.sun.hss.domain:type=xVMServer,name=* UniqueIdentifier > /tmp/unique.tmp
	RESULT="$?"
	let TRIES="$TRIES + 1"
	echo " --- Attempt #" $TRIES | $TEE -a $LOG
	if [ $TRIES -gt $MAXTRIES ]
	then
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Max tries reached, jmxdump erred cannot populate meta data files from domain model." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		exit 46
	fi
done
XVMSUniqueIdentifier=`$CAT /tmp/unique.tmp | $GREP UniqueIdentifier | $GREP -v "\[" | $AWK '{print $3}'`
XVMSApplianceName=`hostname`
XVMSCreationDate=`date +'%e %b %Y, %H\:%M\:%S'`
echo "JMXDump command to generate domain model best implementer IP" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$JMXDUMP a t root wellknown com.sun.hss.domain:type=xVMServer,name=* ApplianceNameIPAddress > /tmp/appip.tmp
RESULT="$?"
TRIES=1
MAXTRIES=5
echo " --- Attempt #" $TRIES | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
while [ "$RESULT" != 0 ]
do
	sleep 1
	$JMXDUMP a t root wellknown com.sun.hss.domain:type=xVMServer,name=* ApplianceNameIPAddress > /tmp/appip.tmp
	RESULT="$?"
	let TRIES="$TRIES + 1"
	echo " --- Attempt #" $TRIES | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	if [ $TRIES -gt $MAXTRIES ]
	then
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Max tries reached, jmxdump erred cannot populate meta data files from domain model." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		exit 48
	fi
done
#
echo "" | $TEE -a $LOG
echo "===============================================" | $TEE -a $LOG
echo " View the output of the built meta data files  " | $TEE -a $LOG
echo "===============================================" | $TEE -a $LOG
echo "" | $TEE -a $LOG
#
XVMSApplianceNameIPAddress=`$CAT /tmp/appip.tmp | $GREP  ApplianceNameIPAddress | $GREP -v "\[" | $AWK '{print $3}'` 
XVMServerBestImplemeter=/var/tmp/$XVMSApplianceName.$XVMSUniqueIdentifier.bmpl
# Sol 11 no longer uses the time() system call 
temp=`/usr/bin/truss -vall ls -l /var/tmp/time 2>&1 | $GREP  "at =" | tail -1 | $AWK '{print $9}'`
XVMServerTimeStamp=`echo 'scale = 0;'$temp' * 1000/1' | bc`
#
# Show example of guest-domain*.properties and create file in /var/tmp
#
echo "Example guest-domain.properties created from CDOM's domain model." | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "#Store the owner during create itself. DO NOT HAND EDIT. GS" | $TEE $LSCGuestDomainProperties | $TEE -a $LOG
echo "#"$LSCDate | $TEE -a $LSCGuestDomainProperties | $TEE -a $LOG
echo "owner.hostname="$LSCHostname | $TEE -a $LSCGuestDomainProperties | $TEE -a $LOG
echo "owner.vsc.id="$LSCUuid | $TEE -a $LSCGuestDomainProperties | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
#
if [ $agentVer -ge "12.3" ]
then
	echo "The scn-agent version is equal to or greater than the 12.3  OC release." | $TEE -a $LOG
	echo "Skipping guest-domain-FO.properties checks as this file has been deprecated." | $TEE -a $LOG
	echo "" | $TEE -a $LOG
else
	echo "Example guest-domain-FO.properties created from CDOM's domain model." | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "#The future owner file. DO NOT HAND EDIT. GS" | $TEE $LSCGuestDomainFOProperties | $TEE -a $LOG
	echo "#"$LSCDate | $TEE -a $LSCGuestDomainFOProperties | $TEE -a $LOG
	echo "previous.owner.processing=false" | $TEE -a $LSCGuestDomainFOProperties | $TEE -a $LOG
	echo "owner.vsc.id="$LSCUuid | $TEE -a $LSCGuestDomainFOProperties | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
fi
#
# Show example of the best implementer file and create in /var/tmp 
#
echo "Example best implementer file created from CDOM's domain model." | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "#created/updated by xVM Server GS" | $TEE $XVMServerBestImplemeter | $TEE -a $LOG
echo "#"$LSCDate | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "zone_ctrl_id="$XVMSUniqueIdentifier | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "hostname="$XVMSApplianceName | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "timestamp="$XVMServerTimeStamp | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "creation_date="$XVMSCreationDate | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "IP="$XVMSApplianceNameIPAddress | $TEE -a $XVMServerBestImplemeter | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
#
echo "" | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo "Starting verification, generating list of guests" | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo "" | $TEE -a $LOG
#
#
# Generate list of VirtServerImage (LDOM Guests) entries in domain model.   We are looking for guest name and path to meta data. 
#
# /opt/sun/xvm/bin/jmxdump a t root wellknown com.sun.hss.domain:type=VirtServerImage,name=* UserFriendlyName ImageDir VirtPoolName
#
echo "JMXDump command to generate domain model guest list" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$JMXDUMP a t root wellknown com.sun.hss.domain:type=VirtServerImage,name=* DomainName ImageDir VirtPoolName > /tmp/guest.tmp
RESULT="$?"
TRIES=1
MAXTRIES=5
echo " --- Attempt #" $TRIES | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
while [ "$RESULT" != 0 ]
do
	sleep 1
	$JMXDUMP a t root wellknown com.sun.hss.domain:type=VirtServerImage,name=* DomainName ImageDir VirtPoolName > /tmp/guest.tmp
	RESULT="$?"
	let TRIES="$TRIES + 1"
	echo " --- Attempt #" $TRIES | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	if [ $TRIES -gt $MAXTRIES ]
	then
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Max tries reached, jmxdump erred cannot determine how many guests from domain model." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		break
		exit 50
	fi
done
#
# Verify number of VSI entries vs LDM entries.  If more VSI than LDM the additional VSI are on the list to be checked.
# If more LDM than VSI then the extra LDM entries not included in the default list.   Additional work is needed to make
# entries for these missing LDM guests.
#
# Generate master guest list to include ldm guests not listed in DM
#
$CAT /tmp/guest.tmp | $EGREP "DomainName|ImageDir" | $EGREP -v "Attributes"  > /tmp/guestlist-vsi.tmp
GUESTVSI=`$GREP DomainName /tmp/guestlist-vsi.tmp | $WC -l`
$LDM list | $EGREP -v "NAME|primary|inactive" > /tmp/guestlist-ldm.tmp 
GUESTLDM=`$EGREP -v "NAME|primary|inactive" /tmp/guestlist-ldm.tmp | $WC -l`
echo "Number of domain model VirtServerImages (LDOM guests): " $GUESTVSI | $TEE -a $LOG
echo "Number of ldm guest instances: " $GUESTLDM | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
if [[ ${GUESTVSI} = ${GUESTLDM} ]]
then
	echo "Number of guests match between domain model and ldm output." | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG	
	$CP /tmp/guestlist-vsi.tmp /tmp/guestlist-master.tmp
else
	echo "Warning:  Number of guests in domain model and ldm command do not match." | $TEE -a $LOG
	echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	echo "Output from the comparison: VSI entries followed by ldm entries."
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	$CAT /tmp/guestlist-vsi.tmp | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	$CAT /tmp/guestlist-ldm.tmp | $TEE -a $LOG
	echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	if [[ $GUESTVSI > $GUESTLDM ]]
	then
		echo "More Domain Model entries than LDM entries, using Domain Model guest list."  | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		$CP /tmp/guestlist-vsi.tmp /tmp/guestlist-master.tmp
	else
		echo "More LDM entries than Domain Model entries, merging Domain Model guest list with extra LDM guest entries."  | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		$CP /tmp/guestlist-vsi.tmp /tmp/guestlist-master.tmp
		#
		# Two possible meta data locations NFS share or local /guests
		#
		$FIND /var/mnt/virtlibs/ -name "servercfg.xml" -print >> /tmp/guestdirs.tmp 2> /dev/null
		# find /var/mnt/virtlibs/ -name "servercfg.xml" -print | awk -F/ '{print "/"$2"/"$3"/"$4"/"$5"/"$6}'
		# /var/mnt/virtlibs/1410912505142/3582590c-8d8e-43b3-82ff-ec464a9bd421
		$FIND /guests/ -name "servercfg.xml" -print >> /tmp/guestdirs.tmp 2> /dev/null
		# find /guests -name "servercfg.xml" -print | awk -F/ '{print "/"$2"/"$3"/"}'
		# /guests/dbb74cf2-7249-4e7e-bb94-e01a1420dba8/		
		for guestname in `$LDM list | $EGREP -v "NAME|primary|inactive" | $AWK '{print $1}'`
		do
			$GREP "= ${guestname}\$" /tmp/guestlist-vsi.tmp
			FOUND="$?"
			if [[ $FOUND != 0 ]] 
			then
				echo "UserFriendlyName = " $guestname >> /tmp/guestlist-master.tmp
				for metadir in `cat /tmp/guestdirs.tmp`
				do
					metaname=`echo $metadir | awk -F/ '{print $2}'`
					if [[ $metaname == "guests" ]]
					then
						guestdir=`$GREP -l $guestname $metadir | $CUT -c 1-44`
					else
						guestdir=`$GREP -l $guestname $metadir | $CUT -c 1-68`	
					fi
				done
				echo "ImageDir = " $guestdir >> /tmp/guestlist-master.tmp
			fi
		done
	fi
fi
#
echo "" | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo "   Starting verification, guest list created    " | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo "" | $TEE -a $LOG
#
# 
# Start main loop
#
for guest in `$GREP -v Attributes /tmp/guestlist-master.tmp | $GREP DomainName | $AWK '{print $3}'`
do
	echo "" | $TEE -a $LOG
	echo "===============" | $TEE -a $LOG
	echo "Guest DomainName: " $guest | $TEE -a $LOG
	echo "===============" | $TEE -a $LOG
	$LDM ls $guest > /dev/null
	FOUND="$?"
	if [[ $FOUND == 0 ]] 
	then
		ImageDir=`$GGREP -A1 $guest /tmp/guestlist-master.tmp | $TAIL -1 | $AWK '{print $3}'`
		guestDir=`basename $ImageDir`
		echo $ImageDir | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG	
		#
		# Does guest-domain.properties exist
		#
		# From bash(1) man page under CONDITIONAL EXPRESSIONS
		#
		#     -s file
        #      True if file exists and has a size greater than zero.
		#
		#
		echo "" | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "        Verifying meta directory exists."         | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "" | $TEE -a $LOG
		$LS -ld $ImageDir > /dev/null 2>&1
		FOUND="$?"
		if [[ $FOUND > 0 ]]
		then 
			echo "Meta directory " $ImageDir " not found." | $TEE -a $LOG
			echo "attempting to create " $ImageDir  | $TEE -a $LOG
			$MKDIR -p $ImageDir > /dev/null 2>&1
			$LS -ld $ImageDir > /dev/null 2>&1
			FOUND="$?"
			if [[ $FOUND > 0 ]]
			then
				echo "Unable to create " $ImageDir | $TEE -a $LOG
				exit 60
			fi
			echo "The "$ImageDir " was recreated." | $TEE -a $LOG
		else
		echo "" | $TEE -a $LOG
		echo "The "$ImageDir " was found." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		fi		
		echo "" | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "    Checking the guest-domain.properties file   " | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "" | $TEE -a $LOG
		#		
		if [[ -s $ImageDir/guest-domain.properties ]]
		then
		#
		# The guest-domain.properties exists
		#
			echo "guest-domain.properties exists, verifying entries." | $TEE -a $LOG
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			#
			# Checking for valid CDOM host UUID
			#
			$GREP $LSCUuid $ImageDir/guest-domain.properties > /dev/null
			FOUND="$?"
			if [[ $FOUND > 0 ]]
			then 
				echo "The CDOM host UUID " $LSCUuid " does not match the entry in guest-domain.properties.  Updating file to match." | $TEE -a $LOG
				echo "copying " $LSCGuestDomainProperties " to " $ImageDir/guest-domain.properties | $TEE -a $LOG
				echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
				$LS -laEc $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~ cat contents of meta data ~~~~~~~~~~" | $TEE -a $LOG
				$CAT $ImageDir/guest-domain.properties | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				$CAT $ImageDir/guest-domain-FO.properties | $TEE -a $LOG				
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				echo "copying " $LSCGuestDomainProperties " to " $ImageDir/guest-domain.properties | $TEE -a $LOG
				cp $LSCGuestDomainProperties $ImageDir/guest-domain.properties
				if [ $agentVer -ge "12.3" ]
				then
					echo "Skipping guest-domain-FO.properties."
				else											
					cp $LSCGuestDomainFOProperties $ImageDir/guest-domain-FO.properties
				fi
				$LS -laE $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				#
				# Check to verify updated file was copied properly.
				#				
				$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
				if [ $agentVer -ge "12.3" ]
				then
					echo "Skipping guest-domain-FO.properties."
				else											
					$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties > /dev/null
				fi
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then
					echo "Warning: script found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
					echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
				fi
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			fi
			#
			# Checking for valid CDOM hostname
			#
			$GREP $LSCHostname $ImageDir/guest-domain.properties > /dev/null
			FOUND="$?"
			if [[ $FOUND > 0 ]]
			then
				echo "The CDOM hostname " $LSCHostname " does not match the entry in guest-domain.properties.  Updating file to match." | $TEE -a $LOG
				echo "Collecting date from the meta data directory " | $TEE -a $LOG
				echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
				$LS -laEc $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~ cat contents of meta data ~~~~~~~~~~" | $TEE -a $LOG
				$CAT $ImageDir/guest-domain.properties | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				$CAT $ImageDir/guest-domain-FO.properties | $TEE -a $LOG				
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				echo "copying " $LSCGuestDomainProperties " to " $ImageDir/guest-domain.properties | $TEE -a $LOG				
				cp $LSCGuestDomainProperties $ImageDir/guest-domain.properties
				if [ $agentVer -ge "12.3" ]
				then
					echo "Skipping guest-domain-FO.properties."
				else							
					cp $LSCGuestDomainFOProperties $ImageDir/guest-domain-FO.properties
				fi
				$LS -laE $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				#
				# Check to verify updated file was copied properly.
				#
				$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
				if [ $agentVer -ge "12.3" ]
				then
					echo "Skipping guest-domain-FO.properties."
				else							
					$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties > /dev/null
				fi
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then
					echo "Warning: Found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
					echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
				fi				
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			fi
		else
			#
			# The guest-domain.properties does not exist,  We replace the missing guest-domain file and update guest-domain-FO file.
			#
			#	
			RESULT=`$LS -s $ImageDir/guest-domain.properties | $AWK '{print $1}'`
			TRIES=1
			MAXTRIES=3
			echo " --- Attempt #" $TRIES | $TEE -a $LOG
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			while [ "$RESULT" = 1 ]
			do
				sleep 15
				RESULT=`$LS -s $ImageDir/guest-domain.properties | $AWK '{print $1}'`
				let TRIES="$TRIES + 1"
				echo " --- Attempt #" $TRIES | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				if [ $TRIES -gt $MAXTRIES ]
				then
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					echo "Max tries reached, guest-domain.properties is 0 length." | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG		
					echo "guest-domain.properties not found for " $guest | $TEE -a $LOG
					echo "Collecting date from the meta data directory " | $TEE -a $LOG
					echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
					$LS -laEc $ImageDir/ | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					echo "copying " $LSCGuestDomainProperties " to " $ImageDir/guest-domain.properties | $TEE -a $LOG
					cp $LSCGuestDomainProperties $ImageDir/guest-domain.properties
					if [ $agentVer -ge "12.3" ]
					then
						echo "Skipping guest-domain-FO.properties."
					else
						cp $LSCGuestDomainFOProperties $ImageDir/guest-domain-FO.properties
					fi
					$LS -laE $ImageDir/ | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					#
					# Check to verify updated file was copied properly.
					#			
					$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
					if [ $agentVer -ge "12.3" ]
					then
						echo "Skipping guest-domain-FO.properties."
					else			
						$GREP GS $ImageDir/guest-domain.properties $ImageDir/guest-domain-FO.properties > /dev/null
					fi
					FOUND="$?"
					if [[ $FOUND > 0 ]]
					then
						echo "Warning: script found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
						echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
					fi			
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				else
					echo "" | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					echo "OC Agent heartbeat rebuilt guest-domain.properties file." | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				fi
			done
		fi
		#
		# Checks for the guest-domain-FO.properties file
		#
		if [ $agentVer -ge "12.3" ]
		then
			echo "The scn-agent version is equal to or greater than the 12.3  OC release." | $TEE -a $LOG
			echo "Skipping guest-domain-FO.properties checks as this file has been deprecated." | $TEE -a $LOG
			echo "" | $TEE -a $LOG
		else
			echo "" | $TEE -a $LOG
			echo "================================================" | $TEE -a $LOG
			echo "   Checking the guest-domain-FO.properties file " | $TEE -a $LOG
			echo "================================================" | $TEE -a $LOG
			echo "" | $TEE -a $LOG
			#		
			if [[ -s $ImageDir/guest-domain-FO.properties ]]
			then
			#
			# The guest-domain-FO.properties file exists.
			#
				echo "guest-domain-FO.properties exists, verifying entries." | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				# 
				# Check for valid CDOM host UUID
				#
				$GREP $LSCUuid $ImageDir/guest-domain-FO.properties > /dev/null
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then 
					echo "The CDOM host UUID " $LSCUuid " does not match the entry in guest-domain-FO.properties.  Updating file to match." | $TEE -a $LOG
					echo "Collecting date from the meta data directory " | $TEE -a $LOG
					echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
					$LS -laEc $ImageDir/ | $TEE -a $LOG
					echo "~~~~~~~~~~~ cat contents of meta data ~~~~~~~~~~" | $TEE -a $LOG
					$CAT $ImageDir/guest-domain-FO.properties | $TEE -a $LOG				
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					echo "copying " $LSCGuestDomainFOProperties " to " $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
					cp $LSCGuestDomainFOProperties $ImageDir/guest-domain-FO.properties
					$LS -laE $ImageDir/ | $TEE -a $LOG
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					#
					# Check to verify updated file was copied properly.
					#
					$GREP GS $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
					$GREP GS $ImageDir/guest-domain-FO.properties > /dev/null
					FOUND="$?"
					if [[ $FOUND > 0 ]]
					then
						echo "Warning: Found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
						echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
					fi				
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				fi
			else
				#
				# The guest-domain-FO.properties does not exist
				#
				echo "guest-domain-FO.properties not found for " $guest | $TEE -a $LOG
				echo "Collecting date from the meta data directory " | $TEE -a $LOG
				echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
				$LS -laEc $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				echo "copying " $LSCGuestDomainFOProperties " to " $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
				cp $LSCGuestDomainFOProperties $ImageDir/guest-domain-FO.properties
				$LS -laE $ImageDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				#
				# Check to verify updated file was copied properly.
				#
				$GREP GS $ImageDir/guest-domain-FO.properties | $TEE -a $LOG
				$GREP GS $ImageDir/guest-domain-FO.properties > /dev/null
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then
					echo "Warning: script found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
					echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
				fi			
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			fi
		fi
		#
		# Best implementer checks
		#
		echo "" | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "        Checking the best implementer file      " | $TEE -a $LOG
		echo "================================================" | $TEE -a $LOG
		echo "" | $TEE -a $LOG
		#
		if [[ -s $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl ]]
		then
		#
		# The best implementer file exists
		#
			echo "The best implementer file exists.  Verifying quantity and contents." | $TEE -a $LOG
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			totalFiles=$(ls ${ImageDir}/../bestimpl/${guestDir}/ 2>/dev/null | wc -l)
			if [ ${totalFiles} -ne 1 ]
			then
			#
			# More than 1 file exists in this directory
			#
				echo "More than 1 file found resetting ..." | $TEE -a $LOG
				echo "Collecting date data from the meta data" | $TEE -a $LOG
				echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
				$LS -laEc $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
				echo "~~~~~~~~~~~ cat comments from meta data ~~~~~~~~~~" | $TEE -a $LOG
				$CAT $ImageDir/../bestimpl/$guestDir/*.bmpl | $TEE -a $LOG
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				rm -rf $ImageDir/../bestimpl/$guestDir/* > /dev/null 2>&1
				cp $XVMServerBestImplemeter $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl
				$LS -laE $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
				#
				# Check to verify updated file was copied properly.
				#
				$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl | $TEE -a $LOG
				$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl > /dev/null
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then
					echo "Warning: script found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
					echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
				fi	
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			else
			#
			# Verifying Unique Identifier in best implementer file
			#
				echo "Checking contents ..." | $TEE -a $LOG				
				echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
				$GREP $XVMSUniqueIdentifier $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl > /dev/null
				FOUND="$?"
				if [[ $FOUND > 0 ]]
				then
					echo "The CDOM host UniqueIdentifier "$XVMSUniqueIdentifier" does not match the entry in "$XVMSUniqueIdentifier".bmpl resetting ..." | $TEE -a $LOG
					echo "Collecting date data from the meta data" | $TEE -a $LOG
					echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
					$LS -laEc $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
					echo "~~~~~~~~~~~ cat comments from meta data ~~~~~~~~~~" | $TEE -a $LOG
					$CAT $ImageDir/../bestimpl/$guestDir/*.bmpl | $TEE -a $LOG					
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					rm -rf $ImageDir/../bestimpl/$guestDir/* > /dev/null 2>&1
					cp $XVMServerBestImplemeter $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl
					$LS -laE $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
					#
					# Check to verify updated file was copied properly.
					#					
					$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl | $TEE -a $LOG
					$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl > /dev/null
					FOUND="$?"
					if [[ $FOUND > 0 ]]
					then
						echo "Warning: Found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
						echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
					fi
					echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
					echo "" | $TEE -a $LOG
				fi
			fi
		else
		#
		# The best implementer file does not exist
		#
			echo "The best implementer file does not exist.  resetting ..." | $TEE -a $LOG
			echo "Collecting date data from the meta data" | $TEE -a $LOG
			echo "~~~~~~~~~~~ ls -laEc output of directory ~~~~~~~~~~" | $TEE -a $LOG
			$LS -laEc $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
			echo "~~~~~~~~~~~ grep comments from meta data ~~~~~~~~~~" | $TEE -a $LOG
			$GREP "^#" $ImageDir/../bestimpl/$guestDir/*.bmpl | $TEE -a $LOG			
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			rm $ImageDir/../bestimpl/$guestDir/* > /dev/null 2>&1
			cp $XVMServerBestImplemeter $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl
			$LS -laE $ImageDir/../bestimpl/$guestDir/ | $TEE -a $LOG
			#
			# Check to verify updated file was copied properly.
			#			
			$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl | $TEE -a $LOG
			$GREP GS $ImageDir/../bestimpl/$guestDir/$XVMSUniqueIdentifier.bmpl > /dev/null
			FOUND="$?"
			if [[ $FOUND > 0 ]]
			then
				echo "Warning: Found problem and tried updating files but check of replaced files failed" | $TEE -a $LOG
				echo "Please review and verify, and contact support to assist." | $TEE -a $LOG
			fi
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			echo "" | $TEE -a $LOG
		fi
		#
		# Library.xml check,  While were looking at all the meta data check other elements OCDoctor provides fixes for.
		#
		$GREP $guestDir $ImageDir/../library.xml > /dev/null
		FOUND="$?"
		if [[ $FOUND > 0 ]]
		then 
			echo "Guest "$guest" was not found in the library.xml." | $TEE -a $LOG
			echo "Running library-check.sh -f "$ImageDir/.." to correct." | $TEE -a $LOG
			echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
			$LIBCHK -f $ImageDir/..
		fi
	else
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
		echo "Guest in shutdown state or not found in ldm list active state." | $TEE -a $LOG
		echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
	fi
done
# Clean up temp files
rm /tmp/guestlist-vsi.tmp
rm /tmp/guestlist-ldm.tmp
rm /tmp/guestlist-master.tmp
rm /tmp/guestdirs.tmp 2> /dev/null
rm /tmp/uuid.tmp
rm /tmp/cdomhost.tmp
rm /tmp/unique.tmp
rm /tmp/appip.tmp
rm /tmp/guest.tmp
rm /var/tmp/time
#
# Verify things look OK
#
echo "" | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo " End of verification, running check script      " | $TEE -a $LOG
echo "================================================" | $TEE -a $LOG
echo "" | $TEE -a $LOG
#
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
echo "Running OCDoctor library-check.sh -x to verify no complaints about meta data." | $TEE -a $LOG
echo "~~~~~~~~~~~~~~~" | $TEE -a $LOG
$LIBCHK -x | $TEE -a $LOG