#!/bin/sh
#
# install_agent_zone
#
# Post-Action script to install agents in brownfield zones
#
# Copyright 2010 Oracle, Inc. All rights reserved.
# Use is subject to license terms.
#
# Author: Jan.Ornstedt@oracle.com
#
# Version 1.0 Jan.Ornstedt@oracle.com
#

# the next file must be copied as Local Config File to 
# the agent together with this Post-Action script
# All agent zip files may be found on the EC in this folder: /var/opt/sun/xvm/images/agent/
AGENTZIP=/var/tmp/oc-agent.zip

# upgrade log files
LOGFILE=`mktemp -p /var/tmp agentlog.XXXXXX`

Log(){
  # copy log entry to logfile and stdout
  printf "$* \n\n" | tee -a $LOGFILE
}

Log "Starting to install agents in Zones ..."

if [ ! -f $AGENTZIP ]
then
  Log "Config File $AGENTZIP not found, exiting!"
  exit 1
fi

UNZIP=`which unzip`
if [ ! -x "$UNZIP" ]
then
  Log "Required program 'unzip' not found, exiting!"
  exit 1
fi

# check zone status
GLOBAL=true
NONGLOBAL=true
ZONEADM='/usr/sbin/zoneadm'
if [ ! -x "$ZONEADM" ]
then
  Log "Error: This server does not support local zones!"
  exit 1
else
  $ZONEADM list | grep global >/dev/null 2>&1 || GLOBAL=false
  $ZONEADM list | grep -v global >/dev/null 2>&1 || NONGLOBAL=false
  if [ "$GLOBAL" = "true" ]
  then
    if [ "$NONGLOBAL" = "false" ]
    then
      Log "Error: No local zones found!"
      exit 1
    fi
  else
    Log "Error: This script should be run in the Global zone not the local zone!"
    exit 1
  fi
fi

# Find zones
ZONES=`$ZONEADM list | grep -v global`

# Params
AGENTADM=/opt/SUNWxvmoc/bin/agentadm
CACAOADM=/usr/sbin/cacaoadm
PROPERTIES=/var/opt/sun/xvm/persistence/scn-agent/connection.properties

if [ ! -f $PROPERTIES ]
then
	Log "Error: Agent software is not installed in this globalzone. This error should not be possible!"
	exit 1
fi

PROXY_IP=`grep '^url=' $PROPERTIES | cut -d/ -f3 | cut -d\\\ -f1`
TOKEN=`grep '^auto-reg-token=' $PROPERTIES | cut -d= -f2 | sed 's/\\\//g'`

# temporary directory
TEMPDIR=/var/tmp/.zone_agent_install_$$
mkdir -m 700 $TEMPDIR

# Create Token fil for authentication
TOKENFILE=$TEMPDIR/token
echo $TOKEN > $TOKENFILE

# unpack xVM agent
#cd $TEMPDIR
$UNZIP -d $TEMPDIR -q $AGENTZIP
if [ $? -ne 0 ]
then
  Log "Could not unzip $AGENTZIP, exiting !"
  exit 1
fi

# Loop through all zones
for i in $ZONES
do
# Find Zonepath
  ZONEPATH=`zonecfg -z $i export | grep path | cut -d= -f2`

# Check if there is any previous config
  CFG=0
  if [ -f "$ZONEPATH/root/$PROPERTIES" ]
  then
    CFG=`grep -cv '^#' "$ZONEPATH/root/$PROPERTIES"`
  fi

# If no zone config exists start configure
  if [ $CFG -eq 0 ]
  then
  	  Log "Will install agent in Zone $i"
  
# Create lofs mountpoint
	  ZONETEMP=$ZONEPATH/root/$TEMPDIR
	  mkdir -m 700 $ZONETEMP
	  mount -F lofs -r $TEMPDIR $ZONETEMP
	  
# Install Agent
	  zlogin -S $i $TEMPDIR/SunConnectionAgent/install -a >> $LOGFILE 2>&1
	  
# Configure Agent
      zlogin -S $i "echo y | $AGENTADM configure -t $TOKENFILE -x $PROXY_IP" >> $LOGFILE 2>&1
      
# Verify agent connection
      CONNECTION=`zlogin -S $i "/usr/bin/sc-console list-connections | grep scn-agent | wc -l"`
      if [ $CONNECTION -eq 0 ]
      then
      	Log "Agent configuration failed! Please login to the global zone and check the logfile: $LOGFILE"
      else
        Log "Agent configuration successful!"
      fi
  
# Clean up lofs mounts
	  umount $ZONETEMP
	  zlogin -S $i rmdir $TEMPDIR
  else
      Log "Zone $i already has previous agent configuration. Skipping"
  fi
done


# Finshed
Log "Zone install finished successfully. Will remove temporary files."

# Remove temp files
rm -rf $TEMPDIR

