#!/bin/bash
# set -x
#
# library-check.sh [ -l library_directory ] [ -f library_directory ] [ -b ] [ -x ]
#
# Author: Juergen.Fleischer@oracle.com
#
#
# Version 3.5 2015/12/16
# 
# Change Log:
#
# Version 3.5:
#
# enhanced check for guest-domain.properties content to not throw false alarms in case it is of 0 size
#
# Version 3.4:
#
# fixed issue caused by SR-IOV devices, they are listed as data links in metadata, but not as vnet 
#
# Version 3.3:
#
# fixed issues caused by hardcoded patterns dsk/c0t500 & dsk/c0t600
#
# Version 3.2:
#
# fixed a false alarm in case of vnics with multiple VLAN IDs
#
# Version 3.1:
#
# fixed a false alarm in case of brownfield vnics named other than 'vnic*'
# added support of zfs volumes as vdisk device to vdisk config check
# added check for virtimage.xml files of size 0 bytes
#
# Version 3.0:
#
# Major rewrite of disk metadata checks to enable them for 12c R2 and later
#
# Version 2.8:
#
# Ensured that we skip disk checks for 12cR3 as well (due to format changes between 12cR1 vs. 12cR[23]
#
# Version 2.7:
#
# for Delta CDOMs, we are obtaining the VSCID now from the Domain Model
#
# Version 2.6:
#
# Added note to output pointing to Geoff's create-guest script in the toolbox
#
# Version 2.5:
#
# changed bugid 18467924 to 18915595 for missing guest-domain.properties file
# fixed exit code when checking for missing guest-domain.properties file
#
# Version 2.4:
#
# fixed a bug in getting the LUN ID of a guest volume causing false alarms
#
# Version 2.3:
#
# corrected check for mismatching guestUUID and UUID values 
#
# Version 2.2:
#
# checking now if guestUUID and UUID are having the same values 
# checking for a missing guest-domain.properties file (bug 18467924)
#
# Version 2.1:
#
# fixed issues when none or multiple metadata directories were found for a guest
#
# Version 2.0:
# 
# skipping of metadata checks implemented for 12cR2 (Diamond)
# fixed a bug when disks of typ nfs:// or dsl:// were used by guests
# 
# Version 1.11:
# 
# added code to fix an empty library.xml file by querying the domain model for the UUID
# 
# Version 1.10:
# 
# added checks for broken guest-domain.properties files (bug 16935439) and best implementor configs
# rewritten some code to avoid usage of ggrep
# enhanced check for unique best implementor in general library check
# skipping "-x" checks on 12.2.x.x CDOMs for now
# fixed errors due to ${LUN,,} commands which will work only in Solaris 11 bash
#
# Version 1.9:
#
# Using now full path to ggrep and 'tr' to convert from upper to lower case
# Fixed issue when called from Operational Plan
#
# Version 1.8:
#
# Addressed false alarms by listing renamed/moved-away stale directories inside the library
#
# Version 1.7:
#
# No longer checking for unique Best Implementors if type is ISOImage
#
# Version 1.6:
#
# Added much more further stuff to verify that the servercfg.xml content is in sync with the running guests
# 
###############################################################################################################



#
# I have introduced a new variable which can be set to define the behaviour when called from an Operational Plan
#
# OPERATIONAL_PLAN_ACTION
#
#       DEFAULT: Run script like called without arguments
#
#       BESTIMPL: Run script like called with '-b' switch
#
#       SERVERCFG: Run script like called with  '-x' switch
#
# to disable this feature, comment out the variable (which is the default). 

# OPERATIONAL_PLAN_ACTION=SERVERCFG

# enable additional DEBUG output by setting variable DEBUG to "true"
DEBUG=false
#DEBUG=true

print_usage() {
cat << EOF

        Usage: library-check.sh [-l library_directory] [ -f library_directory ] [ -b ] [ -x ]

        Without arguments, library-check.sh will scan for local and NFS libraries and
        perform some basic healthchecks on them.

        It will highlight missing or extra virtual images which are not listed in library.xml.
        Output for a healthy library will look like:

        Summary of Library /var/mnt/virtlibs/1334148911654/:
        Library name: LDOM Metadata
        library.xml contains 5 VirtImages elements
        total sum of image subdirs: 5
        subdirs with a virtimage.xml file: 5
        bestimpl subdirs: 5
        Further details of observed findings:

        ------------------------------------------------------------------------------------

        Use the switch '-l' to analyze just a single library:

        library-check.sh -l /var/opt/sun/xvm/locallib/swlib0

        ------------------------------------------------------------------------------------

        In case of a broken 'library.xml' file, you can use '-f' to add all missing
        virtual images to it:

        library-check.sh -f /var/mnt/virtlibs/1345360849060
        Fixing library.xml in Library /var/mnt/virtlibs/1345360849060.

        library.xml file has been fixed, original file saved in /var/mnt/virtlibs/1345360849060/library.xml_saved_2012-08-24_16-48-06.
        
        ------------------------------------------------------------------------------------

        To check for non-unique Best Implementors for a virtual image use the switch '-b':

        library-check.sh -b                                
        OK: Best Implementors in Libraries are all unique
 
        ------------------------------------------------------------------------------------

        To check the consistency of the servercfg.xml config file use the switch '-x':

        library-check.sh -x                                
        Checking consistency in configurations of 'active' LDom Guests...

        Checking guest guest-01 ...
        OK: Number of used vdisks is in sync with listed disks in config file
        OK: Number of used vnets is in sync with listed NICs in config file

        Checking guest guest-34 ...
        OK: Number of used vdisks is in sync with listed disks in config file
        OK: Number of used vnets is in sync with listed NICs in config file

 
EOF
exit 0
}

EXIT_CODE=0
SERVERCFG=false
BESTIMPL=false
FIX=false
PARSE_ARGS=true
JMXDUMP=/opt/sun/xvm/bin/jmxdump

LIBS=/tmp/_our_libraries.$$
ls -d /guests/ /var/mnt/virtlibs/[0-9]*/ /var/opt/sun/xvm/locallib/swlib*/ 2>/dev/null > ${LIBS}

# check if we should parse command line switches
if [ -n "${OC_JOB_ID}" ]; then
    # script is called from an operational profile, check if we defined an Action
    [ -n "${OPERATIONAL_PLAN_ACTION}" ] && PARSE_ARGS=false
fi

if $PARSE_ARGS; then
    while getopts xbl:f: option 2>/dev/null
    do
        case $option in
            x)      SERVERCFG=true;;
            b)      BESTIMPL=true;;
            l)      ( cd $OPTARG 2>/dev/null; pwd ) > ${LIBS};;
            f)      FIX=true; LIB=$( cd $OPTARG 2>/dev/null; pwd );;
            ?)      print_usage;;
        esac
    done
    shift $(($OPTIND - 1))
else
    # script got called from Operational Plan
    [ "${OPERATIONAL_PLAN_ACTION}" == "SERVERCFG" ] && SERVERCFG=true
    [ "${OPERATIONAL_PLAN_ACTION}" == "BESTIMPL" ] && BESTIMPL=true
fi

fix_library_xml() {

    echo "Fixing library.xml in Library ${LIB}."
    echo

    cd $LIB

    if [ ! -f library.xml ]; then
        echo "ERROR: File not found: ${LIB}/library.xml"
        exit 1
    fi

    if [ ! -s library.xml ]; then
        # looks like we stepped into bug 17577507
        # we will try to get the Library UUID from the domain model and will use it to recreate the file

        echo "File library.xml is of length zero, we will query the Agent's Domain Model now to obtain the Library UUID..."

        LIBRARY_TMP=/tmp/_lib_tmp.$$

        ON_NFS_PATH="^ObjectName = .*$(df -h ${LIB} | tail -1 | nawk '{print $1}'| sed 's/:/.*/')"

        $JMXDUMP a t root wellknown 'com.sun.hss.domain:type=VirtImageLibrary,*' CorrelationMap 2>/dev/null > $LIBRARY_TMP

        LINE_NO=$(egrep -n "$ON_NFS_PATH" $LIBRARY_TMP | cut -d: -f1)
        LIBRARY_UUID=$(tail +${LINE_NO} $LIBRARY_TMP | head -7 | nawk ' /VirtImageUUID/ {print substr($8,1,36)}') 
        
        rm -f $LIBRARY_TMP

        if [ -z "$LIBRARY_UUID" ]; then
            printf "\nWARNING: File ${LIB}/library.xml is of length zero and the script is unable to recreate it!\n\n"
            printf "Please contact Ops Center Support!\n\n"
            exit 10
        fi
        cat > ${LIB}/library.xml <<EOF
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<LibraryData>
    <IsReadOnly>false</IsReadOnly>
    <UserFriendlyName>Reconstructed NFS Library</UserFriendlyName>
    <Description>created from scratch by library-check.sh</Description>
    <CreationTime>1384421951087</CreationTime>
    <ModificationTime>1384421951087</ModificationTime>
    <SemanticTags/>
    <UUID>${LIBRARY_UUID}</UUID>
</LibraryData>
EOF

        # ok, we created an empty library.xml file with the correct UUID
        printf "\nCreated an empty skeleton version of library.xml with original UUID, now adding content to it... \n\n"
    fi

    LIBRARY_HEAD=/tmp/_lib_head.$$
    LIBRARY_NEW_CONTENT=/tmp/_lib_new_content.$$
    LIBRARY_TAIL=/tmp/_lib_tail.$$

    if grep '</VirtImagesInLib>' library.xml >/dev/null; then
        # both <VirtImagesInLib> ... </VirtImagesInLib> elements are present
        LINES=$(awk '$1 ~ /<\/VirtImagesInLib>/ {print NR}' library.xml)
        head -$(( LINES - 1 )) library.xml > $LIBRARY_HEAD
        tail +$LINES library.xml > $LIBRARY_TAIL
    elif grep '<VirtImagesInLib/>' library.xml >/dev/null; then
        # single element <VirtImagesInLib/> is present
        LINES=$(awk '$1 ~ /<ModificationTime>/ {print NR}' library.xml)
        head -$LINES library.xml > $LIBRARY_HEAD
        echo "    <VirtImagesInLib>" >> $LIBRARY_HEAD
        echo "    </VirtImagesInLib>" > $LIBRARY_TAIL
        tail +$(( LINES + 2 )) library.xml >> $LIBRARY_TAIL
    else
        # fresh library.xml file without any <VirtImagesInLib* element
        LINES=$(awk '$1 ~ /<ModificationTime>/ {print NR}' library.xml)
        head -$LINES library.xml > $LIBRARY_HEAD
        echo "    <VirtImagesInLib>" >> $LIBRARY_HEAD
        echo "    </VirtImagesInLib>" > $LIBRARY_TAIL
        tail +$(( LINES + 1 )) library.xml >> $LIBRARY_TAIL
    fi

    for i in $(ls *-*-*-*-*/virtimage.xml 2>/dev/null | cut -d/ -f1)
    do
        if ! grep $i library.xml >/dev/null
        then
            echo "        <VirtImageInLib>"    >> $LIBRARY_NEW_CONTENT
            echo "            <Name>$i</Name>" >> $LIBRARY_NEW_CONTENT
            echo "        </VirtImageInLib>"   >> $LIBRARY_NEW_CONTENT
        fi
    done

    if [ -f $LIBRARY_NEW_CONTENT ]; then
        cat $LIBRARY_HEAD $LIBRARY_NEW_CONTENT $LIBRARY_TAIL > library.xml.updated
        ORIG=library.xml_saved_$(date '+%Y-%m-%d_%H-%M-%S')
        cp library.xml $ORIG
        mv library.xml.updated library.xml
        echo "library.xml file has been fixed, original file saved in ${LIB}/${ORIG}."
    else
        echo "library.xml does already contain all virtual images, nothing to do."
    fi

    rm -f $LIBRARY_HEAD $LIBRARY_NEW_CONTENT $LIBRARY_TAIL
        
}

check_bestimpl() {

    #
    # Verify unique best implementors

    check_passed=true

    for LIB in $(cat $LIBS 2>/dev/null)
    do
        for image in $(ls -d ${LIB}/bestimpl/*-*-*-*-* 2>/dev/null)
        do
            # skip any further check if image is of type ISOImage
            virtimage="${LIB}/$(basename $image)/virtimage.xml"
            if [ -f "${virtimage}" ]; then
                grep 'type="ISOImage"' ${virtimage} > /dev/null && continue
            fi
            COUNT=$(ls ${image}/*.bmpl 2>/dev/null | wc -l)
            if [ $COUNT -ne 1 ]; then
                UUID=$(basename $image)
                #
                # check if this image is  still listed in library.xml
                if  grep $UUID ${LIB}/library.xml >/dev/null; then
                    check_passed=false
                    LIB_NAME="$(grep '<UserFriendlyName>' ${LIB}/library.xml | sed 's/^.*<UserFriendlyName>//' | sed 's/<\/.*$//' )"
                    if [ "$COUNT" -eq 0 ]; then
                        echo "ERROR: There is no owner host listed for the following virtual image:"
                        echo
                        echo "       Library Path: $LIB"
                        echo "       Library Name: $LIB_NAME"
                        echo "       Image UUID  : $UUID"
                        echo "       Image Name  : $(grep '<UserFriendlyName>' $LIB/$UUID/virtimage.xml | sed 's/^.*<UserFriendlyName>//' | sed 's/<\/.*$//' )"
                        echo
                        echo "Required Action: Determine the correct owner host and create the following file (can be empty):"
                        echo
                        echo "${LIB}bestimpl/<UUID>.bmpl"
                        echo
                        echo "where <UUID> is the content of /etc/opt/sun/oc/zonectrlr.id on the owner host."
                        echo
                    else
                        echo "ERROR: There is more than one config file in the following directory:"
                        echo
                        echo "# ls -l ${LIB}bestimpl/$UUID"
                        ls -l ${LIB}bestimpl/$UUID
                        echo
                        echo "This issue could trigger various failures during Zone or LDOM Guest jobs (CR 7189364)."
                        echo
                        echo "Required Action: "
                        echo
                        echo "Determine the correct config file, backup and then delete all other file(s)! "
                        echo "Please, open a Service Request if the issue cannot be fixed in a straight forward way!"
                        echo
                        echo "---------------------------------------------------------------------------------"
                        echo
                        echo "The following info could be helpful for identifying the correct config file."
                        echo
                        echo "Affected Virtual Image of the zone/guest and its location:"
                        echo
                        echo "NFS Library Path: $LIB"
                        echo "    Library Name: $LIB_NAME"
                        echo "      Image UUID: $UUID"
                        echo "      Image Name: $(grep '<UserFriendlyName>' $LIB/$UUID/virtimage.xml | sed 's/^.*<UserFriendlyName>//' | sed 's/<\/.*$//' )"
                        echo
                        echo "The correct config file will contain info about the current owner of this zone/guest:"
                        echo
                        echo "Content of above config files:"
                        for bestimpl in $(ls ${LIB}bestimpl/$UUID/*.bmpl)
                        do
                            echo
                            echo "File: $bestimpl"
                            echo "Owner: $(grep ^hostname= $bestimpl|cut -d= -f2)"
                            echo "Creation Date: $(head -2 $bestimpl |tail -1)"
                        done
                        
                        ASSOC_UUIDS=$(grep -l $UUID /var/mnt/virtlibs/*/*/*.xml | cut -d/ -f6 | grep -v $UUID | sort -u)
                        if [ -n "$ASSOC_UUIDS" ]; then
                            echo
                            echo "---------------------------------------------------------------------------------"
                            echo 
                            echo "Additional good info:" 
                            echo 
                            echo "Further Virtual Images have been found which are related to the image above. By design,"
                            echo "all related Virtual Images must have the very same owner (config filename), and will"
                            echo "likely point to the correct config file."
                            echo 
                            echo "Name(s) of the config file(s) found in related Virtual Images:"
                            
                            for assoc_uuid in $ASSOC_UUIDS
                            do
                                echo
                                ls /var/mnt/virtlibs/*/bestimpl/$assoc_uuid
                            done
                            echo
                        fi
                        echo
                        echo "Please, open a Service Request if still in doubt which file(s) to remove!"
                        echo
                        echo "---------------------------------------------------------------------------------"
                    fi
                fi
            fi
        done
    done
    
    $check_passed && echo "OK: Best Implementors in Libraries are all unique"
    
}

check_lib() {

    echo

    for LIB in $(cat ${LIBS} )
    do
        PRINT_DETAILS=false
        echo Summary of Library ${LIB}:

        if [ ! -f ${LIB}/library.xml ]; then
            printf "\nINFO: Directory does not contain a library.xml file, it is probably not used as an Ops Center Library ...\n\n"
            echo '+++++++++++++++++++'
            echo
            continue
        fi
                
        if [ -s ${LIB}/library.xml ]; then
            echo Library name: "$(grep '<UserFriendlyName>' ${LIB}/library.xml | sed 's/^.*<UserFriendlyName>//' | sed 's/<\/.*$//' )"
            
            IMAGES_IN_XML=$(grep '<Name>' ${LIB}/library.xml|wc -l)
            echo library.xml contains $IMAGES_IN_XML VirtImages elements
            
            SUBDIRS=$(ls -d ${LIB}/????????-????-????-????-????????????/ 2>/dev/null|wc -l)
            echo total sum of image subdirs: $SUBDIRS

            VIRTIMAGES=$(ls ${LIB}/????????-????-????-????-????????????/virtimage.xml 2>/dev/null|wc -l)
            echo subdirs with a virtimage.xml file: $VIRTIMAGES
            
            BESTIMPL_SUBDIRS=$(ls -d ${LIB}/bestimpl/????????-????-????-????-???????????? 2>/dev/null|wc -l)
            echo bestimpl subdirs: $BESTIMPL_SUBDIRS
            
            if [ $IMAGES_IN_XML -eq $VIRTIMAGES -a $IMAGES_IN_XML -eq $BESTIMPL_SUBDIRS ]; then
                echo "OK: All virtual images are listed in the library.xml file and have an associated best implementor directory."
                echo
            else
                EXIT_CODE=1
                PRINT_DETAILS=true
            fi 

            # check for unique best implementors but skip VSIs of type ISOImage
            ISO_IMAGES=/tmp/iso_images.$$
            touch $ISO_IMAGES
            NON_UNIQUE_BEST_IMPL=false
            
            grep -l 'type="ISOImage"' ${LIB}/????????-????-????-????-????????????/virtimage.xml 2>/dev/null > $ISO_IMAGES
            ls ${LIB}/????????-????-????-????-????????????/virtimage.xml 2>/dev/null | while read VSI
            do
                if grep $VSI $ISO_IMAGES >/dev/null; then
                    continue;
                fi
                GUID=$(basename $(dirname $VSI) )
                [ $(ls ${LIB}/bestimpl/${GUID} | wc -l) -ne 1 ] && NON_UNIQUE_BEST_IMPL=true
            done
            rm $ISO_IMAGES
            

            if [ $IMAGES_IN_XML -lt $VIRTIMAGES ]; then
                echo "CAUTION: library.xml does not contain all existing virtual images!"
                echo 
                echo "Use the provided info further down to determine if the identified directories"
                echo "contain obsoleted data. In this case remove or rename these directories to avoid false alarms."
                echo
                echo "In case of true missing images, you can use '$0 -f ${LIB}' to add them to library.xml!"
                echo
            fi

	    # check for virtimage.xml files of size 0, this can happen in case of NFS-Server issues
	    for i in $(grep '<Name>' ${LIB}/library.xml | sed 's/^.*<Name>//' | sed 's/<.*$//')
	    do
		if [ ! -s "${LIB}/${i}/virtimage.xml" ]; then
		    echo
		    echo "ERRROR: File ${LIB}/${i}/virtimage.xml is of size 0 Bytes!" 
		    echo "        This could have been caused by NFS-Server outages/issues."
		    echo "        Use an existing good virtimage.xml file as template to manually recreate"
		    echo "        the broken virtimage.xml file or open a SR to get this resolved."
		    echo
		    EXIT_CODE=1
		fi
	    done

            if $NON_UNIQUE_BEST_IMPL; then
                EXIT_CODE=1
                echo "ATTENTION: non-unique best implementors found!"
                echo "Use '$0 -b' to obtain more details on this issue."
                echo
            fi

            if $PRINT_DETAILS; then
                echo
                echo Further details of observed findings:
            fi
            for i in $(grep '<Name>' ${LIB}/library.xml 2>/dev/null|cut -c19-54)
            do
                [ -f ${LIB}/${i}/virtimage.xml ] || echo image ${i} does not exist !
                [ -d ${LIB}/bestimpl/${i} ] || echo no bestimpl entry for image ${i} !
            done
            for i in $(cd ${LIB}; ls ????????-????-????-????-????????????/virtimage.xml 2>/dev/null | cut -d/ -f1)
            do
                grep $i ${LIB}/library.xml >/dev/null || echo image ${i} is not listed in library.xml !
                [ -d ${LIB}/bestimpl/${i} ] || echo no bestimpl entry for image ${i} !
            done
            for i in $(cd ${LIB}/bestimpl 2>/dev/null && ls -d ????????-????-????-????-???????????? 2>/dev/null)
            do
                if ! grep $i ${LIB}/library.xml >/dev/null
                then
                    echo image ${i} is listed in bestimpl with owner host $(grep '^hostname=' ${LIB}/bestimpl/${i}/*|cut -d= -f2), but it is not listed in library.xml !
                    #FOUND=$(ls -d $(dirname $LIB)/*/* $(dirname $LIB)/* | grep -v $LIB | grep ${i})
                    #[ -n "${FOUND}" ] && echo found image in other Library: ${FOUND}
                fi
            done
            for i in $(cd ${LIB}; ls -d ????????-????-????-????-????????????/ 2>/dev/null)
            do
                [ -f ${LIB}/${i}/virtimage.xml ] || echo "WARNING: image ${i} does not contain a file virtimage.xml, looks to be a leftover"
            done
            echo '+++++++++++++++++++'
            echo
        else
            printf "\nERROR: library.xml file is empty, please use '$0 -f ${LIB}' to fix it!\n\n"
            echo '+++++++++++++++++++'
            echo
        fi
    done
}

check_servercfg() {
    # check in detail the metadata of running LDom Guests to detect any kind
    # of corruption and/or out of sync with the running Guest configuration
    #
    # To do:
    # - check all metadata found, not just from running guests
    # - do the same check for metadata of running zones
    #

    VIRTINFO=/usr/sbin/virtinfo
    LDM=/usr/sbin/ldm
    LDM_lp="$LDM ls -l -p"
    LDM_LS="$LDM ls-services"
    LDM_LS_DISK="$LDM ls -o disk"

    TMPDIR=/tmp/library-check.$$
    mkdir $TMPDIR
    LDM_LS_TMPOUT=$TMPDIR/ldm_ls_out
    LDM_LS_DISK_TMPOUT=$TMPDIR/ldm_ls_disk_out
    LDOM_VDS=$TMPDIR/ldom_vds_out
    FILE_W_LUNS=$TMPDIR/file_w_luns
    FILE_W_LUNS_MPGROUP=$TMPDIR/file_w_luns_mpgroup
    FILE_W_LUNS_MPGROUP_TMP=$TMPDIR/file_w_luns_mpgroup_tmp
    FILE_W_LUNS_MPGROUP_CONFIG=$TMPDIR/file_w_luns_mpgroup_config
    FILE_W_LUNS_MPGROUP_CONFIG_TMP=$TMPDIR/file_w_luns_mpgroup_config_tmp
    FILE_W_LUNS_CONFIG=$TMPDIR/file_w_luns_config
    FILE_W_LUNS_CONFIG_S1=$TMPDIR/file_w_luns_config_s1
    FILE_W_LUNS_CONFIG_S2=$TMPDIR/file_w_luns_config_s2
    FILE_W_LUNS_CONFIG_S3=$TMPDIR/file_w_luns_config_s3
    FILE_W_LUNS_CONFIG_S4=$TMPDIR/file_w_luns_config_s4
    FILE_W_LUNS_LIBS_CONFIG=$TMPDIR/file_w_luns_libs_config
    FILE_W_LUNS_SPECIAL=$TMPDIR/file_w_luns_special
    FILE_W_LUNS_IDS=$TMPDIR/file_w_luns_ids
    FILE_W_LUNS_IDS_TMP=$TMPDIR/file_w_luns_ids_tmp
    FILE_W_LOCALDEV_LUN_MAPPING=$TMPDIR/file_w_localdev_lun_mapping
    VIRTIMG_OF_UUID=$TMPDIR/virtimg_of_UUID 

    if [ -x $VIRTINFO -a -x $LDM ]; then
        DOMAINROLE=$($VIRTINFO -pt)
        # check if this system is indeed a CDOM
        if echo "$DOMAINROLE" | grep 'control=true' > /dev/null; then
            echo "Checking consistency in configurations of 'active' LDom Guests on host $(hostname) ... "

            # get a listing of all local devices and their associated LUN IDs
            #
            # we will parse the output of diskinfo for that.
            # 

            /opt/sun/n1gc/bin/xvmserver/diskinfo | egrep 'name : |guid : ' | while read NAME
            do
                # expected output of grep cmd:
                #        name : "c1t0d0",
                #        guid : "600605b0042274e0ff000325302a7d68",
                #        name : "c1t1d0",
                #        guid : "600605b0042274e0ff00032630332830",

                read GUID
                echo "/dev/dsk/$(echo $NAME | awk -F\" '{print $2}')s2 $(echo $GUID | awk -F\" '{print $2}')"
            done > $FILE_W_LOCALDEV_LUN_MAPPING

            if $DEBUG; then
                printf "\nContent of FILE_W_LOCALDEV_LUN_MAPPING\n"
                cat $FILE_W_LOCALDEV_LUN_MAPPING
                printf "\n------------------------------------\n"
                #
                # content of $FILE_W_LOCALDEV_LUN_MAPPING
                #
                # /dev/dsk/c1t0d0s2 600605b0042274e0ff000325302a7d68
                # /dev/dsk/c1t1d0s2 600605b0042274e0ff00032630332830
                #
                #
                #
                # Comment: On T-Series systems with MPxIO enabled, the internal disks are named c0t... as well
                #
                # By default, diskinfo does not present any output. After creating a file
                #
                # cat /opt/sun/n1gc/etc/diskinfo.properties
                # local_disk_controllers = c0
                #
                # diskinfo is seeing them, so that they can get enabled and used.
                #
                # But the GUID output presented by diskinfo looks different:
                #
                # /dev/dsk/c0t5000CCA0163EACE4d0s2 5000cca0163eace4
                # /dev/dsk/c0t5000CCA016154CECd0s2 5000cca016154cec
                # /dev/dsk/c0t5000CCA016373FC0d0s2 5000cca016373fc0
                # /dev/dsk/c0t5000CCA016451730d0s2 5000cca016451730
                #
                # In the above case, we do not get the 32 long GUID of the disk

            fi


            # need to determine the VSCID of this Control Domain, this has changed in Diamond.
            # In Delta, we are using the UUID of the primary Domain, in Diamond we have to query the Domain Model
            #
            # Note: Geoff Shipman pointed me to a Delta config where the UUID of the primary domain was not used 
            # as VSCID in the domain model. Since v2.7, we are handling Delta CDOMs the same like Diamond ones
            JMXDUMP="/opt/sun/xvm/bin/jmxdump a t root wellknown"
            MBEAN="com.sun.hss.domain:type=LDomServerContainer,name=NORM-localhost-global"

            VSCID=$($JMXDUMP $MBEAN Uuid 2>/dev/null | grep ^Uuid | cut -c8-)
            
            ZONECTRLID="$(cat /etc/opt/sun/oc/zonectrlr.id 2>/dev/null)"

            LIST_OF_GUESTS="$($LDM ls | grep -vi inactive | tail +3 | nawk '{print $1}')"
            if [ -z "${LIST_OF_GUESTS}" ]; then
                printf "\nNo running LDom Guests found, exiting ...\n\n"
                exit
            fi

            # we will store all attached/serviced disks in this format
            #
            # SVS-DOM VOL-NAME@VDS-NAME DEV-PATH MPGROUP
            #
            # primary guest-01-vol0@primary-vds0 /dev/dsk/c0t5000CCA016451730d0s2
            # primary ha-guest-06-vol0@primary-vds0 /dev/rdsk/c0t600144F0BCC705920000542AB2FB0015d0s2 ha-guest-06
            # primary ha-guest-12-vol0@primary-vds0 /dev/rdsk/c0t600144F0BCC705920000538610D80002d0s2 ha-guest-12
            # ...
            # secondary ha-guest-06-vol0@secondary-vds0 /dev/rdsk/c0t600144F0BCC705920000542AB2FB0015d0s2 ha-guest-06
            # secondary ha-guest-12-vol0@secondary-vds0 /dev/rdsk/c0t600144F0BCC705920000538610D80002d0s2 ha-guest-12
            #
            $LDM ls-services -p | nawk -F \| '/^VDS\|/ {print substr($3,6) " " substr($2,6)}' | while read SVC_DOM VDS_NAME
            do
                $LDM ls -o disk -p $SVC_DOM | nawk -F \| '/^\|vol=/ {print $2 "=" $4 "=" $5}' | nawk -F = '{print $2 " " $4 " " $6 }' | while read VOLNAME DEVPATH MPGROUP
                do
                    echo "$SVC_DOM ${VOLNAME}@${VDS_NAME} $DEVPATH $MPGROUP" >> $LDOM_VDS
                done    
            done
            if $DEBUG; then
                printf "\nContent of LDOM_VDS\n"
                cat $LDOM_VDS
                printf "\n------------------------------------\n"
            fi

            for GUEST in $LIST_OF_GUESTS
            do
                printf "\n+++++++++++++++++++\n\nChecking guest $GUEST ...\n\n"

                # these files will get re-used for every guest, so ensure they are empty in the beginning
                cp /dev/null $FILE_W_LUNS
                cp /dev/null $FILE_W_LUNS_CONFIG
                cp /dev/null $FILE_W_LUNS_CONFIG_S1
                cp /dev/null $FILE_W_LUNS_CONFIG_S2
                cp /dev/null $FILE_W_LUNS_CONFIG_S3
                cp /dev/null $FILE_W_LUNS_CONFIG_S4
                cp /dev/null $FILE_W_LUNS_IDS_TMP
                cp /dev/null $FILE_W_LUNS_IDS
                cp /dev/null $FILE_W_LUNS_LIBS_CONFIG
                cp /dev/null $FILE_W_LUNS_MPGROUP
                cp /dev/null $FILE_W_LUNS_MPGROUP_TMP
                cp /dev/null $FILE_W_LUNS_MPGROUP_CONFIG
                cp /dev/null $FILE_W_LUNS_MPGROUP_CONFIG_TMP
                cp /dev/null $LDM_LS_TMPOUT
                cp /dev/null $LDM_LS_DISK_TMPOUT
                cp /dev/null $FILE_W_LUNS_SPECIAL

                # now we want to find the correct VirtImage of guest ${GUEST} in the metadata libraries
                # we will use the Guest UUID for it. Any LDom Guest has a unique UUID and OpsCenter is
                # using it as name of the directory containing the metadata of the guest
                #
                # But things can be more complex due to library corruption, brownfield guests, etc.:
                # - no metadata found in any library for a UUID of a guest 
                # - multiple metadata directories found for the same UUID
                # 

                UUID=$($LDM_lp $GUEST | grep ^UUID | cut -d= -f2)
                GUEST_UUID=$($LDM_lp $GUEST | grep '^\|guestUUID=' | cut -d= -f2)
                if [ -z "$GUEST_UUID" ]; then
                    echo "ERROR: Guest does not have an OBP variable 'guestUUID'!"
                    echo "       Use 'ldm set-var ...' to define a variable 'guestUUID=$UUID'"
                    echo
                    # setting the value to the one from $UUID
                    GUEST_UUID=$UUID
                    EXIT_CODE=1
                fi

                if [ -z "$UUID" -a -n "$GUEST_UUID" ]; then
                    UUID=$GUEST_UUID
                fi

                if [ -z "$UUID" ]; then
                    echo "ERROR: Unable to determine the UUID of guest ${GUEST} using the output of 'ldm ls -l ${GUEST}'"   
                    EXIT_CODE=1
                    continue
                fi

                if [ "$UUID" != "$GUEST_UUID" ]; then
                    LDOM_SERVER_VERSION=$($LDM -V 2>/dev/null | nawk ' /Logical Domains Manager/ {print substr($5,1,1) substr($5,3,1)}')
                    if [ -n "$LDOM_SERVER_VERSION" -a $LDOM_SERVER_VERSION -gt 21 ]; then
                        echo "ERROR: OBP Variable guestUUID has a different value than the true Guest UUID!"
                        echo
                        echo "guestUUID=$GUEST_UUID"
                        echo "     UUID=$UUID"
                        echo
                        EXIT_CODE=1
                    fi
                fi

                ls /var/mnt/virtlibs/*/${UUID}/virtimage.xml /guests/${UUID}/virtimage.xml 2>/dev/null > $VIRTIMG_OF_UUID
                NO_OF_VIRTIMG_UUID=$(cat $VIRTIMG_OF_UUID | wc -l)

                if [ $NO_OF_VIRTIMG_UUID -eq 1 ]; then
                    # this should be the standard situation (hopefully)
                    VIRTIMG=$(cat $VIRTIMG_OF_UUID)
                elif [ $NO_OF_VIRTIMG_UUID -eq 0 ]; then
                    # let's check if we can find the metadata by using the guestUUID
                    ls /var/mnt/virtlibs/*/${GUEST_UUID}/virtimage.xml /guests/${GUEST_UUID}/virtimage.xml 2>/dev/null > $VIRTIMG_OF_UUID
                    NO_OF_VIRTIMG_UUID=$(cat $VIRTIMG_OF_UUID | wc -l)
                    if [ $NO_OF_VIRTIMG_UUID -eq 1 ]; then
                        VIRTIMG=$(cat $VIRTIMG_OF_UUID)
                    else
                        echo
                        echo "ERROR: Unable to find the directory with the metadata of guest ${GUEST} in any Library!!!"
                        echo
                        echo "This issue must get fixed to ensure a correct working Agent!"
                        echo
                        EXIT_CODE=1
                        continue
                    fi
                else
                    echo
                    echo "ERROR: Found multiple directories with the metadata for guest ${GUEST}:"
                    echo
                    cat $VIRTIMG_OF_UUID
                    echo
                    echo "This issue must get fixed to ensure a correct working Agent!"
                    echo
                    EXIT_CODE=1
                    continue 
                fi

                
                # when we finally have reached this point the following parameters are set
                #
                # GUEST - guest name
                #
                # VIRTIMG - full path to virtimage.xml file of guest $GUEST
                #

                VSI_PATH=$(dirname $VIRTIMG)
                CONFIG=${VSI_PATH}/servercfg.xml
                GUEST_UUID=$(basename $VSI_PATH)
		LIBRARY_XML=$(dirname $VSI_PATH)/library.xml

                echo "Path to Metadata: $CONFIG"
                echo

		# check if the VSI is listed in library.xml
		if grep "<Name>${GUEST_UUID}</Name>" $LIBRARY_XML >/dev/null 2>&1; then
		    echo "OK: UUID of Guest VSI is listed properly in library.xml file: ${GUEST}"
		else
		    echo
		    echo "ERROR: UUID of Guest VSI is not listed in library.xml file of metadata library: ${GUEST}"
		    echo "Path to library.xml: $LIBRARY_XML"
		    echo "Path to Guest VSI: $VSI_PATH"
		    echo "Guest UUID: $GUEST_UUID"
		    echo
		    echo "Run $0 without arguments to further analyze this issue."
		    echo "Run $0 -f $(dirname $LIBRARY_XML) to gedd the missing VSI UUIDs added to library.xml"
		    echo
		fi    


                # echo "CONFIG = $CONFIG"

                # checking now guest-domain.properties and best implementor files

                # path to best implementor directory and file
                BEST_IMPL_DIR="$(dirname $VSI_PATH)/bestimpl/$GUEST_UUID"
                BEST_IMPL_FILE="${BEST_IMPL_DIR}/${ZONECTRLID}.bmpl"

                # paths to guest-domain.properties and equivalent "Future Owner" file
                GDP=$(dirname $CONFIG)/guest-domain.properties
                GDPFO=$(dirname $CONFIG)/guest-domain-FO.properties

                # checking if the file guest-domain.properties contains the ID of the Control Domain, where the script is running

                CONFIG_IS_BROKEN=false
                if [ -f $GDP ]; then
	            # file guest-domain.properties gets regularly updated with the latest heartbeat timestamp
		    # inbetween the updates there is a short window where the file is of size 0
		    for i in 1 2 3 4 5
		    do
			if [ ! -s $GDP ]; then
				sleep 1
			fi
		    done 	
                    if grep "^owner.vsc.id=$VSCID" $GDP > /dev/null; then
                        echo "OK: correct current ownership: $GUEST"
                    else
                        CONFIG_IS_BROKEN=true
                        printf "\nERROR: Critical issue detected !!!\n\n"
                        printf "The following config file does not list this Control Domain as current owner of guest ${GUEST}:\n\n"
                        printf "File: $GDP\n\n"
                        printf "Content:\n\n"
                        cat $GDP
                        printf "\nWARNING: Restarting the Agent of the Control Domain will stop this Guest (it might get even fully destroyed) !!!\n\n"
                        printf "This issue needs to get corrected immediately (Bug 16935439)\n\n"

                        #if ! grep "^owner.vsc.id=$VSCID" $GDPFO > /dev/null; then
                        #    printf "There is a related config file 'guest-domain-FO.properties' which is listing a wrong owner as well:\n\n"
                        #    printf "File: $GDPFO\n\n"
                        #    printf "Content:\n\n"
                        #    cat $GDPFO
                        #    printf "\nThis file should get corrected as well!\n\n"
                        #fi
                        EXIT_CODE=1
                    fi
                else
                    CONFIG_IS_BROKEN=true
                    printf "\nERROR: guest-domain.properties file is missing in metadata directory (bug 18915595)!\n\n"
                    EXIT_CODE=1
                fi

                # checking if the best implementor config is correct
                NO_OF_BESTIMPL=$(ls $BEST_IMPL_DIR 2>/dev/null| wc -l)
                if [ $NO_OF_BESTIMPL -eq 1 -a -f $BEST_IMPL_FILE ]; then
                    echo "OK: Best Implementor configuration is correct: $GUEST"
                else
                    CONFIG_IS_BROKEN=true
                    printf "\nERROR: Best Implementor configuration of this guest is not correct!\n\n"
                    if  [ $NO_OF_BESTIMPL -ne 1 ]; then
                        printf "Directory $BEST_IMPL_DIR contains $NO_OF_BESTIMPL files, but it should have exactly 1 file!\n\n"
                    fi
                    if [ ! -f $BEST_IMPL_FILE ]; then
                        printf "File $BEST_IMPL_FILE does not exist!\n\n"
                    fi
                    EXIT_CODE=1
                fi
                
                if $CONFIG_IS_BROKEN; then
		    echo
                    echo "NOTE: The script '<OCDoctor>/toolbox/create-guest.sh' can be used to fix broken"
                    echo "      guest-domain.properties, guest-domain-FO.properties and Best Implementor files!"
                    echo
                fi
                

                #
                # checking now for correct disk metadata of each Guest
                #
                
                # get all vol names of a guest and save it in $ALL_VOLNAMES
                ALL_VOLNAMES=$($LDM ls -o disk -p ${GUEST} | tr '|' '=' | nawk -F = '/^VDISK/ {print $5}')

                # get the disk index used for each volume
                $LDM ls -o disk -p ${GUEST} | tr '|' '=' | nawk -F = '/^VDISK/ {print $5 " " $15}' > $LDM_LS_DISK_TMPOUT

                if $DEBUG; then
                    printf "\nContent of LDM_LS_DISK_TMPOUT\n"
                    cat $LDM_LS_DISK_TMPOUT
                    printf "\n------------------------------------\n"
                    # example of expected content of $LDM_LS_DISK_TMPOUT: 
                    # guest-01-vol0@primary-vds0 0
                    # guest-01-vol1@primary-vds0 1
                fi
	
		CHECK_DISK_DETAILS=true	
                # get the device path for each volume, in parallel, device path + mpgroup
                for VOLNAME in $ALL_VOLNAMES
                do
                    $DEBUG && echo "VOLNAME = $VOLNAME"
                    PATH_TO_LUN=$(grep " $VOLNAME " $LDOM_VDS | nawk '{print $3}')
                    $DEBUG && echo "PATH_TO_LUN = $PATH_TO_LUN"
		    # need to modify the PATH_TO_LUN to catch both cases /dev/rdsk/... and /dev/dsk/...
		    MODIFIED_PATH_TO_LUN=$(echo $PATH_TO_LUN | sed 's@^.*/r*dsk/@@')
		    $DEBUG && echo "MODIFIED_PATH_TO_LUN = $MODIFIED_PATH_TO_LUN"

		    # collecting MPGROUP info
		    egrep "[ /]$MODIFIED_PATH_TO_LUN" $LDOM_VDS | nawk '{print $2 " " $3 " " $4}' >> $FILE_W_LUNS_MPGROUP_TMP
                    if echo $PATH_TO_LUN | grep /var/mnt/virtlibs > /dev/null; then
                        # disk is a file in a NFS-Library
                        LUN=$(echo $PATH_TO_LUN | cut -d/ -f6)
                    elif grep ^${PATH_TO_LUN} $FILE_W_LOCALDEV_LUN_MAPPING >/dev/null; then
                        # Local Device is used e.g. /dev/dsk/c3t1d0s2
                        LUN=$(grep ^${PATH_TO_LUN} $FILE_W_LOCALDEV_LUN_MAPPING | nawk '{print $2}')
			# if the LUN does get reported with a number like 500* we have an issue
			# OpsCenter will not recognize it as a known disk and will still report it as 'opague'
			if echo $LUN | grep '^500' > /dev/null; then
			    # we add the LUN to file FILE_W_LUNS_SPECIAL so we know that we have to handle them in special way
			    echo $LUN >> $FILE_W_LUNS_SPECIAL
			fi    
                    elif echo $PATH_TO_LUN|grep 'dsk/c.*t500' > /dev/null; then
                        # Local Device is used, but MPxIO is enabled and diskinfo is not seeing the disk
                        LUN=$(echo $PATH_TO_LUN | sed 's@^.*dsk/c.*t@@' | sed 's/d0.*$//')
                        # we add the LUN to file FILE_W_LUNS_SPECIAL so we know that we have to handle them in special way
                        echo $LUN >> $FILE_W_LUNS_SPECIAL
                    elif echo $PATH_TO_LUN|grep '/guests/.*virtdisk/data' >/dev/null; then
                        # Local Storage library disk is used, e.g. /guests/d6a37edd-b6b7-4300-9a4f-e3884bf6f616/virtdisk/data
                        LUN=$(echo $PATH_TO_LUN|awk -F/ '{print $3}')
                    elif echo $PATH_TO_LUN|grep 'dsk/c.*t600' > /dev/null; then
			# beforehand we checked only for pathes with 'dsk/c0t600', no I found a T2000 system where the FC-LUNs were named like 'dsk/c4t600'
                        # get the LUN ID in lower case
                        # from '/dev/rdsk/c0t600144F08D544600000050F9954E0006d0s2' to '600144f08d544600000050f9954e0006'
                        LUN=$(echo $PATH_TO_LUN | sed 's/^.*t//' | cut -c1-32 | tr '[A-Z]' '[a-z]' )
                    else
                        # unsupported device path found, probably /dev/zvol/dsk/....
			# we will skip just one disk check
			CHECK_DISK_DETAILS=false
                        LUN=$PATH_TO_LUN
                    fi
                    $DEBUG && echo "LUN = $LUN"
                    echo ${LUN} >> $FILE_W_LUNS
                    # get the device id of this disk
                    ID=$(grep "^$VOLNAME " $LDM_LS_DISK_TMPOUT | nawk '{print $2}')
                    $DEBUG && echo "ID = $ID"
                    echo "${LUN} $ID"  >> $FILE_W_LUNS_IDS_TMP
                done
                sort -k 2,2n $FILE_W_LUNS_IDS_TMP > $FILE_W_LUNS_IDS
		sort -k 2,2n $FILE_W_LUNS_MPGROUP_TMP > $FILE_W_LUNS_MPGROUP
                
		
                if $DEBUG; then
		    printf "\nContent of FILE_W_LUNS\n"
		    cat $FILE_W_LUNS
		    printf "\nContent of FILE_W_LUNS_MPGROUP\n"
		    cat $FILE_W_LUNS_MPGROUP
		    printf "\nContent of FILE_W_LUNS_IDS\n"
		    cat $FILE_W_LUNS_IDS
		    printf "\n------------------------------------\n"
                fi
                
                # we check now if a LUN is connected multiple times to a guest
                if [ "$(cat $FILE_W_LUNS | wc -l)" -eq "$(sort -u $FILE_W_LUNS | wc -l)" ]; then
		    echo "OK: Attached LUNs are all unique: $GUEST"
                else
		    echo "ERROR: found LUNs which are attached multiple times to guest ${GUEST}:"
		    NON_UNIQUES_LUNS="$(sort $FILE_W_LUNS | uniq -c | grep -v ' 1 '| awk '{print $2}')"
		    for LUN in ${NON_UNIQUES_LUNS}
		    do
                        grep -i $LUN $LDM_LS_TMPOUT
		    done
		    EXIT_CODE=1
                fi

                # for each vdisk, get the LUN ID and its Library as stored in the config file 
                grep '<vdisk>' $CONFIG | while read LINE
                do
		    $DEBUG && echo "LINE=$LINE"
		    LIB_OF_LUN=$(echo "$LINE" | cut -d, -f1 | cut -d\[ -f2 )
		    LUN=$(echo "$LINE" | cut -d, -f2 | cut -d\] -f1 | tr '[A-Z]' '[a-z]' )
		    echo $LUN >> $FILE_W_LUNS_CONFIG
		    echo $LUN $LIB_OF_LUN >> $FILE_W_LUNS_LIBS_CONFIG
                done
                if $DEBUG; then
		    printf "\nContent of FILE_W_LUNS_LIBS_CONFIG\n"
		    cat $FILE_W_LUNS_LIBS_CONFIG
		    printf "\n------------------------------------\n"
                fi

                
                egrep '^            <source>|^            <idx>' $CONFIG | while read LINE
                do
		    read INDEX_LINE
		    LUN=$(echo "$LINE" | cut -d, -f2 | cut -d\] -f1 )
		    echo ${LUN} | tr '[A-Z]' '[a-z]' >> $FILE_W_LUNS_CONFIG_S1
		    INDEX=$(echo "$INDEX_LINE" | sed 's/^.*<idx>//' |sed 's/<.*$//')
		    echo "${LUN} ${INDEX}" | tr '[A-Z]' '[a-z]' >> $FILE_W_LUNS_CONFIG_S2
		    echo ${INDEX} >> $FILE_W_LUNS_CONFIG_S4
                done
                sort -k 2,2n $FILE_W_LUNS_CONFIG_S2 > $FILE_W_LUNS_CONFIG_S3

                if $DEBUG; then
		    printf "\nContent of FILE_W_LUNS_CONFIG\n"
		    cat $FILE_W_LUNS_CONFIG
		    printf "\nContent of FILE_W_LUNS_CONFIG_S1\n"
		    cat $FILE_W_LUNS_CONFIG_S1
		    printf "\nContent of FILE_W_LUNS_CONFIG_S2\n"
		    cat $FILE_W_LUNS_CONFIG_S2
		    printf "\nContent of FILE_W_LUNS_CONFIG_S3\n"
		    cat $FILE_W_LUNS_CONFIG_S3
		    printf "\nContent of FILE_W_LUNS_CONFIG_S4\n"
		    cat $FILE_W_LUNS_CONFIG_S4
		    printf "\nContent of FILE_W_LUNS_IDS\n"
		    cat $FILE_W_LUNS_IDS
		    printf "\n-----------------------------\n"
                fi

                if [ "$(cat $FILE_W_LUNS_CONFIG_S4 | wc -l)" -eq "$(sort -u $FILE_W_LUNS_CONFIG_S4 | wc -l)" ]; then
		    echo "OK: indexes are all unique: $GUEST"
                else
		    echo "ERROR: found duplicate indexes in config file of guest ${GUEST}:"
		    echo "Config file: $CONFIG"
		    sort $FILE_W_LUNS_CONFIG_S4 | uniq -c | grep -v ' 1 '
		    EXIT_CODE=1
                fi
		    
		if $CHECK_DISK_DETAILS; then

                    if ! diff $FILE_W_LUNS_CONFIG $FILE_W_LUNS_CONFIG_S1; then
			printf "\nERROR: order of LUNS is not in sync (see diff above) - $GUEST!!!\n"
			EXIT_CODE=1
                    fi

                    if ! diff $FILE_W_LUNS_IDS $FILE_W_LUNS_CONFIG_S3 >/dev/null; then
			# if there is a diff, it does not mean that we have an issue
			# Example: a local disk is used and diskinfo is not able to detect it properly
			BROKEN_CONFIG=true
			for LUN_ID in $(cat $FILE_W_LUNS )
			do 
                            if grep "$LUN_ID" $FILE_W_LUNS_SPECIAL >/dev/null; then
				BROKEN_CONFIG=false
				break
                            fi    
			done
			if $BROKEN_CONFIG; then
                            diff $FILE_W_LUNS_IDS $FILE_W_LUNS_CONFIG_S3    
                            printf "\nERROR: disk index in config file is not in sync with running guest $GUEST (see diff above)!!!\n"
                            echo "Guest: $GUEST"
                            echo "CONFIG: $CONFIG"
                            EXIT_CODE=1
			fi  
                    fi
		fi

		# to do: verifying multipathing groups
		
		# checking for issues caused by bug 21295190
		
                # check for devices based on a 'data' file located in the NFS or Local Storage Library
                for VOLNAME in $ALL_VOLNAMES
                do
		    $DEBUG && echo "VOLNAME = $VOLNAME"
		    PATH_TO_LUN=$(grep " $VOLNAME " $LDOM_VDS | nawk '{print $3}')
		    $DEBUG && echo "PATH_TO_LUN = $PATH_TO_LUN"

		    if echo $PATH_TO_LUN | grep '/virtdisk/data$' > /dev/null; then
                        # disk is a file in a NFS or Local Library
                        SHOULD_VIRTIMG_XML=$( echo $PATH_TO_LUN | sed 's@/virtdisk/data$@/virtimage.xml@' )
			if [ ! -f "$SHOULD_VIRTIMG_XML" ]; then
			    # Houston, we have an issue, we got hit by bug 21295190!
			    UUID=${PATH_TO_LUN: -50:36}
			    NO_OF_FOUND_VIRTIMG_XML=$( ls /guests/${UUID}/virtimage.xml /var/mnt/virtlibs/*/${UUID}/virtimage.xml 2>/dev/null | wc -l)
			    if [ "$NO_OF_FOUND_VIRTIMG_XML" -eq 1 ]; then
				echo
				echo "ERROR: Guest: $GUEST"
				echo "ERROR: Disk Devicepath: $PATH_TO_LUN"
				echo "ERROR: The metadata for the above disk got corrupted (probably due to bug 21295190)!"
				echo
				echo "The metadata is distributed across two different libraries:"
				echo
				find /guests/${UUID}/ /var/mnt/virtlibs/*/${UUID} 2>/dev/null
			    else
				echo
				echo "ERROR: Guest: $GUEST"
				echo "ERROR: Disk Devicepath: $PATH_TO_LUN"
				echo "ERROR: The file virtimage.xml is missing in the metadata for this disk device!"
			    fi
			    echo
			    echo "WARNING: Do not shutdown and detach the Guest from the UI, it will become unstartable!!!"
			    echo
			    echo "Open a Service Request to get the issue resolved!"
			    echo
			    EXIT_CODE=1
			fi
		    fi
                done

                DISK_COUNT=$($LDM_lp $GUEST | grep -c ^VDISK) 
                CFG_DISK_1=$(grep -c '<vdisk>' $CONFIG)
                CFG_DISK_2=$(grep -c '<source>' $CONFIG)
		# for testing purposes only
		# DISK_COUNT=0
                if [ $DISK_COUNT -eq $CFG_DISK_1 -a $DISK_COUNT -eq $CFG_DISK_2 ]; then
		    echo "OK: Number of used vdisks is in sync with listed disks in config file: $GUEST" 
                else
		    echo
		    echo "ERROR: Number of used vdisks is not in sync with listed disks in config file - $GUEST!"
		    echo "Config file is $CONFIG"
		    echo
		    echo "Used disks from LDom config:"
		    for VOLNAME in $(nawk '{print $1}' $LDM_LS_DISK_TMPOUT)
		    do
			grep " $VOLNAME " $LDOM_VDS
		    done
		    echo
		    echo "Listed vdisks in CONFIG file:"
		    grep '<vdisk>' $CONFIG
		    echo
		    echo "Listed disk devices in CONFIG file:"
		    grep '<source>' $CONFIG
		    echo
		    EXIT_CODE=1
                fi 

		# removed the check for vnics, it was causing false alarms in case of vnics with multiple VIDs
                NET_COUNT=$($LDM_lp $GUEST | grep -c ^VNET)
                #CFG_VNIC_COUNT=$(nawk '/<vnics>/, /<\/vnics>/' $CONFIG | grep -c '<name>')
                # we have to remove SR-IOV devices which are not listed in the vnet output (they are listed in he IO section)
                CFG_DATALINK_COUNT=$(nawk '/<data-links>/, /<\/data-links>/' $CONFIG | grep -v 'IOVNET' | grep -c '<name>')
		# for testing purposes only
		# NET_COUNT=0
		
                # if [ $NET_COUNT -eq $CFG_VNIC_COUNT -a $NET_COUNT -eq $CFG_DATALINK_COUNT ]; then
                if [ $NET_COUNT -eq $CFG_DATALINK_COUNT ]; then
                    echo "OK: Number of used vnets is in sync with listed NICs in config file: $GUEST"
                    echo
                else
                    echo
                    echo "ERROR: Number of used vnets is not in sync with listed data links in config file - $GUEST !"
                    echo "Config file is $CONFIG"
                    echo
                    echo "Used vnets from LDom config:"
                    $LDM ls -p -o network $GUEST | grep ^VNET | cut -d\| -f2
                    echo
                    #echo "Listed vnics in CONFIG file:"
		    #nawk '/<vnics>/, /<\/vnics>/' $CONFIG | grep '<name>'
                    #echo
                    echo "Listed data links in CONFIG file:"
		    nawk '/<data-links>/, /<\/data-links>/' $CONFIG | grep '<name>'
                    echo
                    EXIT_CODE=1
                fi
            done 
        else
            echo "INFO: This system is not a Control Domain, skipping check" 
        fi 
    else
        echo "INFO: This system is not a Control Domain, skipping check"
    fi

    rm -rf $TMPDIR
}

if $BESTIMPL
then
    check_bestimpl
elif $FIX
then
    fix_library_xml
elif $SERVERCFG
then
    check_servercfg
else
    check_lib
fi

rm -f ${LIBS}

exit $EXIT_CODE
