#!/usr/bin/bash
#
# v2
# --
# * Check if this is being run on an OC-based machine (AC/PC/VC)
# * If yes, extract version of that tier and show only relevant IDR available for download
# * If no, show all available versions and ask what needs to be downloaded
# * For both of the cases above, if unable to fetch IDR list, publish pointer to latest JIDR and exit.
#
# v1
# --
# Script to download latest IDR onto customer specified location
# 1. Check if this is EC or not. Shouldn't need to run on VC/PC.
# 2. Check if provided path is valid or not
# 3. Download!
#

IDR_LOCATION="https://updates.oracle.com/OCDoctor/EC-logs"
IDR_LIST_FILE="available_idr_list"
TMP_LIST_LOC="/tmp/list_file"
WGET_CMD="/usr/bin/wget -q -T 3 -t 2 --no-cache"
WGET="/usr/bin/wget"
PROXY_STRING=""
OC_VERS=""

isDisconnected() {

    $WGET_CMD ${PROXY_STRING} $IDR_LOCATION/$IDR_LIST_FILE -O $TMP_LIST_LOC

    if [ ! -s $TMP_LIST_LOC ]; then
        echo "[ERROR] Unable to contact updates.oracle.com to download JIDR(web proxy needed or disconnected mode)"
        echo "[ERROR] Please run 'OCDoctor.sh --get-latest-JIDR' from a server that can reach updates.oracle.com "
        echo "[ERROR] OR           "
        if [ -z "$OC_VERS" ]; then
            echo "[ERROR] Please read the MOS note for the appropriate JIDR to obtain the download URL."
        else
            echo "[ERROR] Please read the MOS note for the JIDR for $OC_VERS to obtain the download URL."
        fi
        exit
    fi
}

getOCVersion() {
    # Check only for EC/PC since agent can be at an earlier version than
    # EC or PC and isn't indicative of what version of OC is being run
    if [ -f /opt/SUNWxvmoc/bin/ecadm ]; then
        OC_VERS=`/opt/SUNWxvmoc/bin/ecadm -V`
        elif [ -f /opt/SUNWxvmoc/bin/proxyadm ]; then
        OC_VERS=`/opt/SUNWxvmoc/bin/proxyadm -V`
    else
        echo "[INFO] Unable to find OC version on this system."
    fi
}

readProxy() {

    if [ -z "$https_proxy" ]; then
        PROXYPORT=""
        PROXYUSER=""
        PROXYPASS=""
        PROXY=""
        echo "Does this system need a web proxy to connect to the Internet?"
        printf "Type the proxy server here, for example 'www-proxy.us.oracle.com' (hit Enter if there is no proxy): "
        read PROXY
        if [ ! "$PROXY" = "" ]; then
            printf "Proxy port: "
            read PROXYPORT
            export http_proxy=${PROXY}:${PROXYPORT}
            export https_proxy=${PROXY}:${PROXYPORT}
            PROXY_STRING=" --proxy -e https_proxy=${http_proxy} -e http_proxy=${http_proxy} "
            printf "Proxy username (Enter for no username): "
            read PROXYUSER
            if [ ! -z "$PROXYUSER" ]; then
                printf "Proxy password: "
                stty -echo
                read PROXYPASS
                stty echo
                PROXY_STRING=${PROXY_STRING}" --proxy-user=${PROXYUSER} --proxy-passwd=${PROXYPASS}"
            fi
        fi
    fi
}

get_JIDR() {

    # Non OC server with Internet connectivity
    if [ -z "$OC_VERS" ]; then
        printf "[INFO] Printing list of all available IDRs\n"
        cut -d'|' -f1,3 $TMP_LIST_LOC | sed 's/|/: /'
        printf "Enter number of JIDR which needs to be downloaded\n"
        read NUM


        NUM_IDRs=`wc -l $TMP_LIST_LOC | awk '{print $1}'`
        if [ $NUM -lt 1 -o $NUM -gt $NUM_IDRs ]; then
            echo "[ERROR] Invalid choice..Enter a number among the list shown above"
            echo "[ERROR] Exiting"
            exit
        fi
        line=`grep "^$NUM" $TMP_LIST_LOC`
    else
        # EC/PC/VC in Connected mode
        line=`grep "${OC_VERS}" $TMP_LIST_LOC | tail -1`
    fi
 
    if [ -z "$line" ]; then
        echo "[ERROR] No JIDR found for $OC_VERS. Exiting.."
        exit
    fi

    # Last field in metadata file..trim it
    IDR_SIZE=`echo $line | cut -d'|' -f4 | tr -d '\r\n'`
    IDR_NAME=`echo $line | cut -d'|' -f3 `

    echo
    echo "[INFO] The following JIDR will be downloaded: ${IDR_NAME}"
    echo "[INFO] This JIDR requires ${IDR_SIZE} of space"
    echo
    printf "Please enter absolute path of directory where JIDR can be stored: "

    read DOWN_DIR
    echo ""

    if [ ! -d $DOWN_DIR ]; then
        echo "[ERROR] $DOWN_DIR is not a valid directory path"
        echo "[ERROR] Exiting"
        exit
    fi

    echo "[INFO] $DOWN_DIR is where JIDR will be saved"
    printf "Press 'y' to confirm, 'n' to abort: "
    read CONF
    if [ "$CONF" = "n" ]; then
        echo "Exiting"
        exit
    fi

    IDR_PATH=`echo $line | cut -d'|' -f2`

    echo
    echo "[INFO] Downloading IDR $IDR_NAME to $DOWN_DIR.."
    $WGET_CMD ${PROXY_STRING} $IDR_LOCATION/$IDR_PATH -c --show-progress -P $DOWN_DIR
    status=$?
    if [ $status -eq 0 ]; then
        echo "[INFO] Download complete!"
    else
        echo "[ERROR] Unable to download JIDR bundle. Error: $status"
    fi
}

###### MAIN ######
# Spacing after the OCDoctor version banner
echo

# No wget, no JIDR
if [ ! -f $WGET ]; then
    echo
    echo "[ERROR] Couldn't find $WGET, can't download JIDR without it"
    echo "[ERROR] Exiting"
    exit
fi

# Set http proxy if needed
readProxy

# Check if running on an OC machine
getOCVersion

if [ ! -z "$OC_VERS" ]; then
    echo "[INFO] Detected OC software running at version: $OC_VERS"
fi

# Check if disconnected
isDisconnected

# Get the latest IDR
get_JIDR

rm $TMP_LIST_LOC
