#!/bin/bash

MY_DIR=`dirname $0`
source $MY_DIR/../../functions

UNAME=`uname`
if [ $UNAME = "Linux" ];then
	### 64 bit tuning is currently only available on Solaris
	echo "This java tuning is only available for Solaris"
	exit
fi

### Original Update 3 value: -server -Xmx2000m -Xms2000m -XX:PermSize=128m -XX:MaxPermSize=128m -Xss256k -XX:+UseParallelOldGC -Dsun.security.pkcs11.enable-solaris=false -Djava.endorsed.dirs=/usr/lib/cacao/lib/endorsed -Dxvmserver=false

### Only applies to 1177 and higher ###
get_version ""
if [ $VERSIONNUM -lt "2501177" ];then
        echo "OK - This 64 bit tuning is for newer versions only (Update 3 and above)"
        exit
fi

### Is the Java version the correct one?
JAVA="/usr/jdk/latest/bin/java"
if [ ! -f $JAVA ]; then
	echo "ERROR - couldn't find $JAVA, can't enable 64 bit"
	echo ""
	exit
fi
VERSION=`$JAVA -version 2>&1 | head -1 | awk -F"\"" '{print $2}'`
NEEDED="1.6.0_14"
if [ ! "$VERSION" = "$NEEDED" ]; then
	echo "ERROR - expected java version $NEEDED, but found the following:"
	echo ""
	echo "# $JAVA -version"
	$JAVA -version
	echo ""
	echo "If this is a newer version of Java, there are known issues with turning 64 bits on. Skipping..."
	echo ""
	exit
fi 


### Do we already have 64 bits enabled? ###
CACAOADM="/usr/sbin/cacaoadm"
JAVA_FLAGS=`$CACAOADM get-param java-flags -v | grep "\-d64"`
if [ ! "$JAVA_FLAGS" = "" ]; then
	echo "OK: Java is already enabled to use 64 bits, checking if any adjustments are needed..."
	JAVA64_ON=1
else
	echo "Analyzing memory for 64 bit support - this is recommended in environments larger than 200 agents as it allows using more memory."
	JAVA64_ON=0
fi



### MEM
MEMORY=$(prtconf | nawk '$1 == "Memory" {print $3}')

### SWAP
if swap -l 1>/dev/null 2>&1
then
	SWAP=$(( $(swap -l | nawk ' $1 != "swapfile" { SWAPSIZE = SWAPSIZE + $4 }; END {print SWAPSIZE}') / 2048 ))
else
	SWAP=0
fi

#MEMORY=16000
#SWAP=6000

echo "Memory found: ${MEMORY}M  Swap found: ${SWAP}M"


### Calculate ###
OSNEEDS=2800
MIN=2048
MAX=8192
MAX_INITIAL=4096

AVAIL_SWAP=`expr $SWAP - $OSNEEDS`
AVAIL_MEM=`expr $MEMORY - $OSNEEDS`

if [ $AVAIL_SWAP -gt $MAX ]; then
	AVAIL_SWAP=$MAX
fi
if [ $AVAIL_MEM -gt $MAX ]; then
	AVAIL_MEM=$MAX
fi

if [ $AVAIL_SWAP -lt $AVAIL_MEM ]; then
	MISSING_SWAP=`expr $MIN - $AVAIL_SWAP`
	RECOMMENDED_SWAP=`expr $AVAIL_MEM - $AVAIL_SWAP`
	if [ $AVAIL_SWAP -lt $MIN ]; then
		if [ $RECOMMENDED_SWAP -lt $MISSING_SWAP ]; then
			#### That means that we don't have enough memory either
			echo ""
			echo "Not enough RAM"
			echo ""
		else
			echo ""
			echo "NOTICE: In order to switch to 64 bit, you will need to increase the swap size with at least additional ${MISSING_SWAP}M, but additional ${RECOMMENDED_SWAP}M is recommended" 
			echo ""
		fi
	else
		echo ""
		echo "NOTICE: To better utilize 64 bit, consider increasing swap size with additional ${RECOMMENDED_SWAP}M and rerun this test to utilize that new space."
		echo ""
	fi
	AVAIL=$AVAIL_SWAP
else
	### If we have more swap than memory, or both swap and mem reached max 
	AVAIL=$AVAIL_MEM
	if [ $AVAIL -lt $MIN ]; then
		echo ""
		echo "Not enough RAM found (only ${MEMORY}M)"
		echo ""
	fi
fi	

echo "Virtual memory (Swap or RAM) available for the Java runtime environment: ${AVAIL}M"
echo

if [ $AVAIL -lt $MIN ]; then
	echo "NOTICE: Not enough available virtual memory to justify switching to 64 bits, skipping"
	echo
	exit
fi

INITIAL_MEM=$AVAIL
if [ $INITIAL_MEM -gt $MAX_INITIAL ]; then
	INITIAL_MEM=$MAX_INITIAL
fi


####### Figure out current command line #######
JAVA_FLAGS=`$CACAOADM get-param java-flags -v`
NEWFLAGS=""
ARR=$(echo $JAVA_FLAGS | tr " ")
for x in $ARR
do
	USED_X=0

	############## Xmx change #############
	OUTPUT=`echo $x | grep Xmx`
	if [ ! "$OUTPUT" = "" ]; then
		if [ "$NEWFLAGS" = "" ]; then
			NEWFLAGS="-Xmx${AVAIL}m"	
		else
			NEWFLAGS="$NEWFLAGS -Xmx${AVAIL}m"	
		fi
		USED_X=1
	fi
	############## Xms change #############
	OUTPUT=`echo $x | grep Xms`
	if [ ! "$OUTPUT" = "" ]; then
		if [ "$NEWFLAGS" = "" ]; then
			NEWFLAGS="-Xms${INITIAL_MEM}m"	
		else
			NEWFLAGS="$NEWFLAGS -Xms${INITIAL_MEM}m"	
		fi
		USED_X=1
	fi
	############## MaxPermSize change #############
	OUTPUT=`echo $x | grep "\:MaxPermSize\=128m"`
	if [ ! "$OUTPUT" = "" ]; then
		if [ "$NEWFLAGS" = "" ]; then
			NEWFLAGS="-XX:MaxPermSize=256m"	
		else
			NEWFLAGS="$NEWFLAGS -XX:MaxPermSize=256m"	
		fi
		USED_X=1
	fi
	############## Remove PermSize  #############
	OUTPUT=`echo $x | grep "\:PermSize\=128m"`
	if [ ! "$OUTPUT" = "" ]; then
		USED_X=1
	fi
	############## Remove d64 since we add it at the end anyways  #############
	OUTPUT=`echo $x | grep "d64"`
	if [ ! "$OUTPUT" = "" ]; then
		USED_X=1
	fi
	############# The rest #############
	if [ "$USED_X" = "0" ]; then
		if [ "$NEWFLAGS" = "" ]; then
			NEWFLAGS="$x"	
		else
			NEWFLAGS="$NEWFLAGS $x"	
		fi
	fi
done
NEWFLAGS="$NEWFLAGS -d64"


if [ "$JAVA_FLAGS" = "$NEWFLAGS" ];then
	echo "OK - all flags are set properly"
	echo
else
	if [ "$OCDOCTOR_FIX" = "1" ];then
	        echo "Configuring this automatically..."
		SAT_BIN="/opt/SUNWxvmoc/bin/satadm"
		STATUS=`$SAT_BIN status`
		if [ "$STATUS" = "online" ];then
			printf "Would you like to stop the Enterprise Controller now? Answering no will abort automatic 64 bit configuration (y/n): "
			read yesno
			if [ "$yesno" = "yes" ]; then
				yesno="y"
			fi
			if [ "$yesno" = "Y" ]; then
				yesno="y"
			fi
			if [ "$yesno" = "y" ]; then
				echo "--- Stopping cacao ---"
				$CACAOADM stop
				echo "--- Setting java-flags ---"
				$CACAOADM set-param java-flags="$NEWFLAGS"
				echo "--- Starting cacao ---"
				$CACAOADM start
				echo ""
				echo "Done."
				echo ""
			else
				echo "Skipping... try running without --fix to see the manual procedure"
				echo ""
				exit
			fi
		else
			echo "--- Setting java-flags ---"
			$CACAOADM set-param java-flags="$NEWFLAGS"
			echo ""
			echo "Done. Don't forget to start the Enterprise Controller when you're ready."
			echo ""
		fi

	else
		if [ "$JAVA64_ON" = "0" ]; then
			echo "To enable 64 bits, run the following:"
		else
			echo "Current flags used:"
			echo $JAVA_FLAGS
			echo ""
			echo "To readjust the Java flags to the new values, run the following:"
		fi
		echo ""
		echo "# $CACAOADM stop                                        - NOTICE - this will shutdown the Enterprise Controller"
		echo "# $CACAOADM set-param java-flags=\"$NEWFLAGS\""
		echo "# $CACAOADM start"
		echo ""
		echo "OR run again with --fix to automatically configure this:"
	        echo "# ./OCDoctor.sh --tuning --fix"
		echo
	fi
fi


